@@ -5,7 +5,7 @@ dist_path = Path('dist')
 install_path = Path('install')
 
 wheel_file = next(dist_path.glob('*.whl'))
-sdk_name = wheel_file.stem.replace('vtk-', 'vtk-wheel-sdk-')
+sdk_name = wheel_file.stem.replace('-', '-wheel-sdk-', 1)
 tarball_name = f'{sdk_name}.tar.xz'
 
 install_path.rename(sdk_name)
