@@ -29,10 +29,19 @@ yum install -y --setopt=install_weak_deps=False \
     tbb-devel libiodbc-devel libarchive-devel freeglut-devel sqlite-devel \
     fontconfig-devel
 
-# Python dependencies
+# Python 3.8 via scl
 yum install -y --setopt=install_weak_deps=False \
-    python3 python3-devel python3-numpy \
-    python3-pip python3-mpi4py-mpich python3-mpi4py-openmpi python3-matplotlib
+    centos-release-scl
+
+yum install -y --setopt=install_weak_deps=False \
+    rh-python38-python rh-python38-python-devel \
+    rh-python38-python-pip rh-python38-python-numpy
+
+# Install a venv which provides `matplotlib`
+scl enable rh-python38 -- python3 -m venv /opt/python38/venv
+/opt/python38/venv/bin/pip install matplotlib
+# Ignore mpi4py for now as the CI job doesn't build with MPI anyways.
+# scl enable rh-python38 -- /opt/python38/venv/bin/pip install mpi4py
 
 # EPEL for more tools
 yum install -y --setopt=install_weak_deps=False \
