@@ -1,7 +1,7 @@
 # Put all files into a single directory.
-ls release/
 mv -v build/dist/*.whl release/
 mv -v build/*-wheel-sdk-*.tar.xz release/
+ls release/
 
 # Print the hashes of everything to be uploaded.
 sha256sum release/*
@@ -13,5 +13,5 @@ minor="$( sed -ne '/set.VTK_MINOR_VERSION/s/.* \([0-9]*\))/\1/p' CMake/vtkVersio
 readonly minor
 
 # Tell the rest of the job about the destination.
-RSYNC_DESTINATION="vtk_release/$major.$minor/"
+RSYNC_DESTINATION="$major.$minor/"
 export RSYNC_DESTINATION
