@@ -278,15 +278,19 @@ void pqRenderViewSelectionReaction::setRepresentation(pqDataRepresentation* repr
 
     if (this->Representation != nullptr)
     {
-      QObject::disconnect(this->RepresentationConnection);
+      this->disconnect(this->Representation);
+      this->Representation = nullptr;
     }
 
-    this->Representation = representation;
-
-    if (this->Representation != nullptr)
+    if (representation)
     {
-      this->RepresentationConnection = this->connect(
-        this->Representation, SIGNAL(colorArrayNameModified()), SLOT(updateEnableState()));
+      this->Representation = representation;
+
+      if (this->Representation != nullptr)
+      {
+        QObject::connect(this->Representation, &pqDataRepresentation::colorArrayNameModified, this,
+          &pqRenderViewSelectionReaction::updateEnableState);
+      }
     }
 
     // update enable state.
@@ -335,6 +339,7 @@ void pqRenderViewSelectionReaction::beginSelection()
     case SELECT_FRUSTUM_CELLS:
     case SELECT_FRUSTUM_POINTS:
     case SELECT_BLOCKS:
+    case SELECT_FRUSTUM_BLOCKS:
     case SELECT_CUSTOM_BOX:
       this->View->setCursor(Qt::CrossCursor);
       vtkSMPropertyHelper(rmp, "InteractionMode").Set(vtkPVRenderView::INTERACTION_MODE_SELECTION);
@@ -567,6 +572,10 @@ void pqRenderViewSelectionReaction::selectionChanged(vtkObject*, unsigned long,
       this->View->selectBlock(region, selectionModifier);
       break;
 
+    case SELECT_FRUSTUM_BLOCKS:
+      this->View->selectFrustumBlocks(region, selectionModifier);
+      break;
+
     case SELECT_CUSTOM_BOX:
       Q_EMIT this->selectedCustomBox(region);
       Q_EMIT this->selectedCustomBox(region[0], region[1], region[2], region[3]);
