@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkChartXYZ.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkChartXYZ.h"
 
@@ -45,6 +33,7 @@
 #include <list>
 #include <sstream>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkChartXYZ);
 
 //------------------------------------------------------------------------------
@@ -1222,11 +1211,11 @@ bool vtkChartXYZ::Rotate(const vtkContextMouseEvent& mouse)
   }
 
   // Figure out how much the mouse has moved in plot coordinates
-  vtkVector2d screenPos(mouse.GetScreenPos().Cast<double>().GetData());
-  vtkVector2d lastScreenPos(mouse.GetLastScreenPos().Cast<double>().GetData());
+  vtkVector2d scenePos(mouse.GetScenePos().Cast<double>().GetData());
+  vtkVector2d lastScenePos(mouse.GetLastScenePos().Cast<double>().GetData());
 
-  double dx = screenPos[0] - lastScreenPos[0];
-  double dy = screenPos[1] - lastScreenPos[1];
+  double dx = scenePos[0] - lastScenePos[0];
+  double dy = scenePos[1] - lastScenePos[1];
 
   double delta_elevation = -20.0 / this->Scene->GetSceneHeight();
   double delta_azimuth = -20.0 / this->Scene->GetSceneWidth();
@@ -1245,7 +1234,7 @@ bool vtkChartXYZ::Rotate(const vtkContextMouseEvent& mouse)
 }
 
 //------------------------------------------------------------------------------
-bool vtkChartXYZ::Rotate(const RotateDirection rotateDirection)
+bool vtkChartXYZ::Rotate(RotateDirection rotateDirection)
 {
   if (this->Scene->GetSceneHeight() == 0 || this->Scene->GetSceneWidth() == 0)
   {
@@ -1278,11 +1267,11 @@ bool vtkChartXYZ::Rotate(const RotateDirection rotateDirection)
 bool vtkChartXYZ::Pan(const vtkContextMouseEvent& mouse)
 {
   // Figure out how much the mouse has moved in plot coordinates
-  vtkVector2d screenPos(mouse.GetScreenPos().Cast<double>().GetData());
-  vtkVector2d lastScreenPos(mouse.GetLastScreenPos().Cast<double>().GetData());
+  vtkVector2d scenePos(mouse.GetScenePos().Cast<double>().GetData());
+  vtkVector2d lastScenePos(mouse.GetLastScenePos().Cast<double>().GetData());
 
-  double dx = (screenPos[0] - lastScreenPos[0]);
-  double dy = (screenPos[1] - lastScreenPos[1]);
+  double dx = (scenePos[0] - lastScenePos[0]);
+  double dy = (scenePos[1] - lastScenePos[1]);
 
   this->Translation->Translate(dx, dy, 0.0);
 
@@ -1297,13 +1286,13 @@ bool vtkChartXYZ::Pan(const vtkContextMouseEvent& mouse)
 bool vtkChartXYZ::Zoom(const vtkContextMouseEvent& mouse)
 {
   // Figure out how much the mouse has moved and scale accordingly
-  vtkVector2d screenPos(mouse.GetScreenPos().Cast<double>().GetData());
-  vtkVector2d lastScreenPos(mouse.GetLastScreenPos().Cast<double>().GetData());
+  vtkVector2d scenePos(mouse.GetScenePos().Cast<double>().GetData());
+  vtkVector2d lastScenePos(mouse.GetLastScenePos().Cast<double>().GetData());
 
   float delta = 0.0f;
   if (this->Scene->GetSceneHeight() > 0)
   {
-    delta = static_cast<float>(mouse.GetLastScreenPos()[1] - mouse.GetScreenPos()[1]) /
+    delta = static_cast<float>(mouse.GetLastScenePos()[1] - mouse.GetScenePos()[1]) /
       this->Scene->GetSceneHeight();
   }
 
@@ -1322,11 +1311,11 @@ bool vtkChartXYZ::Zoom(const vtkContextMouseEvent& mouse)
 bool vtkChartXYZ::Spin(const vtkContextMouseEvent& mouse)
 {
   // Figure out how much the mouse has moved in plot coordinates
-  vtkVector2d screenPos(mouse.GetScreenPos().Cast<double>().GetData());
-  vtkVector2d lastScreenPos(mouse.GetLastScreenPos().Cast<double>().GetData());
+  vtkVector2d scenePos(mouse.GetScenePos().Cast<double>().GetData());
+  vtkVector2d lastScenePos(mouse.GetLastScenePos().Cast<double>().GetData());
 
-  double newAngle = vtkMath::DegreesFromRadians(atan2(screenPos[1], screenPos[0]));
-  double oldAngle = vtkMath::DegreesFromRadians(atan2(lastScreenPos[1], lastScreenPos[0]));
+  double newAngle = vtkMath::DegreesFromRadians(atan2(scenePos[1], scenePos[0]));
+  double oldAngle = vtkMath::DegreesFromRadians(atan2(lastScenePos[1], lastScenePos[0]));
 
   this->Rotation->RotateZ(-(newAngle - oldAngle));
 
@@ -1340,8 +1329,8 @@ bool vtkChartXYZ::Spin(const vtkContextMouseEvent& mouse)
 //------------------------------------------------------------------------------
 bool vtkChartXYZ::KeyPressEvent(const vtkContextKeyEvent& evt)
 {
-  std::string key = evt.GetInteractor()->GetKeySym();
-
+  char* ckey = evt.GetInteractor()->GetKeySym();
+  std::string key = ckey != nullptr ? ckey : "";
   if (key == "x")
   {
     this->LookDownX();
@@ -1974,3 +1963,4 @@ void vtkChartXYZ::SetClippingPlanesEnabled(bool v)
     this->Scene->SetDirty(true);
   }
 }
+VTK_ABI_NAMESPACE_END
