@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkScatterPlotMatrix.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkScatterPlotMatrix.h"
 
@@ -23,6 +11,7 @@
 #include "vtkChartXYZ.h"
 #include "vtkCommand.h"
 #include "vtkContext2D.h"
+#include "vtkContextActor.h"
 #include "vtkContextMouseEvent.h"
 #include "vtkContextScene.h"
 #include "vtkDataSetAttributes.h"
@@ -36,7 +25,9 @@
 #include "vtkPlotPoints.h"
 #include "vtkPlotPoints3D.h"
 #include "vtkPoints2D.h"
+#include "vtkRenderWindow.h"
 #include "vtkRenderWindowInteractor.h"
+#include "vtkRenderer.h"
 #include "vtkStdString.h"
 #include "vtkStringArray.h"
 #include "vtkTable.h"
@@ -50,6 +41,7 @@
 #include <map>
 #include <vector>
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkScatterPlotMatrix::PIMPL
 {
 public:
@@ -78,6 +70,10 @@ public:
     this->SelectedChartBGBrush->SetColor(0, 204, 0, 102);
     this->SelectedRowColumnBGBrush->SetColor(204, 0, 0, 102);
     this->TooltipItem = vtkSmartPointer<vtkTooltipItem>::New();
+
+    this->BigChart3DRenderer->AddActor(this->BigChart3DActor);
+    this->BigChart3DRenderer->SetBackground(1.0, 1.0, 1.0);
+    this->BigChart3DActor->GetScene()->SetRenderer(this->BigChart3DRenderer);
   }
 
   ~PIMPL()
@@ -105,6 +101,15 @@ public:
     std::string title;
   };
 
+  enum class AnimationPhaseEnum
+  {
+    Ready,
+    Start,
+    Rotate,
+    Stop,
+    Finalize
+  };
+
   class pimplChartSetting
   {
   public:
@@ -204,13 +209,15 @@ public:
   bool AnimationCallbackInitialized;
   unsigned long int TimerId;
   bool TimerCallbackInitialized;
-  int AnimationPhase;
+  AnimationPhaseEnum AnimationPhase;
   float CurrentAngle;
   float IncAngle;
   float FinalAngle;
   vtkVector2i NextActivePlot;
 
   vtkNew<vtkChartXYZ> BigChart3D;
+  vtkNew<vtkContextActor> BigChart3DActor;
+  vtkNew<vtkRenderer> BigChart3DRenderer;
   vtkNew<vtkAxis> TestAxis; // Used to get ranges/number of ticks
   vtkSmartPointer<vtkTooltipItem> TooltipItem;
   vtkSmartPointer<vtkStringArray> IndexedLabelsArray;
@@ -219,7 +226,7 @@ public:
 namespace
 {
 
-// This is just here for now - quick and dirty historgram calculations...
+// This is just here for now - quick and dirty histogram calculations...
 bool PopulateHistograms(vtkTable* input, vtkTable* output, vtkStringArray* s, int NumberOfBins)
 {
   // The output table will have the twice the number of columns, they will be
@@ -397,6 +404,11 @@ void vtkScatterPlotMatrix::Update()
 
 bool vtkScatterPlotMatrix::Paint(vtkContext2D* painter)
 {
+  // Do not paint ourselves in the rotation phase.
+  if (this->Private->AnimationPhase == PIMPL::AnimationPhaseEnum::Rotate)
+  {
+    return false;
+  }
   this->CurrentPainter = painter;
   this->Update();
   bool ret = this->Superclass::Paint(painter);
@@ -633,7 +645,7 @@ void vtkScatterPlotMatrix::StartAnimation(vtkRenderWindowInteractor* interactor)
     // This defines the interval at which the animation will proceed. 25Hz?
     this->Private->TimerId = interactor->CreateRepeatingTimer(1000 / 50);
     this->Private->AnimationIter = this->Private->AnimationPath.begin();
-    this->Private->AnimationPhase = 0;
+    this->Private->AnimationPhase = PIMPL::AnimationPhaseEnum::Ready;
   }
 }
 
@@ -643,23 +655,23 @@ void vtkScatterPlotMatrix::AdvanceAnimation()
 
   // 1: Remove decoration from the big chart.
   // 2: Set three dimensions to plot in the BigChart3D.
-  // 3: Make BigChart invisible, and BigChart3D visible.
+  // 3: Make BigChart invisible, and BigChart3D visible. Turn off erase for main scene renderer.
   // 4: Rotate between the two dimensions we are transitioning between.
   //    -> Loop from start to end angle to complete the effect.
   // 5: Make the new dimensionality active, update BigChart.
-  // 5: Make BigChart3D invisible and BigChart visible.
+  // 5: Make BigChart3D invisible and BigChart visible. Turn on erase for the main scene renderer.
   // 6: Stop the timer.
   this->InvokeEvent(vtkCommand::AnimationCueTickEvent);
+  auto renWin = this->Scene->GetRenderer()->GetRenderWindow();
   switch (this->Private->AnimationPhase)
   {
-    case 0: // Remove decoration from the big chart, load up the 3D chart
+    case PIMPL::AnimationPhaseEnum::Ready: // Remove decoration from the big chart, load up the 3D
+                                           // chart
     {
       this->Private->NextActivePlot = *this->Private->AnimationIter;
       vtkChartXYZ* chart = this->Private->BigChart3D;
-      chart->SetVisible(false);
       chart->SetAutoRotate(true);
       chart->SetDecorateAxes(false);
-      chart->SetFitToScene(false);
 
       int yColumn = this->GetSize().GetY() - this->ActivePlot.GetY() - 1;
       bool isX = false;
@@ -669,6 +681,22 @@ void vtkScatterPlotMatrix::AdvanceAnimation()
       float zSize;
       this->Private->FinalAngle = 90.0;
 
+      double viewport[4] = {};
+      float chart3dSize[4] = { 0, 0, size.GetWidth(), size.GetHeight() };
+      // stretch and squeeze the viewport in x and y directions in order
+      // to accommodate the 3D chart in the rotation phase.
+      viewport[0] = size.GetX() / this->Scene->GetViewWidth();
+      viewport[1] = size.GetY() / this->Scene->GetViewHeight();
+      viewport[2] = (size.GetX() + size.GetWidth() + this->Gutter.GetX() + this->Borders[2]) /
+        this->Scene->GetViewWidth();
+      viewport[3] = (size.GetY() + size.GetHeight() + this->Gutter.GetY() + this->Borders[3]) /
+        this->Scene->GetViewHeight();
+      // DO NOT MODIFY. These magic numbers were found by trial and error to
+      // position the chart correctly so that the 3D axes are not clipped out
+      // of the 3D viewport during animation.
+      const float scaleFactor = 0.08;
+      const float orthogonalScaleFactor = 0.008;
+
       if (this->Private->NextActivePlot.GetY() == this->ActivePlot.GetY())
       {
         // Horizontal move.
@@ -676,6 +704,10 @@ void vtkScatterPlotMatrix::AdvanceAnimation()
         isX = false;
         this->Private->IncAngle = this->Private->FinalAngle / this->NumberOfFrames;
         zSize = size.GetWidth();
+        chart3dSize[0] = scaleFactor * this->Scene->GetSceneWidth();
+        chart3dSize[1] = orthogonalScaleFactor * this->Scene->GetSceneHeight();
+        viewport[0] -= scaleFactor;
+        viewport[1] -= orthogonalScaleFactor;
       }
       else
       {
@@ -684,9 +716,15 @@ void vtkScatterPlotMatrix::AdvanceAnimation()
         isX = true;
         this->Private->IncAngle = -this->Private->FinalAngle / this->NumberOfFrames;
         zSize = size.GetHeight();
+        chart3dSize[0] = orthogonalScaleFactor * this->Scene->GetSceneWidth();
+        chart3dSize[1] = scaleFactor * this->Scene->GetSceneHeight();
+        viewport[0] -= orthogonalScaleFactor;
+        viewport[1] -= scaleFactor;
       }
       chart->SetAroundX(isX);
-      chart->SetGeometry(size);
+      chart->SetGeometry(vtkRectf{ chart3dSize });
+      // show the 3d renderer in place of the 2d chart.
+      this->Private->BigChart3DRenderer->SetViewport(viewport);
 
       std::string names[3];
       names[0] = this->VisibleColumns->GetValue(this->ActivePlot.GetX());
@@ -709,19 +747,25 @@ void vtkScatterPlotMatrix::AdvanceAnimation()
       }
       chart->RecalculateTransform();
       this->GetScene()->SetDirty(true);
-      ++this->Private->AnimationPhase;
+      this->Private->AnimationPhase = PIMPL::AnimationPhaseEnum::Start;
       return;
     }
-    case 1: // Make BigChart invisible, and BigChart3D visible.
+    case PIMPL::AnimationPhaseEnum::Start: // Make BigChart invisible, and BigChart3D visible.
       this->Private->BigChart->SetVisible(false);
-      this->AddItem(this->Private->BigChart3D);
+      // add 3d chart to 3d scene.
+      this->Private->BigChart3DActor->GetScene()->AddItem(this->Private->BigChart3D);
+      // add 3d renderer to render window.
+      renWin->AddRenderer(this->Private->BigChart3DRenderer);
+      // DO NOT ERASE main scene renderer. Allow the 2D scatter plots to remain on the color
+      // attachment. we don't want to draw those unncessarily during animation because it tanks
+      // framerate.
+      this->Scene->GetRenderer()->EraseOff();
       this->Private->CurrentAngle = 0.0;
       this->Private->BigChart3D->SetAngle(0.0);
-      this->Private->BigChart3D->SetVisible(true);
       this->GetScene()->SetDirty(true);
-      ++this->Private->AnimationPhase;
+      this->Private->AnimationPhase = PIMPL::AnimationPhaseEnum::Rotate;
       return;
-    case 2: // Rotation of the 3D chart from start to end angle.
+    case PIMPL::AnimationPhaseEnum::Rotate: // Rotation of the 3D chart from start to end angle.
       if (fabs(this->Private->CurrentAngle) < (this->Private->FinalAngle - 0.001))
       {
         this->Private->CurrentAngle += this->Private->IncAngle;
@@ -729,26 +773,29 @@ void vtkScatterPlotMatrix::AdvanceAnimation()
       }
       else
       {
-        ++this->Private->AnimationPhase;
+        this->Private->AnimationPhase = PIMPL::AnimationPhaseEnum::Stop;
       }
-      this->GetScene()->SetDirty(true);
       return;
-    case 3: // Transition to new dimensionality, update the big chart.
+    case PIMPL::AnimationPhaseEnum::Stop: // Transition to new dimensionality, update the big chart.
+      this->Scene->GetRenderer()->EraseOn();
       this->SetActivePlot(this->Private->NextActivePlot);
       this->Private->BigChart->Update();
       this->GetScene()->SetDirty(true);
-      ++this->Private->AnimationPhase;
+      this->Private->AnimationPhase = PIMPL::AnimationPhaseEnum::Finalize;
       break;
-    case 4:
+    case PIMPL::AnimationPhaseEnum::Finalize:
+      // remove 3d chart from 3d scene.
+      this->Private->BigChart3DActor->GetScene()->RemoveItem(this->Private->BigChart3D);
+      // remove 3d renderer from render window.
+      renWin->RemoveRenderer(this->Private->BigChart3DRenderer);
+      // turn on erase for the main scene.
       this->GetScene()->SetDirty(true);
       ++this->Private->AnimationIter;
       // Clean up - we are done.
-      this->Private->AnimationPhase = 0;
+      this->Private->AnimationPhase = PIMPL::AnimationPhaseEnum::Ready;
       if (this->Private->AnimationIter == this->Private->AnimationPath.end())
       {
         this->Private->BigChart->SetVisible(true);
-        this->RemoveItem(this->Private->BigChart3D);
-        this->Private->BigChart3D->SetVisible(false);
         this->Private->Interactor->DestroyTimer(this->Private->TimerId);
         this->Private->TimerId = 0;
         this->Private->TimerCallbackInitialized = false;
@@ -1664,7 +1711,7 @@ void vtkScatterPlotMatrix::SetGridVisibility(int plotType, bool visible)
   if (plotType != NOPLOT)
   {
     this->Private->ChartSettings[plotType]->ShowGrid = visible;
-    // How to update
+    this->ActivePlotValid = false;
     this->Modified();
   }
 }
@@ -1675,7 +1722,7 @@ void vtkScatterPlotMatrix::SetGridColor(int plotType, const vtkColor4ub& color)
   if (plotType >= 0 && plotType < vtkScatterPlotMatrix::NOPLOT)
   {
     this->Private->ChartSettings[plotType]->GridColor = color;
-    // How to update
+    this->ActivePlotValid = false;
     this->Modified();
   }
 }
@@ -1686,7 +1733,7 @@ void vtkScatterPlotMatrix::SetAxisLabelVisibility(int plotType, bool visible)
   if (plotType != NOPLOT)
   {
     this->Private->ChartSettings[plotType]->ShowAxisLabels = visible;
-    // How to update
+    this->ActivePlotValid = false;
     this->Modified();
   }
 }
@@ -1697,7 +1744,7 @@ void vtkScatterPlotMatrix::SetAxisLabelNotation(int plotType, int notation)
   if (plotType != NOPLOT)
   {
     this->Private->ChartSettings[plotType]->LabelNotation = notation;
-    // How to update
+    this->ActivePlotValid = false;
     this->Modified();
   }
 }
@@ -1708,7 +1755,7 @@ void vtkScatterPlotMatrix::SetAxisLabelPrecision(int plotType, int precision)
   if (plotType != NOPLOT)
   {
     this->Private->ChartSettings[plotType]->LabelPrecision = precision;
-    // How to update
+    this->ActivePlotValid = false;
     this->Modified();
   }
 }
@@ -1719,7 +1766,7 @@ void vtkScatterPlotMatrix::SetTooltipNotation(int plotType, int notation)
   if (plotType != NOPLOT)
   {
     this->Private->ChartSettings[plotType]->TooltipNotation = notation;
-    // How to update
+    this->ActivePlotValid = false;
     this->Modified();
   }
 }
@@ -1730,7 +1777,7 @@ void vtkScatterPlotMatrix::SetTooltipPrecision(int plotType, int precision)
   if (plotType != NOPLOT)
   {
     this->Private->ChartSettings[plotType]->TooltipPrecision = precision;
-    // How to update
+    this->ActivePlotValid = false;
     this->Modified();
   }
 }
@@ -1795,6 +1842,7 @@ void vtkScatterPlotMatrix::UpdateChartSettings(int plotType)
       this->Private->BigChart->GetAxis(vtkAxis::RIGHT), this->Private->ChartSettings[ACTIVEPLOT]);
     this->Private->UpdateChart(this->Private->BigChart, this->Private->ChartSettings[ACTIVEPLOT]);
     this->Private->BigChart->SetSelectionMode(this->SelectionMode);
+    this->ActivePlotValid = false;
   }
   this->Modified();
 }
@@ -1975,3 +2023,4 @@ void vtkScatterPlotMatrix::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Title: " << this->Title << endl;
   os << indent << "SelectionMode: " << this->SelectionMode << endl;
 }
+VTK_ABI_NAMESPACE_END
