@@ -7,6 +7,9 @@
 #include "vtkDataArray.h"
 #include "vtkFloatArray.h"
 #include "vtkSOADataArrayTemplate.h"
+#include "vtkSmartPointer.h"
+#include "vtkTypeInt32Array.h"
+#include "vtkTypedDataArray.h"
 #ifdef VTK_USE_SCALED_SOA_ARRAYS
 #include "vtkScaledSOADataArrayTemplate.h"
 #endif
@@ -148,7 +151,7 @@ void TestIota(Range range)
 // TupleRange:
 //==============================================================================
 //==============================================================================
-template <typename ArrayType>
+template <typename ArrayType, typename ForceValueTypeForVtkDataArray>
 struct UnitTestValueRangeAPI
 {
   static constexpr vtk::ComponentIdType NumComps = 3;
@@ -162,16 +165,20 @@ struct UnitTestValueRangeAPI
     array->SetNumberOfComponents(NumComps);
 
     this->TestEmptyRange(vtk::DataArrayValueRange(array));
-    this->TestEmptyRange(vtk::DataArrayValueRange(da));
-    this->TestEmptyRange(vtk::DataArrayValueRange<NumComps>(array));
-    this->TestEmptyRange(vtk::DataArrayValueRange<NumComps>(da));
+    this->TestEmptyRange(
+      vtk::DataArrayValueRange<vtk::detail::DynamicTupleSize, ForceValueTypeForVtkDataArray>(da));
+    this->TestEmptyRange(vtk::DataArrayValueRange(array));
+    this->TestEmptyRange(vtk::DataArrayValueRange<NumComps, ForceValueTypeForVtkDataArray>(da));
 
     array->SetNumberOfTuples(this->NumTuples);
 
     this->TestEmptyRange(vtk::DataArrayValueRange(array, 4, 4));
-    this->TestEmptyRange(vtk::DataArrayValueRange(da, 4, 4));
-    this->TestEmptyRange(vtk::DataArrayValueRange<NumComps>(array, 4, 4));
-    this->TestEmptyRange(vtk::DataArrayValueRange<NumComps>(da, 4, 4));
+    this->TestEmptyRange(
+      vtk::DataArrayValueRange<vtk::detail::DynamicTupleSize, ForceValueTypeForVtkDataArray>(
+        da, 4, 4));
+    this->TestEmptyRange(vtk::DataArrayValueRange(array, 4, 4));
+    this->TestEmptyRange(
+      vtk::DataArrayValueRange<NumComps, ForceValueTypeForVtkDataArray>(da, 4, 4));
 
     FillValueRangeIota(vtk::DataArrayValueRange<NumComps>(array));
 
@@ -183,7 +190,8 @@ struct UnitTestValueRangeAPI
       DispatchRangeTests<ArrayType, vtk::detail::DynamicTupleSize>(range, array, 0, NumValues);
     }
     { // Full, dynamic-size, generic-typed range
-      auto range = vtk::DataArrayValueRange(da);
+      auto range =
+        vtk::DataArrayValueRange<vtk::detail::DynamicTupleSize, ForceValueTypeForVtkDataArray>(da);
       DispatchRangeTests<vtkDataArray, vtk::detail::DynamicTupleSize>(range, array, 0, NumValues);
     }
     { // Full, fixed-size, real typed range
@@ -191,7 +199,7 @@ struct UnitTestValueRangeAPI
       DispatchRangeTests<ArrayType, NumComps>(range, array, 0, NumValues);
     }
     { // Full, fixed-size, generic-typed range
-      auto range = vtk::DataArrayValueRange<NumComps>(da);
+      auto range = vtk::DataArrayValueRange<NumComps, ForceValueTypeForVtkDataArray>(da);
       DispatchRangeTests<vtkDataArray, NumComps>(range, array, 0, NumValues);
     }
     { // Partial, dynamic-size, real typed range
@@ -199,7 +207,9 @@ struct UnitTestValueRangeAPI
       DispatchRangeTests<ArrayType, vtk::detail::DynamicTupleSize>(range, array, pStart, pEnd);
     }
     { // Partial, dynamic-size, generic-typed range
-      auto range = vtk::DataArrayValueRange(da, pStart, pEnd);
+      auto range =
+        vtk::DataArrayValueRange<vtk::detail::DynamicTupleSize, ForceValueTypeForVtkDataArray>(
+          da, pStart, pEnd);
       DispatchRangeTests<vtkDataArray, vtk::detail::DynamicTupleSize>(range, array, pStart, pEnd);
     }
     { // Partial, fixed-size, real typed range
@@ -207,7 +217,8 @@ struct UnitTestValueRangeAPI
       DispatchRangeTests<ArrayType, NumComps>(range, array, pStart, pEnd);
     }
     { // Partial, fixed-size, generic-typed range
-      auto range = vtk::DataArrayValueRange<NumComps>(da, pStart, pEnd);
+      auto range =
+        vtk::DataArrayValueRange<NumComps, ForceValueTypeForVtkDataArray>(da, pStart, pEnd);
       DispatchRangeTests<vtkDataArray, NumComps>(range, array, pStart, pEnd);
     }
   }
@@ -253,6 +264,8 @@ struct UnitTestValueRangeAPI
     CHECK_EQUAL(range.cend() - range.cbegin(), range.size());
     CHECK_EQUAL_NODUMP(*range.begin(), range[0]);
     CHECK_EQUAL_NODUMP(*(range.begin() + 1), range[1]);
+    CHECK_EQUAL_NODUMP(reinterpret_cast<std::intptr_t>(range.data()),
+      reinterpret_cast<std::intptr_t>(array->GetVoidPointer(0)));
 
     TestIota(range);
   }
@@ -262,11 +275,11 @@ struct UnitTestValueRangeAPI
   {
     using ConstRange = typename std::add_const<Range>::type;
     using MutableRange = typename std::remove_const<Range>::type;
+    using ActualValueType = vtk::GetAPIType<RangeArrayType, ForceValueTypeForVtkDataArray>;
     (void)range; // decltype doesn't actually count as a usage.
 
     CHECK_IS_BASE_TYPE_OF(typename Range::ArrayType, RangeArrayType);
-    CHECK_TYPEDEF(typename Range::ValueType, vtk::GetAPIType<RangeArrayType>);
-    CHECK_TYPEDEF(typename Range::ValueType, vtk::GetAPIType<RangeArrayType>);
+    CHECK_TYPEDEF(typename Range::ValueType, ActualValueType);
     CHECK_TYPEDEF(typename Range::size_type, vtk::ValueIdType);
     CHECK_TYPEDEF(typename Range::size_type, decltype(range.size()));
     CHECK_TYPEDEF(typename Range::iterator, decltype(std::declval<MutableRange>().begin()));
@@ -280,6 +293,8 @@ struct UnitTestValueRangeAPI
     CHECK_TYPEDEF(typename Range::ArrayType, decltype(*range.GetArray()));
     CHECK_TYPEDEF(vtk::ValueIdType, decltype(range.GetBeginValueId()));
     CHECK_TYPEDEF(vtk::ValueIdType, decltype(range.GetEndValueId()));
+    CHECK_TYPEDEF(
+      typename Range::ValueType, typename vtk::detail::StripPointers<decltype(range.data())>::type);
 
     static_assert(Range::TupleSizeTag == RangeTupleSize, "Range::TupleSizeTag incorrect.");
   }
@@ -326,7 +341,7 @@ struct UnitTestValueRangeAPI
   }
 };
 
-template <typename ArrayType>
+template <typename ArrayType, typename ForceValueTypeForVtkDataArray>
 struct UnitTestValueIteratorAPI
 {
   static constexpr vtk::ComponentIdType NumComps = 3;
@@ -338,7 +353,7 @@ struct UnitTestValueIteratorAPI
     vtkNew<ArrayType> array;
     array->SetNumberOfComponents(NumComps);
     array->SetNumberOfTuples(NumTuples);
-    FillValueRangeIota(vtk::DataArrayValueRange<NumComps>(array));
+    FillValueRangeIota(vtk::DataArrayValueRange(array));
 
     auto da = static_cast<vtkDataArray*>(array);
 
@@ -347,15 +362,16 @@ struct UnitTestValueIteratorAPI
       DispatchRangeTests(range);
     }
     { // Full, dynamic-size, generic-typed range
-      auto range = vtk::DataArrayValueRange(da);
+      auto range =
+        vtk::DataArrayValueRange<vtk::detail::DynamicTupleSize, ForceValueTypeForVtkDataArray>(da);
       DispatchRangeTests(range);
     }
     { // Full, fixed-size, real typed range
-      auto range = vtk::DataArrayValueRange<NumComps>(array);
+      auto range = vtk::DataArrayValueRange(array);
       DispatchRangeTests(range);
     }
     { // Full, fixed-size, generic-typed range
-      auto range = vtk::DataArrayValueRange<NumComps>(da);
+      auto range = vtk::DataArrayValueRange<NumComps, ForceValueTypeForVtkDataArray>(da);
       DispatchRangeTests(range);
     }
   }
@@ -833,7 +849,7 @@ struct UnitTestValueIteratorAPI
   }
 };
 
-template <typename ArrayType>
+template <typename ArrayType, typename ForceValueTypeForVtkDataArray>
 struct UnitTestValueReferenceAPI
 {
   static constexpr vtk::ComponentIdType NumComps = 9;
@@ -844,7 +860,7 @@ struct UnitTestValueReferenceAPI
     vtkNew<ArrayType> array;
     array->SetNumberOfComponents(NumComps);
     array->SetNumberOfTuples(NumTuples);
-    FillValueRangeIota(vtk::DataArrayValueRange<NumComps>(array));
+    FillValueRangeIota(vtk::DataArrayValueRange(array));
 
     auto da = static_cast<vtkDataArray*>(array);
 
@@ -853,7 +869,8 @@ struct UnitTestValueReferenceAPI
       DispatchRangeTests(range);
     }
     { // Full, dynamic-size, generic-typed range
-      auto range = vtk::DataArrayValueRange(da);
+      auto range =
+        vtk::DataArrayValueRange<vtk::detail::DynamicTupleSize, ForceValueTypeForVtkDataArray>(da);
       DispatchRangeTests(range);
     }
     { // Full, fixed-size, real typed range
@@ -861,7 +878,7 @@ struct UnitTestValueReferenceAPI
       DispatchRangeTests(range);
     }
     { // Full, fixed-size, generic-typed range
-      auto range = vtk::DataArrayValueRange<NumComps>(da);
+      auto range = vtk::DataArrayValueRange<NumComps, ForceValueTypeForVtkDataArray>(da);
       DispatchRangeTests(range);
     }
   }
@@ -1717,17 +1734,77 @@ struct UnitTestEdgeCases
   }
 };
 
-template <typename ArrayType>
+template <typename ArrayType, typename ForceValueTypeForVtkDataArray = double>
 void RunTestsForArray()
 {
   std::cerr << "ValueRangeAPI:\n";
-  UnitTestValueRangeAPI<ArrayType>{}();
+  UnitTestValueRangeAPI<ArrayType, ForceValueTypeForVtkDataArray>{}();
   std::cerr << "ValueIteratorAPI:\n";
-  UnitTestValueIteratorAPI<ArrayType>{}();
+  UnitTestValueIteratorAPI<ArrayType, ForceValueTypeForVtkDataArray>{}();
   std::cerr << "ValueReferenceAPI:\n";
-  UnitTestValueReferenceAPI<ArrayType>{}();
+  UnitTestValueReferenceAPI<ArrayType, ForceValueTypeForVtkDataArray>{}();
 }
 
+// Exercise DataArrayValueRange for vtkGenericDataArray.
+template <typename ValueT>
+class MockDataArray : public vtkGenericDataArray<MockDataArray<ValueT>, ValueT>
+{
+  using GenericDataArrayType = vtkGenericDataArray<MockDataArray<ValueT>, ValueT>;
+
+public:
+  vtkTemplateTypeMacro(MockDataArray<ValueT>, GenericDataArrayType);
+  using ValueType = typename Superclass::ValueType;
+  static MockDataArray* New() { VTK_STANDARD_NEW_BODY(MockDataArray<ValueT>); }
+  void* GetVoidPointer(vtkIdType idx) override { return this->Buffer->GetBuffer() + idx; }
+  ValueType GetValue(vtkIdType valueIdx) const { return this->Buffer->GetBuffer()[valueIdx]; }
+  void SetValue(vtkIdType valueIdx, ValueType value)
+  {
+    this->Buffer->GetBuffer()[valueIdx] = value;
+  }
+  void GetTypedTuple(vtkIdType tupleIdx, ValueType* tuple) const
+  {
+    const vtkIdType valueIdx = tupleIdx * this->NumberOfComponents;
+    std::copy(this->Buffer->GetBuffer() + valueIdx,
+      this->Buffer->GetBuffer() + valueIdx + this->NumberOfComponents, tuple);
+  }
+  void SetTypedTuple(vtkIdType tupleIdx, const ValueType* tuple)
+  {
+    const vtkIdType valueIdx = tupleIdx * this->NumberOfComponents;
+    std::copy(tuple, tuple + this->NumberOfComponents, this->Buffer->GetBuffer() + valueIdx);
+  }
+  ValueType GetTypedComponent(vtkIdType tupleIdx, int compIdx) const
+  {
+    return this->Buffer->GetBuffer()[this->NumberOfComponents * tupleIdx + compIdx];
+  }
+  void SetTypedComponent(vtkIdType tupleIdx, int compIdx, ValueType value)
+  {
+    const vtkIdType valueIdx = tupleIdx * this->NumberOfComponents + compIdx;
+    this->SetValue(valueIdx, value);
+  }
+
+protected:
+  vtkNew<vtkBuffer<ValueT>> Buffer;
+  bool AllocateTuples(vtkIdType numTuples)
+  {
+    vtkIdType numValues = numTuples * this->GetNumberOfComponents();
+    if (this->Buffer->Allocate(numValues))
+    {
+      this->Size = this->Buffer->GetSize();
+      return true;
+    }
+    return false;
+  }
+  bool ReallocateTuples(vtkIdType numTuples)
+  {
+    if (this->Buffer->Reallocate(numTuples * this->GetNumberOfComponents()))
+    {
+      this->Size = this->Buffer->GetSize();
+      return true;
+    }
+    return false;
+  }
+  friend class vtkGenericDataArray<MockDataArray<ValueT>, ValueT>;
+};
 } // end anon namespace
 
 int TestDataArrayValueRange(int, char*[])
@@ -1742,6 +1819,8 @@ int TestDataArrayValueRange(int, char*[])
 #endif
   std::cerr << "vtkFloatArray:\n";
   RunTestsForArray<vtkFloatArray>();
+  std::cerr << "MockDataArray<vtkTypeInt32>:\n";
+  RunTestsForArray<MockDataArray<vtkTypeInt32>, /*ForceValueTypeForVtkDataArray=*/vtkTypeInt32>();
 
   std::cerr << "\nEdgeCases:\n";
   UnitTestEdgeCases{}();
