@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkAbstractArray.h
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 //
 /**
  * @class   vtkAbstractArray
@@ -67,8 +55,13 @@
 #include "vtkCommonCoreModule.h" // For export macro
 #include "vtkIdList.h"           // For InsertTuples
 #include "vtkObject.h"
-#include "vtkVariant.h" // for variant arguments
+#include "vtkVariant.h"       // for variant arguments
+#include "vtkWrappingHints.h" // For VTK_MARSHALAUTO
 
+#include "vtk_nlohmannjson.h"
+#include VTK_NLOHMANN_JSON(json_fwd.hpp)
+
+VTK_ABI_NAMESPACE_BEGIN
 class vtkArrayIterator;
 class vtkDataArray;
 class vtkIdList;
@@ -80,11 +73,13 @@ class vtkInformationInformationVectorKey;
 class vtkInformationVariantVectorKey;
 class vtkVariantArray;
 
-class VTKCOMMONCORE_EXPORT vtkAbstractArray : public vtkObject
+class VTKCOMMONCORE_EXPORT VTK_MARSHALAUTO vtkAbstractArray : public vtkObject
 {
 public:
   vtkTypeMacro(vtkAbstractArray, vtkObject);
   void PrintSelf(ostream& os, vtkIndent indent) override;
+  void PrintValues(ostream& os);
+  nlohmann::json SerializeValues();
 
   /**
    * Allocate memory for this array. Delete old storage only if necessary.
@@ -172,6 +167,7 @@ public:
    * conjunction with SetValue() method for fast insertion. Preserves existing
    * data and returns true if allocation succeeds, or false otherwise.
    */
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_IS_INTERNAL)
   virtual bool SetNumberOfValues(vtkIdType numValues);
 
   /**
@@ -185,6 +181,7 @@ public:
    * construction for subclasses that support component insertion, which may
    * result in an incomplete trailing tuple.
    */
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_IS_INTERNAL)
   inline vtkIdType GetNumberOfValues() const { return (this->MaxId + 1); }
 
   /**
@@ -427,6 +424,11 @@ public:
    */
   virtual int IsNumeric() const = 0;
 
+  /**
+   * This method will return true if and only if an array contains integer-valued data.
+   */
+  virtual bool IsIntegral() const;
+
   /**
    * Subclasses must override this method and provide the right kind
    * of templated vtkArrayIteratorTemplate.
@@ -578,7 +580,7 @@ public:
    * keys they do not wish to be copied. The subclass will not need to
    * explicitly copy the keys as it's handled here.
    */
-  virtual int CopyInformation(vtkInformation* infoFrom, int deep = 1);
+  virtual int CopyInformation(vtkInformation* infoFrom, vtkTypeBool deep = 1);
 
   /**
    * This key is a hint to end user interface that this array
@@ -659,6 +661,7 @@ public:
     TypedDataArray,
     MappedDataArray,
     ScaleSoADataArrayTemplate,
+    ImplicitArray,
 
     DataArrayTemplate = AoSDataArrayTemplate //! Legacy
   };
@@ -748,7 +751,7 @@ struct vtkArrayDownCast_impl
  * it, while others will fallback to the slower SafeDownCast.
 
  * A more detailed description of this class and related tools can be found
- * \ref VTK-7-1-ArrayDispatch "here".
+ * [here](https://docs.vtk.org/en/latest/design_documents/array_dispatch.html).
  */
 template <typename ArrayT>
 ArrayT* vtkArrayDownCast(vtkAbstractArray* array)
@@ -758,6 +761,8 @@ ArrayT* vtkArrayDownCast(vtkAbstractArray* array)
   return vtkArrayDownCast_impl<ArrayT>()(array);
 }
 
+VTK_ABI_NAMESPACE_END
+
 ///@{
 /**
  * This macro is used to tell vtkArrayDownCast to use FastDownCast instead of
@@ -788,5 +793,4 @@ ArrayT* vtkArrayDownCast(vtkAbstractArray* array)
     }                                                                                              \
   }
 ///@}
-
 #endif
