@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkCellArray.h
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkCellArray
  * @brief   object to represent cell connectivity
@@ -129,14 +117,15 @@
  * version of the vtkCellArray API, the VTK_CELL_ARRAY_V2 preprocessor
  * definition may be used to detect which API is being compiled against.
  *
- * @sa vtkCellTypes vtkCellLinks
+ * @sa vtkAbstractCellArray vtkStructuredCellArray vtkCellTypes vtkCellLinks
  */
 
 #ifndef vtkCellArray_h
 #define vtkCellArray_h
 
+#include "vtkAbstractCellArray.h"
 #include "vtkCommonDataModelModule.h" // For export macro
-#include "vtkObject.h"
+#include "vtkWrappingHints.h"         // For VTK_MARSHALMANUAL
 
 #include "vtkAOSDataArrayTemplate.h" // Needed for inline methods
 #include "vtkCell.h"                 // Needed for inline methods
@@ -174,10 +163,11 @@
  */
 #define VTK_CELL_ARRAY_V2
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkCellArrayIterator;
 class vtkIdTypeArray;
 
-class VTKCOMMONDATAMODEL_EXPORT vtkCellArray : public vtkObject
+class VTKCOMMONDATAMODEL_EXPORT VTK_MARSHALMANUAL vtkCellArray : public vtkAbstractCellArray
 {
 public:
   using ArrayType32 = vtkTypeInt32Array;
@@ -189,7 +179,7 @@ public:
    * printing.
    */
   static vtkCellArray* New();
-  vtkTypeMacro(vtkCellArray, vtkObject);
+  vtkTypeMacro(vtkCellArray, vtkAbstractCellArray);
   void PrintSelf(ostream& os, vtkIndent indent) override;
   void PrintDebug(ostream& os);
   ///@}
@@ -282,7 +272,7 @@ public:
   /**
    * Free any memory and reset to an empty state.
    */
-  void Initialize();
+  void Initialize() override;
 
   /**
    * Reuse list. Reset to initial state without freeing memory.
@@ -311,7 +301,7 @@ public:
   /**
    * Get the number of cells in the array.
    */
-  vtkIdType GetNumberOfCells() const
+  vtkIdType GetNumberOfCells() const override
   {
     if (this->Storage.Is64Bit())
     {
@@ -327,7 +317,7 @@ public:
    * Get the number of elements in the offsets array. This will be the number of
    * cells + 1.
    */
-  vtkIdType GetNumberOfOffsets() const
+  vtkIdType GetNumberOfOffsets() const override
   {
     if (this->Storage.Is64Bit())
     {
@@ -342,7 +332,7 @@ public:
   /**
    * Get the offset (into the connectivity) for a specified cell id.
    */
-  vtkIdType GetOffset(vtkIdType cellId)
+  vtkIdType GetOffset(vtkIdType cellId) override
   {
     if (this->Storage.Is64Bit())
     {
@@ -360,7 +350,7 @@ public:
    * GetNumberOfConnectivityEntries(), which refers to the legacy memory
    * layout.
    */
-  vtkIdType GetNumberOfConnectivityIds() const
+  vtkIdType GetNumberOfConnectivityIds() const override
   {
     if (this->Storage.Is64Bit())
     {
@@ -379,7 +369,6 @@ public:
    */
   VTK_NEWINSTANCE vtkCellArrayIterator* NewIterator();
 
-#ifndef __VTK_WRAP__ // The wrappers have issues with some of these templates
   /**
    * Set the internal data arrays to the supplied offsets and connectivity
    * arrays.
@@ -390,15 +379,14 @@ public:
    *
    * @{
    */
-  void SetData(vtkTypeInt32Array* offsets, vtkTypeInt32Array* connectivity);
-  void SetData(vtkTypeInt64Array* offsets, vtkTypeInt64Array* connectivity);
   void SetData(vtkIdTypeArray* offsets, vtkIdTypeArray* connectivity);
   void SetData(vtkAOSDataArrayTemplate<int>* offsets, vtkAOSDataArrayTemplate<int>* connectivity);
   void SetData(vtkAOSDataArrayTemplate<long>* offsets, vtkAOSDataArrayTemplate<long>* connectivity);
   void SetData(
     vtkAOSDataArrayTemplate<long long>* offsets, vtkAOSDataArrayTemplate<long long>* connectivity);
+  void SetData(vtkTypeInt32Array* offsets, vtkTypeInt32Array* connectivity);
+  void SetData(vtkTypeInt64Array* offsets, vtkTypeInt64Array* connectivity);
   /**@}*/
-#endif // __VTK_WRAP__
 
   /**
    * Sets the internal arrays to the supplied offsets and connectivity arrays.
@@ -441,15 +429,15 @@ public:
    * storage is such that copying of data can be avoided, and instead
    * a pointer to vtkIdType can be used.
    */
-  bool IsStorageShareable() const
+  bool IsStorageShareable() const override
   {
     if (this->Storage.Is64Bit())
     {
-      return this->Storage.GetArrays64().ValueTypeIsSameAsIdType;
+      return VisitState<ArrayType64>::ValueTypeIsSameAsIdType;
     }
     else
     {
-      return this->Storage.GetArrays32().ValueTypeIsSameAsIdType;
+      return VisitState<ArrayType32>::ValueTypeIsSameAsIdType;
     }
   }
 
@@ -550,7 +538,7 @@ public:
    * * 0 = Cell array empty
    * * n (positive integer) = homogeneous array of cell size n
    */
-  vtkIdType IsHomogeneous();
+  vtkIdType IsHomogeneous() override;
 
   /**
    * @warning This method is not thread-safe. Consider using the NewIterator()
@@ -591,19 +579,6 @@ public:
    */
   int GetNextCell(vtkIdList* pts);
 
-  /**
-   * Return the point ids for the cell at @a cellId.
-   *
-   * @warning Subsequent calls to this method may invalidate previous call
-   * results if the internal storage type is not the same as vtkIdType and
-   * cannot be shared through the @a cellPoints pointer. In other words, the
-   * method may not be thread safe. Check if shareable (using
-   * IsStorageShareable()), or use a vtkCellArrayIterator to guarantee thread
-   * safety.
-   */
-  void GetCellAtId(vtkIdType cellId, vtkIdType& cellSize, vtkIdType const*& cellPoints)
-    VTK_SIZEHINT(cellPoints, cellSize) VTK_EXPECTS(0 <= cellId && cellId < GetNumberOfCells());
-
   /**
    * Return the point ids for the cell at @a cellId.
    *
@@ -613,9 +588,10 @@ public:
    * the method will use ptIds, which is an object that is created by each thread,
    * to guarantee thread safety.
    */
-  void GetCellAtId(
-    vtkIdType cellId, vtkIdType& cellSize, vtkIdType const*& cellPoints, vtkIdList* ptIds)
-    VTK_SIZEHINT(cellPoints, cellSize) VTK_EXPECTS(0 <= cellId && cellId < GetNumberOfCells());
+  using vtkAbstractCellArray::GetCellAtId;
+  void GetCellAtId(vtkIdType cellId, vtkIdType& cellSize, vtkIdType const*& cellPoints,
+    vtkIdList* ptIds) VTK_SIZEHINT(cellPoints, cellSize)
+    VTK_EXPECTS(0 <= cellId && cellId < GetNumberOfCells()) override;
 
   /**
    * Return the point ids for the cell at @a cellId. This always copies
@@ -623,12 +599,22 @@ public:
    * vtkIdList). This method is thread safe.
    */
   void GetCellAtId(vtkIdType cellId, vtkIdList* pts)
-    VTK_EXPECTS(0 <= cellId && cellId < GetNumberOfCells());
+    VTK_EXPECTS(0 <= cellId && cellId < GetNumberOfCells()) override;
+
+  /**
+   * Return the point ids for the cell at @a cellId. This always copies
+   * the cell ids into cellSize and cellPoints. This method is thread safe.
+   *
+   * Note: the cellPoints need to have the correct size already allocated otherwise memory
+   * issues can occur.
+   */
+  void GetCellAtId(vtkIdType cellId, vtkIdType& cellSize, vtkIdType* cellPoints) VTK_SIZEHINT(
+    cellPoints, cellSize) VTK_EXPECTS(0 <= cellId && cellId < GetNumberOfCells()) override;
 
   /**
    * Return the size of the cell at @a cellId.
    */
-  vtkIdType GetCellSize(const vtkIdType cellId) const;
+  vtkIdType GetCellSize(vtkIdType cellId) const override;
 
   /**
    * Insert a cell object. Return the cell id of the cell.
@@ -733,17 +719,17 @@ public:
    * Returns the size of the largest cell. The size is the number of points
    * defining the cell.
    */
-  int GetMaxCellSize();
+  int GetMaxCellSize() override;
 
   /**
    * Perform a deep copy (no reference counting) of the given cell array.
    */
-  void DeepCopy(vtkCellArray* ca);
+  void DeepCopy(vtkAbstractCellArray* ca) override;
 
   /**
    * Shallow copy @a ca into this cell array.
    */
-  void ShallowCopy(vtkCellArray* ca);
+  void ShallowCopy(vtkAbstractCellArray* ca) override;
 
   /**
    * Append cells from src into this. Point ids are offset by @a pointOffset.
@@ -1051,6 +1037,16 @@ public:
 
   /** @} */
 
+  /**
+   * Control the default internal storage size. Useful for saving memory when
+   * most cases can be handled with 32bit indices, but large models may require
+   * a run-time switch to 64bit indices.
+   * @{
+   */
+  static bool GetDefaultStorageIs64Bit() { return vtkCellArray::DefaultStorageIs64Bit; }
+  static void SetDefaultStorageIs64Bit(bool val) { vtkCellArray::DefaultStorageIs64Bit = val; }
+  /** @} */
+
 #endif // __VTK_WRAP__
 
   //=================== Begin Legacy Methods ===================================
@@ -1216,18 +1212,18 @@ protected:
       this->Arrays = new ArraySwitch;
 #endif
 
-      // Default to the compile-time setting:
-#ifdef VTK_USE_64BIT_IDS
+      // Default can be changed, to save memory
+      if (vtkCellArray::GetDefaultStorageIs64Bit())
+      {
+        this->Arrays->Int64 = new VisitState<ArrayType64>;
+        this->StorageIs64Bit = true;
+      }
+      else
+      {
+        this->Arrays->Int32 = new VisitState<ArrayType32>;
+        this->StorageIs64Bit = false;
+      }
 
-      this->Arrays->Int64 = new VisitState<ArrayType64>;
-      this->StorageIs64Bit = true;
-
-#else // VTK_USE_64BIT_IDS
-
-      this->Arrays->Int32 = new VisitState<ArrayType32>;
-      this->StorageIs64Bit = false;
-
-#endif // VTK_USE_64BIT_IDS
 #ifdef VTK_USE_MEMKIND
       if (vtkObjectBase::GetUsingMemkind())
       {
@@ -1336,11 +1332,12 @@ protected:
   };
 
   Storage Storage;
-  vtkNew<vtkIdList> TempCell;
   vtkIdType TraversalCellId{ 0 };
 
   vtkNew<vtkIdTypeArray> LegacyData; // For GetData().
 
+  static bool DefaultStorageIs64Bit;
+
 private:
   vtkCellArray(const vtkCellArray&) = delete;
   void operator=(const vtkCellArray&) = delete;
@@ -1377,9 +1374,11 @@ vtkCellArray::VisitState<ArrayT>::GetCellRange(vtkIdType cellId)
   return vtk::DataArrayValueRange<1>(
     this->GetConnectivity(), this->GetBeginOffset(cellId), this->GetEndOffset(cellId));
 }
+VTK_ABI_NAMESPACE_END
 
 namespace vtkCellArray_detail
 {
+VTK_ABI_NAMESPACE_BEGIN
 
 struct InsertNextCellImpl
 {
@@ -1436,7 +1435,7 @@ struct UpdateCellCountImpl
 struct GetCellSizeImpl
 {
   template <typename CellStateT>
-  vtkIdType operator()(CellStateT& state, const vtkIdType cellId)
+  vtkIdType operator()(CellStateT& state, vtkIdType cellId)
   {
     return state.GetCellSize(cellId);
   }
@@ -1449,14 +1448,35 @@ struct GetCellAtIdImpl
   {
     using ValueType = typename CellStateT::ValueType;
 
-    const auto cellPts = state.GetCellRange(cellId);
+    const vtkIdType beginOffset = state.GetBeginOffset(cellId);
+    const vtkIdType endOffset = state.GetEndOffset(cellId);
+    const vtkIdType cellSize = endOffset - beginOffset;
+    const auto cellConnectivity = state.GetConnectivity()->GetPointer(beginOffset);
 
-    ids->SetNumberOfIds(cellPts.size());
+    // ValueType differs from vtkIdType, so we have to copy into a temporary buffer:
+    ids->SetNumberOfIds(cellSize);
     vtkIdType* idPtr = ids->GetPointer(0);
-
-    for (ValueType ptId : cellPts)
+    for (ValueType i = 0; i < cellSize; ++i)
     {
-      *idPtr++ = static_cast<vtkIdType>(ptId);
+      idPtr[i] = static_cast<vtkIdType>(cellConnectivity[i]);
+    }
+  }
+
+  template <typename CellStateT>
+  void operator()(
+    CellStateT& state, const vtkIdType cellId, vtkIdType& cellSize, vtkIdType* cellPoints)
+  {
+    using ValueType = typename CellStateT::ValueType;
+
+    const vtkIdType beginOffset = state.GetBeginOffset(cellId);
+    const vtkIdType endOffset = state.GetEndOffset(cellId);
+    cellSize = endOffset - beginOffset;
+    const ValueType* cellConnectivity = state.GetConnectivity()->GetPointer(beginOffset);
+
+    // ValueType differs from vtkIdType, so we have to copy into a temporary buffer:
+    for (vtkIdType i = 0; i < cellSize; ++i)
+    {
+      cellPoints[i] = static_cast<vtkIdType>(cellConnectivity[i]);
     }
   }
 
@@ -1495,16 +1515,17 @@ struct GetCellAtIdImpl
   {
     using ValueType = typename CellStateT::ValueType;
 
-    const auto cellPts = state.GetCellRange(cellId);
-    cellSize = cellPts.size();
+    const vtkIdType beginOffset = state.GetBeginOffset(cellId);
+    const vtkIdType endOffset = state.GetEndOffset(cellId);
+    cellSize = endOffset - beginOffset;
+    const ValueType* cellConnectivity = state.GetConnectivity()->GetPointer(beginOffset);
 
-    // ValueType differs from vtkIdType, so we have to copy into a temporary
-    // buffer:
+    // ValueType differs from vtkIdType, so we have to copy into a temporary buffer:
     temp->SetNumberOfIds(cellSize);
     vtkIdType* tempPtr = temp->GetPointer(0);
-    for (ValueType ptId : cellPts)
+    for (vtkIdType i = 0; i < cellSize; ++i)
     {
-      *tempPtr++ = static_cast<vtkIdType>(ptId);
+      tempPtr[i] = static_cast<vtkIdType>(cellConnectivity[i]);
     }
 
     cellPoints = temp->GetPointer(0);
@@ -1522,8 +1543,10 @@ struct ResetImpl
   }
 };
 
+VTK_ABI_NAMESPACE_END
 } // end namespace vtkCellArray_detail
 
+VTK_ABI_NAMESPACE_BEGIN
 //----------------------------------------------------------------------------
 inline void vtkCellArray::InitTraversal()
 {
@@ -1564,13 +1587,6 @@ inline vtkIdType vtkCellArray::GetCellSize(const vtkIdType cellId) const
   return this->Visit(vtkCellArray_detail::GetCellSizeImpl{}, cellId);
 }
 
-//----------------------------------------------------------------------------
-inline void vtkCellArray::GetCellAtId(vtkIdType cellId, vtkIdType& cellSize,
-  vtkIdType const*& cellPoints) VTK_SIZEHINT(cellPoints, cellSize)
-{
-  this->Visit(vtkCellArray_detail::GetCellAtIdImpl{}, cellId, cellSize, cellPoints, this->TempCell);
-}
-
 //----------------------------------------------------------------------------
 inline void vtkCellArray::GetCellAtId(vtkIdType cellId, vtkIdType& cellSize,
   vtkIdType const*& cellPoints, vtkIdList* ptIds) VTK_SIZEHINT(cellPoints, cellSize)
@@ -1584,6 +1600,12 @@ inline void vtkCellArray::GetCellAtId(vtkIdType cellId, vtkIdList* pts)
   this->Visit(vtkCellArray_detail::GetCellAtIdImpl{}, cellId, pts);
 }
 
+//----------------------------------------------------------------------------
+inline void vtkCellArray::GetCellAtId(vtkIdType cellId, vtkIdType& cellSize, vtkIdType* cellPoints)
+{
+  this->Visit(vtkCellArray_detail::GetCellAtIdImpl{}, cellId, cellSize, cellPoints);
+}
+
 //----------------------------------------------------------------------------
 inline vtkIdType vtkCellArray::InsertNextCell(vtkIdType npts, const vtkIdType* pts)
   VTK_SIZEHINT(pts, npts)
@@ -1639,4 +1661,5 @@ inline void vtkCellArray::Reset()
   this->Visit(vtkCellArray_detail::ResetImpl{});
 }
 
+VTK_ABI_NAMESPACE_END
 #endif // vtkCellArray.h
