@@ -1,19 +1,8 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkDataObjectTree.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkDataObjectTree.h"
 
+#include "vtkCellGrid.h"
 #include "vtkDataObjectTreeInternals.h"
 #include "vtkDataObjectTreeIterator.h"
 #include "vtkDataSet.h"
@@ -21,10 +10,14 @@
 #include "vtkInformationIntegerKey.h"
 #include "vtkInformationStringKey.h"
 #include "vtkInformationVector.h"
-#include "vtkMultiPieceDataSet.h"
+#include "vtkLegacy.h"
 #include "vtkObjectFactory.h"
+#include "vtkPartitionedDataSet.h"
+#include "vtkUniformGridAMR.h"
+#include "vtkUniformGridAMRDataIterator.h"
 
 //------------------------------------------------------------------------------
+VTK_ABI_NAMESPACE_BEGIN
 vtkDataObjectTree::vtkDataObjectTree()
 {
   this->Internals = new vtkDataObjectTreeInternals;
@@ -132,7 +125,7 @@ void vtkDataObjectTree::SetChildMetaData(unsigned int index, vtkInformation* inf
 }
 
 //------------------------------------------------------------------------------
-int vtkDataObjectTree::HasChildMetaData(unsigned int index)
+vtkTypeBool vtkDataObjectTree::HasChildMetaData(unsigned int index)
 {
   if (index < this->Internals->Children.size())
   {
@@ -150,7 +143,7 @@ void vtkDataObjectTree::CopyStructure(vtkCompositeDataSet* compositeSource)
   {
     return;
   }
-  vtkDataObjectTree* dObjTree = vtkDataObjectTree::SafeDownCast(compositeSource);
+  auto dObjTree = vtkDataObjectTree::SafeDownCast(compositeSource);
   if (dObjTree == this)
   {
     return;
@@ -159,32 +152,7 @@ void vtkDataObjectTree::CopyStructure(vtkCompositeDataSet* compositeSource)
   this->Superclass::CopyStructure(compositeSource);
   this->Internals->Children.clear();
 
-  if (!dObjTree)
-  {
-    // WARNING:
-    // If we copy the structure of from a non-tree composite data set
-    // we create a special structure of two levels, the first level
-    // is just a single multipiece and the second level are all the data sets.
-    // This is likely to change in the future!
-    vtkNew<vtkMultiPieceDataSet> mds;
-    this->SetChild(0, mds);
-
-    vtkNew<vtkInformation> info;
-    info->Set(vtkCompositeDataSet::NAME(), "All Blocks");
-    this->SetChildMetaData(0, info);
-
-    int totalNumBlocks = 0;
-    vtkCompositeDataIterator* iter = compositeSource->NewIterator();
-    iter->SkipEmptyNodesOff();
-    for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
-    {
-      totalNumBlocks++;
-    }
-    iter->Delete();
-
-    mds->SetNumberOfChildren(totalNumBlocks);
-  }
-  else
+  if (dObjTree)
   {
     this->Internals->Children.resize(dObjTree->Internals->Children.size());
 
@@ -215,6 +183,46 @@ void vtkDataObjectTree::CopyStructure(vtkCompositeDataSet* compositeSource)
       }
     }
   }
+  else if (auto amr = vtkUniformGridAMR::SafeDownCast(compositeSource))
+  {
+    this->SetNumberOfChildren(amr->GetNumberOfLevels());
+    vtkNew<vtkPartitionedDataSet> tempPds;
+    for (unsigned int level = 0; level < amr->GetNumberOfLevels(); level++)
+    {
+      auto child = vtk::TakeSmartPointer(this->CreateForCopyStructure(tempPds));
+      child->SetNumberOfChildren(amr->GetNumberOfDataSets(level));
+      this->SetChild(level, child);
+
+      vtkNew<vtkInformation> info;
+      info->Set(vtkCompositeDataSet::NAME(), "Level " + std::to_string(level));
+      this->SetChildMetaData(level, info);
+    }
+  }
+  else
+  {
+    // WARNING:
+    // If we copy the structure from a non-tree composite data set
+    // we create a special structure of two levels, the first level
+    // is just a single partitioned dataSet and the second level are all the data sets.
+    // This is likely to change in the future!
+    vtkNew<vtkPartitionedDataSet> tempPds;
+    auto pds = vtk::TakeSmartPointer(this->CreateForCopyStructure(tempPds));
+    this->SetChild(0, pds);
+
+    vtkNew<vtkInformation> info;
+    info->Set(vtkCompositeDataSet::NAME(), "All Blocks");
+    this->SetChildMetaData(0, info);
+
+    int totalNumBlocks = 0;
+    auto iter = vtk::TakeSmartPointer(compositeSource->NewIterator());
+    iter->SkipEmptyNodesOff();
+    for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+    {
+      ++totalNumBlocks;
+    }
+
+    pds->SetNumberOfChildren(totalNumBlocks);
+  }
   this->Modified();
 }
 
@@ -227,9 +235,9 @@ vtkDataObjectTree* vtkDataObjectTree::CreateForCopyStructure(vtkDataObjectTree*
 //------------------------------------------------------------------------------
 vtkDataObjectTreeIterator* vtkDataObjectTree::NewTreeIterator()
 {
-  vtkDataObjectTreeIterator* iter = vtkDataObjectTreeIterator::New();
-  iter->SetDataSet(this);
-  return iter;
+  vtkDataObjectTreeIterator* dObjTreeIter = vtkDataObjectTreeIterator::New();
+  dObjTreeIter->SetDataSet(this);
+  return dObjTreeIter;
 }
 
 //------------------------------------------------------------------------------
@@ -241,10 +249,9 @@ vtkCompositeDataIterator* vtkDataObjectTree::NewIterator()
 //------------------------------------------------------------------------------
 void vtkDataObjectTree::SetDataSet(vtkCompositeDataIterator* iter, vtkDataObject* dataObj)
 {
-  vtkDataObjectTreeIterator* treeIter = vtkDataObjectTreeIterator::SafeDownCast(iter);
-  if (treeIter)
+  if (auto dObjTreeIter = vtkDataObjectTreeIterator::SafeDownCast(iter))
   {
-    this->SetDataSetFrom(treeIter, dataObj);
+    this->SetDataSetFrom(dObjTreeIter, dataObj);
     return;
   }
 
@@ -254,34 +261,58 @@ void vtkDataObjectTree::SetDataSet(vtkCompositeDataIterator* iter, vtkDataObject
     return;
   }
 
-  // WARNING: We are doing something special here. See comments
-  // in CopyStructure()
-
-  unsigned int index = iter->GetCurrentFlatIndex();
-  if (this->GetNumberOfChildren() != 1)
+  auto uniformGridAMRIter = vtkUniformGridAMRDataIterator::SafeDownCast(iter);
+  if (uniformGridAMRIter)
   {
-    vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
-    return;
+    if (uniformGridAMRIter->GetCurrentLevel() < this->GetNumberOfChildren())
+    {
+      auto parent =
+        vtkDataObjectTree::SafeDownCast(this->GetChild(uniformGridAMRIter->GetCurrentLevel()));
+      if (parent)
+      {
+        parent->SetChild(uniformGridAMRIter->GetCurrentIndex(), dataObj);
+      }
+      else
+      {
+        vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
+      }
+    }
+    else
+    {
+      vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
+    }
   }
-  vtkMultiPieceDataSet* parent = vtkMultiPieceDataSet::SafeDownCast(this->GetChild(0));
-  if (!parent)
+  else
   {
-    vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
-    return;
+    // WARNING: We are doing something special here. See comments
+    // in CopyStructure()
+    unsigned int index = iter->GetCurrentFlatIndex();
+    if (this->GetNumberOfChildren() != 1)
+    {
+      vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
+      return;
+    }
+    auto parent = vtkPartitionedDataSet::SafeDownCast(this->GetChild(0));
+    if (!parent)
+    {
+      vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
+      return;
+    }
+    parent->SetChild(index, dataObj);
   }
-  parent->SetChild(index, dataObj);
 }
 
 //------------------------------------------------------------------------------
-void vtkDataObjectTree::SetDataSetFrom(vtkDataObjectTreeIterator* iter, vtkDataObject* dataObj)
+void vtkDataObjectTree::SetDataSetFrom(
+  vtkDataObjectTreeIterator* dObjTreeIter, vtkDataObject* dataObj)
 {
-  if (!iter || iter->IsDoneWithTraversal())
+  if (!dObjTreeIter || dObjTreeIter->IsDoneWithTraversal())
   {
     vtkErrorMacro("Invalid iterator location.");
     return;
   }
 
-  vtkDataObjectTreeIndex index = iter->GetCurrentIndex();
+  vtkDataObjectTreeIndex index = dObjTreeIter->GetCurrentIndex();
 
   if (index.empty())
   {
@@ -296,8 +327,7 @@ void vtkDataObjectTree::SetDataSetFrom(vtkDataObjectTreeIterator* iter, vtkDataO
   {
     if (!parent || parent->GetNumberOfChildren() <= index[cc])
     {
-      vtkErrorMacro("Structure does not match. "
-                    "You must use CopyStructure before calling this method.");
+      vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
       return;
     }
     parent = vtkDataObjectTree::SafeDownCast(parent->GetChild(index[cc]));
@@ -305,8 +335,7 @@ void vtkDataObjectTree::SetDataSetFrom(vtkDataObjectTreeIterator* iter, vtkDataO
 
   if (!parent || parent->GetNumberOfChildren() <= index.back())
   {
-    vtkErrorMacro("Structure does not match. "
-                  "You must use CopyStructure before calling this method.");
+    vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
     return;
   }
 
@@ -322,8 +351,59 @@ vtkDataObject* vtkDataObjectTree::GetDataSet(vtkCompositeDataIterator* composite
     return nullptr;
   }
 
-  vtkDataObjectTreeIterator* iter = vtkDataObjectTreeIterator::SafeDownCast(compositeIter);
-  if (!iter)
+  if (auto dObjTreeIter = vtkDataObjectTreeIterator::SafeDownCast(compositeIter))
+  {
+    vtkDataObjectTreeIndex index = dObjTreeIter->GetCurrentIndex();
+
+    if (index.empty())
+    {
+      // Sanity check.
+      vtkErrorMacro("Invalid index returned by iterator.");
+      return nullptr;
+    }
+
+    vtkDataObjectTree* parent = this;
+    int numIndices = static_cast<int>(index.size());
+    for (int cc = 0; cc < numIndices - 1; cc++)
+    {
+      if (!parent || parent->GetNumberOfChildren() <= index[cc])
+      {
+        vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
+        return nullptr;
+      }
+      parent = vtkDataObjectTree::SafeDownCast(parent->GetChild(index[cc]));
+    }
+
+    if (!parent || parent->GetNumberOfChildren() <= index.back())
+    {
+      vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
+      return nullptr;
+    }
+
+    return parent->GetChild(index.back());
+  }
+  else if (auto amrIter = vtkUniformGridAMRDataIterator::SafeDownCast(compositeIter))
+  {
+    if (amrIter->GetCurrentLevel() < this->GetNumberOfChildren())
+    {
+      auto parent = vtkDataObjectTree::SafeDownCast(this->GetChild(amrIter->GetCurrentLevel()));
+      if (parent)
+      {
+        return parent->GetChild(amrIter->GetCurrentIndex());
+      }
+      else
+      {
+        vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
+        return nullptr;
+      }
+    }
+    else
+    {
+      vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
+      return nullptr;
+    }
+  }
+  else
   {
     // WARNING: We are doing something special here. See comments
     // in CopyStructure()
@@ -335,7 +415,7 @@ vtkDataObject* vtkDataObjectTree::GetDataSet(vtkCompositeDataIterator* composite
       vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
       return nullptr;
     }
-    vtkMultiPieceDataSet* parent = vtkMultiPieceDataSet::SafeDownCast(this->GetChild(0));
+    auto parent = vtkPartitionedDataSet::SafeDownCast(this->GetChild(0));
     if (!parent)
     {
       vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
@@ -351,50 +431,19 @@ vtkDataObject* vtkDataObjectTree::GetDataSet(vtkCompositeDataIterator* composite
       return nullptr;
     }
   }
-
-  vtkDataObjectTreeIndex index = iter->GetCurrentIndex();
-
-  if (index.empty())
-  {
-    // Sanity check.
-    vtkErrorMacro("Invalid index returned by iterator.");
-    return nullptr;
-  }
-
-  vtkDataObjectTree* parent = this;
-  int numIndices = static_cast<int>(index.size());
-  for (int cc = 0; cc < numIndices - 1; cc++)
-  {
-    if (!parent || parent->GetNumberOfChildren() <= index[cc])
-    {
-      vtkErrorMacro("Structure does not match. "
-                    "You must use CopyStructure before calling this method.");
-      return nullptr;
-    }
-    parent = vtkDataObjectTree::SafeDownCast(parent->GetChild(index[cc]));
-  }
-
-  if (!parent || parent->GetNumberOfChildren() <= index.back())
-  {
-    vtkErrorMacro("Structure does not match. "
-                  "You must use CopyStructure before calling this method.");
-    return nullptr;
-  }
-
-  return parent->GetChild(index.back());
 }
 
 //------------------------------------------------------------------------------
 vtkInformation* vtkDataObjectTree::GetMetaData(vtkCompositeDataIterator* compositeIter)
 {
-  vtkDataObjectTreeIterator* iter = vtkDataObjectTreeIterator::SafeDownCast(compositeIter);
-  if (!iter || iter->IsDoneWithTraversal())
+  auto dObjTreeIter = vtkDataObjectTreeIterator::SafeDownCast(compositeIter);
+  if (!dObjTreeIter || dObjTreeIter->IsDoneWithTraversal())
   {
     vtkErrorMacro("Invalid iterator location.");
     return nullptr;
   }
 
-  vtkDataObjectTreeIndex index = iter->GetCurrentIndex();
+  vtkDataObjectTreeIndex index = dObjTreeIter->GetCurrentIndex();
 
   if (index.empty())
   {
@@ -409,8 +458,7 @@ vtkInformation* vtkDataObjectTree::GetMetaData(vtkCompositeDataIterator* composi
   {
     if (!parent || parent->GetNumberOfChildren() <= index[cc])
     {
-      vtkErrorMacro("Structure does not match. "
-                    "You must use CopyStructure before calling this method.");
+      vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
       return nullptr;
     }
     parent = vtkDataObjectTree::SafeDownCast(parent->GetChild(index[cc]));
@@ -418,8 +466,7 @@ vtkInformation* vtkDataObjectTree::GetMetaData(vtkCompositeDataIterator* composi
 
   if (!parent || parent->GetNumberOfChildren() <= index.back())
   {
-    vtkErrorMacro("Structure does not match. "
-                  "You must use CopyStructure before calling this method.");
+    vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
     return nullptr;
   }
 
@@ -427,16 +474,16 @@ vtkInformation* vtkDataObjectTree::GetMetaData(vtkCompositeDataIterator* composi
 }
 
 //------------------------------------------------------------------------------
-int vtkDataObjectTree::HasMetaData(vtkCompositeDataIterator* compositeIter)
+vtkTypeBool vtkDataObjectTree::HasMetaData(vtkCompositeDataIterator* compositeIter)
 {
-  vtkDataObjectTreeIterator* iter = vtkDataObjectTreeIterator::SafeDownCast(compositeIter);
-  if (!iter || iter->IsDoneWithTraversal())
+  auto dObjTreeIter = vtkDataObjectTreeIterator::SafeDownCast(compositeIter);
+  if (!dObjTreeIter || dObjTreeIter->IsDoneWithTraversal())
   {
     vtkErrorMacro("Invalid iterator location.");
     return 0;
   }
 
-  vtkDataObjectTreeIndex index = iter->GetCurrentIndex();
+  vtkDataObjectTreeIndex index = dObjTreeIter->GetCurrentIndex();
 
   if (index.empty())
   {
@@ -451,8 +498,7 @@ int vtkDataObjectTree::HasMetaData(vtkCompositeDataIterator* compositeIter)
   {
     if (!parent || parent->GetNumberOfChildren() <= index[cc])
     {
-      vtkErrorMacro("Structure does not match. "
-                    "You must use CopyStructure before calling this method.");
+      vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
       return 0;
     }
     parent = vtkDataObjectTree::SafeDownCast(parent->GetChild(index[cc]));
@@ -460,8 +506,7 @@ int vtkDataObjectTree::HasMetaData(vtkCompositeDataIterator* compositeIter)
 
   if (!parent || parent->GetNumberOfChildren() <= index.back())
   {
-    vtkErrorMacro("Structure does not match. "
-                  "You must use CopyStructure before calling this method.");
+    vtkErrorMacro("Structure is not expected. Did you forget to use copy structure?");
     return 0;
   }
 
@@ -469,7 +514,7 @@ int vtkDataObjectTree::HasMetaData(vtkCompositeDataIterator* compositeIter)
 }
 
 //------------------------------------------------------------------------------
-void vtkDataObjectTree::ShallowCopy(vtkDataObject* src)
+void vtkDataObjectTree::CompositeShallowCopy(vtkCompositeDataSet* src)
 {
   if (src == this)
   {
@@ -477,22 +522,20 @@ void vtkDataObjectTree::ShallowCopy(vtkDataObject* src)
   }
 
   this->Internals->Children.clear();
-  this->Superclass::ShallowCopy(src);
+  this->Superclass::CompositeShallowCopy(src);
 
-  vtkDataObjectTree* from = vtkDataObjectTree::SafeDownCast(src);
-  if (from)
+  if (auto from = vtkDataObjectTree::SafeDownCast(src))
   {
     unsigned int numChildren = from->GetNumberOfChildren();
     this->SetNumberOfChildren(numChildren);
     for (unsigned int cc = 0; cc < numChildren; cc++)
     {
-      vtkDataObject* child = from->GetChild(cc);
-      if (child)
+      if (auto child = from->GetChild(cc))
       {
-        if (child->IsA("vtkDataObjectTree"))
+        if (auto childTree = vtkDataObjectTree::SafeDownCast(child))
         {
-          vtkDataObject* clone = child->NewInstance();
-          clone->ShallowCopy(child);
+          vtkDataObjectTree* clone = childTree->NewInstance();
+          clone->CompositeShallowCopy(childTree);
           this->SetChild(cc, clone);
           clone->FastDelete();
         }
@@ -504,7 +547,7 @@ void vtkDataObjectTree::ShallowCopy(vtkDataObject* src)
       if (from->HasChildMetaData(cc))
       {
         vtkInformation* toInfo = this->GetChildMetaData(cc);
-        toInfo->Copy(from->GetChildMetaData(cc), /*deep=*/0);
+        toInfo->Copy(from->GetChildMetaData(cc), 0);
       }
     }
   }
@@ -522,15 +565,13 @@ void vtkDataObjectTree::DeepCopy(vtkDataObject* src)
   this->Internals->Children.clear();
   this->Superclass::DeepCopy(src);
 
-  vtkDataObjectTree* from = vtkDataObjectTree::SafeDownCast(src);
-  if (from)
+  if (auto from = vtkDataObjectTree::SafeDownCast(src))
   {
     unsigned int numChildren = from->GetNumberOfChildren();
     this->SetNumberOfChildren(numChildren);
     for (unsigned int cc = 0; cc < numChildren; cc++)
     {
-      vtkDataObject* fromChild = from->GetChild(cc);
-      if (fromChild)
+      if (auto fromChild = from->GetChild(cc))
       {
         vtkDataObject* toChild = fromChild->NewInstance();
         toChild->DeepCopy(fromChild);
@@ -548,7 +589,7 @@ void vtkDataObjectTree::DeepCopy(vtkDataObject* src)
 }
 
 //------------------------------------------------------------------------------
-void vtkDataObjectTree::RecursiveShallowCopy(vtkDataObject* src)
+void vtkDataObjectTree::ShallowCopy(vtkDataObject* src)
 {
   if (src == this)
   {
@@ -558,15 +599,13 @@ void vtkDataObjectTree::RecursiveShallowCopy(vtkDataObject* src)
   this->Internals->Children.clear();
   this->Superclass::ShallowCopy(src);
 
-  vtkDataObjectTree* from = vtkDataObjectTree::SafeDownCast(src);
-  if (from)
+  if (auto from = vtkDataObjectTree::SafeDownCast(src))
   {
     unsigned int numChildren = from->GetNumberOfChildren();
     this->SetNumberOfChildren(numChildren);
     for (unsigned int cc = 0; cc < numChildren; cc++)
     {
-      vtkDataObject* child = from->GetChild(cc);
-      if (child)
+      if (auto child = from->GetChild(cc))
       {
         auto clone = child->NewInstance();
         clone->ShallowCopy(child);
@@ -583,6 +622,13 @@ void vtkDataObjectTree::RecursiveShallowCopy(vtkDataObject* src)
   this->Modified();
 }
 
+//------------------------------------------------------------------------------
+void vtkDataObjectTree::RecursiveShallowCopy(vtkDataObject* src)
+{
+  VTK_LEGACY_REPLACED_BODY(RecursiveShallowCopy, "VTK 9.3", ShallowCopy);
+  this->ShallowCopy(src);
+}
+
 //------------------------------------------------------------------------------
 void vtkDataObjectTree::Initialize()
 {
@@ -594,16 +640,15 @@ void vtkDataObjectTree::Initialize()
 vtkIdType vtkDataObjectTree::GetNumberOfPoints()
 {
   vtkIdType numPts = 0;
-  vtkDataObjectTreeIterator* iter = vtkDataObjectTreeIterator::SafeDownCast(this->NewIterator());
-  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+  auto dObjTreeIter = vtk::TakeSmartPointer(this->NewIterator());
+  for (dObjTreeIter->InitTraversal(); !dObjTreeIter->IsDoneWithTraversal();
+       dObjTreeIter->GoToNextItem())
   {
-    vtkDataSet* ds = vtkDataSet::SafeDownCast(iter->GetCurrentDataObject());
-    if (ds)
+    if (auto ds = vtkDataSet::SafeDownCast(dObjTreeIter->GetCurrentDataObject()))
     {
       numPts += ds->GetNumberOfPoints();
     }
   }
-  iter->Delete();
   return numPts;
 }
 
@@ -611,16 +656,19 @@ vtkIdType vtkDataObjectTree::GetNumberOfPoints()
 vtkIdType vtkDataObjectTree::GetNumberOfCells()
 {
   vtkIdType numCells = 0;
-  vtkDataObjectTreeIterator* iter = vtkDataObjectTreeIterator::SafeDownCast(this->NewIterator());
-  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+  auto dObjTreeIter = vtk::TakeSmartPointer(this->NewIterator());
+  for (dObjTreeIter->InitTraversal(); !dObjTreeIter->IsDoneWithTraversal();
+       dObjTreeIter->GoToNextItem())
   {
-    vtkDataSet* ds = vtkDataSet::SafeDownCast(iter->GetCurrentDataObject());
-    if (ds)
+    if (auto ds = vtkDataSet::SafeDownCast(dObjTreeIter->GetCurrentDataObject()))
     {
       numCells += ds->GetNumberOfCells();
     }
+    else if (auto cg = vtkCellGrid::SafeDownCast(dObjTreeIter->GetCurrentDataObject()))
+    {
+      numCells += cg->GetNumberOfCells();
+    }
   }
-  iter->Delete();
   return numCells;
 }
 
@@ -628,13 +676,13 @@ vtkIdType vtkDataObjectTree::GetNumberOfCells()
 unsigned long vtkDataObjectTree::GetActualMemorySize()
 {
   unsigned long memSize = 0;
-  vtkDataObjectTreeIterator* iter = vtkDataObjectTreeIterator::SafeDownCast(this->NewIterator());
-  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+  auto dObjTreeIter = vtk::TakeSmartPointer(this->NewIterator());
+  for (dObjTreeIter->InitTraversal(); !dObjTreeIter->IsDoneWithTraversal();
+       dObjTreeIter->GoToNextItem())
   {
-    vtkDataObject* dobj = iter->GetCurrentDataObject();
+    vtkDataObject* dobj = dObjTreeIter->GetCurrentDataObject();
     memSize += dobj->GetActualMemorySize();
   }
-  iter->Delete();
   return memSize;
 }
 
@@ -663,3 +711,4 @@ void vtkDataObjectTree::PrintSelf(ostream& os, vtkIndent indent)
     }
   }
 }
+VTK_ABI_NAMESPACE_END
