@@ -21,19 +21,23 @@
 
 #include "vtkCommonDataModelModule.h" // For export macro
 #include "vtkDataSet.h"
+#include "vtkSmartPointer.h"  // For vtkSmartPointer ivars
+#include "vtkWrappingHints.h" // For VTK_MARSHALAUTO
 
 #include "vtkStructuredData.h" // Needed for inline methods
 
 VTK_ABI_NAMESPACE_BEGIN
 class vtkDataArray;
+class vtkStructuredCellArray;
 class vtkLine;
 class vtkMatrix3x3;
 class vtkMatrix4x4;
 class vtkPixel;
+class vtkPoints;
 class vtkVertex;
 class vtkVoxel;
 
-class VTKCOMMONDATAMODEL_EXPORT vtkImageData : public vtkDataSet
+class VTKCOMMONDATAMODEL_EXPORT VTK_MARSHALAUTO vtkImageData : public vtkDataSet
 {
 public:
   static vtkImageData* New();
@@ -48,6 +52,11 @@ public:
    */
   void CopyStructure(vtkDataSet* ds) override;
 
+  /**
+   * Restore object to initial state. Release memory back to system.
+   */
+  void Initialize() override;
+
   /**
    * Return what type of dataset this is.
    */
@@ -63,16 +72,13 @@ public:
    */
   vtkIdType GetNumberOfCells() override;
   vtkIdType GetNumberOfPoints() override;
+  vtkPoints* GetPoints() override;
   double* GetPoint(vtkIdType ptId) VTK_SIZEHINT(3) override;
   void GetPoint(vtkIdType id, double x[3]) override;
   vtkCell* GetCell(vtkIdType cellId) override;
   vtkCell* GetCell(int i, int j, int k) override;
   void GetCell(vtkIdType cellId, vtkGenericCell* cell) override;
   void GetCellBounds(vtkIdType cellId, double bounds[6]) override;
-  virtual vtkIdType FindPoint(double x, double y, double z)
-  {
-    return this->vtkDataSet::FindPoint(x, y, z);
-  }
   vtkIdType FindPoint(double x[3]) override;
   vtkIdType FindCell(double x[3], vtkCell* cell, vtkIdType cellId, double tol2, int& subId,
     double pcoords[3], double* weights) override;
@@ -82,13 +88,9 @@ public:
     double pcoords[3], double* weights) override;
   int GetCellType(vtkIdType cellId) override;
   vtkIdType GetCellSize(vtkIdType cellId) override;
-  using vtkDataSet::GetCellPoints;
-  void GetCellPoints(vtkIdType cellId, vtkIdList* ptIds) override
-  {
-    int dimensions[3];
-    this->GetDimensions(dimensions);
-    vtkStructuredData::GetCellPoints(cellId, ptIds, this->DataDescription, dimensions);
-  }
+  void GetCellPoints(vtkIdType cellId, vtkIdType& npts, vtkIdType const*& pts, vtkIdList* ptIds)
+    VTK_SIZEHINT(pts, npts) override;
+  void GetCellPoints(vtkIdType cellId, vtkIdList* ptIds) override;
   void GetPointCells(vtkIdType ptId, vtkIdList* cellIds) override
   {
     int dimensions[3];
@@ -101,6 +103,21 @@ public:
   void GetCellNeighbors(vtkIdType cellId, vtkIdList* ptIds, vtkIdList* cellIds) override;
   ///@}
 
+  /**
+   * Return the image data connectivity array.
+   *
+   * NOTE: the returned object should not be modified.
+   */
+  vtkStructuredCellArray* GetCells();
+
+  /**
+   * Get the array of all cell types in the image data. Each single-component
+   * integer value is the same. The array is of size GetNumberOfCells().
+   *
+   * NOTE: the returned object should not be modified.
+   */
+  vtkConstantArray<int>* GetCellTypesArray();
+
   /**
    * Get cell neighbors around cell located at `seedloc`, except cell of id `cellId`.
    *
@@ -110,10 +127,31 @@ public:
    */
   void GetCellNeighbors(vtkIdType cellId, vtkIdList* ptIds, vtkIdList* cellIds, int* seedLoc);
 
+  ///@{
   /**
-   * Restore data object to initial state.
+   * Methods for supporting blanking of cells. Blanking turns on or off
+   * points in the structured grid, and hence the cells connected to them.
+   * These methods should be called only after the dimensions of the
+   * grid are set.
    */
-  void Initialize() override;
+  virtual void BlankPoint(vtkIdType ptId);
+  virtual void UnBlankPoint(vtkIdType ptId);
+  virtual void BlankPoint(int i, int j, int k);
+  virtual void UnBlankPoint(int i, int j, int k);
+  ///@}
+
+  ///@{
+  /**
+   * Methods for supporting blanking of cells. Blanking turns on or off
+   * cells in the structured grid.
+   * These methods should be called only after the dimensions of the
+   * grid are set.
+   */
+  virtual void BlankCell(vtkIdType ptId);
+  virtual void UnBlankCell(vtkIdType ptId);
+  virtual void BlankCell(int i, int j, int k);
+  virtual void UnBlankCell(int i, int j, int k);
+  ///@}
 
   /**
    * Return non-zero value if specified point is visible.
@@ -134,12 +172,18 @@ public:
    * 0 otherwise.
    */
   bool HasAnyBlankPoints() override;
+
   /**
    * Returns 1 if there is any visibility constraint on the cells,
    * 0 otherwise.
    */
   bool HasAnyBlankCells() override;
 
+  /**
+   * Get the data description of the image data.
+   */
+  vtkGetMacro(DataDescription, int);
+
   /**
    * Given the node dimensions of this grid instance, this method computes the
    * node dimensions. The value in each dimension can will have a lowest value
@@ -437,6 +481,19 @@ public:
   vtkGetObjectMacro(IndexToPhysicalMatrix, vtkMatrix4x4);
   ///@}
 
+  ///@{
+  /**
+   * Set the transformation matrix from the index space to the physical space
+   * coordinate system of the dataset. The transform is a 4 by 4 matrix.
+   * The supplied matrix pointer is not stored in the the image object but the matrix
+   * values are used for updating the Origin, Spacing, and DirectionMatrix.
+   * \sa SetOrigin
+   * \sa SetSpacing
+   * \sa SetDirectionMatrix
+   */
+  void ApplyIndexToPhysicalMatrix(vtkMatrix4x4* source);
+  ///@}
+
   ///@{
   /**
    * Convert coordinates from index space (ijk) to physical space (xyz).
@@ -457,6 +514,19 @@ public:
   vtkGetObjectMacro(PhysicalToIndexMatrix, vtkMatrix4x4);
   ///@}
 
+  ///@{
+  /**
+   * Get the transformation matrix from the physical space to the index space
+   * coordinate system of the dataset. The transform is a 4 by 4 matrix.
+   * The supplied matrix pointer is not stored in the the image object but the matrix
+   * values are used for updating the Origin, Spacing, and DirectionMatrix.
+   * \sa SetOrigin
+   * \sa SetSpacing
+   * \sa SetDirectionMatrix
+   */
+  void ApplyPhysicalToIndexMatrix(vtkMatrix4x4* source);
+  ///@}
+
   ///@{
   /**
    * Convert coordinates from physical space (xyz) to index space (ijk).
@@ -598,6 +668,10 @@ protected:
 
   int Extent[6];
 
+  vtkSmartPointer<vtkPoints> StructuredPoints;
+  vtkSmartPointer<vtkStructuredCellArray> StructuredCells;
+  vtkSmartPointer<vtkConstantArray<int>> StructuredCellTypes;
+
   // The first method assumes Active Scalars
   void ComputeIncrements();
   // This one is given the number of components of the
@@ -615,33 +689,21 @@ protected:
   // for the index to physical methods
   void ComputeTransforms();
 
-  // Cell utilities
-  vtkCell* GetCellTemplateForDataDescription();
-  bool GetCellTemplateForDataDescription(vtkGenericCell* cell);
-  bool GetIJKMinForCellId(vtkIdType cellId, int ijkMin[3]);
-  bool GetIJKMaxForIJKMin(int ijkMin[3], int ijkMax[3]);
-  void AddPointsToCellTemplate(vtkCell* cell, int ijkMin[3], int ijkMax[3]);
-
-  vtkTimeStamp ExtentComputeTime;
-
-  void SetDataDescription(int desc);
-  int GetDataDescription() { return this->DataDescription; }
+  void BuildImplicitStructures();
+  void BuildPoints();
+  void BuildCells();
+  void BuildCellTypes();
 
 private:
   void InternalImageDataCopy(vtkImageData* src);
 
   friend class vtkUniformGrid;
 
-  // for the GetCell method
-  vtkVertex* Vertex;
-  vtkLine* Line;
-  vtkPixel* Pixel;
-  vtkVoxel* Voxel;
-
   // for the GetPoint method
   double Point[3];
 
   int DataDescription;
+  bool DirectionMatrixIsIdentity;
 
   vtkImageData(const vtkImageData&) = delete;
   void operator=(const vtkImageData&) = delete;
@@ -675,13 +737,13 @@ inline double* vtkImageData::GetPoint(vtkIdType id)
 //----------------------------------------------------------------------------
 inline vtkIdType vtkImageData::GetNumberOfPoints()
 {
-  const int* extent = this->Extent;
-  vtkIdType dims[3];
-  dims[0] = extent[1] - extent[0] + 1;
-  dims[1] = extent[3] - extent[2] + 1;
-  dims[2] = extent[5] - extent[4] + 1;
+  return vtkStructuredData::GetNumberOfPoints(this->Extent);
+}
 
-  return dims[0] * dims[1] * dims[2];
+//----------------------------------------------------------------------------
+inline vtkIdType vtkImageData::GetNumberOfCells()
+{
+  return vtkStructuredData::GetNumberOfCells(this->Extent);
 }
 
 //----------------------------------------------------------------------------
