@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkPointLocator.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkPointLocator.h"
 
 #include "vtkBoundingBox.h"
@@ -24,6 +12,7 @@
 
 #include <algorithm> //std::sort
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkPointLocator);
 
 static const int VTK_INITIAL_SIZE = 1000;
@@ -145,6 +134,13 @@ void vtkPointLocator::FreeSearchStructure()
     delete[] this->HashTable;
     this->HashTable = nullptr;
   }
+
+  // The hash table has been invalidated.
+  // Reset the variables calculated from the dataset to their original values defined in the
+  // constructor.
+  vtkMath::UninitializeBounds(this->Bounds);
+  this->H[0] = this->H[1] = this->H[2] = 0.0;
+  this->Divisions[0] = this->Divisions[1] = this->Divisions[2] = 50;
 }
 
 //------------------------------------------------------------------------------
@@ -863,15 +859,16 @@ void vtkPointLocator::BuildLocatorInternal()
   vtkDebugMacro(<< "Hashing points...");
   this->Level = 1; // only single lowest level
 
+  // Delete the current hash table values and reset dataset metrics to their original values.
+  this->FreeSearchStructure();
+
   if (!this->DataSet || (numPts = this->DataSet->GetNumberOfPoints()) < 1)
   {
-    vtkErrorMacro(<< "No points to subdivide");
+    // Missing datasets and datasets with no points are valid inputs and should not log an error.
+    // Searching for the closest point id is always -1 for this data.
     return;
   }
 
-  //  Make sure the appropriate data is available
-  this->FreeSearchStructure();
-
   //  Size the root bucket.  Initialize bucket data structure, compute
   //  level and divisions.
   const double* bounds = this->DataSet->GetBounds();
@@ -1732,3 +1729,4 @@ void vtkPointLocator::PrintSelf(ostream& os, vtkIndent indent)
     os << indent << "Points: (none)\n";
   }
 }
+VTK_ABI_NAMESPACE_END
