@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkClientSocket.h
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkClientSocket
  * @brief   Encapsulates a client socket.
@@ -22,6 +10,7 @@
 
 #include "vtkCommonSystemModule.h" // For export macro
 #include "vtkSocket.h"
+VTK_ABI_NAMESPACE_BEGIN
 class vtkServerSocket;
 
 class VTKCOMMONSYSTEM_EXPORT vtkClientSocket : public vtkSocket
@@ -59,4 +48,5 @@ private:
   void operator=(const vtkClientSocket&) = delete;
 };
 
+VTK_ABI_NAMESPACE_END
 #endif
