@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkTimerLog.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 // .NAME vtkTimerLog - Maintains timing table for performance analysis
 // .SECTION Description
 // vtkTimerLog contains walltime and cputime measurements associated
@@ -49,8 +37,9 @@
 #include "vtkObjectFactory.h"
 
 //==============================================================================
+VTK_ABI_NAMESPACE_BEGIN
 static unsigned int vtkTimerLogCleanupCounter = 0;
-std::vector<vtkTimerLogEntry>* vtkTimerLogEntryVectorPtr = nullptr;
+static std::vector<vtkTimerLogEntry>* vtkTimerLogEntryVectorPtr = nullptr;
 
 vtkTimerLogCleanup::vtkTimerLogCleanup()
 {
@@ -798,3 +787,4 @@ int vtkTimerLog::GetMaxEntries()
 {
   return vtkTimerLog::MaxEntries;
 }
+VTK_ABI_NAMESPACE_END
