@@ -1,34 +1,6 @@
-/*=========================================================================
-
-   Program: ParaView
-   Module:  pqRenderView.cxx
-
-   Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
-   All rights reserved.
-
-   ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.2.
-
-   See License_v1.2.txt for the full ParaView license.
-   A copy of this license can be obtained by contacting
-   Kitware Inc.
-   28 Corporate Drive
-   Clifton Park, NY 12065
-   USA
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR
-CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
+// SPDX-FileCopyrightText: Copyright (c) Sandia Corporation
+// SPDX-License-Identifier: BSD-3-Clause
 #include "pqRenderView.h"
 
 // ParaView Server Manager includes.
@@ -38,6 +10,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkPVDataInformation.h"
 #include "vtkPVRenderView.h"
 #include "vtkPVRenderViewSettings.h"
+#include "vtkPVXMLElement.h"
 #include "vtkProcessModule.h"
 #include "vtkSMIntVectorProperty.h"
 #include "vtkSMInteractionUndoStackBuilder.h"
@@ -116,6 +89,23 @@ public:
   }
 
   ~pqInternal() = default;
+
+  // Convenience method to update the view widget cursor
+  void UpdateCursor(QWidget* widget)
+  {
+    QCursor cursor = this->CursorVisible ? this->Cursor : QCursor(Qt::BlankCursor);
+    if (auto* qvtkWidget = qobject_cast<pqQVTKWidget*>(widget))
+    {
+      qvtkWidget->setCursorCustom(cursor);
+    }
+    else
+    {
+      widget->setCursor(cursor);
+    }
+  }
+
+  QCursor Cursor;
+  bool CursorVisible = true;
 };
 
 namespace
@@ -201,6 +191,12 @@ void pqRenderView::initialize()
   // initialize the interaction undo-redo stack.
   vtkSMRenderViewProxy* viewProxy = this->getRenderViewProxy();
   this->Internal->UndoStackBuilder->SetRenderView(viewProxy);
+
+  if (this->getProxy()->GetHints() &&
+    this->getProxy()->GetHints()->FindNestedElementByName("HideCursor"))
+  {
+    this->setCursorVisible(false);
+  }
 }
 
 //-----------------------------------------------------------------------------
@@ -229,10 +225,10 @@ void pqRenderView::onResetCameraEvent()
 }
 
 //-----------------------------------------------------------------------------
-void pqRenderView::resetCamera(bool closest)
+void pqRenderView::resetCamera(bool closest, double offsetRatio)
 {
   this->fakeInteraction(true);
-  this->getRenderViewProxy()->ResetCamera(closest);
+  this->getRenderViewProxy()->ResetCamera(closest, offsetRatio);
   this->fakeInteraction(false);
   this->render();
 }
@@ -807,7 +803,7 @@ void pqRenderView::selectOnSurfaceInternal(int rect[4], QList<pqOutputPort*>& pq
       .arg("SelectSurfacePoints")
       .arg("Rectangle", rectVector)
       .arg("Modifier", modifier.empty() ? nullptr : modifier.c_str())
-      .arg("comment", "create a surface points selection");
+      .arg("comment", qPrintable(tr("create a surface points selection")));
   }
   else
   {
@@ -823,7 +819,7 @@ void pqRenderView::selectOnSurfaceInternal(int rect[4], QList<pqOutputPort*>& pq
         .arg("SelectSurfaceBlocks")
         .arg("Rectangle", rectVector)
         .arg("Modifier", modifier.empty() ? nullptr : modifier.c_str())
-        .arg("comment", "create a block selection");
+        .arg("comment", qPrintable(tr("create a block selection")));
     }
     else
     {
@@ -831,7 +827,7 @@ void pqRenderView::selectOnSurfaceInternal(int rect[4], QList<pqOutputPort*>& pq
         .arg("SelectSurfaceCells")
         .arg("Rectangle", rectVector)
         .arg("Modifier", modifier.empty() ? nullptr : modifier.c_str())
-        .arg("comment", "create a surface cells selection");
+        .arg("comment", qPrintable(tr("create a surface cells selection")));
     }
   }
 
@@ -898,7 +894,7 @@ void pqRenderView::selectPolygonInternal(vtkIntArray* polygon, QList<pqOutputPor
       .arg("SelectSurfacePoints")
       .arg("Polygon", polygonVector)
       .arg("Modifier", modifier.empty() ? nullptr : modifier.c_str())
-      .arg("comment", "create a surface points polygon selection");
+      .arg("comment", qPrintable(tr("create a surface points polygon selection")));
   }
   else
   {
@@ -912,7 +908,7 @@ void pqRenderView::selectPolygonInternal(vtkIntArray* polygon, QList<pqOutputPor
       .arg("SelectSurfaceCells")
       .arg("Polygon", polygonVector)
       .arg("Modifier", modifier.empty() ? nullptr : modifier.c_str())
-      .arg("comment", "create a surface cells polygon selection");
+      .arg("comment", qPrintable(tr("create a surface cells polygon selection")));
   }
 
   END_UNDO_EXCLUDE();
@@ -924,7 +920,7 @@ void pqRenderView::selectPolygonInternal(vtkIntArray* polygon, QList<pqOutputPor
 void pqRenderView::selectFrustumCells(int rect[4], int selectionModifier)
 {
   vtkSMRenderViewProxy* renderModuleP = this->getRenderViewProxy();
-  std::string modifier = GetSelectionModifierAsString(selectionModifier);
+  const std::string modifier = GetSelectionModifierAsString(selectionModifier);
 
   vtkNew<vtkCollection> selectedRepresentations;
   vtkNew<vtkCollection> selectionSources;
@@ -939,17 +935,13 @@ void pqRenderView::selectFrustumCells(int rect[4], int selectionModifier)
     return;
   }
 
-  std::vector<int> rectVector(4);
-  for (size_t i = 0; i < 4; ++i)
-  {
-    rectVector[i] = rect[i];
-  }
+  const std::vector<int> rectVector(rect, rect + 4);
 
   SM_SCOPED_TRACE(CallFunction)
     .arg("SelectCellsThrough")
     .arg("Rectangle", rectVector)
     .arg("Modifier", modifier.empty() ? nullptr : modifier.c_str())
-    .arg("comment", "create a frustum selection of cells");
+    .arg("comment", qPrintable(tr("create a frustum selection of cells")));
 
   END_UNDO_EXCLUDE();
 
@@ -964,7 +956,7 @@ void pqRenderView::selectFrustumCells(int rect[4], int selectionModifier)
 void pqRenderView::selectFrustumPoints(int rect[4], int selectionModifier)
 {
   vtkSMRenderViewProxy* renderModuleP = this->getRenderViewProxy();
-  std::string modifier = GetSelectionModifierAsString(selectionModifier);
+  const std::string modifier = GetSelectionModifierAsString(selectionModifier);
 
   vtkNew<vtkCollection> selectedRepresentations;
   vtkNew<vtkCollection> selectionSources;
@@ -979,13 +971,13 @@ void pqRenderView::selectFrustumPoints(int rect[4], int selectionModifier)
     return;
   }
 
-  std::vector<int> rectVector(rect, rect + 4);
+  const std::vector<int> rectVector(rect, rect + 4);
 
   SM_SCOPED_TRACE(CallFunction)
     .arg("SelectPointsThrough")
     .arg("Rectangle", rectVector)
     .arg("Modifier", modifier.empty() ? nullptr : modifier.c_str())
-    .arg("comment", "create a frustum selection of points");
+    .arg("comment", qPrintable(tr("create a frustum selection of points")));
 
   END_UNDO_EXCLUDE();
 
@@ -996,6 +988,34 @@ void pqRenderView::selectFrustumPoints(int rect[4], int selectionModifier)
   this->emitSelectionSignal(output_ports, selectionModifier);
 }
 
+//-----------------------------------------------------------------------------
+void pqRenderView::selectFrustumBlocks(int rect[4], int selectionModifier)
+{
+  vtkSMRenderViewProxy* renderModuleP = this->getRenderViewProxy();
+  const std::string modifier = GetSelectionModifierAsString(selectionModifier);
+
+  vtkNew<vtkCollection> selectedRepresentations;
+  vtkNew<vtkCollection> selectionSources;
+  QList<pqOutputPort*> output_ports;
+
+  BEGIN_UNDO_EXCLUDE();
+  if (!renderModuleP->SelectFrustumCells(
+        rect, selectedRepresentations, selectionSources, this->UseMultipleRepresentationSelection))
+  {
+    END_UNDO_EXCLUDE();
+    this->emitSelectionSignal(output_ports, selectionModifier);
+    return;
+  }
+
+  END_UNDO_EXCLUDE();
+
+  this->collectSelectionPorts(
+    selectedRepresentations, selectionSources, output_ports, selectionModifier, true);
+
+  // Fire selection event to let the world know that this view selected something.
+  this->emitSelectionSignal(output_ports, selectionModifier);
+}
+
 //-----------------------------------------------------------------------------
 void pqRenderView::selectBlock(int rectangle[4], int selectionModifier)
 {
@@ -1072,7 +1092,7 @@ void pqRenderView::updateInteractionMode(pqOutputPort* opPort)
   // FIXME: move this logic to server-manager.
   SM_SCOPED_TRACE(PropertiesModified)
     .arg(this->getProxy())
-    .arg("comment", "changing interaction mode based on data extents");
+    .arg("comment", qPrintable(tr("changing interaction mode based on data extents")));
   if (is2DDataSet)
   {
     // Update camera position
@@ -1123,10 +1143,26 @@ void pqRenderView::onGenericFilmicPresetsChange()
 }
 
 //-----------------------------------------------------------------------------
-void pqRenderView::setCursor(const QCursor& c)
+void pqRenderView::setCursor(const QCursor& cursor)
 {
-  if (QWidget* wdg = this->widget())
+  this->Internal->Cursor = cursor;
+  this->Internal->UpdateCursor(this->widget());
+}
+
+//-----------------------------------------------------------------------------
+QCursor pqRenderView::cursor()
+{
+  if (auto* qvtkWidget = qobject_cast<pqQVTKWidget*>(this->widget()))
   {
-    wdg->setCursor(c);
+    return qvtkWidget->cursorCustom();
   }
+
+  return this->widget()->cursor();
+}
+
+//-----------------------------------------------------------------------------
+void pqRenderView::setCursorVisible(bool visible)
+{
+  this->Internal->CursorVisible = visible;
+  this->Internal->UpdateCursor(this->widget());
 }
