@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkAppendFilter.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkAppendFilter.h"
 
 #include "vtkBoundingBox.h"
@@ -32,6 +20,7 @@
 #include <string>
 #include <unordered_map>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkAppendFilter);
 
 //------------------------------------------------------------------------------
@@ -103,6 +92,28 @@ vtkDataSetCollection* vtkAppendFilter::GetInputList()
   return this->InputList;
 }
 
+//------------------------------------------------------------------------------
+// Helper
+namespace
+{
+struct RenumberingVisitor
+{
+  // Insert full cell
+  template <typename CellStateT>
+  void operator()(CellStateT& state, vtkIdType* idMap, vtkIdType offset)
+  {
+    using ValueType = typename CellStateT::ValueType;
+    auto* conn = state.GetConnectivity();
+    const vtkIdType nids = conn->GetNumberOfValues();
+    for (vtkIdType i = 0; i < nids; ++i)
+    {
+      ValueType tmp = conn->GetValue(i);
+      conn->SetValue(i, idMap[tmp + offset]);
+    }
+  }
+};
+}
+
 //------------------------------------------------------------------------------
 // Append data sets into single unstructured grid
 int vtkAppendFilter::RequestData(vtkInformation* vtkNotUsed(request),
@@ -276,7 +287,7 @@ int vtkAppendFilter::RequestData(vtkInformation* vtkNotUsed(request),
   vtkIdType ptOffset = 0;
   float decimal = 0.0;
   inputs->InitTraversal(iter);
-  int abort = 0;
+  bool abort = false;
   double p[3];
   while (!abort && (dataSet = inputs->GetNextDataSet(iter)))
   {
@@ -329,7 +340,7 @@ int vtkAppendFilter::RequestData(vtkInformation* vtkNotUsed(request),
       {
         decimal += 0.05;
         this->UpdateProgress(decimal);
-        abort = this->GetAbortExecute();
+        abort = this->CheckAbort();
       }
     }
 
@@ -340,20 +351,16 @@ int vtkAppendFilter::RequestData(vtkInformation* vtkNotUsed(request),
       newPtIds->Reset();
       if (ug && dataSet->GetCellType(cellId) == VTK_POLYHEDRON)
       {
-        vtkIdType nfaces;
-        const vtkIdType* facePtIds;
-        ug->GetFaceStream(cellId, nfaces, facePtIds);
-        for (vtkIdType id = 0; id < nfaces; ++id)
+        vtkNew<vtkCellArray> faces;
+        ug->GetPolyhedronFaces(cellId, faces);
+        faces->Visit(RenumberingVisitor{}, globalIndices, ptOffset);
+        dataSet->GetCellPoints(cellId, ptIds);
+        for (vtkIdType id = 0; id < ptIds->GetNumberOfIds(); ++id)
         {
-          vtkIdType nPoints = facePtIds[0];
-          newPtIds->InsertNextId(nPoints);
-          for (vtkIdType j = 1; j <= nPoints; ++j)
-          {
-            newPtIds->InsertNextId(globalIndices[facePtIds[j] + ptOffset]);
-          }
-          facePtIds += nPoints + 1;
+          newPtIds->InsertId(id, globalIndices[ptIds->GetId(id) + ptOffset]);
         }
-        output->InsertNextCell(VTK_POLYHEDRON, nfaces, newPtIds->GetPointer(0));
+        output->InsertNextCell(
+          VTK_POLYHEDRON, newPtIds->GetNumberOfIds(), newPtIds->GetPointer(0), faces);
       }
       else
       {
@@ -371,7 +378,7 @@ int vtkAppendFilter::RequestData(vtkInformation* vtkNotUsed(request),
       {
         decimal += 0.05;
         this->UpdateProgress(decimal);
-        abort = this->GetAbortExecute();
+        abort = this->CheckAbort();
       }
     }
     ptOffset += dataSetNumPts;
@@ -521,3 +528,4 @@ void vtkAppendFilter::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "OutputPointsPrecision: " << this->OutputPointsPrecision << "\n";
   os << indent << "Tolerance: " << this->Tolerance << "\n";
 }
+VTK_ABI_NAMESPACE_END
