@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkBinnedDecimation.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkBinnedDecimation.h"
 
 #include "vtkArrayDispatch.h"
@@ -39,6 +27,7 @@
 #include <atomic>
 #include <vector>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkBinnedDecimation);
 
 //----------------------------------------------------------------------------
@@ -71,11 +60,13 @@ struct BinPoints
   double hX, hY, hZ;
   double fX, fY, fZ, bX, bY, bZ;
   vtkIdType xD, yD, zD, xyD;
+  vtkBinnedDecimation* Filter;
 
-  BinPoints(
-    PointsT* pts, TIds* binIds, const int* dims, const double* bounds, const double* spacing)
+  BinPoints(PointsT* pts, TIds* binIds, const int* dims, const double* bounds,
+    const double* spacing, vtkBinnedDecimation* filter)
     : Points(pts)
     , BinIds(binIds)
+    , Filter(filter)
   {
     for (auto i = 0; i < 3; ++i)
     {
@@ -129,8 +120,24 @@ struct BinPoints
     const auto points = vtk::DataArrayTupleRange<3>(this->Points, ptId, endPtId);
     double x[3];
     TIds* bins = this->BinIds + ptId;
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endPtId - ptId) / 10 + 1, (vtkIdType)1000);
+
     for (const auto tuple : points)
     {
+      if (ptId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
+      ptId++;
+
       x[0] = static_cast<double>(tuple[0]);
       x[1] = static_cast<double>(tuple[1]);
       x[2] = static_cast<double>(tuple[2]);
@@ -152,9 +159,10 @@ struct GenerateTriangles
   vtkIdType* OutTris;
   vtkIdType* OutTriOffsets;
   ArrayList* Arrays;
+  vtkBinnedDecimation* Filter;
 
   GenerateTriangles(TIds* bins, TPtMap* ptMap, vtkCellArray* tris, TIds* triMap, vtkIdType* outTris,
-    vtkIdType* outTriOffsets, ArrayList* arrays)
+    vtkIdType* outTriOffsets, ArrayList* arrays, vtkBinnedDecimation* filter)
     : BinIds(bins)
     , PointMap(ptMap)
     , Tris(tris)
@@ -162,6 +170,7 @@ struct GenerateTriangles
     , OutTris(outTris)
     , OutTriOffsets(outTriOffsets)
     , Arrays(arrays)
+    , Filter(filter)
   {
   }
 
@@ -177,9 +186,22 @@ struct GenerateTriangles
     const TIds* triMap = this->TriMap;
     vtkIdType *outTris = this->OutTris, *outTri;
     vtkIdType *outTriOffsets = this->OutTriOffsets, *outOffsets;
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endTriId - triId) / 10 + 1, (vtkIdType)1000);
 
     for (; triId < endTriId; ++triId)
     {
+      if (triId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       if ((triMap[triId + 1] - triMap[triId]) > 0) // spit out triangle
       {
         cellIter->GetCellAtId(triId, npts, tri);
@@ -212,12 +234,15 @@ struct SelectOutput
   vtkCellArray* Tris;
   TIds* TriMap;
   vtkSMPThreadLocal<vtkSmartPointer<vtkCellArrayIterator>> CellIterator;
+  vtkBinnedDecimation* Filter;
 
-  SelectOutput(TIds* bins, unsigned char* ptUses, vtkCellArray* tris, TIds* triMap)
+  SelectOutput(TIds* bins, unsigned char* ptUses, vtkCellArray* tris, TIds* triMap,
+    vtkBinnedDecimation* filter)
     : BinIds(bins)
     , PointUses(ptUses)
     , Tris(tris)
     , TriMap(triMap)
+    , Filter(filter)
   {
   }
 
@@ -230,9 +255,22 @@ struct SelectOutput
     vtkCellArrayIterator* cellIter = this->CellIterator.Local();
     TIds* triMap = this->TriMap + triId;
     unsigned char* ptUses = this->PointUses;
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endTriId - triId) / 10 + 1, (vtkIdType)1000);
 
     for (; triId < endTriId; ++triId, ++triMap)
     {
+      if (triId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       cellIter->GetCellAtId(triId, npts, tri);
       // All three points have to be in different bins
       if (this->BinIds[tri[0]] != this->BinIds[tri[1]] &&
@@ -262,11 +300,13 @@ struct InitializePointMap
   const TIds* BinIds;
   const unsigned char* PointUses;
   TIds* PointMap;
+  vtkBinnedDecimation* Filter;
 
-  InitializePointMap(TIds* binIds, unsigned char* ptUses, TIds* ptMap)
+  InitializePointMap(TIds* binIds, unsigned char* ptUses, TIds* ptMap, vtkBinnedDecimation* filter)
     : BinIds(binIds)
     , PointUses(ptUses)
     , PointMap(ptMap)
+    , Filter(filter)
   {
   }
 
@@ -275,9 +315,22 @@ struct InitializePointMap
     const TIds* binIds = this->BinIds;
     const unsigned char* ptUses = this->PointUses + ptId;
     TIds* ptMap = this->PointMap;
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endPtId - ptId) / 10 + 1, (vtkIdType)1000);
 
     for (; ptId < endPtId; ++ptId)
     {
+      if (ptId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       if (*ptUses++ > 0)
       {
         ptMap[binIds[ptId]] = (-1); // mark unvisited
@@ -291,7 +344,7 @@ struct InitializePointMap
 template <typename PointsT, typename TIds>
 void ReuseDecimate(vtkIdType numPts, PointsT* pts, vtkIdType numTris, vtkCellArray* tris,
   vtkCellData* inCD, vtkCellData* outCD, vtkIdType numBins, const int* dims, const double* bounds,
-  const double* spacing, vtkPolyData* output)
+  const double* spacing, vtkPolyData* output, vtkBinnedDecimation* filter)
 {
   // Setup execution. Several arrays are used to transform the data.
   // The bin id of each point
@@ -306,21 +359,20 @@ void ReuseDecimate(vtkIdType numPts, PointsT* pts, vtkIdType numTris, vtkCellArr
   TIds* triMap = new TIds[numTris + 1];
 
   // Bin points to generate a bin index for each point
-  BinPoints<PointsT, TIds> binPoints(pts, binIds, dims, bounds, spacing);
+  BinPoints<PointsT, TIds> binPoints(pts, binIds, dims, bounds, spacing, filter);
   vtkSMPTools::For(0, numPts, binPoints);
 
   // Select which triangles and points are sent to the output.
-  SelectOutput<TIds> selectOutput(binIds, ptUses, tris, triMap);
+  SelectOutput<TIds> selectOutput(binIds, ptUses, tris, triMap, filter);
   vtkSMPTools::For(0, numTris, selectOutput);
 
   // Initialize the point map, only the bins that contain something
-  InitializePointMap<TIds> initPtMap(binIds, ptUses, ptMap);
+  InitializePointMap<TIds> initPtMap(binIds, ptUses, ptMap, filter);
   vtkSMPTools::For(0, numPts, initPtMap);
 
   // Prefix sums to roll up the points and cells, and setup offsets for
   // subsequent threading. This could be threaded, although the gains
   // are likely modest.
-  vtkIdType numOutPts = 0;
   for (vtkIdType ptId = 0; ptId < numPts; ++ptId)
   {
     if (ptUses[ptId] > 0)
@@ -328,7 +380,6 @@ void ReuseDecimate(vtkIdType numPts, PointsT* pts, vtkIdType numTris, vtkCellArr
       if (ptMap[binIds[ptId]] < 0)
       {
         ptMap[binIds[ptId]] = ptId;
-        ++numOutPts;
       }
     }
   }
@@ -357,8 +408,8 @@ void ReuseDecimate(vtkIdType numPts, PointsT* pts, vtkIdType numTris, vtkCellArr
   }
 
   // Produce output triangles.
-  GenerateTriangles<TIds, TIds> genTris(
-    binIds, ptMap, tris, triMap, outTris, outTriOffsets, (outCD != nullptr ? (&arrays) : nullptr));
+  GenerateTriangles<TIds, TIds> genTris(binIds, ptMap, tris, triMap, outTris, outTriOffsets,
+    (outCD != nullptr ? (&arrays) : nullptr), filter);
   vtkSMPTools::For(0, numTris, genTris);
   outTrisArray->SetData(outOffsets, outConn);
 
@@ -375,7 +426,8 @@ struct PointReuseWorker
 {
   template <typename DataT>
   void operator()(DataT* pts, bool largeIds, vtkCellArray* tris, vtkCellData* inCD,
-    vtkCellData* outCD, int* divs, double bounds[6], double spacing[3], vtkPolyData* output)
+    vtkCellData* outCD, int* divs, double bounds[6], double spacing[3], vtkPolyData* output,
+    vtkBinnedDecimation* filter)
   {
     vtkIdType numPts = pts->GetNumberOfTuples();
     vtkIdType numTris = tris->GetNumberOfCells();
@@ -385,12 +437,12 @@ struct PointReuseWorker
     if (!largeIds)
     {
       ReuseDecimate<DataT, int>(
-        numPts, pts, numTris, tris, inCD, outCD, numBins, divs, bounds, spacing, output);
+        numPts, pts, numTris, tris, inCD, outCD, numBins, divs, bounds, spacing, output, filter);
     }
     else
     {
       ReuseDecimate<DataT, vtkIdType>(
-        numPts, pts, numTris, tris, inCD, outCD, numBins, divs, bounds, spacing, output);
+        numPts, pts, numTris, tris, inCD, outCD, numBins, divs, bounds, spacing, output, filter);
     }
   }
 };
@@ -409,12 +461,15 @@ struct MapOutput
   vtkCellArray* Tris;
   TIds* TriMap;
   vtkSMPThreadLocal<vtkSmartPointer<vtkCellArrayIterator>> CellIterator;
+  vtkBinnedDecimation* Filter;
 
-  MapOutput(TIds* bins, std::atomic<TIds>* ptMap, vtkCellArray* tris, TIds* triMap)
+  MapOutput(TIds* bins, std::atomic<TIds>* ptMap, vtkCellArray* tris, TIds* triMap,
+    vtkBinnedDecimation* filter)
     : BinIds(bins)
     , PointMap(ptMap)
     , Tris(tris)
     , TriMap(triMap)
+    , Filter(filter)
   {
   }
 
@@ -447,9 +502,22 @@ struct MapOutput
     TIds* triMap = this->TriMap + triId;
     std::atomic<TIds>* ptMap = this->PointMap;
     TIds binIds[3];
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endTriId - triId) / 10 + 1, (vtkIdType)1000);
 
     for (; triId < endTriId; ++triId, ++triMap)
     {
+      if (triId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       cellIter->GetCellAtId(triId, npts, tri);
       binIds[0] = this->BinIds[tri[0]];
       binIds[1] = this->BinIds[tri[1]];
@@ -483,11 +551,14 @@ struct CountPoints
   const int* Dims;
   std::atomic<TIds>* PointMap;
   int* SliceOffsets;
+  vtkBinnedDecimation* Filter;
 
-  CountPoints(const int* dims, std::atomic<TIds>* ptMap, int* sliceOffsets)
+  CountPoints(
+    const int* dims, std::atomic<TIds>* ptMap, int* sliceOffsets, vtkBinnedDecimation* filter)
     : Dims(dims)
     , PointMap(ptMap)
     , SliceOffsets(sliceOffsets)
+    , Filter(filter)
   {
   }
 
@@ -496,9 +567,22 @@ struct CountPoints
   void operator()(vtkIdType slice, vtkIdType endSlice)
   {
     int binOffset = slice * this->Dims[0] * this->Dims[1];
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endSlice - slice) / 10 + 1, (vtkIdType)1000);
 
     for (; slice < endSlice; ++slice)
     {
+      if (slice % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       vtkIdType numSlicePts = 0;
       for (auto j = 0; j < this->Dims[1]; ++j)
       {
@@ -543,9 +627,11 @@ struct GenerateBinPoints
   PointsT* InPoints;
   ArrayList* Arrays;
   float* OutPoints;
+  vtkBinnedDecimation* Filter;
 
   GenerateBinPoints(int genMode, const double* bounds, const double* spacing, const int* dims,
-    int* sliceOffsets, std::atomic<TIds>* ptMap, PointsT* inPts, ArrayList* arrays, float* outPts)
+    int* sliceOffsets, std::atomic<TIds>* ptMap, PointsT* inPts, ArrayList* arrays, float* outPts,
+    vtkBinnedDecimation* filter)
     : PointGenerationMode(genMode)
     , Bounds(bounds)
     , Spacing(spacing)
@@ -555,6 +641,7 @@ struct GenerateBinPoints
     , InPoints(inPts)
     , Arrays(arrays)
     , OutPoints(outPts)
+    , Filter(filter)
   {
   }
 
@@ -566,9 +653,22 @@ struct GenerateBinPoints
     float* xOut;
     double xIn[3];
     const auto pts = vtk::DataArrayTupleRange<3>(this->InPoints);
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endSlice - slice) / 10 + 1, (vtkIdType)1000);
 
     for (; slice < endSlice; ++slice)
     {
+      if (slice % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       for (auto j = 0; j < this->Dims[1]; ++j)
       {
         for (auto i = 0; i < this->Dims[0]; ++i)
@@ -616,14 +716,14 @@ template <typename PointsT, typename TIds>
 void BinPointsDecimate(int genMode, vtkIdType numPts, PointsT* pts, vtkPointData* inPD,
   vtkPointData* outPD, vtkIdType numTris, vtkCellArray* tris, vtkCellData* inCD, vtkCellData* outCD,
   vtkIdType numBins, const int* dims, const double* bounds, const double* spacing,
-  vtkPolyData* output)
+  vtkPolyData* output, vtkBinnedDecimation* filter)
 {
   // Setup execution. Several arrays are used to transform the data.
   // The bin id of each point.
   TIds* binIds = new TIds[numPts];
 
   // Now bin points to generate a bin index for each point.
-  BinPoints<PointsT, TIds> binPoints(pts, binIds, dims, bounds, spacing);
+  BinPoints<PointsT, TIds> binPoints(pts, binIds, dims, bounds, spacing, filter);
   vtkSMPTools::For(0, numPts, binPoints);
 
   // The ptMap is the output point id assigned to each bin (if the bin
@@ -639,13 +739,13 @@ void BinPointsDecimate(int genMode, vtkIdType numPts, PointsT* pts, vtkPointData
 
   // Begin to construct mappings of input points and cells, to output points
   // and cells.
-  MapOutput<TIds> mapOutput(binIds, ptMap, tris, triMap);
+  MapOutput<TIds> mapOutput(binIds, ptMap, tris, triMap, filter);
   vtkSMPTools::For(0, numTris, mapOutput);
 
   // Now generate the new points. First generate new point ids, and then
   // produce the actual points.
   int* sliceOffsets = new int[dims[2] + 1];
-  CountPoints<TIds> countPts(dims, ptMap, sliceOffsets);
+  CountPoints<TIds> countPts(dims, ptMap, sliceOffsets, filter);
   vtkSMPTools::For(0, dims[2], countPts);
   int numNewPts = sliceOffsets[dims[2]];
 
@@ -661,7 +761,7 @@ void BinPointsDecimate(int genMode, vtkIdType numPts, PointsT* pts, vtkPointData
 
   GenerateBinPoints<PointsT, TIds> genPts(genMode, bounds, spacing, dims, sliceOffsets, ptMap, pts,
     (outPD != nullptr ? (&ptArrays) : nullptr),
-    vtkFloatArray::FastDownCast(newPts->GetData())->GetPointer(0));
+    vtkFloatArray::FastDownCast(newPts->GetData())->GetPointer(0), filter);
   vtkSMPTools::For(0, dims[2], genPts);
   output->SetPoints(newPts);
 
@@ -691,8 +791,8 @@ void BinPointsDecimate(int genMode, vtkIdType numPts, PointsT* pts, vtkPointData
     arrays.AddArrays(numOutTris, inCD, outCD);
   }
 
-  GenerateTriangles<TIds, std::atomic<TIds>> genTris(
-    binIds, ptMap, tris, triMap, outTris, outTriOffsets, (outCD != nullptr ? (&arrays) : nullptr));
+  GenerateTriangles<TIds, std::atomic<TIds>> genTris(binIds, ptMap, tris, triMap, outTris,
+    outTriOffsets, (outCD != nullptr ? (&arrays) : nullptr), filter);
   vtkSMPTools::For(0, numTris, genTris);
   outTrisArray->SetData(outOffsets, outConn);
 
@@ -712,7 +812,7 @@ struct BinPointsWorker
   template <typename DataT>
   void operator()(DataT* pts, vtkPointData* inPD, vtkPointData* outPD, bool largeIds, int genMode,
     vtkCellArray* tris, vtkCellData* inCD, vtkCellData* outCD, int* divs, double bounds[6],
-    double spacing[3], vtkPolyData* output)
+    double spacing[3], vtkPolyData* output, vtkBinnedDecimation* filter)
   {
     vtkIdType numPts = pts->GetNumberOfTuples();
     vtkIdType numTris = tris->GetNumberOfCells();
@@ -721,12 +821,12 @@ struct BinPointsWorker
     if (!largeIds)
     {
       BinPointsDecimate<DataT, int>(genMode, numPts, pts, inPD, outPD, numTris, tris, inCD, outCD,
-        numBins, divs, bounds, spacing, output);
+        numBins, divs, bounds, spacing, output, filter);
     }
     else
     {
       BinPointsDecimate<DataT, vtkIdType>(genMode, numPts, pts, inPD, outPD, numTris, tris, inCD,
-        outCD, numBins, divs, bounds, spacing, output);
+        outCD, numBins, divs, bounds, spacing, output, filter);
     }
   }
 };
@@ -756,8 +856,8 @@ struct BinPointTuples : public BinPoints<PointsT, TIds>
   BinTuple<TIds>* BinTuples;
 
   BinPointTuples(PointsT* pts, BinTuple<TIds>* binTuples, const int* dims, const double* bounds,
-    const double* spacing)
-    : BinPoints<PointsT, TIds>(pts, nullptr, dims, bounds, spacing)
+    const double* spacing, vtkBinnedDecimation* filter)
+    : BinPoints<PointsT, TIds>(pts, nullptr, dims, bounds, spacing, filter)
     , BinTuples(binTuples)
   {
   }
@@ -767,8 +867,21 @@ struct BinPointTuples : public BinPoints<PointsT, TIds>
     const auto points = vtk::DataArrayTupleRange<3>(this->Points, ptId, endPtId);
     double x[3];
     BinTuple<TIds>* bins = this->BinTuples + ptId;
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endPtId - ptId) / 10 + 1, (vtkIdType)1000);
     for (const auto tuple : points)
     {
+      if (ptId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       (*bins).PtId = ptId++;
       x[0] = static_cast<double>(tuple[0]);
       x[1] = static_cast<double>(tuple[1]);
@@ -786,11 +899,13 @@ struct MarkBinnedTris
   vtkCellArray* Tris;
   TIds* TriMap;
   vtkSMPThreadLocal<vtkSmartPointer<vtkCellArrayIterator>> CellIterator;
+  vtkBinnedDecimation* Filter;
 
-  MarkBinnedTris(BinTuple<TIds>* bt, vtkCellArray* tris, TIds* triMap)
+  MarkBinnedTris(BinTuple<TIds>* bt, vtkCellArray* tris, TIds* triMap, vtkBinnedDecimation* filter)
     : BinTuples(bt)
     , Tris(tris)
     , TriMap(triMap)
+    , Filter(filter)
   {
   }
 
@@ -803,9 +918,22 @@ struct MarkBinnedTris
     vtkCellArrayIterator* cellIter = this->CellIterator.Local();
     TIds* triMap = this->TriMap + triId;
     TIds binIds[3];
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endTriId - triId) / 10 + 1, (vtkIdType)1000);
 
     for (; triId < endTriId; ++triId, ++triMap)
     {
+      if (triId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       cellIter->GetCellAtId(triId, npts, tri);
       binIds[0] = this->BinTuples[tri[0]].Bin;
       binIds[1] = this->BinTuples[tri[1]].Bin;
@@ -837,15 +965,17 @@ struct BinAveTriangles
   vtkIdType* OutTris;
   vtkIdType* OutTriOffsets;
   ArrayList* Arrays;
+  vtkBinnedDecimation* Filter;
 
   BinAveTriangles(const BinTuple<TIds>* bt, vtkCellArray* tris, TIds* triMap, vtkIdType* outTris,
-    vtkIdType* outTriOffsets, ArrayList* arrays)
+    vtkIdType* outTriOffsets, ArrayList* arrays, vtkBinnedDecimation* filter)
     : BinTuples(bt)
     , Tris(tris)
     , TriMap(triMap)
     , OutTris(outTris)
     , OutTriOffsets(outTriOffsets)
     , Arrays(arrays)
+    , Filter(filter)
   {
   }
 
@@ -860,9 +990,22 @@ struct BinAveTriangles
     const TIds* triMap = this->TriMap;
     vtkIdType *outTris = this->OutTris, *outTri;
     vtkIdType *outTriOffsets = this->OutTriOffsets, *outOffsets;
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endTriId - triId) / 10 + 1, (vtkIdType)1000);
 
     for (; triId < endTriId; ++triId)
     {
+      if (triId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       if ((triMap[triId + 1] - triMap[triId]) > 0) // spit out triangle
       {
         cellIter->GetCellAtId(triId, npts, tri);
@@ -892,11 +1035,14 @@ struct GenerateAveTriangles
   const BinTuple<TIds>* BinTuples;
   const TIds* Offsets;
   vtkIdType* OutTris;
+  vtkBinnedDecimation* Filter;
 
-  GenerateAveTriangles(const BinTuple<TIds>* bt, TIds* offsets, vtkIdType* outTris)
+  GenerateAveTriangles(
+    const BinTuple<TIds>* bt, TIds* offsets, vtkIdType* outTris, vtkBinnedDecimation* filter)
     : BinTuples(bt)
     , Offsets(offsets)
     , OutTris(outTris)
+    , Filter(filter)
   {
   }
 
@@ -905,9 +1051,22 @@ struct GenerateAveTriangles
     const BinTuple<TIds>* binTuples = this->BinTuples;
     const TIds* offsets = this->Offsets;
     vtkIdType* outTri = this->OutTris + 3 * triId;
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endTriId - triId) / 10 + 1, (vtkIdType)1000);
 
     for (; triId < endTriId; ++triId, outTri += 3)
     {
+      if (triId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       outTri[0] = (*(binTuples + offsets[outTri[0]])).PtId;
       outTri[1] = (*(binTuples + offsets[outTri[1]])).PtId;
       outTri[2] = (*(binTuples + offsets[outTri[2]])).PtId;
@@ -925,14 +1084,18 @@ struct MapOffsets
   TIds NumPts;
   TIds NumBins;
   TIds BatchSize;
+  vtkBinnedDecimation* Filter;
 
-  MapOffsets(BinTuple<TIds>* bt, TIds* offsets, TIds numPts, TIds numBins, TIds numBatches)
+  MapOffsets(BinTuple<TIds>* bt, TIds* offsets, TIds numPts, TIds numBins, TIds numBatches,
+    vtkBinnedDecimation* filter)
     : BinTuples(bt)
     , Offsets(offsets)
     , NumPts(numPts)
     , NumBins(numBins)
+    , Filter(filter)
   {
-    this->BatchSize = static_cast<int>(ceil(static_cast<double>(numPts) / numBatches));
+    // Round up BatchSize so that final batch isn't larger than the rest
+    this->BatchSize = static_cast<int>((numPts - 1) / numBatches + 1);
   }
 
   // Traverse sorted points (i.e., tuples) and update bin offsets.
@@ -940,14 +1103,18 @@ struct MapOffsets
   {
     TIds* offsets = this->Offsets;
     const BinTuple<TIds>* curPt = this->BinTuples + batch * this->BatchSize;
-    const BinTuple<TIds>* endBatchPt = this->BinTuples + batchEnd * this->BatchSize;
     const BinTuple<TIds>* endPt = this->BinTuples + this->NumPts;
-    const BinTuple<TIds>* prevPt;
-    endBatchPt = (endBatchPt > endPt ? endPt : endBatchPt);
+    // Use endPt if we're on the last batch, otherwise use BatchSize
+    const BinTuple<TIds>* endBatchPt = endPt;
+    if (batchEnd * this->BatchSize <= this->NumPts)
+    {
+      endBatchPt = this->BinTuples + batchEnd * this->BatchSize;
+    }
 
     // Special case at the very beginning of the bin tuples array.  If
     // the first point is in bin# N, then all bins up and including
     // N must refer to the first point.
+    const BinTuple<TIds>* prevPt;
     if (curPt == this->BinTuples)
     {
       prevPt = this->BinTuples;
@@ -962,13 +1129,30 @@ struct MapOffsets
       prevPt = curPt;
     } // else in the middle of a batch
 
+    bool isFirst = vtkSMPTools::GetSingleThread();
+
     // Okay we have a starting point for a bin run. Now we can begin
     // filling in the offsets in this batch. A previous thread should
     // have/will have completed the previous and subsequent runs outside
     // of the [batch,batchEnd) range
+    vtkIdType checkAbortInterval =
+      std::min((vtkIdType)(endBatchPt - prevPt) / 10 + 1, (vtkIdType)1000);
+
     for (curPt = prevPt; curPt < endBatchPt;)
     {
-      for (; curPt->Bin == prevPt->Bin && curPt <= endBatchPt; ++curPt)
+      if (batch % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
+      batch++;
+      for (; curPt < endBatchPt && curPt->Bin == prevPt->Bin; ++curPt)
       {
         // advance
       }
@@ -997,11 +1181,13 @@ struct CountAvePts
   const int* Dims;
   const TIds* Offsets;
   int* SliceOffsets;
+  vtkBinnedDecimation* Filter;
 
-  CountAvePts(const int* dims, const TIds* offsets, int* sliceOffsets)
+  CountAvePts(const int* dims, const TIds* offsets, int* sliceOffsets, vtkBinnedDecimation* filter)
     : Dims(dims)
     , Offsets(offsets)
     , SliceOffsets(sliceOffsets)
+    , Filter(filter)
   {
   }
 
@@ -1010,9 +1196,22 @@ struct CountAvePts
   void operator()(vtkIdType slice, vtkIdType endSlice)
   {
     int binNum = slice * this->Dims[0] * this->Dims[1];
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endSlice - slice) / 10 + 1, (vtkIdType)1000);
 
     for (; slice < endSlice; ++slice)
     {
+      if (slice % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       vtkIdType numSlicePts = 0;
       for (auto j = 0; j < this->Dims[1]; ++j)
       {
@@ -1056,9 +1255,11 @@ struct GenerateAveBinPoints
   ArrayList* Arrays;
   float* OutPoints;
   vtkSMPThreadLocal<std::vector<vtkIdType>> PtIds;
+  vtkBinnedDecimation* Filter;
 
   GenerateAveBinPoints(const int* dims, PointsT* inPts, const int* sliceOffsets,
-    BinTuple<TIds>* binTuples, const TIds* offsets, ArrayList* arrays, float* outPts)
+    BinTuple<TIds>* binTuples, const TIds* offsets, ArrayList* arrays, float* outPts,
+    vtkBinnedDecimation* filter)
     : Dims(dims)
     , InPoints(inPts)
     , SliceOffsets(sliceOffsets)
@@ -1066,6 +1267,7 @@ struct GenerateAveBinPoints
     , Offsets(offsets)
     , Arrays(arrays)
     , OutPoints(outPts)
+    , Filter(filter)
   {
   }
 
@@ -1081,9 +1283,22 @@ struct GenerateAveBinPoints
     BinTuple<TIds>* pIds;
     TIds pId;
     std::vector<vtkIdType> v = this->PtIds.Local();
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endSlice - slice) / 10 + 1, (vtkIdType)1000);
 
     for (; slice < endSlice; ++slice)
     {
+      if (slice % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
       for (auto j = 0; j < this->Dims[1]; ++j)
       {
         for (auto i = 0; i < this->Dims[0]; ++i)
@@ -1135,14 +1350,15 @@ struct GenerateAveBinPoints
 template <typename PointsT, typename TIds>
 void AvePointsDecimate(vtkIdType numPts, PointsT* pts, vtkPointData* inPD, vtkPointData* outPD,
   vtkIdType numTris, vtkCellArray* tris, vtkCellData* inCD, vtkCellData* outCD, vtkIdType numBins,
-  const int* dims, const double* bounds, const double* spacing, vtkPolyData* output)
+  const int* dims, const double* bounds, const double* spacing, vtkPolyData* output,
+  vtkBinnedDecimation* filter)
 {
   // Setup execution. Several arrays are used to transform the data.
   // Define the bin id and associated point id of each point.
   BinTuple<TIds>* binTuples = new BinTuple<TIds>[numPts];
 
   // Now bin points to generate a bin index for each point.
-  BinPointTuples<PointsT, TIds> binPoints(pts, binTuples, dims, bounds, spacing);
+  BinPointTuples<PointsT, TIds> binPoints(pts, binTuples, dims, bounds, spacing, filter);
   vtkSMPTools::For(0, numPts, binPoints);
 
   // Initially, triMap indicates which triangles are output. And then
@@ -1151,7 +1367,7 @@ void AvePointsDecimate(vtkIdType numPts, PointsT* pts, vtkPointData* inPD, vtkPo
 
   // Begin to construct mappings of input points and cells, to output points
   // and cells. First identify the triangles to be sent to the output.
-  MarkBinnedTris<TIds> markBinnedTris(binTuples, tris, triMap);
+  MarkBinnedTris<TIds> markBinnedTris(binTuples, tris, triMap, filter);
   vtkSMPTools::For(0, numTris, markBinnedTris);
 
   // Create a mapping of the input triangles to the output triangles.
@@ -1181,8 +1397,8 @@ void AvePointsDecimate(vtkIdType numPts, PointsT* pts, vtkPointData* inPD, vtkPo
     arrays.AddArrays(numOutTris, inCD, outCD);
   }
 
-  BinAveTriangles<TIds> binTris(
-    binTuples, tris, triMap, outTris, outTriOffsets, (outCD != nullptr ? (&arrays) : nullptr));
+  BinAveTriangles<TIds> binTris(binTuples, tris, triMap, outTris, outTriOffsets,
+    (outCD != nullptr ? (&arrays) : nullptr), filter);
   vtkSMPTools::For(0, numTris, binTris);
   outTrisArray->SetData(outOffsets, outConn);
 
@@ -1195,7 +1411,7 @@ void AvePointsDecimate(vtkIdType numPts, PointsT* pts, vtkPointData* inPD, vtkPo
   // into the sorted bin tuples (i.e., runs of points in each bin).
   TIds* offsets = new TIds[numBins + 1];
   TIds numBatches = (numPts < 10000 ? 1 : 100); // totally arbitrary
-  MapOffsets<TIds> offMapper(binTuples, offsets, numPts, numBins, numBatches);
+  MapOffsets<TIds> offMapper(binTuples, offsets, numPts, numBins, numBatches, filter);
   vtkSMPTools::For(0, numBatches, offMapper);
   offsets[numBins] = numPts;
 
@@ -1204,7 +1420,7 @@ void AvePointsDecimate(vtkIdType numPts, PointsT* pts, vtkPointData* inPD, vtkPo
   // have to count the new points, and then accumulate them with a prefix
   // sum. For convenience, this is done on a bin slice-by-slice manner.
   int* sliceOffsets = new int[dims[2] + 1];
-  CountAvePts<TIds> countPts(dims, offsets, sliceOffsets);
+  CountAvePts<TIds> countPts(dims, offsets, sliceOffsets, filter);
   vtkSMPTools::For(0, dims[2], countPts);
   int numNewPts = sliceOffsets[dims[2]];
 
@@ -1226,12 +1442,12 @@ void AvePointsDecimate(vtkIdType numPts, PointsT* pts, vtkPointData* inPD, vtkPo
   // Do the core work of averaging point coordinates and attributes.
   GenerateAveBinPoints<PointsT, TIds> genPts(dims, pts, sliceOffsets, binTuples, offsets,
     (outPD != nullptr ? (&ptArrays) : nullptr),
-    vtkFloatArray::FastDownCast(newPts->GetData())->GetPointer(0));
+    vtkFloatArray::FastDownCast(newPts->GetData())->GetPointer(0), filter);
   vtkSMPTools::For(0, dims[2], genPts);
   output->SetPoints(newPts);
 
   // Finally map the triangle connectivity list to the new point ids.
-  GenerateAveTriangles<TIds> genAveTris(binTuples, offsets, outTris);
+  GenerateAveTriangles<TIds> genAveTris(binTuples, offsets, outTris, filter);
   vtkSMPTools::For(0, numOutTris, genAveTris);
 
   // Clean up
@@ -1249,7 +1465,7 @@ struct AvePointsWorker
   template <typename DataT>
   void operator()(DataT* pts, vtkPointData* inPD, vtkPointData* outPD, bool largeIds,
     vtkCellArray* tris, vtkCellData* inCD, vtkCellData* outCD, int* divs, double bounds[6],
-    double spacing[3], vtkPolyData* output)
+    double spacing[3], vtkPolyData* output, vtkBinnedDecimation* filter)
   {
     vtkIdType numPts = pts->GetNumberOfTuples();
     vtkIdType numTris = tris->GetNumberOfCells();
@@ -1258,12 +1474,12 @@ struct AvePointsWorker
     if (!largeIds)
     {
       AvePointsDecimate<DataT, int>(numPts, pts, inPD, outPD, numTris, tris, inCD, outCD, numBins,
-        divs, bounds, spacing, output);
+        divs, bounds, spacing, output, filter);
     }
     else
     {
       AvePointsDecimate<DataT, vtkIdType>(numPts, pts, inPD, outPD, numTris, tris, inCD, outCD,
-        numBins, divs, bounds, spacing, output);
+        numBins, divs, bounds, spacing, output, filter);
     }
   }
 };
@@ -1448,10 +1664,10 @@ int vtkBinnedDecimation::RequestData(vtkInformation* vtkNotUsed(request),
     using PointReuseDispatch = vtkArrayDispatch::DispatchByValueType<Reals>;
     PointReuseWorker deciWorker;
     if (!PointReuseDispatch::Execute(inPts->GetData(), deciWorker, this->LargeIds, inTris, inCD,
-          outCD, this->NumberOfDivisions, this->Bounds, this->DivisionSpacing, output))
+          outCD, this->NumberOfDivisions, this->Bounds, this->DivisionSpacing, output, this))
     { // Fallback to slowpath for other point types
       deciWorker(inPts->GetData(), this->LargeIds, inTris, inCD, outCD, this->NumberOfDivisions,
-        this->Bounds, this->DivisionSpacing, output);
+        this->Bounds, this->DivisionSpacing, output, this);
     }
   }
 
@@ -1463,10 +1679,10 @@ int vtkBinnedDecimation::RequestData(vtkInformation* vtkNotUsed(request),
     BinPointsWorker deciWorker;
     if (!BinPointsDispatch::Execute(inPts->GetData(), deciWorker, inPD, outPD, this->LargeIds,
           this->PointGenerationMode, inTris, inCD, outCD, this->NumberOfDivisions, this->Bounds,
-          this->DivisionSpacing, output))
+          this->DivisionSpacing, output, this))
     { // Fallback to slowpath for other point types
       deciWorker(inPts->GetData(), inPD, outPD, this->LargeIds, this->PointGenerationMode, inTris,
-        inCD, outCD, this->NumberOfDivisions, this->Bounds, this->DivisionSpacing, output);
+        inCD, outCD, this->NumberOfDivisions, this->Bounds, this->DivisionSpacing, output, this);
     }
   }
 
@@ -1476,11 +1692,11 @@ int vtkBinnedDecimation::RequestData(vtkInformation* vtkNotUsed(request),
     using AvePointsDispatch = vtkArrayDispatch::DispatchByValueType<Reals>;
     AvePointsWorker deciWorker;
     if (!AvePointsDispatch::Execute(inPts->GetData(), deciWorker, inPD, outPD, this->LargeIds,
-          inTris, inCD, outCD, this->NumberOfDivisions, this->Bounds, this->DivisionSpacing,
-          output))
+          inTris, inCD, outCD, this->NumberOfDivisions, this->Bounds, this->DivisionSpacing, output,
+          this))
     { // Fallback to slowpath for other point types
       deciWorker(inPts->GetData(), inPD, outPD, this->LargeIds, inTris, inCD, outCD,
-        this->NumberOfDivisions, this->Bounds, this->DivisionSpacing, output);
+        this->NumberOfDivisions, this->Bounds, this->DivisionSpacing, output, this);
     }
   }
 
@@ -1651,3 +1867,4 @@ void vtkBinnedDecimation::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Pass Point Data : " << this->ProducePointData << endl;
   os << indent << "Produce Cell Data : " << this->ProduceCellData << endl;
 }
+VTK_ABI_NAMESPACE_END
