@@ -1,21 +1,10 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkPolyDataPlaneCutter.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkPolyDataPlaneCutter.h"
 
 #include "vtkArrayDispatch.h"
 #include "vtkArrayListTemplate.h" // For processing attribute data
+#include "vtkBatch.h"
 #include "vtkCellArray.h"
 #include "vtkCellArrayIterator.h"
 #include "vtkCellData.h"
@@ -34,11 +23,12 @@
 #include "vtkPolygon.h"
 #include "vtkSMPThreadLocal.h"
 #include "vtkSMPTools.h"
-#include "vtkStaticCellLinks.h"
 #include "vtkStaticEdgeLocatorTemplate.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkObjectFactoryNewMacro(vtkPolyDataPlaneCutter);
+VTK_ABI_NAMESPACE_END
 
 //------------------------------------------------------------------------------
 // A high level overview of the algorithms is as follows. All steps are
@@ -66,6 +56,7 @@ struct EvaluatePoints
   double Origin[3];                  // Plane origin
   double Normal[3];                  // Plane normal
   std::vector<unsigned char>& PtMap; // 0/1 values indicating below and above plane
+  vtkPolyDataPlaneCutter* Filter;
   bool Intersects;
 
   // These are used to determine whether the plane
@@ -73,9 +64,11 @@ struct EvaluatePoints
   vtkSMPThreadLocal<unsigned char> BelowPlane;
   vtkSMPThreadLocal<unsigned char> AbovePlane;
 
-  EvaluatePoints(TP* pts, vtkPlane* plane, std::vector<unsigned char>& ptMap)
+  EvaluatePoints(
+    TP* pts, vtkPlane* plane, std::vector<unsigned char>& ptMap, vtkPolyDataPlaneCutter* filter)
     : Points(pts)
     , PtMap(ptMap)
+    , Filter(filter)
   {
     plane->GetOrigin(Origin);
     plane->GetNormal(Normal);
@@ -92,9 +85,23 @@ struct EvaluatePoints
   {
     const auto pts = vtk::DataArrayTupleRange<3>(this->Points);
     double p[3], *n = this->Normal, *o = this->Origin;
+    bool isFirst = vtkSMPTools::GetSingleThread();
+    vtkIdType checkAbortInterval = std::min((endPtId - ptId) / 10 + 1, (vtkIdType)1000);
 
     for (; ptId < endPtId; ++ptId)
     {
+      if (ptId % checkAbortInterval == 0)
+      {
+        if (isFirst)
+        {
+          this->Filter->CheckAbort();
+        }
+        if (this->Filter->GetAbortOutput())
+        {
+          break;
+        }
+      }
+
       auto pt = pts[ptId];
       p[0] = pt[0];
       p[1] = pt[1];
@@ -146,10 +153,11 @@ struct EvaluatePointsWorker
   bool Intersects;
 
   template <typename DataT>
-  void operator()(DataT* pts, vtkPlane* plane, std::vector<unsigned char>& ptMap)
+  void operator()(
+    DataT* pts, vtkPlane* plane, std::vector<unsigned char>& ptMap, vtkPolyDataPlaneCutter* filter)
   {
     vtkIdType numPts = pts->GetNumberOfTuples();
-    EvaluatePoints<DataT> ep(pts, plane, ptMap);
+    EvaluatePoints<DataT> ep(pts, plane, ptMap, filter);
     vtkSMPTools::For(0, numPts, ep);
     this->Intersects = ep.Intersects;
   }
@@ -177,6 +185,30 @@ bool CellIntersectsPlane(
   return (belowPlane && abovePlane);
 } // CellIntersectsPlane
 
+struct PolyCutterBatchData
+{
+  vtkIdType LinesOffset;
+
+  PolyCutterBatchData()
+    : LinesOffset(0)
+  {
+  }
+  ~PolyCutterBatchData() = default;
+  PolyCutterBatchData& operator+=(const PolyCutterBatchData& other)
+  {
+    this->LinesOffset += other.LinesOffset;
+    return *this;
+  }
+  PolyCutterBatchData operator+(const PolyCutterBatchData& other) const
+  {
+    PolyCutterBatchData result = *this;
+    result += other;
+    return result;
+  }
+};
+using PolyCutterBatch = vtkBatch<PolyCutterBatchData>;
+using PolyCutterBatches = vtkBatches<PolyCutterBatchData>;
+
 // Gather information on the size of the output. Basically, count the
 // number of line segments created in each batch. Then roll up these
 // counts to create offsets which are later used to generate the output
@@ -186,21 +218,20 @@ struct EvaluateCells
   const std::vector<unsigned char>& PtMap;
   vtkCellArray* Cells;
   vtkIdType NumCells;
-  vtkIdType BatchSize;
-  vtkIdType NumBatches;
-  std::vector<vtkIdType> LineOffsets;
+  vtkPolyDataPlaneCutter* Filter;
+  PolyCutterBatches Batches;
   std::vector<unsigned char> CellMap;
   vtkIdType NumLines;
   vtkSMPThreadLocal<vtkSmartPointer<vtkCellArrayIterator>> CellIterator;
 
-  EvaluateCells(const std::vector<unsigned char>& ptMap, vtkCellArray* cells, int batchSize)
+  EvaluateCells(const std::vector<unsigned char>& ptMap, vtkCellArray* cells, int batchSize,
+    vtkPolyDataPlaneCutter* filter)
     : PtMap(ptMap)
     , Cells(cells)
-    , BatchSize(batchSize)
+    , Filter(filter)
   {
     this->NumCells = this->Cells->GetNumberOfCells();
-    this->NumBatches = ((this->NumCells - 1) / batchSize) + 1;
-    this->LineOffsets.resize(this->NumBatches + 1); // added one for offset
+    this->Batches.Initialize(this->NumCells, batchSize);
     this->CellMap.resize(this->NumCells);
   }
 
@@ -211,17 +242,25 @@ struct EvaluateCells
     vtkIdType npts;
     const vtkIdType* cell;
     vtkCellArrayIterator* cellIter = this->CellIterator.Local();
+    bool isFirst = vtkSMPTools::GetSingleThread();
 
     // Over batches of cells
     for (; batchNum < endBatchNum; ++batchNum)
     {
-      vtkIdType cellId = batchNum * this->BatchSize;
-      vtkIdType endCellId =
-        (cellId + this->BatchSize > this->NumCells ? this->NumCells : cellId + this->BatchSize);
+      if (isFirst)
+      {
+        this->Filter->CheckAbort();
+      }
+      if (this->Filter->GetAbortOutput())
+      {
+        break;
+      }
+      auto& batch = this->Batches[batchNum];
+      auto& batchNumberOfLines = batch.Data.LinesOffset;
       vtkIdType numLines = 0;
 
       // For all cells making up this batch
-      for (; cellId < endCellId; ++cellId)
+      for (vtkIdType cellId = batch.BeginId; cellId < batch.EndId; ++cellId)
       {
         cellIter->GetCellAtId(cellId, npts, cell);
 
@@ -239,7 +278,7 @@ struct EvaluateCells
 
       // Keep track of the number of cell intersections for each batch. A
       // subsequent prefix sum will produce offsets from this information.
-      this->LineOffsets[batchNum] = numLines;
+      batchNumberOfLines = numLines;
     } // for each batch of cells
   }
 
@@ -247,19 +286,16 @@ struct EvaluateCells
   // generated in the next pass.
   void Reduce()
   {
+    // trim batches that have no intersections
+    this->Batches.TrimBatches(
+      [](const PolyCutterBatch& batch) { return batch.Data.LinesOffset == 0; });
+
     // Prefix sum over the batches to roll up total output.
-    vtkIdType numLines, totalNumLines = 0;
-    for (auto batchNum = 0; batchNum < this->NumBatches; ++batchNum)
-    {
-      numLines = this->LineOffsets[batchNum];
-      this->LineOffsets[batchNum] = totalNumLines;
-      totalNumLines += numLines;
-    }
-    this->LineOffsets[this->NumBatches] = totalNumLines;
-    this->NumLines = totalNumLines;
+    const auto globalSum = this->Batches.BuildOffsetsAndGetGlobalSum();
+    this->NumLines = globalSum.LinesOffset;
   } // Reduce
 
-  void Execute() { vtkSMPTools::For(0, this->NumBatches, *this); }
+  void Execute() { vtkSMPTools::For(0, this->Batches.GetNumberOfBatches(), *this); }
 }; // EvaluateCells
 
 // Represent cut edges. A cut edge has two values: (V0,V1) defining the
@@ -288,16 +324,18 @@ struct ExtractLines
   std::vector<EdgeTupleType>& Edges;
   ArrayList* Arrays;
   vtkSMPThreadLocal<vtkSmartPointer<vtkCellArrayIterator>> CellIterator;
+  vtkPolyDataPlaneCutter* Filter;
 
   ExtractLines(EvaluateCells& ec, const std::vector<unsigned char>& ptMap, vtkCellArray* cells,
     std::vector<unsigned char>& cellMap, vtkIdTypeArray* lineConn, vtkIdTypeArray* lineOffsets,
-    std::vector<EdgeTupleType>& e, ArrayList* arrays)
+    std::vector<EdgeTupleType>& e, ArrayList* arrays, vtkPolyDataPlaneCutter* filter)
     : EC(ec)
     , PtMap(ptMap)
     , Cells(cells)
     , CellMap(cellMap)
     , Edges(e)
     , Arrays(arrays)
+    , Filter(filter)
   {
     this->NumCells = this->Cells->GetNumberOfCells();
     this->LineConn = lineConn->GetPointer(0);
@@ -315,27 +353,33 @@ struct ExtractLines
     const std::vector<unsigned char>& cellMap = this->CellMap;
     std::vector<EdgeTupleType>& edges = this->Edges;
     ArrayList* arrays = this->Arrays;
+    bool isFirst = vtkSMPTools::GetSingleThread();
 
     // For each batch, process the intersected cells in the batch.
     for (; batchNum < endBatchNum; ++batchNum)
     {
-      vtkIdType cellId = batchNum * this->EC.BatchSize;
-      vtkIdType endCellId =
-        (cellId + this->EC.BatchSize > this->NumCells ? this->NumCells
-                                                      : cellId + this->EC.BatchSize);
+      if (isFirst)
+      {
+        this->Filter->CheckAbort();
+      }
+      if (this->Filter->GetAbortOutput())
+      {
+        break;
+      }
+      auto& batch = this->EC.Batches[batchNum];
 
       // What's going on here is the line offsets are being updated, and
       // the merged edges are being created with indices into the line
       // connectivity array. Later, after edge sorting and when the final
       // point ids are defined, the edge connectivity array is updated with
       // the output point ids.
-      vtkIdType lineNum = this->EC.LineOffsets[batchNum];
+      vtkIdType lineNum = batch.Data.LinesOffset;
       vtkIdType lineConnIdx = 2 * lineNum;
       vtkIdType* lineOffsets = this->LineOffsets + lineNum;
       vtkIdType lineOffset = 2 * lineNum;
 
       // For all cells in this batch
-      for (; cellId < endCellId; ++cellId)
+      for (vtkIdType cellId = batch.BeginId; cellId < batch.EndId; ++cellId)
       {
         if (cellMap[cellId] != 0) // if the cell is cut
         {
@@ -371,7 +415,7 @@ struct ExtractLines
 
   void Reduce() {}
 
-  void Execute() { vtkSMPTools::For(0, this->EC.NumBatches, *this); }
+  void Execute() { vtkSMPTools::For(0, this->EC.Batches.GetNumberOfBatches(), *this); }
 }; // ExtractLines
 
 // Update the line connectivity with new point ids.
@@ -381,13 +425,15 @@ struct OutputLines
   const EdgeTupleType* MergeEdges;
   const vtkIdType* MergeOffsets;
   vtkIdType* OutLinesConn;
+  vtkPolyDataPlaneCutter* Filter;
 
   OutputLines(vtkIdType numNewPts, const EdgeTupleType* mergeEdges, const vtkIdType* mergeOffsets,
-    vtkIdTypeArray* outLines)
+    vtkIdTypeArray* outLines, vtkPolyDataPlaneCutter* filter)
     : NumNewPts(numNewPts)
     , MergeEdges(mergeEdges)
     , MergeOffsets(mergeOffsets)
     , OutLinesConn(outLines->GetPointer(0))
+    , Filter(filter)
   {
   }
 
@@ -401,9 +447,23 @@ struct OutputLines
     vtkSMPTools::For(
       0, numNewPts, [&, edges, offsets, linesConn](vtkIdType newPtId, vtkIdType endNewPtId) {
         const EdgeTupleType* edge;
+        bool isFirst = vtkSMPTools::GetSingleThread();
+        vtkIdType checkAbortInterval = std::min((endNewPtId - newPtId) / 10 + 1, (vtkIdType)1000);
 
         for (; newPtId < endNewPtId; ++newPtId)
         {
+          if (newPtId % checkAbortInterval == 0)
+          {
+            if (isFirst)
+            {
+              this->Filter->CheckAbort();
+            }
+            if (this->Filter->GetAbortOutput())
+            {
+              break;
+            }
+          }
+
           vtkIdType numEdges = offsets[newPtId + 1] - offsets[newPtId];
           vtkIdType updatedId = newPtId;
           for (auto i = 0; i < numEdges; ++i)
@@ -423,7 +483,7 @@ struct OutputPointsWorker
   template <typename InPtsT, typename OutPtsT>
   void operator()(InPtsT* inPts, OutPtsT* outPts, vtkIdType numNewPts,
     const EdgeTupleType* mergeEdges, const vtkIdType* mergeOffsets, vtkPlane* plane,
-    ArrayList* arrays)
+    ArrayList* arrays, vtkPolyDataPlaneCutter* filter)
   {
     // Interpolate new points on cut edges. Since we are going to the trouble of
     // computing the t parametric coordinate along the edge, also interpolate the
@@ -437,9 +497,22 @@ struct OutputPointsWorker
         const auto in = vtk::DataArrayTupleRange<3>(inPts);
         auto out = vtk::DataArrayTupleRange<3>(outPts);
         double x0[3], x1[3];
+        bool isFirst = vtkSMPTools::GetSingleThread();
+        vtkIdType checkAbortInterval = std::min((endNewPtId - newPtId) / 10 + 1, (vtkIdType)1000);
 
         for (; newPtId < endNewPtId; ++newPtId)
         {
+          if (newPtId % checkAbortInterval == 0)
+          {
+            if (isFirst)
+            {
+              filter->CheckAbort();
+            }
+            if (filter->GetAbortOutput())
+            {
+              break;
+            }
+          }
           const EdgeTupleType* edge = mergeEdges + mergeOffsets[newPtId];
           const auto x0T = in[edge->V0];
           x0[0] = x0T[0];
@@ -468,6 +541,7 @@ struct OutputPointsWorker
 
 } // anonymous namespace
 
+VTK_ABI_NAMESPACE_BEGIN
 //==============================================================================
 //------------------------------------------------------------------------------
 // Here is the VTK class proper.
@@ -517,8 +591,6 @@ vtkMTimeType vtkPolyDataPlaneCutter::GetMTime()
 int vtkPolyDataPlaneCutter::RequestData(vtkInformation* vtkNotUsed(request),
   vtkInformationVector** inputVector, vtkInformationVector* outputVector)
 {
-  vtkLog(INFO, "Executing vtkPolyData plane cutter");
-
   // Get the input and output
   vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
   vtkInformation* outInfo = outputVector->GetInformationObject(0);
@@ -541,9 +613,9 @@ int vtkPolyDataPlaneCutter::RequestData(vtkInformation* vtkNotUsed(request),
   using EvaluatePointsDispatch = vtkArrayDispatch::DispatchByValueType<vtkArrayDispatch::Reals>;
   EvaluatePointsWorker epWorker;
   std::vector<unsigned char> ptMap(numPts);
-  if (!EvaluatePointsDispatch::Execute(inPts->GetData(), epWorker, this->Plane, ptMap))
+  if (!EvaluatePointsDispatch::Execute(inPts->GetData(), epWorker, this->Plane, ptMap, this))
   {
-    epWorker(inPts->GetData(), this->Plane, ptMap);
+    epWorker(inPts->GetData(), this->Plane, ptMap, this);
   }
 
   // Return quickly when no cells are cut.
@@ -556,7 +628,7 @@ int vtkPolyDataPlaneCutter::RequestData(vtkInformation* vtkNotUsed(request),
   // the number of output line segments). We are going to process cells in
   // batches, and to avoid the use of thread local storage and to facilitate
   // threading, keep track of information regarding the size of the output.
-  EvaluateCells evalCells(ptMap, cells, this->BatchSize);
+  EvaluateCells evalCells(ptMap, cells, this->BatchSize, this);
   evalCells.Execute();
 
   // Build the cell array for the output lines. Also generate cut edges (and
@@ -581,7 +653,7 @@ int vtkPolyDataPlaneCutter::RequestData(vtkInformation* vtkNotUsed(request),
 
   // Extract the line segments.
   ExtractLines extLines(evalCells, ptMap, cells, evalCells.CellMap, lineConn, lineOffsets,
-    mergeEdges, (this->InterpolateAttributes ? &cellArrays : nullptr));
+    mergeEdges, (this->InterpolateAttributes ? &cellArrays : nullptr), this);
   extLines.Execute();
   lineOffsets->SetComponent(numLines, 0, 2 * numLines);
 
@@ -597,7 +669,7 @@ int vtkPolyDataPlaneCutter::RequestData(vtkInformation* vtkNotUsed(request),
   // each group of duplicate edges generates one new cut point). Now update
   // the line connectivity array with new point ids.
   vtkNew<vtkCellArray> outLines;
-  OutputLines ol(numOutPts, mergeEdges.data(), mergeOffsets, lineConn);
+  OutputLines ol(numOutPts, mergeEdges.data(), mergeOffsets, lineConn, this);
   ol.Execute();
   outLines->SetData(lineOffsets, lineConn);
 
@@ -636,10 +708,10 @@ int vtkPolyDataPlaneCutter::RequestData(vtkInformation* vtkNotUsed(request),
   OutputPointsWorker opWorker;
   if (!OutputPointsDispatch::Execute(inPts->GetData(), outPts->GetData(), opWorker, numOutPts,
         mergeEdges.data(), mergeOffsets, this->Plane,
-        (this->InterpolateAttributes ? &ptArrays : nullptr)))
+        (this->InterpolateAttributes ? &ptArrays : nullptr), this))
   {
     opWorker(inPts->GetData(), outPts->GetData(), numOutPts, mergeEdges.data(), mergeOffsets,
-      this->Plane, (this->InterpolateAttributes ? &ptArrays : nullptr));
+      this->Plane, (this->InterpolateAttributes ? &ptArrays : nullptr), this);
   }
 
   // If normals requested, then create an array of point normals.
@@ -662,6 +734,7 @@ int vtkPolyDataPlaneCutter::RequestData(vtkInformation* vtkNotUsed(request),
 
   return 1;
 }
+VTK_ABI_NAMESPACE_END
 
 // Support convexity check on input
 namespace // begin anonymous namespace
@@ -726,6 +799,7 @@ struct CheckConvex
 
 } // anonymous namespace
 
+VTK_ABI_NAMESPACE_BEGIN
 //------------------------------------------------------------------------------
 // Assess whether the specified data object can be processed by this filter. The
 // input data must be of type vtkPolyData, and contain only convex polygons.
@@ -774,3 +848,4 @@ void vtkPolyDataPlaneCutter::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Output Points Precision: " << this->OutputPointsPrecision << "\n";
   os << indent << "Batch Size: " << this->BatchSize << "\n";
 }
+VTK_ABI_NAMESPACE_END
