@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkAbstractInterpolatedVelocityField.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkAbstractInterpolatedVelocityField.h"
 
 #include "vtkAbstractCellLocator.h"
@@ -31,6 +19,7 @@
 #include "vtkUnstructuredGrid.h"
 
 //------------------------------------------------------------------------------
+VTK_ABI_NAMESPACE_BEGIN
 vtkCxxSetObjectMacro(vtkAbstractInterpolatedVelocityField, FindCellStrategy, vtkFindCellStrategy);
 
 //------------------------------------------------------------------------------
@@ -271,33 +260,43 @@ int vtkAbstractInterpolatedVelocityField::FunctionValues(vtkDataSet* dataset, do
 
     if (this->ForceSurfaceTangentVector)
     {
-      dataset->GetCellPoints(this->LastCellId, this->PointIds);
-      if (this->PointIds->GetNumberOfIds() < 3)
+      // check cell is a 2D cell
+      if (this->CurrentCell->GetCellDimension() != 2)
       {
-        vtkErrorMacro(<< "Cannot compute normal on cells with less than 3 points");
+        vtkWarningMacro(
+          << "Cannot compute normal on non 2D cells, skipping forcing vector on tangent");
       }
       else
       {
-        double p1[3];
-        double p2[3];
-        double p3[3];
-        dataset->GetPoint(this->PointIds->GetId(0), p1);
-        dataset->GetPoint(this->PointIds->GetId(1), p2);
-        dataset->GetPoint(this->PointIds->GetId(2), p3);
+        dataset->GetCellPoints(this->LastCellId, this->PointIds);
+        if (this->PointIds->GetNumberOfIds() < 3)
+        {
+          vtkWarningMacro(<< "Cannot compute normal on cells with less than 3 points, skipping "
+                             "forcing vector on tangent");
+        }
+        else
+        {
+          double p1[3];
+          double p2[3];
+          double p3[3];
+          dataset->GetPoint(this->PointIds->GetId(0), p1);
+          dataset->GetPoint(this->PointIds->GetId(1), p2);
+          dataset->GetPoint(this->PointIds->GetId(2), p3);
 
-        // Compute orthogonal component
-        const double v1[3] = { p2[0] - p1[0], p2[1] - p1[1], p2[2] - p1[2] };
-        const double v2[3] = { p3[0] - p1[0], p3[1] - p1[1], p3[2] - p1[2] };
+          // Compute orthogonal component
+          const double v1[3] = { p2[0] - p1[0], p2[1] - p1[1], p2[2] - p1[2] };
+          const double v2[3] = { p3[0] - p1[0], p3[1] - p1[1], p3[2] - p1[2] };
 
-        double normal[3];
-        vtkMath::Cross(v1, v2, normal);
-        vtkMath::Normalize(normal);
-        const double k = vtkMath::Dot(normal, f);
+          double normal[3];
+          vtkMath::Cross(v1, v2, normal);
+          vtkMath::Normalize(normal);
+          const double k = vtkMath::Dot(normal, f);
 
-        // Remove non-orthogonal component.
-        f[0] -= (normal[0] * k);
-        f[1] -= (normal[1] * k);
-        f[2] -= (normal[2] * k);
+          // Remove non-orthogonal component.
+          f[0] -= (normal[0] * k);
+          f[1] -= (normal[1] * k);
+          f[2] -= (normal[2] * k);
+        }
       }
     }
 
@@ -412,7 +411,7 @@ bool vtkAbstractInterpolatedVelocityField::FindAndUpdateCell(
     else
     {
       this->CacheMiss++;
-      if (this->SurfaceDataset)
+      if (this->SurfaceDataset && strategy)
       {
         // if we are on a surface dataset, we can use the strategy to find the closest point
         closestPointFound = strategy->FindClosestPointWithinRadius(x, tol, this->LastClosestPoint,
@@ -605,3 +604,4 @@ void vtkAbstractInterpolatedVelocityField::PrintSelf(ostream& os, vtkIndent inde
   os << indent << "FindCell Strategy: " << endl;
   this->FindCellStrategy->PrintSelf(os, indent);
 }
+VTK_ABI_NAMESPACE_END
