@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkLinearTransformCellLocator.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkLinearTransformCellLocator.h"
 
 #include "vtkArrayDispatch.h"
@@ -36,6 +24,7 @@
 
 static constexpr vtkIdType VTK_MAX_SAMPLE_POINTS = 100;
 
+VTK_ABI_NAMESPACE_BEGIN
 //------------------------------------------------------------------------------
 vtkStandardNewMacro(vtkLinearTransformCellLocator);
 
@@ -74,40 +63,6 @@ void vtkLinearTransformCellLocator::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "UseAllPoints: " << this->UseAllPoints << "\n";
 }
 
-//------------------------------------------------------------------------------
-static vtkSmartPointer<vtkPoints> GetPoints(vtkDataSet* ds)
-{
-  vtkSmartPointer<vtkPoints> points;
-  if (auto pointSet = vtkPointSet::SafeDownCast(ds))
-  {
-    points = pointSet->GetPoints();
-  }
-  else if (auto imageData = vtkImageData::SafeDownCast(ds))
-  {
-    points = vtkSmartPointer<vtkPoints>::New();
-    points->SetDataTypeToDouble();
-    points->SetNumberOfPoints(imageData->GetNumberOfPoints());
-    vtkSMPTools::For(0, imageData->GetNumberOfPoints(), [&](vtkIdType begin, vtkIdType end) {
-      double point[3];
-      for (vtkIdType i = begin; i < end; ++i)
-      {
-        imageData->GetPoint(i, point);
-        points->SetPoint(i, point);
-      }
-    });
-  }
-  else if (auto recGrid = vtkRectilinearGrid::SafeDownCast(ds))
-  {
-    points = vtkSmartPointer<vtkPoints>::New();
-    recGrid->GetPoints(points);
-  }
-  else
-  {
-    vtkGenericWarningMacro(<< "Unsupported dataset type: " << ds->GetClassName());
-  }
-  return points;
-}
-
 //------------------------------------------------------------------------------
 struct ComputeTransformationWorker
 {
@@ -264,8 +219,8 @@ bool vtkLinearTransformCellLocator::ComputeTransformation()
     vtkErrorMacro("Number of points in the dataset is less than 2.");
     return false;
   }
-  auto initialPoints = GetPoints(this->CellLocator->GetDataSet());
-  auto newPoints = GetPoints(this->DataSet);
+  auto initialPoints = this->CellLocator->GetDataSet()->GetPoints();
+  auto newPoints = this->DataSet->GetPoints();
   vtkSmartPointer<vtkPoints> initialPointsSample, newPointsSample;
   vtkDataArray* initialPointsSampleData;
   vtkDataArray* newPointsSampleData;
@@ -386,12 +341,12 @@ void vtkLinearTransformCellLocator::ShallowCopy(vtkAbstractCellLocator* locator)
     vtkErrorMacro("Cannot cast " << locator->GetClassName() << " to " << this->GetClassName());
   }
   // we only copy what's actually used by vtkLinearTransformCellLocator
-  this->SetDataSet(cellLocator->GetDataSet());
   this->SetCellLocator(cellLocator->GetCellLocator());
   this->Transform = cellLocator->Transform;
   this->InverseTransform = cellLocator->InverseTransform;
   this->IsLinearTransformation = cellLocator->IsLinearTransformation;
   this->UseAllPoints = cellLocator->UseAllPoints;
+  this->BuildTime.Modified();
 }
 
 //------------------------------------------------------------------------------
@@ -425,7 +380,7 @@ int vtkLinearTransformCellLocator::IntersectWithLine(const double p1[3], const d
 
 //------------------------------------------------------------------------------
 int vtkLinearTransformCellLocator::IntersectWithLine(const double p1[3], const double p2[3],
-  const double tol, vtkPoints* points, vtkIdList* cellIds, vtkGenericCell* cell)
+  double tol, vtkPoints* points, vtkIdList* cellIds, vtkGenericCell* cell)
 {
   if (!this->CellLocator)
   {
@@ -536,3 +491,4 @@ bool vtkLinearTransformCellLocator::InsideCellBounds(double x[3], vtkIdType cell
   this->InverseTransform->InternalTransformPoint(x, xTransform);
   return this->CellLocator->InsideCellBounds(xTransform, cellId);
 }
+VTK_ABI_NAMESPACE_END
