@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkVectorFieldTopology.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include <vtkVectorFieldTopology.h>
 
 // VTK includes
@@ -27,8 +15,8 @@
 #include <vtkDataSetTriangleFilter.h>
 #include <vtkDoubleArray.h>
 #include <vtkFeatureEdges.h>
+#include <vtkGenerateIds.h>
 #include <vtkGeometryFilter.h>
-#include <vtkIdFilter.h>
 #include <vtkImageData.h>
 #include <vtkInformation.h>
 #include <vtkInformationVector.h>
@@ -55,7 +43,6 @@
 #include <vtkTriangle.h>
 #include <vtkUnstructuredGrid.h>
 #include <vtkVector.h>
-#include <vtkVectorFieldTopology.h>
 #include <vtkVertex.h>
 
 // Eigen3
@@ -68,6 +55,7 @@
 #define epsilon (1e-10)
 
 //----------------------------------------------------------------------------
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkVectorFieldTopology);
 
 //----------------------------------------------------------------------------
@@ -79,6 +67,8 @@ vtkVectorFieldTopology::vtkVectorFieldTopology()
   // by default process active point vectors
   this->SetInputArrayToProcess(
     0, 0, 0, vtkDataObject::FIELD_ASSOCIATION_POINTS, vtkDataSetAttributes::VECTORS);
+
+  this->StreamSurface->SetContainerAlgorithm(this);
 }
 
 //----------------------------------------------------------------------------
@@ -334,6 +324,10 @@ int vtkVectorFieldTopology::ComputeCriticalPoints2D(
 {
   for (int cellId = 0; cellId < tridataset->GetNumberOfCells(); cellId++)
   {
+    if (this->CheckAbort())
+    {
+      break;
+    }
     auto cell = tridataset->GetCell(cellId);
     if (cell->GetCellType() != VTK_TRIANGLE)
     {
@@ -372,7 +366,7 @@ int vtkVectorFieldTopology::ComputeCriticalPoints2D(
     {
       valueMatrix->Invert();
 
-      // barycentric corrdinates of the zero: lambda = f(T)^-1 (-values[0])
+      // barycentric coordinates of the zero: lambda = f(T)^-1 (-values[0])
       double lambda[3] = { -values[0][0], -values[0][1], -values[0][2] };
       valueMatrix->MultiplyPoint(lambda, lambda);
 
@@ -427,6 +421,10 @@ int vtkVectorFieldTopology::ComputeCriticalPoints3D(
 {
   for (int cellId = 0; cellId < tridataset->GetNumberOfCells(); cellId++)
   {
+    if (this->CheckAbort())
+    {
+      break;
+    }
     auto cell = tridataset->GetCell(cellId);
     if (cell->GetCellType() != VTK_TETRA)
     {
@@ -550,10 +548,12 @@ int vtkVectorFieldTopology::ComputeBoundarySwitchPoints(
 
   vtkNew<vtkGeometryFilter> geometry;
   geometry->SetInputData(tridataset);
+  geometry->SetContainerAlgorithm(this);
   geometry->Update();
 
   vtkNew<vtkFeatureEdges> surface;
   surface->SetInputData(geometry->GetOutput());
+  surface->SetContainerAlgorithm(this);
   surface->Update();
 
   vtkNew<vtkCellLocator> cellLocator;
@@ -566,7 +566,10 @@ int vtkVectorFieldTopology::ComputeBoundarySwitchPoints(
   // main loop
   for (int i = 0; i < surface->GetOutput()->GetNumberOfCells(); i++)
   {
-
+    if (this->CheckAbort())
+    {
+      break;
+    }
     // compute tangent and line normal of the line in the ith cell
     vtkCell* cell = surface->GetOutput()->GetCell(i);
 
@@ -635,14 +638,14 @@ int vtkVectorFieldTopology::ComputeBoundarySwitchPoints(
 
     x = vn0 / y;
 
-    // if the location is inbwteen the two end points of the line
+    // if the location is in between the two end points of the line
     if (x > 0 && x < 1)
     {
       InterpolateVector(0, 1, x, vector0, vector1, vector);
 
       double vectorNorm = vtkMath::Norm(vector);
 
-      // if the verctor at the boundary switch point is not a zero vector
+      // if the vector at the boundary switch point is not a zero vector
       if (vectorNorm > 1e-16)
       {
         vtkMath::MultiplyScalar(vector, 1 / vectorNorm);
@@ -705,6 +708,7 @@ int vtkVectorFieldTopology::ComputeSurface(int numberOfSeparatingSurfaces, bool
   circle->SetRadius(dist);
   circle->SetCenter(zeroPos);
   circle->SetNormal(normal);
+  circle->SetContainerAlgorithm(this);
   circle->Update();
 
   // close circle exactly with a point instead of an edge to correctly treat points exiting the
@@ -715,11 +719,8 @@ int vtkVectorFieldTopology::ComputeSurface(int numberOfSeparatingSurfaces, bool
   vtkNew<vtkDoubleArray> integrationTimeArray;
   integrationTimeArray->SetName("IntegrationTime");
   currentCircle->GetPointData()->AddArray(integrationTimeArray);
-  integrationTimeArray->Resize(currentCircle->GetNumberOfPoints());
-  for (int i = 0; i < currentCircle->GetNumberOfPoints(); ++i)
-  {
-    integrationTimeArray->SetTuple1(i, (double)0);
-  }
+  integrationTimeArray->SetNumberOfTuples(currentCircle->GetNumberOfPoints());
+  integrationTimeArray->Fill(0.0);
 
   this->StreamSurface->SetInputData(0, dataset);
   this->StreamSurface->SetInputData(1, currentCircle);
@@ -746,6 +747,7 @@ int vtkVectorFieldTopology::ComputeSurface(int numberOfSeparatingSurfaces, bool
   vtkNew<vtkAppendPolyData> appendSurfaces;
   appendSurfaces->AddInputData(this->StreamSurface->GetOutput());
   appendSurfaces->AddInputData(streamSurfaces);
+  appendSurfaces->SetContainerAlgorithm(this);
   appendSurfaces->Update();
   streamSurfaces->DeepCopy(appendSurfaces->GetOutput());
   this->StreamSurface->SetInputData(0, nullptr);
@@ -778,11 +780,16 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchPoints(
   streamTracer->SetInputArrayToProcess(
     0, 0, 0, vtkDataObject::FIELD_ASSOCIATION_POINTS, this->NameOfVectorArray);
   streamTracer->SetInterpolatorType(this->InterpolatorType);
+  streamTracer->SetContainerAlgorithm(this);
 
   int numberOfSeparatingLines = 0;
 
   for (int i = 0; i < boundarySwitchPoints->GetNumberOfPoints(); i++)
   {
+    if (this->CheckAbort())
+    {
+      break;
+    }
     if (boundarySwitchPoints->GetPointData()->GetArray("BoundarySwitchType")->GetTuple1(i) == 1)
       continue;
 
@@ -933,6 +940,7 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchPoints(
 
         // combine lines of this separatrix with existing ones
         vtkNew<vtkAppendPolyData> appendFilter;
+        appendFilter->SetContainerAlgorithm(this);
         appendFilter->AddInputData(separatrices);
         appendFilter->AddInputData(streamTracer->GetOutput());
         appendFilter->Update();
@@ -960,10 +968,12 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchLines(vtkPolyData*
 
   vtkNew<vtkGeometryFilter> geometry;
   geometry->SetInputData(dataset);
+  geometry->SetContainerAlgorithm(this);
   geometry->Update();
 
   vtkNew<vtkDataSetSurfaceFilter> surface;
   surface->SetInputData(geometry->GetOutput());
+  surface->SetContainerAlgorithm(this);
   surface->Update();
 
   vtkNew<vtkDoubleArray> normals;
@@ -1008,6 +1018,7 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchLines(vtkPolyData*
   // and compute the dot product between vector and surface normal
   vtkNew<vtkCellDataToPointData> cell2point;
   cell2point->SetInputData(surface->GetOutput());
+  cell2point->SetContainerAlgorithm(this);
   cell2point->Update();
 
   for (int i = 0; i < surface->GetOutput()->GetNumberOfPoints(); i++)
@@ -1029,6 +1040,7 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchLines(vtkPolyData*
   contourFilter->SetValue(0, 0);
   // (id=0 for first array, port=0, connection=0, pointData=0 and cellData=1, name)
   contourFilter->SetInputArrayToProcess(0, 0, 0, 0, "ScalarProduct");
+  contourFilter->SetContainerAlgorithm(this);
   contourFilter->Update();
 
   if (contourFilter->GetOutput()->GetNumberOfCells() == 0)
@@ -1119,7 +1131,7 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchLines(vtkPolyData*
     normalArray->SetTuple3(i, x[0], x[1], x[2]);
   }
 
-  // the outputs of the countour filter are potential boundary switch lines.
+  // the outputs of the contour filter are potential boundary switch lines.
   // computed shifted boundary switch lines as seeds for computing separating surfaces
   for (int i = 0; i < contourFilter->GetOutput()->GetNumberOfCells(); i++)
   {
@@ -1169,10 +1181,11 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchLines(vtkPolyData*
     }
   }
 
-  // use probe filter to interpolate the verctors at seeds points and center points of  lines
+  // use probe filter to interpolate the vectors at seeds points and center points of  lines
   vtkNew<vtkProbeFilter> probe;
   probe->SetInputData(offsetPoints);
   probe->SetSourceData(dataset);
+  probe->SetContainerAlgorithm(this);
   probe->Update();
 
   vtkNew<vtkPointLocator> pointLocator;
@@ -1192,6 +1205,7 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchLines(vtkPolyData*
   streamSurface->SetTerminalSpeed(epsilon);
   streamSurface->SetUseIterativeSeeding(useIterativeSeeding);
   streamSurface->SetInterpolatorTypeToCellLocator();
+  streamSurface->SetContainerAlgorithm(this);
 
   vtkNew<vtkPolyData> seeds;
 
@@ -1199,6 +1213,10 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchLines(vtkPolyData*
   // lines.
   for (int i = 0; i < contourFilter->GetOutput()->GetNumberOfCells(); i++)
   {
+    if (this->CheckAbort())
+    {
+      break;
+    }
     vtkCell* cell = contourFilter->GetOutput()->GetCell(i);
     double p0[3], p1[3], tangent[3], center[3];
 
@@ -1337,6 +1355,7 @@ int vtkVectorFieldTopology::ComputeSeparatricesBoundarySwitchLines(vtkPolyData*
               vtkNew<vtkAppendPolyData> appendFilter;
               appendFilter->AddInputData(separatrices);
               appendFilter->AddInputData(streamSurface->GetOutput());
+              appendFilter->SetContainerAlgorithm(this);
               appendFilter->Update();
 
               separatrices->DeepCopy(appendFilter->GetOutput());
@@ -1395,12 +1414,17 @@ int vtkVectorFieldTopology::ComputeSeparatrices(vtkPolyData* criticalPoints,
   streamTracer->SetInputArrayToProcess(
     0, 0, 0, vtkDataObject::FIELD_ASSOCIATION_POINTS, this->NameOfVectorArray);
   streamTracer->SetInterpolatorType(this->InterpolatorType);
+  streamTracer->SetContainerAlgorithm(this);
 
   int numberOfSeparatingLines = 0;
   int numberOfSeparatingSurfaces = 0;
 
   for (int pointId = 0; pointId < criticalPoints->GetNumberOfPoints(); pointId++)
   {
+    if (this->CheckAbort())
+    {
+      break;
+    }
     // classification
     Eigen::Matrix<double, 3, 3> eigenMatrix;
     int flatIdx = 0;
@@ -1416,17 +1440,12 @@ int vtkVectorFieldTopology::ComputeSeparatrices(vtkPolyData* criticalPoints,
 
     Eigen::EigenSolver<Eigen::Matrix<double, 3, 3>> eigenS(eigenMatrix);
 
-    int countReal = 0;
     int countComplex = 0;
     int countPos = 0;
     int countNeg = 0;
     for (int i = 0; i < this->Dimension; i++)
     {
-      if (imag(eigenS.eigenvalues()[i]) == 0.0)
-      {
-        countReal++;
-      }
-      else
+      if (imag(eigenS.eigenvalues()[i]) != 0.0)
       {
         countComplex++;
       }
@@ -1609,6 +1628,7 @@ int vtkVectorFieldTopology::ComputeSeparatrices(vtkPolyData* criticalPoints,
               vtkNew<vtkAppendPolyData> appendFilter;
               appendFilter->AddInputData(separatrices);
               appendFilter->AddInputData(streamTracer->GetOutput());
+              appendFilter->SetContainerAlgorithm(this);
               appendFilter->Update();
               separatrices->DeepCopy(appendFilter->GetOutput());
               numberOfSeparatingLines++;
@@ -1632,7 +1652,7 @@ int vtkVectorFieldTopology::ComputeSeparatrices(vtkPolyData* criticalPoints,
 int vtkVectorFieldTopology::RemoveBoundary(vtkSmartPointer<vtkUnstructuredGrid> tridataset)
 {
   // assign id to each point
-  vtkNew<vtkIdFilter> idFilter;
+  vtkNew<vtkGenerateIds> idFilter;
   idFilter->SetInputData(tridataset);
   idFilter->SetPointIdsArrayName("ids");
   idFilter->Update();
@@ -1643,10 +1663,12 @@ int vtkVectorFieldTopology::RemoveBoundary(vtkSmartPointer<vtkUnstructuredGrid>
   {
     vtkNew<vtkGeometryFilter> geometryFilter;
     geometryFilter->SetInputData(idFilter->GetOutput());
+    geometryFilter->SetContainerAlgorithm(this);
     geometryFilter->Update();
 
     vtkNew<vtkFeatureEdges> surfaceFilter;
     surfaceFilter->SetInputData(geometryFilter->GetOutput());
+    surfaceFilter->SetContainerAlgorithm(this);
     surfaceFilter->Update();
     boundary = surfaceFilter->GetOutput();
   }
@@ -1654,6 +1676,7 @@ int vtkVectorFieldTopology::RemoveBoundary(vtkSmartPointer<vtkUnstructuredGrid>
   {
     vtkNew<vtkDataSetSurfaceFilter> surfaceFilter;
     surfaceFilter->SetInputData(idFilter->GetOutput());
+    surfaceFilter->SetContainerAlgorithm(this);
     surfaceFilter->Update();
     boundary = surfaceFilter->GetOutput();
   }
@@ -1669,6 +1692,10 @@ int vtkVectorFieldTopology::RemoveBoundary(vtkSmartPointer<vtkUnstructuredGrid>
   }
   for (int ptId = 0; ptId < boundary->GetNumberOfPoints(); ptId++)
   {
+    // The point data is `double`, but it stores ids and the value of
+    // `isBoundary` is treated as a boolean, so while the types may appear as
+    // if these are swapped, they are not.
+    // NOLINTNEXTLINE(bugprone-swapped-arguments)
     isBoundary->SetTuple1(boundary->GetPointData()->GetArray("ids")->GetTuple1(ptId), 1);
   }
 
@@ -1736,6 +1763,7 @@ int vtkVectorFieldTopology::ImageDataPrepare(
   // Triangulate the input data
   vtkNew<vtkDataSetTriangleFilter> triangulateFilter;
   triangulateFilter->SetInputData(dataset);
+  triangulateFilter->SetContainerAlgorithm(this);
   triangulateFilter->Update();
   tridataset->DeepCopy(triangulateFilter->GetOutput());
 
@@ -1781,6 +1809,7 @@ int vtkVectorFieldTopology::UnstructuredGridPrepare(
     // Triangulate the input data
     vtkNew<vtkDataSetTriangleFilter> triangulateFilter;
     triangulateFilter->SetInputData(dataset);
+    triangulateFilter->SetContainerAlgorithm(this);
     triangulateFilter->Update();
     tridataset->DeepCopy(triangulateFilter->GetOutput());
   }
@@ -1959,3 +1988,4 @@ int vtkVectorFieldTopology::RequestData(vtkInformation* vtkNotUsed(request),
 
   return success;
 }
+VTK_ABI_NAMESPACE_END
