@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkContourTriangulator.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkContourTriangulator.h"
 
 #include "vtkCellArray.h"
@@ -33,6 +21,7 @@
 #include <utility>
 #include <vector>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkContourTriangulator);
 
 //------------------------------------------------------------------------------
@@ -84,13 +73,15 @@ int vtkContourTriangulator::RequestData(vtkInformation* vtkNotUsed(request),
   output->GetPointData()->PassData(input->GetPointData());
 
   this->TriangulationError = !vtkContourTriangulator::TriangulateContours(
-    input, input->GetNumberOfVerts(), lines->GetNumberOfCells(), polys, nullptr);
+    input, input->GetNumberOfVerts(), lines->GetNumberOfCells(), polys, nullptr, this);
 
   if (this->TriangulationError && this->TriangulationErrorDisplay)
   {
     vtkErrorMacro("Triangulation failed, output might have holes.");
   }
 
+  this->CheckAbort();
+
   return 1;
 }
 
@@ -554,6 +545,7 @@ void vtkCCSMakePolysFromLines(vtkPolyData* data, vtkIdType firstLine, vtkIdType
   vtkCCSBitArray usedLines;
 
   // Require cell links to get lines from pointIds
+  data->EditableOn();
   data->BuildLinks(data->GetPoints()->GetNumberOfPoints());
 
   size_t numNewPolys = 0;
@@ -1602,6 +1594,14 @@ void vtkCCSPrepareForPolyInPoly(
   // Find the bounding box and tolerance for the polygon
   tol2 = (vtkCCSPolygonBounds(outerPoly, points, bounds) *
     (VTK_CCS_POLYGON_TOLERANCE * VTK_CCS_POLYGON_TOLERANCE));
+
+  // Expand the bounds
+  double tol = sqrt(tol2);
+  for (int i = 0; i < 3; i++)
+  {
+    bounds[2 * i] -= tol;
+    bounds[2 * i + 1] += tol;
+  }
 }
 
 // ---------------------------------------------------
@@ -2469,7 +2469,7 @@ int vtkCCSCutHoleyPolys(std::vector<vtkCCSPoly>& polys, vtkPoints* points,
 //#define VTK_CCS_SHOW_FAILED_POLYS
 
 int vtkContourTriangulator::TriangulateContours(vtkPolyData* data, vtkIdType firstLine,
-  vtkIdType numLines, vtkCellArray* polys, const double normal[3])
+  vtkIdType numLines, vtkCellArray* polys, const double normal[3], vtkPolyDataAlgorithm* self)
 {
   int triangulationFailure = 0;
 
@@ -2573,6 +2573,10 @@ int vtkContourTriangulator::TriangulateContours(vtkPolyData* data, vtkIdType fir
   // Go through all polys and triangulate them
   for (size_t polyId = 0; polyId < polyGroups.size(); polyId++)
   {
+    if (self && self->CheckAbort())
+    {
+      break;
+    }
     // If group is empty, then poly was a hole without a containing poly
     if (polyGroups[polyId].empty())
     {
@@ -2627,3 +2631,4 @@ int vtkContourTriangulator::TriangulatePolygon(
   }
   return success;
 }
+VTK_ABI_NAMESPACE_END
