@@ -1,17 +1,6 @@
-/*=========================================================================
-
-  Program:   ParaView
-  Module:    vtkTemporalFractal.cxx
-
-  Copyright (c) Kitware, Inc.
-  All rights reserved.
-  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-FileCopyrightText: Copyright (c) Kitware, Inc.
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkTemporalFractal.h"
 
 #include "vtkCellData.h"
@@ -36,6 +25,7 @@
 #include <cassert>
 
 //------------------------------------------------------------------------------
+VTK_ABI_NAMESPACE_BEGIN
 class TemporalFractalOutputUtil : public vtkObject
 {
 public:
@@ -85,7 +75,7 @@ public:
       unsigned int level = this->Levels[i];
       vtkUniformGrid* grid = vtkUniformGrid::SafeDownCast(this->DataSets[i]);
       assert(grid);
-      gridDescription = grid->GetGridDescription();
+      gridDescription = grid->GetDataDescription();
       double* gridOrigin = grid->GetOrigin();
       for (int d = 0; d < 3; d++)
       {
@@ -992,7 +982,7 @@ void vtkTemporalFractal::AddTestArray(vtkHierarchicalBoxDataSet* output)
       int x, y, z;
       int ext[6];
       grid->GetExtent(ext);
-      // we need cell extents bu we just get point extents
+      // we need cell extents but we just get point extents
       if (ext[5] > 0)
       {
         --ext[5];
@@ -1019,6 +1009,7 @@ void vtkTemporalFractal::AddTestArray(vtkHierarchicalBoxDataSet* output)
         }
       }
       assert("check: valid_debugcounter" && debugcounter == numCells);
+      (void)debugcounter;
       array->SetName("TestX");
       grid->GetCellData()->AddArray(array);
       array->Delete();
@@ -1056,7 +1047,7 @@ void vtkTemporalFractal::AddVectorArray(vtkHierarchicalBoxDataSet* output)
       int x, y, z;
       int ext[6];
       grid->GetExtent(ext);
-      // we need cell extents bu we just get point extents
+      // we need cell extents but we just get point extents
       if (ext[5] > 0)
       {
         --ext[5];
@@ -1101,6 +1092,10 @@ void vtkTemporalFractal::AddFractalArray(vtkCompositeDataSet* output)
 
   for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
   {
+    if (this->CheckAbort())
+    {
+      break;
+    }
     if (!this->GenerateRectilinearGrids)
     {
       vtkUniformGrid* grid = vtkUniformGrid::SafeDownCast(iter->GetCurrentDataObject());
@@ -1213,7 +1208,6 @@ void vtkTemporalFractal::AddDepthArray(vtkHierarchicalBoxDataSet* output)
 {
   int levels = output->GetNumberOfLevels();
   int level = 0;
-  int blockId = 0;
   while (level < levels)
   {
     int blocks = output->GetNumberOfDataSets(level);
@@ -1237,7 +1231,6 @@ void vtkTemporalFractal::AddDepthArray(vtkHierarchicalBoxDataSet* output)
       grid->GetCellData()->AddArray(array);
       array->Delete();
       ++block;
-      ++blockId;
     }
     ++level;
   }
@@ -1566,3 +1559,4 @@ void vtkTemporalFractal::PrintSelf(ostream& os, vtkIndent indent)
   os << indent
      << "GenerateRectilinearGrids: " << (this->GenerateRectilinearGrids ? "True" : "False") << endl;
 }
+VTK_ABI_NAMESPACE_END
