@@ -1,17 +1,5 @@
-/*=========================================================================
-
-Program:   Visualization Toolkit
-Module:    vtkHyperTreeGridToDualGrid.cxx
-
-Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-All rights reserved.
-See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-This software is distributed WITHOUT ANY WARRANTY; without even
-the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkHyperTreeGridToDualGrid.h"
 
 #include "vtkBitArray.h"
@@ -29,6 +17,7 @@ PURPOSE.  See the above copyright notice for more information.
 #include "vtkPoints.h"
 #include "vtkUnstructuredGrid.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkHyperTreeGridToDualGrid);
 
 static const unsigned int CornerNeighborCursorsTable3D0[8] = { 0, 1, 3, 4, 9, 10, 12, 13 };
@@ -113,12 +102,10 @@ int vtkHyperTreeGridToDualGrid::ProcessTrees(vtkHyperTreeGrid* input, vtkDataObj
   this->Connectivity = vtkIdTypeArray::New();
 
   // Primal cell centers are dual points
-  // JB On ne peut pas se reduire a dimensionner le tableau de points au nombre
-  // de Vertices, en effet, surtout si l'on veut conserver le mapping 1:1
-  // entre les noeuds de l'HTG et ces points du maillage dual.
-  // En effet, si l'on definit un GlobalIndex ou un IndexStart specifique
-  // cette ecriture simpliste ne fonctionnait plus... tableau trop petit
-  // car GetGlobalIndex retourne une valeur > this->GetNumberOfCells().
+  // TODO: We cannot only dimension the point array as the number of vertices,
+  // especially if we want to keep a 1:1 mapping between HTG nodes and the dual mesh.
+  // If we define a specific GlobalIndex or IndexStart, it would be too small,
+  // because GetGlobalIndex returns a value greater than the number of cells.
   this->Points->SetNumberOfPoints(input->GetGlobalNodeIndexMax() + 1);
 
   // TODO: find out why we get some uninitialized point coords instead
@@ -165,6 +152,10 @@ int vtkHyperTreeGridToDualGrid::ProcessTrees(vtkHyperTreeGrid* input, vtkDataObj
   vtkNew<vtkHyperTreeGridNonOrientedMooreSuperCursor> cursor;
   while (it.GetNextTree(index))
   {
+    if (this->CheckAbort())
+    {
+      break;
+    }
     // Initialize new Moore cursor at root of current tree
     input->InitializeNonOrientedMooreSuperCursor(cursor, index);
     // Convert hyper tree into unstructured mesh recursively
@@ -252,6 +243,10 @@ void vtkHyperTreeGridToDualGrid::TraverseDualRecursively(
     int numChildren = input->GetNumberOfChildren();
     for (int child = 0; child < numChildren; ++child)
     {
+      if (this->CheckAbort())
+      {
+        break;
+      }
       cursor->ToChild(child);
       // Recurse
       this->TraverseDualRecursively(cursor, input);
@@ -587,6 +582,10 @@ void vtkHyperTreeGridToDualGrid::TraverseDualRecursively(
     int numChildren = input->GetNumberOfChildren();
     for (int child = 0; child < numChildren; ++child)
     {
+      if (this->CheckAbort())
+      {
+        break;
+      }
       cursor->ToChild(child);
       // Recurse
       this->TraverseDualRecursively(cursor, mask, input);
@@ -1308,3 +1307,4 @@ void vtkHyperTreeGridToDualGrid::GenerateDualCornerFromLeaf3D(
     } // if ( owner )
   }   // c
 }
+VTK_ABI_NAMESPACE_END
