@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkDIYKdTreeUtilities.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkDIYKdTreeUtilities.h"
 
@@ -24,6 +12,7 @@
 #include "vtkIdTypeArray.h"
 #include "vtkLogger.h"
 #include "vtkMath.h"
+#include "vtkMathUtilities.h"
 #include "vtkNew.h"
 #include "vtkObjectFactory.h"
 #include "vtkPartitionedDataSet.h"
@@ -33,6 +22,7 @@
 #include "vtkUnsignedCharArray.h"
 #include "vtkUnstructuredGrid.h"
 
+#include <array>
 #include <iterator>
 #include <map>
 #include <memory>
@@ -50,6 +40,7 @@
 #include VTK_DIY2(diy/algorithms.hpp)
 // clang-format on
 
+VTK_ABI_NAMESPACE_BEGIN
 namespace
 {
 struct PointTT
@@ -189,14 +180,30 @@ std::vector<vtkBoundingBox> vtkDIYKdTreeUtilities::GenerateCuts(
   // determine global domain bounds.
   vtkDIYUtilities::AllReduce(comm, bbox);
 
-  if (!bbox.IsValid())
+  if (!bbox.IsValid() || bbox.GetMaxLength() == 0.)
   {
     // nothing to split since global bounds are empty.
     return std::vector<vtkBoundingBox>();
   }
 
-  // I am removing this. it doesn't not make sense to inflate here.
-  // bbox.Inflate(0.1 * bbox.GetDiagonalLength());
+  // Need to inflate the bounding box to ensure each dimension is not zero,
+  // (or too much close to zero) since we build a 3D kd-tree in any case.
+  // Building a kd-tree with same dimension as the bounding box seems to
+  // cause issues in some cases, for example in 1D, depending of the number
+  // of ranks used.
+  const double* minPoint = bbox.GetMinPoint();
+  const double* maxPoint = bbox.GetMaxPoint();
+
+  std::array<double, 3> delta = { 0., 0., 0. };
+  for (unsigned int dim = 0; dim < 3; dim++)
+  {
+    if (vtkMathUtilities::FuzzyCompare(minPoint[dim] - maxPoint[dim], 0.))
+    {
+      delta[dim] = std::numeric_limits<double>::epsilon();
+    }
+  }
+
+  bbox.Inflate(delta[0], delta[1], delta[2]);
 
   if (number_of_partitions == 1)
   {
@@ -335,7 +342,7 @@ vtkSmartPointer<vtkPartitionedDataSet> vtkDIYKdTreeUtilities::Exchange(
             if (target_rank == myrank)
             {
               // short-circuit messages to self.
-              (*block)[partId].push_back(part);
+              (*block)[partId].emplace_back(part);
             }
             else
             {
@@ -590,3 +597,4 @@ void vtkDIYKdTreeUtilities::ResizeCuts(std::vector<vtkBoundingBox>& cuts, int si
   cuts.swap(new_cuts);
   assert(static_cast<int>(cuts.size()) == size);
 }
+VTK_ABI_NAMESPACE_END
