@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkOverlappingCellsDetector.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkOverlappingCellsDetector.h"
 
@@ -64,6 +52,7 @@
 #include VTK_DIY2(diy/assigner.hpp)
 // clang-format on
 
+VTK_ABI_NAMESPACE_BEGIN
 namespace
 {
 constexpr char SPHERE_RADIUS_ARRAY_NAME[21] = "SphereRadius";
@@ -835,3 +824,4 @@ void vtkOverlappingCellsDetector::PrintSelf(ostream& os, vtkIndent indent)
      << std::endl;
   os << indent << "Tolerance: " << this->Tolerance << std::endl;
 }
+VTK_ABI_NAMESPACE_END
