@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkPLagrangianParticleTracker.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-    This software is distributed WITHOUT ANY WARRANTY; without even
-    the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-    PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkPLagrangianParticleTracker.h"
 
 #include "vtkAppendFilter.h"
@@ -42,6 +30,7 @@
 #define LAGRANGIAN_PARTICLE_CONTROL_TAG 625
 
 // Class used to serialize and stream a particle
+VTK_ABI_NAMESPACE_BEGIN
 class MessageStream
 {
 public:
@@ -1110,7 +1099,7 @@ bool vtkPLagrangianParticleTracker::UpdateSurfaceCacheIfNeeded(vtkDataObject*& s
       {
         // Rank 0 reconstruct Composite tree
         vtkCompositeDataSet* mb = vtkCompositeDataSet::SafeDownCast(surfaces);
-        this->TmpSurfaceInputMB->ShallowCopy(mb);
+        this->TmpSurfaceInputMB->CompositeShallowCopy(mb);
         vtkCompositeDataIterator* iter = mb->NewIterator();
         iter->SkipEmptyNodesOff();
         for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
@@ -1179,3 +1168,4 @@ void vtkPLagrangianParticleTracker::DeleteParticle(vtkLagrangianParticle* partic
     this->OutOfDomainParticleMap[particle->GetId()] = particle;
   }
 }
+VTK_ABI_NAMESPACE_END
