@@ -1,31 +1,21 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkCGNSReaderInternal.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
-// Copyright 2013-2014 Mickael Philit.
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-FileCopyrightText: Copyright 2013-2014 Mickael Philit
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkCGNSReaderInternal.h"
 
 #include "cgio_helpers.h"
 #include "vtkCellType.h"
+#include "vtkIdTypeArray.h"
 #include "vtkMultiProcessStream.h"
 
 #include <algorithm>
 
 namespace CGNSRead
 {
+VTK_ABI_NAMESPACE_BEGIN
 //------------------------------------------------------------------------------
-int setUpRind(const int cgioNum, const double rindId, int* rind)
+int setUpRind(int cgioNum, double rindId, int* rind)
 {
   CGNSRead::char_33 dataType;
   if (cgio_get_data_type(cgioNum, rindId, dataType) != CG_OK)
@@ -56,8 +46,7 @@ int setUpRind(const int cgioNum, const double rindId, int* rind)
 }
 
 //------------------------------------------------------------------------------
-int getFirstNodeId(
-  const int cgioNum, const double parentId, const char* label, double* id, const char* name)
+int getFirstNodeId(int cgioNum, double parentId, const char* label, double* id, const char* name)
 {
   int nId, n, nChildren, len;
   int ier = 0;
@@ -128,10 +117,10 @@ int getFirstNodeId(
 }
 
 //------------------------------------------------------------------------------
-int get_section_connectivity(const int cgioNum, const double cgioSectionId, const int dim,
-  const cgsize_t* srcStart, const cgsize_t* srcEnd, const cgsize_t* srcStride,
-  const cgsize_t* memStart, const cgsize_t* memEnd, const cgsize_t* memStride,
-  const cgsize_t* memDim, vtkIdType* localElements)
+int get_section_connectivity(int cgioNum, double cgioSectionId, int dim, const cgsize_t* srcStart,
+  const cgsize_t* srcEnd, const cgsize_t* srcStride, const cgsize_t* memStart,
+  const cgsize_t* memEnd, const cgsize_t* memStride, const cgsize_t* memDim,
+  vtkIdType* localElements)
 {
   const char* connectivityPath = "ElementConnectivity";
   double cgioElemConnectId;
@@ -179,6 +168,7 @@ int get_section_connectivity(const int cgioNum, const double cgioSectionId, cons
       if (data == nullptr)
       {
         std::cerr << "Allocation failed for temporary connectivity array\n";
+        return 1;
       }
 
       if (cgio_read_data_type(cgioNum, cgioElemConnectId, srcStart, srcEnd, srcStride, "I4", dim,
@@ -225,10 +215,10 @@ int get_section_connectivity(const int cgioNum, const double cgioSectionId, cons
 }
 
 //------------------------------------------------------------------------------
-int get_section_start_offset(const int cgioNum, const double cgioSectionId, const int dim,
-  const cgsize_t* srcStart, const cgsize_t* srcEnd, const cgsize_t* srcStride,
-  const cgsize_t* memStart, const cgsize_t* memEnd, const cgsize_t* memStride,
-  const cgsize_t* memDim, vtkIdType* localElementsIdx)
+int get_section_start_offset(int cgioNum, double cgioSectionId, int dim, const cgsize_t* srcStart,
+  const cgsize_t* srcEnd, const cgsize_t* srcStride, const cgsize_t* memStart,
+  const cgsize_t* memEnd, const cgsize_t* memStride, const cgsize_t* memDim,
+  vtkIdType* localElementsIdx)
 {
   const char* offsetPath = "ElementStartOffset";
   double cgioElemOffsetId;
@@ -603,7 +593,7 @@ static const int PYRA_30_ToVTK[30] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 9, 12, 11,
 static const int QUAD_16_ToVTK[16] = { 0, 1, 2, 3, 4, 5, 6, 7, 9, 8, 11, 10, 12, 13, 15, 14 };
 
 //------------------------------------------------------------------------------
-inline const int* getTranslator(const int cellType)
+inline const int* getTranslator(int cellType)
 {
   switch (cellType)
   {
@@ -648,7 +638,7 @@ inline const int* getTranslator(const int cellType)
 }
 
 //------------------------------------------------------------------------------
-void CGNS2VTKorder(const vtkIdType size, const int* cells_types, vtkIdType* elements)
+void CGNS2VTKorder(vtkIdType size, const int* cells_types, vtkIdType* elements)
 {
   const int maxPointsPerCells = 64;
   int tmp[maxPointsPerCells];
@@ -675,11 +665,12 @@ void CGNS2VTKorder(const vtkIdType size, const int* cells_types, vtkIdType* elem
 }
 
 //------------------------------------------------------------------------------
-void CGNS2VTKorderMonoElem(const vtkIdType size, const int cell_type, vtkIdType* elements)
+void ReorderMonoCellPointsCGNS2VTK(
+  vtkIdType size, int cell_type, vtkIdType numPointsPerCell, vtkIdType* elements)
 {
-  const int maxPointsPerCells = 64;
+  vtkNew<vtkIdTypeArray> tempArray;
+  tempArray->SetNumberOfTuples(numPointsPerCell);
 
-  int tmp[maxPointsPerCells];
   const int* translator;
   translator = getTranslator(cell_type);
   if (translator == nullptr)
@@ -690,15 +681,13 @@ void CGNS2VTKorderMonoElem(const vtkIdType size, const int cell_type, vtkIdType*
   vtkIdType pos = 0;
   for (vtkIdType icell = 0; icell < size; ++icell)
   {
-    vtkIdType numPointsPerCell = elements[pos];
-    pos++;
     for (vtkIdType ip = 0; ip < numPointsPerCell; ++ip)
     {
-      tmp[ip] = elements[translator[ip] + pos];
+      tempArray->SetValue(ip, elements[translator[ip] + pos]);
     }
     for (vtkIdType ip = 0; ip < numPointsPerCell; ++ip)
     {
-      elements[pos + ip] = tmp[ip];
+      elements[pos + ip] = tempArray->GetValue(ip);
     }
     pos += numPointsPerCell;
   }
@@ -713,7 +702,7 @@ bool testValidVector(const CGNSVector& item)
 
 //------------------------------------------------------------------------------
 void fillVectorsFromVars(std::vector<CGNSRead::CGNSVariable>& vars,
-  std::vector<CGNSRead::CGNSVector>& vectors, const int physicalDim)
+  std::vector<CGNSRead::CGNSVector>& vectors, int physicalDim)
 {
   // get number of scalars and vectors
   const std::size_t nvar = vars.size();
@@ -730,18 +719,30 @@ void fillVectorsFromVars(std::vector<CGNSRead::CGNSVariable>& vars,
   {
     len = strlen(vars[n].name) - 1;
     // CGNS convention uses CamelCase for vector naming (VectorX)
-    // For added convenience, snake_case is supported (vector_x)
+    // but we can also detect Vector_X or Vector_x
     switch (vars[n].name[len])
     {
       case 'X':
+        if (len > 0 && vars[n].name[len - 1] == '_')
+        {
+          len -= 1;
+        }
         vars[n].xyzIndex = 1;
         vars[n].isComponent = true;
         break;
       case 'Y':
+        if (len > 0 && vars[n].name[len - 1] == '_')
+        {
+          len -= 1;
+        }
         vars[n].xyzIndex = 2;
         vars[n].isComponent = true;
         break;
       case 'Z':
+        if (len > 0 && vars[n].name[len - 1] == '_')
+        {
+          len -= 1;
+        }
         vars[n].xyzIndex = 3;
         vars[n].isComponent = true;
         break;
@@ -877,11 +878,18 @@ bool vtkCGNSMetaData::Parse(const char* cgnsFileName)
   // use cgio routine to open the file
   if (cgio_open_file(cgnsFileName, CGIO_MODE_READ, CG_FILE_NONE, &cgioNum) != CG_OK)
   {
-    cgio_error_exit("cgio_file_open");
+    char message[81];
+    cgio_error_message(message);
+    vtkErrorWithObjectMacro(nullptr, "Error loading CGNS file with cgio_file_open: " << message);
+    return false;
   }
   if (cgio_get_root_id(cgioNum, &rootId) != CG_OK)
   {
-    cgio_error_exit("cgio_get_root_id");
+    char message[81];
+    cgio_error_message(message);
+    vtkErrorWithObjectMacro(
+      nullptr, "Error accessing CGNS root node with cgio_get_root_id: " << message);
+    return false;
   }
 
   // Get base id list :
@@ -1373,4 +1381,5 @@ bool ReadPatch(vtkCGNSReader* reader, const BaseInformation&, const ZoneInformat
   return true;
 }
 
+VTK_ABI_NAMESPACE_END
 } // end of namespace
