@@ -1,29 +1,25 @@
-/*=========================================================================
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
-  Program:   Visualization Toolkit
-  Module:    Test3DTilesWriter.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// VTK_DEPRECATED_IN_9_4_0()
+#define VTK_DEPRECATION_LEVEL 0
 
+#include "vtkAlgorithm.h"
 #include "vtkAppendPolyData.h"
+#include "vtkCamera.h"
 #include "vtkCellData.h"
 #include "vtkCesium3DTilesWriter.h"
 #include "vtkCityGMLReader.h"
 #include "vtkCompositeDataIterator.h"
+#include "vtkCompositePolyDataMapper.h"
 #include "vtkDataObject.h"
 #include "vtkDirectory.h"
 #include "vtkDoubleArray.h"
+#include "vtkGLTFImporter.h"
 #include "vtkGLTFReader.h"
 #include "vtkIncrementalOctreeNode.h"
 #include "vtkIncrementalOctreePointLocator.h"
+#include "vtkJPEGReader.h"
 #include "vtkLogger.h"
 #include "vtkMathUtilities.h"
 #include "vtkMultiBlockDataSet.h"
@@ -31,9 +27,16 @@
 #include "vtkOBJReader.h"
 #include "vtkPointData.h"
 #include "vtkPoints.h"
+#include "vtkRegressionTestImage.h"
+#include "vtkRenderWindow.h"
+#include "vtkRenderWindowInteractor.h"
+#include "vtkRenderer.h"
 #include "vtkSmartPointer.h"
 #include "vtkStringArray.h"
+#include "vtkTestUtilities.h"
 #include "vtkTesting.h"
+#include "vtkTexture.h"
+#include "vtkVersionMacros.h"
 #include "vtksys/FStream.hxx"
 #include "vtksys/SystemTools.hxx"
 
@@ -47,6 +50,7 @@
 #include VTK_NLOHMANN_JSON(json.hpp)
 
 using namespace vtksys;
+
 //------------------------------------------------------------------------------
 void SetField(vtkDataObject* obj, const char* name, const char* value)
 {
@@ -173,7 +177,7 @@ vtkSmartPointer<vtkMultiBlockDataSet> ReadCityGMLBuildings(int numberOfBuildings
 //------------------------------------------------------------------------------
 using ReaderType = vtkSmartPointer<vtkMultiBlockDataSet> (*)(int numberOfBuildings, int lod,
   const std::vector<std::string>& files, std::array<double, 3>& fileOffset);
-std::map<std::string, ReaderType> READER = { { ".obj", ReadOBJBuildings },
+static std::map<std::string, ReaderType> READER = { { ".obj", ReadOBJBuildings },
   { ".gml", ReadCityGMLBuildings } };
 
 //------------------------------------------------------------------------------
@@ -223,10 +227,10 @@ std::vector<std::string> getFiles(const std::vector<std::string>& input)
 
 //------------------------------------------------------------------------------
 
-void tiler(const std::vector<std::string>& input, int inputType, bool addColor,
-  const std::string& output, bool contentGLTF, int numberOfBuildings, int buildingsPerTile, int lod,
-  const std::vector<double>& inputOffset, bool saveTiles, bool saveTextures, std::string crs,
-  const int utmZone, char utmHemisphere)
+vtkSmartPointer<vtkMultiBlockDataSet> tiler(const std::vector<std::string>& input, int inputType,
+  bool addColor, const std::string& output, bool contentGLTF, int numberOfBuildings,
+  int buildingsPerTile, int lod, const std::vector<double>& inputOffset, bool saveTiles,
+  bool saveTextures, std::string crs, const int utmZone, char utmHemisphere)
 {
   vtkSmartPointer<vtkMultiBlockDataSet> mbData;
   vtkSmartPointer<vtkPolyData> polyData;
@@ -276,6 +280,7 @@ void tiler(const std::vector<std::string>& input, int inputType, bool addColor,
     writer->SetInputDataObject(polyData);
   }
   writer->SetContentGLTF(contentGLTF);
+  writer->ContentGLTFSaveGLBOff();
   writer->SetInputType(inputType);
   writer->SetDirectoryName(output.c_str());
   writer->SetTextureBaseDirectory(textureBaseDirectory.c_str());
@@ -291,11 +296,13 @@ void tiler(const std::vector<std::string>& input, int inputType, bool addColor,
   }
   writer->SetCRS(crs.c_str());
   writer->Write();
+  return mbData;
 }
 
 bool TrianglesDiffer(std::array<std::array<double, 3>, 3>& in, std::string gltfFileName)
 {
   vtkNew<vtkGLTFReader> reader;
+  reader->SetOutputPointsPrecision(vtkAlgorithm::DOUBLE_PRECISION);
   reader->SetFileName(gltfFileName.c_str());
   reader->Update();
   vtkMultiBlockDataSet* mbOutput = reader->GetOutput();
@@ -403,7 +410,7 @@ bool JsonEqual(nlohmann::json& l, nlohmann::json& r) noexcept
   return false;
 }
 
-std::array<std::array<double, 3>, 3> triangleJacksonville = {
+static std::array<std::array<double, 3>, 3> triangleJacksonville = {
   { { { 799099.7216079829959199, -5452032.6613515587523580, 3201501.3033391013741493 } },
     { { 797899.9930383440805599, -5452124.7368548354133964, 3201444.7161126118153334 } },
     { { 797971.0970941731939092, -5452573.6701772613450885, 3200667.5626786206848919 } } }
@@ -523,6 +530,21 @@ void TestBerlinBuildings(const std::string& dataRoot, const std::string& tempDir
   }
 }
 
+void TestChurchBuildings(
+  const std::string& dataRoot, const std::string& tempDirectory, vtkRenderWindow* renderWindow)
+{
+  std::cout << "Test merge textures church (citygml)" << std::endl;
+  vtkSmartPointer<vtkMultiBlockDataSet> input =
+    tiler(std::vector<std::string>{ { dataRoot + "/Data/CityGML/Part-4-Buildings-V4-one.gml" } },
+      vtkCesium3DTilesWriter::Buildings, false /*addColor*/, tempDirectory + "/church-3dtiles",
+      true /*contentGLTF*/, 1, 1, 3 /*lod*/, std::vector<double>{ { 435200, 3354000, 0 } },
+      true /*saveTiles*/, true /*saveTextures*/, "", 17, 'N');
+  vtkNew<vtkGLTFImporter> importer;
+  importer->SetFileName((tempDirectory + "/church-3dtiles/0/0.gltf").c_str());
+  importer->SetRenderWindow(renderWindow);
+  importer->Update();
+}
+
 int TestCesium3DTilesWriter(int argc, char* argv[])
 {
   vtkNew<vtkTesting> testHelper;
@@ -540,6 +562,7 @@ int TestCesium3DTilesWriter(int argc, char* argv[])
 
   std::string dataRoot = testHelper->GetDataRoot();
   std::string tempDirectory = testHelper->GetTempDirectory();
+  int retVal = 0;
   try
   {
     TestJacksonvilleBuildings(dataRoot, tempDirectory);
@@ -549,14 +572,38 @@ int TestCesium3DTilesWriter(int argc, char* argv[])
     TestJacksonvillePoints(dataRoot, tempDirectory, true /*contentGLTF*/);
     TestJacksonvilleColorPoints(dataRoot, tempDirectory, false /*contentGLTF*/);
     TestJacksonvilleColorPoints(dataRoot, tempDirectory, true /*contentGLTF*/);
-
     TestJacksonvilleMesh(dataRoot, tempDirectory);
+
+    vtkNew<vtkRenderer> renderer;
+    renderer->SetBackground(0.5, 0.7, 0.7);
+    vtkNew<vtkRenderWindow> renWin;
+    renWin->AddRenderer(renderer);
+
+    TestChurchBuildings(dataRoot, tempDirectory, renWin);
+
+    vtkNew<vtkRenderWindowInteractor> interactor;
+    interactor->SetRenderWindow(renWin);
+
+    renderer->ResetCamera();
+    renderer->GetActiveCamera()->Azimuth(90);
+    renderer->GetActiveCamera()->Roll(-90);
+    renderer->GetActiveCamera()->Zoom(1.5);
+
+    renWin->SetSize(400, 400);
+    renWin->Render();
+    interactor->Initialize();
+    renWin->Render();
+
+    retVal = vtkRegressionTestImage(renWin);
+    if (retVal == vtkRegressionTester::DO_INTERACTOR)
+    {
+      interactor->Start();
+    }
   }
-  catch (std::runtime_error& e)
+  catch (std::exception& e)
   {
     vtkLog(ERROR, << e.what());
     return EXIT_FAILURE;
   }
-
-  return EXIT_SUCCESS;
+  return !retVal;
 }
