@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtk3DSImporter.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtk3DSImporter.h"
 
 #include "vtkActor.h"
@@ -30,7 +18,9 @@
 
 #include <sstream>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtk3DSImporter);
+VTK_ABI_NAMESPACE_END
 
 // Silence warning like
 // "dereferencing type-punned pointer will break strict-aliasing rules"
@@ -111,6 +101,8 @@ static float read_float(vtk3DSImporter* importer);
 static void read_point(vtk3DSImporter* importer, vtk3DSVector v);
 static char* read_string(vtk3DSImporter* importer);
 
+VTK_ABI_NAMESPACE_BEGIN
+
 vtk3DSImporter::vtk3DSImporter()
 {
   this->OmniList = nullptr;
@@ -151,8 +143,7 @@ std::string vtk3DSImporter::GetOutputsDescription()
 {
   std::stringstream ss;
   size_t idx = 0;
-  for (auto mesh = this->MeshList; mesh != (vtk3DSMesh*)nullptr;
-       mesh = (vtk3DSMesh*)mesh->next, idx++)
+  for (auto mesh = this->MeshList; mesh != nullptr; mesh = (vtk3DSMesh*)mesh->next, idx++)
   {
     if (mesh->aPolyData)
     {
@@ -192,7 +183,7 @@ void vtk3DSImporter::ImportActors(vtkRenderer* renderer)
   vtkActor* actor;
 
   // walk the list of meshes, creating actors
-  for (mesh = this->MeshList; mesh != (vtk3DSMesh*)nullptr; mesh = (vtk3DSMesh*)mesh->next)
+  for (mesh = this->MeshList; mesh != nullptr; mesh = (vtk3DSMesh*)mesh->next)
   {
     if (mesh->faces == 0)
     {
@@ -264,8 +255,7 @@ void vtk3DSImporter::ImportCameras(vtkRenderer* renderer)
   vtk3DSCamera* camera;
 
   // walk the list of cameras and create vtk cameras
-  for (camera = this->CameraList; camera != (vtk3DSCamera*)nullptr;
-       camera = (vtk3DSCamera*)camera->next)
+  for (camera = this->CameraList; camera != nullptr; camera = (vtk3DSCamera*)camera->next)
   {
     camera->aCamera = aCamera = vtkCamera::New();
     aCamera->SetPosition(camera->pos[0], camera->pos[1], camera->pos[2]);
@@ -285,7 +275,7 @@ void vtk3DSImporter::ImportLights(vtkRenderer* renderer)
   vtkLight* aLight;
 
   // just walk the list of omni lights, creating vtk lights
-  for (omniLight = this->OmniList; omniLight != (vtk3DSOmniLight*)nullptr;
+  for (omniLight = this->OmniList; omniLight != nullptr;
        omniLight = (vtk3DSOmniLight*)omniLight->next)
   {
     omniLight->aLight = aLight = vtkLight::New();
@@ -297,7 +287,7 @@ void vtk3DSImporter::ImportLights(vtkRenderer* renderer)
   }
 
   // now walk the list of spot lights, creating vtk lights
-  for (spotLight = this->SpotLightList; spotLight != (vtk3DSSpotLight*)nullptr;
+  for (spotLight = this->SpotLightList; spotLight != nullptr;
        spotLight = (vtk3DSSpotLight*)spotLight->next)
   {
     spotLight->aLight = aLight = vtkLight::New();
@@ -319,7 +309,7 @@ void vtk3DSImporter::ImportProperties(vtkRenderer* vtkNotUsed(renderer))
   vtk3DSMatProp* m;
 
   // just walk the list of material properties, creating vtk properties
-  for (m = this->MatPropList; m != (vtk3DSMatProp*)nullptr; m = (vtk3DSMatProp*)m->next)
+  for (m = this->MatPropList; m != nullptr; m = (vtk3DSMatProp*)m->next)
   {
     if (m->self_illum)
     {
@@ -367,6 +357,109 @@ void vtk3DSImporter::ImportProperties(vtkRenderer* vtkNotUsed(renderer))
   }
 }
 
+vtk3DSImporter::~vtk3DSImporter()
+{
+  vtk3DSOmniLight* omniLight;
+  vtk3DSSpotLight* spotLight;
+
+  // walk the light list and delete vtk objects
+  for (omniLight = this->OmniList; omniLight != nullptr;
+       omniLight = (vtk3DSOmniLight*)omniLight->next)
+  {
+    omniLight->aLight->Delete();
+  }
+  VTK_LIST_KILL(this->OmniList);
+
+  // walk the spot light list and delete vtk objects
+  for (spotLight = this->SpotLightList; spotLight != nullptr;
+       spotLight = (vtk3DSSpotLight*)spotLight->next)
+  {
+    spotLight->aLight->Delete();
+  }
+  VTK_LIST_KILL(this->SpotLightList);
+
+  vtk3DSCamera* camera;
+  // walk the camera list and delete vtk objects
+  for (camera = this->CameraList; camera != nullptr; camera = (vtk3DSCamera*)camera->next)
+  {
+    camera->aCamera->Delete();
+  }
+  VTK_LIST_KILL(this->CameraList);
+
+  // walk the mesh list and delete malloced datra and vtk objects
+  vtk3DSMesh* mesh;
+  for (mesh = this->MeshList; mesh != nullptr; mesh = (vtk3DSMesh*)mesh->next)
+  {
+    if (mesh->anActor != nullptr)
+    {
+      mesh->anActor->Delete();
+    }
+    if (mesh->aMapper != nullptr)
+    {
+      mesh->aMapper->Delete();
+    }
+    if (mesh->aNormals != nullptr)
+    {
+      mesh->aNormals->Delete();
+    }
+    if (mesh->aStripper != nullptr)
+    {
+      mesh->aStripper->Delete();
+    }
+    if (mesh->aPoints != nullptr)
+    {
+      mesh->aPoints->Delete();
+    }
+    if (mesh->aCellArray != nullptr)
+    {
+      mesh->aCellArray->Delete();
+    }
+    if (mesh->aPolyData != nullptr)
+    {
+      mesh->aPolyData->Delete();
+    }
+    if (mesh->vertex)
+    {
+      free(mesh->vertex);
+    }
+    if (mesh->face)
+    {
+      free(mesh->face);
+    }
+    if (mesh->mtl)
+    {
+      free(mesh->mtl);
+    }
+  }
+
+  // then delete the list structure
+
+  VTK_LIST_KILL(this->MeshList);
+  VTK_LIST_KILL(this->MaterialList);
+
+  // objects allocated in Material Property List
+  vtk3DSMatProp* m;
+  // just walk the list of material properties, deleting vtk properties
+  for (m = this->MatPropList; m != nullptr; m = (vtk3DSMatProp*)m->next)
+  {
+    m->aProperty->Delete();
+  }
+
+  // then delete the list structure
+  VTK_LIST_KILL(this->MatPropList);
+
+  delete[] this->FileName;
+}
+
+void vtk3DSImporter::PrintSelf(ostream& os, vtkIndent indent)
+{
+  this->Superclass::PrintSelf(os, indent);
+  os << indent << "File Name: " << (this->FileName ? this->FileName : "(none)") << "\n";
+
+  os << indent << "Compute Normals: " << (this->ComputeNormals ? "On\n" : "Off\n");
+}
+VTK_ABI_NAMESPACE_END
+
 /* Insert a new node into the list */
 static void list_insert(vtk3DSList** root, vtk3DSList* new_node)
 {
@@ -378,7 +471,7 @@ static void list_insert(vtk3DSList** root, vtk3DSList* new_node)
 static void* list_find(vtk3DSList** root, const char* name)
 {
   vtk3DSList* p;
-  for (p = *root; p != (vtk3DSList*)nullptr; p = (vtk3DSList*)p->next)
+  for (p = *root; p != nullptr; p = (vtk3DSList*)p->next)
   {
     if (strcmp(p->name, name) == 0)
     {
@@ -393,7 +486,7 @@ static void list_kill(vtk3DSList** root)
 {
   vtk3DSList* temp;
 
-  while (*root != (vtk3DSList*)nullptr)
+  while (*root != nullptr)
   {
     temp = *root;
     *root = (vtk3DSList*)(*root)->next;
@@ -827,7 +920,7 @@ static void parse_face_array(vtk3DSImporter* importer, vtk3DSMesh* mesh, vtk3DSC
 
   for (i = 0; i < mesh->faces; i++)
   {
-    if (mesh->mtl[i] == (vtk3DSMaterial*)nullptr)
+    if (mesh->mtl[i] == nullptr)
     {
       mesh->mtl[i] = update_materials(importer, "Default", 0);
     }
@@ -1224,106 +1317,3 @@ static void cleanup_name(char* name)
 
   free(tmp);
 }
-
-vtk3DSImporter::~vtk3DSImporter()
-{
-  vtk3DSOmniLight* omniLight;
-  vtk3DSSpotLight* spotLight;
-
-  // walk the light list and delete vtk objects
-  for (omniLight = this->OmniList; omniLight != (vtk3DSOmniLight*)nullptr;
-       omniLight = (vtk3DSOmniLight*)omniLight->next)
-  {
-    omniLight->aLight->Delete();
-  }
-  VTK_LIST_KILL(this->OmniList);
-
-  // walk the spot light list and delete vtk objects
-  for (spotLight = this->SpotLightList; spotLight != (vtk3DSSpotLight*)nullptr;
-       spotLight = (vtk3DSSpotLight*)spotLight->next)
-  {
-    spotLight->aLight->Delete();
-  }
-  VTK_LIST_KILL(this->SpotLightList);
-
-  vtk3DSCamera* camera;
-  // walk the camera list and delete vtk objects
-  for (camera = this->CameraList; camera != (vtk3DSCamera*)nullptr;
-       camera = (vtk3DSCamera*)camera->next)
-  {
-    camera->aCamera->Delete();
-  }
-  VTK_LIST_KILL(this->CameraList);
-
-  // walk the mesh list and delete malloced datra and vtk objects
-  vtk3DSMesh* mesh;
-  for (mesh = this->MeshList; mesh != (vtk3DSMesh*)nullptr; mesh = (vtk3DSMesh*)mesh->next)
-  {
-    if (mesh->anActor != nullptr)
-    {
-      mesh->anActor->Delete();
-    }
-    if (mesh->aMapper != nullptr)
-    {
-      mesh->aMapper->Delete();
-    }
-    if (mesh->aNormals != nullptr)
-    {
-      mesh->aNormals->Delete();
-    }
-    if (mesh->aStripper != nullptr)
-    {
-      mesh->aStripper->Delete();
-    }
-    if (mesh->aPoints != nullptr)
-    {
-      mesh->aPoints->Delete();
-    }
-    if (mesh->aCellArray != nullptr)
-    {
-      mesh->aCellArray->Delete();
-    }
-    if (mesh->aPolyData != nullptr)
-    {
-      mesh->aPolyData->Delete();
-    }
-    if (mesh->vertex)
-    {
-      free(mesh->vertex);
-    }
-    if (mesh->face)
-    {
-      free(mesh->face);
-    }
-    if (mesh->mtl)
-    {
-      free(mesh->mtl);
-    }
-  }
-
-  // then delete the list structure
-
-  VTK_LIST_KILL(this->MeshList);
-  VTK_LIST_KILL(this->MaterialList);
-
-  // objects allocated in Material Property List
-  vtk3DSMatProp* m;
-  // just walk the list of material properties, deleting vtk properties
-  for (m = this->MatPropList; m != (vtk3DSMatProp*)nullptr; m = (vtk3DSMatProp*)m->next)
-  {
-    m->aProperty->Delete();
-  }
-
-  // then delete the list structure
-  VTK_LIST_KILL(this->MatPropList);
-
-  delete[] this->FileName;
-}
-
-void vtk3DSImporter::PrintSelf(ostream& os, vtkIndent indent)
-{
-  this->Superclass::PrintSelf(os, indent);
-  os << indent << "File Name: " << (this->FileName ? this->FileName : "(none)") << "\n";
-
-  os << indent << "Compute Normals: " << (this->ComputeNormals ? "On\n" : "Off\n");
-}
