@@ -1,15 +1,5 @@
-/*=========================================================================
-  Program:   Visualization Toolkit
-  Module:    vtkOBJImporter.cxx
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-=========================================================================*/
-
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkOBJImporter.h"
 
 #include "vtkActor.h"
@@ -37,6 +27,7 @@
 #include <set>
 #include <sstream>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkOBJImporter);
 vtkStandardNewMacro(vtkOBJPolyDataProcessor);
 
@@ -439,7 +430,9 @@ int vtkOBJPolyDataProcessor::RequestData(vtkInformation* vtkNotUsed(request),
 
   bool gotFirstUseMaterialTag = false;
 
+#ifndef NDEBUG
   int numPolysWithTCoords = 0;
+#endif
   bool hasTCoords = false;                 // has vt x y z
   bool hasPolysWithTextureIndices = false; // has f i/t/n or f i/t
   bool hasNormals = false;                 // has f i/t/n or f i//n
@@ -575,8 +568,16 @@ int vtkOBJPolyDataProcessor::RequestData(vtkInformation* vtkNotUsed(request),
             int iVert;
             if (sscanf(pLine, "%d", &iVert) == 1)
             {
-              pointElems->InsertCellPoint(iVert - 1);
-              nVerts++;
+              if (iVert <= 0)
+              {
+                vtkErrorMacro(<< "Unexpected point indices value");
+                everything_ok = false;
+              }
+              else
+              {
+                pointElems->InsertCellPoint(iVert - 1);
+                nVerts++;
+              }
             }
             else if (strcmp(pLine, "\\\n") == 0)
             {
@@ -638,14 +639,30 @@ int vtkOBJPolyDataProcessor::RequestData(vtkInformation* vtkNotUsed(request),
             int iVert, dummyInt;
             if (sscanf(pLine, "%d/%d", &iVert, &dummyInt) == 2)
             {
-              // we simply ignore texture information
-              lineElems->InsertCellPoint(iVert - 1);
-              nVerts++;
+              if (iVert <= 0)
+              {
+                vtkErrorMacro(<< "Unexpected point indices value");
+                everything_ok = false;
+              }
+              else
+              {
+                // we simply ignore texture information
+                lineElems->InsertCellPoint(iVert - 1);
+                nVerts++;
+              }
             }
             else if (sscanf(pLine, "%d", &iVert) == 1)
             {
-              lineElems->InsertCellPoint(iVert - 1);
-              nVerts++;
+              if (iVert <= 0)
+              {
+                vtkErrorMacro(<< "Unexpected point indices value");
+                everything_ok = false;
+              }
+              else
+              {
+                lineElems->InsertCellPoint(iVert - 1);
+                nVerts++;
+              }
             }
             else if (strcmp(pLine, "\\\n") == 0)
             {
@@ -726,17 +743,30 @@ int vtkOBJPolyDataProcessor::RequestData(vtkInformation* vtkNotUsed(request),
               {
                 iNormal = lastVertexIndex + iNormal + 1;
               }
-              hasPolysWithTextureIndices = true;
-              polys->InsertCellPoint(iVert - 1); // convert to 0-based index
-              nVerts++;
-              tcoord_polys->InsertCellPoint(iTCoord - 1);
-              nTCoords++;
-              normal_polys->InsertCellPoint(iNormal - 1);
-              nNormals++;
-              if (iTCoord != iVert)
-                tcoords_same_as_verts = false;
-              if (iNormal != iVert)
-                normals_same_as_verts = false;
+
+              if (iVert <= 0 || iTCoord <= 0 || iNormal <= 0)
+              {
+                vtkErrorMacro(<< "Unexpected point indice value");
+                everything_ok = false;
+              }
+              else
+              {
+                hasPolysWithTextureIndices = true;
+                polys->InsertCellPoint(iVert - 1); // convert to 0-based index
+                nVerts++;
+                tcoord_polys->InsertCellPoint(iTCoord - 1);
+                nTCoords++;
+                normal_polys->InsertCellPoint(iNormal - 1);
+                nNormals++;
+                if (iTCoord != iVert)
+                {
+                  tcoords_same_as_verts = false;
+                }
+                if (iNormal != iVert)
+                {
+                  normals_same_as_verts = false;
+                }
+              }
             }
             else if (sscanf(pLine, "%d//%d", &iVert, &iNormal) == 2)
             {
@@ -748,13 +778,23 @@ int vtkOBJPolyDataProcessor::RequestData(vtkInformation* vtkNotUsed(request),
               {
                 iNormal = lastVertexIndex + iNormal + 1;
               }
-              hasPolysWithTextureIndices = false;
-              polys->InsertCellPoint(iVert - 1);
-              nVerts++;
-              normal_polys->InsertCellPoint(iNormal - 1);
-              nNormals++;
-              if (iNormal != iVert)
-                normals_same_as_verts = false;
+              if (iVert <= 0 || iNormal <= 0)
+              {
+                vtkErrorMacro(<< "Unexpected point indice value");
+                everything_ok = false;
+              }
+              else
+              {
+                hasPolysWithTextureIndices = false;
+                polys->InsertCellPoint(iVert - 1);
+                nVerts++;
+                normal_polys->InsertCellPoint(iNormal - 1);
+                nNormals++;
+                if (iNormal != iVert)
+                {
+                  normals_same_as_verts = false;
+                }
+              }
             }
             else if (sscanf(pLine, "%d/%d", &iVert, &iTCoord) == 2)
             {
@@ -766,13 +806,23 @@ int vtkOBJPolyDataProcessor::RequestData(vtkInformation* vtkNotUsed(request),
               {
                 iTCoord = lastVertexIndex + iTCoord + 1;
               }
-              hasPolysWithTextureIndices = true;
-              polys->InsertCellPoint(iVert - 1);
-              nVerts++;
-              tcoord_polys->InsertCellPoint(iTCoord - 1);
-              nTCoords++;
-              if (iTCoord != iVert)
-                tcoords_same_as_verts = false;
+              if (iVert <= 0 || iTCoord <= 0)
+              {
+                vtkErrorMacro(<< "Unexpected point indice value");
+                everything_ok = false;
+              }
+              else
+              {
+                hasPolysWithTextureIndices = true;
+                polys->InsertCellPoint(iVert - 1);
+                nVerts++;
+                tcoord_polys->InsertCellPoint(iTCoord - 1);
+                nTCoords++;
+                if (iTCoord != iVert)
+                {
+                  tcoords_same_as_verts = false;
+                }
+              }
             }
             else if (sscanf(pLine, "%d", &iVert) == 1)
             {
@@ -780,9 +830,17 @@ int vtkOBJPolyDataProcessor::RequestData(vtkInformation* vtkNotUsed(request),
               {
                 iVert = lastVertexIndex + iVert + 1;
               }
-              hasPolysWithTextureIndices = false;
-              polys->InsertCellPoint(iVert - 1);
-              nVerts++;
+              if (iVert <= 0)
+              {
+                vtkErrorMacro(<< "Unexpected point indice value");
+                everything_ok = false;
+              }
+              else
+              {
+                hasPolysWithTextureIndices = false;
+                polys->InsertCellPoint(iVert - 1);
+                nVerts++;
+              }
             }
             else if (strcmp(pLine, "\\\n") == 0)
             {
@@ -831,7 +889,9 @@ int vtkOBJPolyDataProcessor::RequestData(vtkInformation* vtkNotUsed(request),
         normal_polys->UpdateCellCount(nNormals);
 
         // also make a note of whether any cells have tcoords, and whether any have normals
+#ifndef NDEBUG
         numPolysWithTCoords += (int)(nTCoords) > 0;
+#endif
         if ((!hasTCoords) && (nTCoords > 0))
         {
           vtkDebugMacro("got texture coords in obj file! nTCoords = " << nTCoords);
@@ -1128,3 +1188,4 @@ vtkPolyData* vtkOBJPolyDataProcessor::GetOutput(int idx)
     return nullptr;
   }
 }
+VTK_ABI_NAMESPACE_END
