@@ -1,47 +1,6 @@
-/*=========================================================================
-
-Program:   Visualization Toolkit
-Module:    vtkMPASReader.cxx
-
-Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-All rights reserved.
-See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-This software is distributed WITHOUT ANY WARRANTY; without even
-the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
-/*=========================================================================
-
-  Copyright (c) 2002-2005 Los Alamos National Laboratory
-
-  This software and ancillary information known as vtk_ext (and herein
-  called "SOFTWARE") is made available under the terms described below.
-  The SOFTWARE has been approved for release with associated LA_CC
-  Number 99-44, granted by Los Alamos National Laboratory in July 1999.
-
-  Unless otherwise indicated, this SOFTWARE has been authored by an
-  employee or employees of the University of California, operator of the
-  Los Alamos National Laboratory under Contract No. W-7405-ENG-36 with
-  the United States Department of Energy.
-
-  The United States Government has rights to use, reproduce, and
-  distribute this SOFTWARE.  The public may copy, distribute, prepare
-  derivative works and publicly display this SOFTWARE without charge,
-  provided that this Notice and any statement of authorship are
-  reproduced on all copies.
-
-  Neither the U. S. Government, the University of California, nor the
-  Advanced Computing Laboratory makes any warranty, either express or
-  implied, nor assumes any liability or responsibility for the use of
-  this SOFTWARE.
-
-  If SOFTWARE is modified to produce derivative works, such modified
-  SOFTWARE should be clearly marked, so as not to confuse it with the
-  version available from Los Alamos National Laboratory.
-
-  =========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-FileCopyrightText: Copyright (c) 2002-2005 Los Alamos National Laboratory
+// SPDX-License-Identifier: BSD-3-Clause-Sandia-LANL-California-USGov
 
 // Christine Ahrens (cahrens@lanl.gov)
 // Version 1.3
@@ -120,6 +79,7 @@ PURPOSE.  See the above copyright notice for more information.
     }                                                                                              \
   } while (false)
 
+VTK_ABI_NAMESPACE_BEGIN
 namespace
 {
 
@@ -3049,3 +3009,4 @@ int vtkMPASReader::GetNumberOfPointVars()
 {
   return static_cast<int>(this->Internals->pointVars.size());
 }
+VTK_ABI_NAMESPACE_END
