@@ -1,17 +1,6 @@
-/*=========================================================================
-
-  Program:   ParaView
-  Module:    vtkXMLCompositeDataWriter.cxx
-
-  Copyright (c) Kitware, Inc.
-  All rights reserved.
-  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-FileCopyrightText: Copyright (c) Kitware, Inc.
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkXMLCompositeDataWriter.h"
 
 #include "vtkCallbackCommand.h"
@@ -47,6 +36,7 @@
 
 //------------------------------------------------------------------------------
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkXMLCompositeDataWriterInternals
 {
   // These are used to by GetDefaultFileExtension(). This helps us avoid
@@ -256,7 +246,7 @@ int vtkXMLCompositeDataWriter::WriteNonCompositeData(
   vtkXMLWriter* writer = this->GetWriter(myWriterIndex);
   if (!writer)
   {
-    return 0;
+    return 1;
   }
 
   vtkDataSet* curDS = vtkDataSet::SafeDownCast(dObj);
@@ -332,11 +322,11 @@ int vtkXMLCompositeDataWriter::WriteData()
 
   vtkInformation* meta = input->GetInformation();
   bool hasTime = meta->Has(vtkDataObject::DATA_TIME_STEP()) != 0;
-  if ((fieldData && fieldData->GetNumberOfArrays()) || hasTime)
+  if ((fieldData && fieldData->GetNumberOfArrays()) || (hasTime && this->GetWriteTimeValue()))
   {
     vtkNew<vtkFieldData> fieldDataCopy;
     fieldDataCopy->ShallowCopy(fieldData);
-    if (hasTime)
+    if (hasTime && this->GetWriteTimeValue())
     {
       vtkNew<vtkDoubleArray> time;
       time->SetNumberOfTuples(1);
@@ -488,6 +478,7 @@ void vtkXMLCompositeDataWriter::CreateWriters(vtkCompositeDataSet* hdInput)
       writer->SetEncodeAppendedData(this->GetEncodeAppendedData());
       writer->SetHeaderType(this->GetHeaderType());
       writer->SetIdType(this->GetIdType());
+      writer->SetWriteTimeValue(this->GetWriteTimeValue());
 
       // Pass input.
       writer->SetInputDataObject(iter->GetCurrentDataObject());
@@ -613,3 +604,4 @@ void vtkXMLCompositeDataWriter::RemoveWrittenFiles(const char* SubDirectory)
   this->DeleteAFile();
   this->InputInformation = nullptr;
 }
+VTK_ABI_NAMESPACE_END
