@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   ParaView
-  Module:    vtkXMLCollectionReader.cxx
-
-  Copyright (c) Kitware, Inc.
-  All rights reserved.
-  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkXMLCollectionReader.h"
 
 #include "vtkCallbackCommand.h"
@@ -39,6 +27,8 @@
 #include "vtkXMLPStructuredGridReader.h"
 #include "vtkXMLPTableReader.h"
 #include "vtkXMLPUnstructuredGridReader.h"
+#include "vtkXMLPartitionedDataSetCollectionReader.h"
+#include "vtkXMLPartitionedDataSetReader.h"
 #include "vtkXMLPolyDataReader.h"
 #include "vtkXMLRectilinearGridReader.h"
 #include "vtkXMLStructuredGridReader.h"
@@ -125,6 +115,8 @@ const vtkXMLCollectionReaderInternals::ReaderConstructorsType
     { "vtu", GET_NEW_FUNCTOR(vtkXMLUnstructuredGridReader) },
     { "vti", GET_NEW_FUNCTOR(vtkXMLImageDataReader) },
     { "vtr", GET_NEW_FUNCTOR(vtkXMLRectilinearGridReader) },
+    { "vtpc", GET_NEW_FUNCTOR(vtkXMLPartitionedDataSetCollectionReader) },
+    { "vtpd", GET_NEW_FUNCTOR(vtkXMLPartitionedDataSetReader) },
     { "vtm", GET_NEW_FUNCTOR(vtkXMLMultiBlockDataReader) },
     { "vtmb", GET_NEW_FUNCTOR(vtkXMLMultiBlockDataReader) },
     // legacy reader - produces vtkMultiBlockDataSet.
