@@ -1,10 +1,14 @@
-// See license.md for copyright information.
+// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
+// SPDX-License-Identifier: BSD-3-Clause
 #ifndef vtkVectorJSON_h
 #define vtkVectorJSON_h
 
 #include "vtkVector.h"
 
-#include "nlohmann/json.hpp" // for json
+#include "vtk_nlohmannjson.h"
+// clang-format off
+#include VTK_NLOHMANN_JSON(json.hpp) // for json
+// clang-format on
 
 #include <stdexcept>
 
