@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkCubeAxesActor.h
-  Language:  C++
-
-Copyright (c) 1993-2001 Ken Martin, Will Schroeder, Bill Lorensen
-All rights reserve
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkPolarAxesActor
  * @brief   create an actor of a polar axes -
@@ -33,18 +21,17 @@ All rights reserve
 #ifndef vtkPolarAxesActor_h
 #define vtkPolarAxesActor_h
 
-#define VTK_MAXIMUM_NUMBER_OF_RADIAL_AXES 50
-#define VTK_DEFAULT_NUMBER_OF_RADIAL_AXES 5
-#define VTK_MAXIMUM_NUMBER_OF_POLAR_AXIS_TICKS 200
-#define VTK_MAXIMUM_RATIO 1000.0
-#define VTK_POLAR_ARC_RESOLUTION_PER_DEG 0.2
-
 #include "vtkActor.h"
 #include "vtkAxisActor.h"                 // access to enum values
+#include "vtkDeprecation.h"               // For deprecation macro
+#include "vtkNew.h"                       // used for vtkNew
 #include "vtkRenderingAnnotationModule.h" // For export macro
+#include "vtkSmartPointer.h"              // used for vtkSmartPointer
+#include "vtkWrappingHints.h"             // For VTK_MARSHALAUTO
 #include <list>                           // To process exponent list as reference
 #include <string>                         // used for ivar
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkCamera;
 class vtkPolyData;
 class vtkPolyDataMapper;
@@ -52,7 +39,7 @@ class vtkProperty;
 class vtkStringArray;
 class vtkTextProperty;
 
-class VTKRENDERINGANNOTATION_EXPORT vtkPolarAxesActor : public vtkActor
+class VTKRENDERINGANNOTATION_EXPORT VTK_MARSHALAUTO vtkPolarAxesActor : public vtkActor
 {
 public:
   vtkTypeMacro(vtkPolarAxesActor, vtkActor);
@@ -84,8 +71,8 @@ public:
 
   ///@{
   /**
-   * Enable/Disable log scale
-   * Default: false
+   * Enable/Disable log scale.
+   * Default: false.
    */
   vtkSetMacro(Log, bool);
   vtkGetMacro(Log, bool);
@@ -94,7 +81,8 @@ public:
 
   ///@{
   /**
-   * Gets/Sets the number of radial axes
+   * Gets/Sets the number of radial axes.
+   * Default: 0.
    */
   vtkSetClampMacro(RequestedNumberOfRadialAxes, vtkIdType, 0, VTK_MAXIMUM_NUMBER_OF_RADIAL_AXES);
   vtkGetMacro(RequestedNumberOfRadialAxes, vtkIdType);
@@ -102,26 +90,40 @@ public:
 
   ///@{
   /**
-   * Set/Get a number of ticks that one would like to display along polar axis
-   * NB: it modifies DeltaRangeMajor to correspond to this number
+   * Gets/Sets the number of polar axes.
+   * Default: 5.
    */
+  vtkSetClampMacro(RequestedNumberOfPolarAxes, vtkIdType, 0, VTK_MAXIMUM_NUMBER_OF_POLAR_AXES);
+  vtkGetMacro(RequestedNumberOfPolarAxes, vtkIdType);
+  ///@}
+
+  ///@{
+  /**
+   * Set/Get whether the number of polar axis ticks and arcs should be automatically calculated.
+   * Default: false.
+   */
+  VTK_DEPRECATED_IN_9_3_0("Use SetDeltaRangeMajor instead or enable AxisTickMatchesPolarAxes")
+  vtkSetMacro(AutoSubdividePolarAxis, bool);
+  VTK_DEPRECATED_IN_9_3_0("Use SetDeltaRangeMajor instead or enable AxisTickMatchesPolarAxes")
+  vtkGetMacro(AutoSubdividePolarAxis, bool);
+  void AutoSubdividePolarAxisOn();
+  void AutoSubdividePolarAxisOff();
+  ///@}
+
+  ///@{
+  /**
+   * Set/Get a number of ticks that one would like to display along polar axis.
+   * NB: it modifies DeltaRangeMajor to correspond to this number.
+   */
+  VTK_DEPRECATED_IN_9_3_0("Use SetDeltaRangeMajor instead or enable AxisTickMatchesPolarAxes")
   virtual void SetNumberOfPolarAxisTicks(int);
   int GetNumberOfPolarAxisTicks();
   ///@}
 
-  ///@{
-  /**
-   * Set/Get whether the number of polar axis ticks and arcs should be automatically calculated
-   * Default: true
-   */
-  vtkSetMacro(AutoSubdividePolarAxis, bool);
-  vtkGetMacro(AutoSubdividePolarAxis, bool);
-  vtkBooleanMacro(AutoSubdividePolarAxis, bool);
-  ///@}
-
   ///@{
   /**
    * Define the range values displayed on the polar Axis.
+   * Default: (0, 10).
    */
   vtkSetVector2Macro(Range, double);
   vtkGetVectorMacro(Range, double, 2);
@@ -130,6 +132,7 @@ public:
   ///@{
   /**
    * Set/Get the minimal radius of the polar coordinates.
+   * Default: 0.
    */
   virtual void SetMinimumRadius(double);
   vtkGetMacro(MinimumRadius, double);
@@ -138,6 +141,7 @@ public:
   ///@{
   /**
    * Set/Get the maximum radius of the polar coordinates.
+   * Default: 1.
    */
   virtual void SetMaximumRadius(double);
   vtkGetMacro(MaximumRadius, double);
@@ -146,6 +150,7 @@ public:
   ///@{
   /**
    * Set/Get the minimum radius of the polar coordinates (in degrees).
+   * Default: 0.
    */
   virtual void SetMinimumAngle(double);
   vtkGetMacro(MinimumAngle, double);
@@ -154,6 +159,7 @@ public:
   ///@{
   /**
    * Set/Get the maximum radius of the polar coordinates (in degrees).
+   * Default: 90.
    */
   virtual void SetMaximumAngle(double);
   vtkGetMacro(MaximumAngle, double);
@@ -161,9 +167,9 @@ public:
 
   ///@{
   /**
-   * Set/Get the minimum radial angle distinguishable from polar axis
-   * NB: This is used only when polar axis is visible
-   * Default: 0.5
+   * Set/Get the minimum radial angle distinguishable from polar axis.
+   * NB: This is used only when polar axis is visible.
+   * Default: 0.5.
    */
   vtkSetClampMacro(SmallestVisiblePolarAngle, double, 0., 5.);
   vtkGetMacro(SmallestVisiblePolarAngle, double);
@@ -174,6 +180,7 @@ public:
    * Set/Get the location of the ticks.
    * Inside: tick end toward positive direction of perpendicular axes.
    * Outside: tick end toward negative direction of perpendicular axes.
+   * Default: VTK_TICKS_BOTH.
    */
   vtkSetClampMacro(TickLocation, int, vtkAxisActor::VTK_TICKS_INSIDE, vtkAxisActor::VTK_TICKS_BOTH);
   vtkGetMacro(TickLocation, int);
@@ -198,13 +205,51 @@ public:
   vtkGetMacro(ScreenSize, double);
   ///@}
 
+  ///@{
+  /**
+   * Set/Get the polar title offset.
+   * X-component is used only if text is not aligned to center.
+   * Default: (20, 10).
+   */
+  vtkSetVector2Macro(PolarTitleOffset, double);
+  vtkGetVectorMacro(PolarTitleOffset, double, 2);
+  ///@}
+
+  ///@{
+  /**
+   * Set/Get the radial title offset.
+   * X-component is used only if text is not aligned to center.
+   * Default: (20, 0).
+   */
+  vtkSetVector2Macro(RadialTitleOffset, double);
+  vtkGetVectorMacro(RadialTitleOffset, double, 2);
+  ///@}
+
+  ///@{
+  /**
+   * Set/Get the polar label Y-offset.
+   * Default: 10.
+   */
+  vtkGetMacro(PolarLabelOffset, double);
+  vtkSetMacro(PolarLabelOffset, double);
+  ///@}
+
+  ///@{
+  /**
+   * Set/Get the polar exponent Y-offset.
+   * Default: 5.
+   */
+  vtkGetMacro(PolarExponentOffset, double);
+  vtkSetMacro(PolarExponentOffset, double);
+  ///@}
+
   ///@{
   /**
    * Set/Get the camera to perform scaling and translation of the
    * vtkPolarAxesActor.
    */
   virtual void SetCamera(vtkCamera*);
-  vtkGetObjectMacro(Camera, vtkCamera);
+  vtkCamera* GetCamera();
   ///@}
 
   ///@{
@@ -259,14 +304,16 @@ public:
   ///@{
   /**
    * Enable and disable the use of distance based LOD for titles and labels.
+   * Default: true.
    */
-  vtkSetMacro(EnableDistanceLOD, int);
-  vtkGetMacro(EnableDistanceLOD, int);
+  vtkSetMacro(EnableDistanceLOD, bool);
+  vtkGetMacro(EnableDistanceLOD, bool);
   ///@}
 
   ///@{
   /**
    * Set distance LOD threshold [0.0 - 1.0] for titles and labels.
+   * Default: 0.7.
    */
   vtkSetClampMacro(DistanceLODThreshold, double, 0.0, 1.0);
   vtkGetMacro(DistanceLODThreshold, double);
@@ -275,14 +322,16 @@ public:
   ///@{
   /**
    * Enable and disable the use of view angle based LOD for titles and labels.
+   * Default: true.
    */
-  vtkSetMacro(EnableViewAngleLOD, int);
-  vtkGetMacro(EnableViewAngleLOD, int);
+  vtkSetMacro(EnableViewAngleLOD, bool);
+  vtkGetMacro(EnableViewAngleLOD, bool);
   ///@}
 
   ///@{
   /**
    * Set view angle LOD threshold [0.0 - 1.0] for titles and labels.
+   * Default: 0.3.
    */
   vtkSetClampMacro(ViewAngleLODThreshold, double, 0., 1.);
   vtkGetMacro(ViewAngleLODThreshold, double);
@@ -291,37 +340,41 @@ public:
   ///@{
   /**
    * Turn on and off the visibility of the polar axis.
+   * Default: true.
    */
-  vtkSetMacro(PolarAxisVisibility, vtkTypeBool);
-  vtkGetMacro(PolarAxisVisibility, vtkTypeBool);
-  vtkBooleanMacro(PolarAxisVisibility, vtkTypeBool);
+  vtkSetMacro(PolarAxisVisibility, bool);
+  vtkGetMacro(PolarAxisVisibility, bool);
+  vtkBooleanMacro(PolarAxisVisibility, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of inner radial grid lines
+   * Default: true.
    */
-  vtkSetMacro(DrawRadialGridlines, vtkTypeBool);
-  vtkGetMacro(DrawRadialGridlines, vtkTypeBool);
-  vtkBooleanMacro(DrawRadialGridlines, vtkTypeBool);
+  vtkSetMacro(DrawRadialGridlines, bool);
+  vtkGetMacro(DrawRadialGridlines, bool);
+  vtkBooleanMacro(DrawRadialGridlines, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of inner polar arcs grid lines
+   * Default: true.
    */
-  vtkSetMacro(DrawPolarArcsGridlines, vtkTypeBool);
-  vtkGetMacro(DrawPolarArcsGridlines, vtkTypeBool);
-  vtkBooleanMacro(DrawPolarArcsGridlines, vtkTypeBool);
+  vtkSetMacro(DrawPolarArcsGridlines, bool);
+  vtkGetMacro(DrawPolarArcsGridlines, bool);
+  vtkBooleanMacro(DrawPolarArcsGridlines, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of titles for polar axis.
+   * Default: true.
    */
-  vtkSetMacro(PolarTitleVisibility, vtkTypeBool);
-  vtkGetMacro(PolarTitleVisibility, vtkTypeBool);
-  vtkBooleanMacro(PolarTitleVisibility, vtkTypeBool);
+  vtkSetMacro(PolarTitleVisibility, bool);
+  vtkGetMacro(PolarTitleVisibility, bool);
+  vtkBooleanMacro(PolarTitleVisibility, bool);
   ///@}
 
   enum TitleLocation
@@ -333,7 +386,8 @@ public:
   ///@{
   /**
    * Get/Set the alignment of the radial axes title related to the axis.
-   * Possible Alignment: VTK_TITLE_BOTTOM, VTK_TITLE_EXTERN
+   * Possible Alignment: VTK_TITLE_BOTTOM, VTK_TITLE_EXTERN.
+   * Default: VTK_TITLE_BOTTOM.
    */
   vtkSetClampMacro(RadialAxisTitleLocation, int, VTK_TITLE_BOTTOM, VTK_TITLE_EXTERN);
   vtkGetMacro(RadialAxisTitleLocation, int);
@@ -342,7 +396,8 @@ public:
   ///@{
   /**
    * Get/Set the alignment of the polar axes title related to the axis.
-   * Possible Alignment: VTKTITLE_BOTTOM, VTK_TITLE_EXTERN
+   * Possible Alignment: VTKTITLE_BOTTOM, VTK_TITLE_EXTERN.
+   * Default: VTK_TITLE_BOTTOM.
    */
   vtkSetClampMacro(PolarAxisTitleLocation, int, VTK_TITLE_BOTTOM, VTK_TITLE_EXTERN);
   vtkGetMacro(PolarAxisTitleLocation, int);
@@ -351,84 +406,112 @@ public:
   ///@{
   /**
    * Turn on and off the visibility of labels for polar axis.
+   * Default: true.
    */
-  vtkSetMacro(PolarLabelVisibility, vtkTypeBool);
-  vtkGetMacro(PolarLabelVisibility, vtkTypeBool);
-  vtkBooleanMacro(PolarLabelVisibility, vtkTypeBool);
+  vtkSetMacro(PolarLabelVisibility, bool);
+  vtkGetMacro(PolarLabelVisibility, bool);
+  vtkBooleanMacro(PolarLabelVisibility, bool);
   ///@}
 
   ///@{
   /**
    * If On, the ticks are drawn from the angle of the polarAxis (i.e. this->MinimalRadius)
    * and continue counterclockwise with the step DeltaAngle Major/Minor. if Off, the start angle is
-   * 0.0, i.e.
-   * the angle on the major radius of the ellipse.
+   * 0.0, i.e. the angle on the major radius of the ellipse.
+   * Default: true.
    */
-  vtkSetMacro(ArcTicksOriginToPolarAxis, vtkTypeBool);
-  vtkGetMacro(ArcTicksOriginToPolarAxis, vtkTypeBool);
-  vtkBooleanMacro(ArcTicksOriginToPolarAxis, vtkTypeBool);
+  vtkSetMacro(ArcTicksOriginToPolarAxis, bool);
+  vtkGetMacro(ArcTicksOriginToPolarAxis, bool);
+  vtkBooleanMacro(ArcTicksOriginToPolarAxis, bool);
   ///@}
 
   ///@{
   /**
    * If On, the radial axes are drawn from the angle of the polarAxis (i.e. this->MinimalRadius)
    * and continue counterclockwise with the step DeltaAngleRadialAxes. if Off, the start angle is
-   * 0.0, i.e.
-   * the angle on the major radius of the ellipse.
+   * 0.0, i.e. the angle on the major radius of the ellipse.
+   * Default: true.
    */
-  vtkSetMacro(RadialAxesOriginToPolarAxis, vtkTypeBool);
-  vtkGetMacro(RadialAxesOriginToPolarAxis, vtkTypeBool);
-  vtkBooleanMacro(RadialAxesOriginToPolarAxis, vtkTypeBool);
+  vtkSetMacro(RadialAxesOriginToPolarAxis, bool);
+  vtkGetMacro(RadialAxesOriginToPolarAxis, bool);
+  vtkBooleanMacro(RadialAxesOriginToPolarAxis, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the overall visibility of ticks.
+   * Default: true.
    */
-  vtkSetMacro(PolarTickVisibility, vtkTypeBool);
-  vtkGetMacro(PolarTickVisibility, vtkTypeBool);
-  vtkBooleanMacro(PolarTickVisibility, vtkTypeBool);
+  vtkSetMacro(PolarTickVisibility, bool);
+  vtkGetMacro(PolarTickVisibility, bool);
+  vtkBooleanMacro(PolarTickVisibility, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of major ticks on polar axis and last radial axis.
+   * Default: true.
    */
-  vtkSetMacro(AxisTickVisibility, vtkTypeBool);
-  vtkGetMacro(AxisTickVisibility, vtkTypeBool);
-  vtkBooleanMacro(AxisTickVisibility, vtkTypeBool);
+  vtkSetMacro(AxisTickVisibility, bool);
+  vtkGetMacro(AxisTickVisibility, bool);
+  vtkBooleanMacro(AxisTickVisibility, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of minor ticks on polar axis and last radial axis.
+   * Default: false.
    */
-  vtkSetMacro(AxisMinorTickVisibility, vtkTypeBool);
-  vtkGetMacro(AxisMinorTickVisibility, vtkTypeBool);
-  vtkBooleanMacro(AxisMinorTickVisibility, vtkTypeBool);
+  vtkSetMacro(AxisMinorTickVisibility, bool);
+  vtkGetMacro(AxisMinorTickVisibility, bool);
+  vtkBooleanMacro(AxisMinorTickVisibility, bool);
+  ///@}
+
+  ///@{
+  /**
+   * Turn on and off the use of polar axes range for axis major ticks.
+   * Default: true.
+   */
+  vtkSetMacro(AxisTickMatchesPolarAxes, bool);
+  vtkGetMacro(AxisTickMatchesPolarAxes, bool);
+  vtkBooleanMacro(AxisTickMatchesPolarAxes, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of major ticks on the last arc.
+   * Default: true.
    */
-  vtkSetMacro(ArcTickVisibility, vtkTypeBool);
-  vtkGetMacro(ArcTickVisibility, vtkTypeBool);
-  vtkBooleanMacro(ArcTickVisibility, vtkTypeBool);
+  vtkSetMacro(ArcTickVisibility, bool);
+  vtkGetMacro(ArcTickVisibility, bool);
+  vtkBooleanMacro(ArcTickVisibility, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of minor ticks on the last arc.
+   * Default: false.
    */
-  vtkSetMacro(ArcMinorTickVisibility, vtkTypeBool);
-  vtkGetMacro(ArcMinorTickVisibility, vtkTypeBool);
-  vtkBooleanMacro(ArcMinorTickVisibility, vtkTypeBool);
+  vtkSetMacro(ArcMinorTickVisibility, bool);
+  vtkGetMacro(ArcMinorTickVisibility, bool);
+  vtkBooleanMacro(ArcMinorTickVisibility, bool);
+  ///@}
+
+  ///@{
+  /**
+   * Turn on and off the use of radial axes angle for arc major ticks.
+   * Default: true.
+   */
+  vtkSetMacro(ArcTickMatchesRadialAxes, bool);
+  vtkGetMacro(ArcTickMatchesRadialAxes, bool);
+  vtkBooleanMacro(ArcTickMatchesRadialAxes, bool);
   ///@}
 
   ///@{
   /**
    * Set/Get the size of the major ticks on the last arc.
+   * If set to 0, compute it as a ratio of maximum radius.
+   * Default 0.
    */
   vtkSetMacro(ArcMajorTickSize, double);
   vtkGetMacro(ArcMajorTickSize, double);
@@ -437,6 +520,8 @@ public:
   ///@{
   /**
    * Set/Get the size of the major ticks on the polar axis.
+   * If set to 0, compute it as a ratio of maximum radius.
+   * Default 0.
    */
   vtkSetMacro(PolarAxisMajorTickSize, double);
   vtkGetMacro(PolarAxisMajorTickSize, double);
@@ -445,6 +530,8 @@ public:
   ///@{
   /**
    * Set/Get the size of the major ticks on the last radial axis.
+   * If set to 0, compute it as a ratio of maximum radius.
+   * Default 0.
    */
   vtkSetMacro(LastRadialAxisMajorTickSize, double);
   vtkGetMacro(LastRadialAxisMajorTickSize, double);
@@ -452,7 +539,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the ratio between major and minor Polar Axis ticks size
+   * Set/Get the ratio between major and minor Polar Axis ticks size.
+   * Default: 0.3.
    */
   vtkSetMacro(PolarAxisTickRatioSize, double);
   vtkGetMacro(PolarAxisTickRatioSize, double);
@@ -460,7 +548,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the ratio between major and minor Last Radial axis ticks size
+   * Set/Get the ratio between major and minor Last Radial axis ticks size.
+   * Default: 0.3.
    */
   vtkSetMacro(LastAxisTickRatioSize, double);
   vtkGetMacro(LastAxisTickRatioSize, double);
@@ -468,7 +557,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the ratio between major and minor Arc ticks size
+   * Set/Get the ratio between major and minor Arc ticks size.
+   * Default: 0.3.
    */
   vtkSetMacro(ArcTickRatioSize, double);
   vtkGetMacro(ArcTickRatioSize, double);
@@ -476,7 +566,17 @@ public:
 
   ///@{
   /**
-   * Set/Get the size of the thickness of polar axis ticks
+   * Set/Get the ratio between maximum radius and major ticks size.
+   * Default: 0.02.
+   */
+  vtkSetMacro(TickRatioRadiusSize, double);
+  vtkGetMacro(TickRatioRadiusSize, double);
+  ///@}
+
+  ///@{
+  /**
+   * Set/Get the size of the thickness of polar axis ticks.
+   * Default: 1.
    */
   vtkSetMacro(PolarAxisMajorTickThickness, double);
   vtkGetMacro(PolarAxisMajorTickThickness, double);
@@ -484,7 +584,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the size of the thickness of last radial axis ticks
+   * Set/Get the size of the thickness of last radial axis ticks.
+   * Default: 1.
    */
   vtkSetMacro(LastRadialAxisMajorTickThickness, double);
   vtkGetMacro(LastRadialAxisMajorTickThickness, double);
@@ -492,7 +593,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the size of the thickness of the last arc ticks
+   * Set/Get the size of the thickness of the last arc ticks.
+   * Default: 1.
    */
   vtkSetMacro(ArcMajorTickThickness, double);
   vtkGetMacro(ArcMajorTickThickness, double);
@@ -500,7 +602,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the ratio between major and minor Polar Axis ticks thickness
+   * Set/Get the ratio between major and minor Polar Axis ticks thickness.
+   * Default: 0.5.
    */
   vtkSetMacro(PolarAxisTickRatioThickness, double);
   vtkGetMacro(PolarAxisTickRatioThickness, double);
@@ -508,7 +611,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the ratio between major and minor Last Radial axis ticks thickness
+   * Set/Get the ratio between major and minor Last Radial axis ticks thickness.
+   * Default: 0.5.
    */
   vtkSetMacro(LastAxisTickRatioThickness, double);
   vtkGetMacro(LastAxisTickRatioThickness, double);
@@ -516,7 +620,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the ratio between major and minor Arc ticks thickness
+   * Set/Get the ratio between major and minor Arc ticks thickness.
+   * Default: 0.5.
    */
   vtkSetMacro(ArcTickRatioThickness, double);
   vtkGetMacro(ArcTickRatioThickness, double);
@@ -524,7 +629,8 @@ public:
 
   ///@{
   /**
-   * Set/Get the step between 2 major ticks, in range value (values displayed on the axis).
+   * Set/Get the range between 2 major ticks (values displayed on the axis).
+   * Default: 1.
    */
   vtkSetMacro(DeltaRangeMajor, double);
   vtkGetMacro(DeltaRangeMajor, double);
@@ -532,15 +638,27 @@ public:
 
   ///@{
   /**
-   * Set/Get the step between 2 minor ticks, in range value (values displayed on the axis).
+   * Set/Get the range between 2 minor ticks.
+   * Default: 0.5.
    */
   vtkSetMacro(DeltaRangeMinor, double);
   vtkGetMacro(DeltaRangeMinor, double);
   ///@}
 
+  ///@{
+  /**
+   * Set/Get requested delta range for polar axes.
+   * If set to 0, compute it depending on count.
+   * Default: 0.
+   */
+  vtkSetMacro(RequestedDeltaRangePolarAxes, double);
+  vtkGetMacro(RequestedDeltaRangePolarAxes, double);
+  ///@}
+
   ///@{
   /**
    * Set/Get the angle between 2 major ticks on the last arc.
+   * Default: 45.
    */
   vtkSetMacro(DeltaAngleMajor, double);
   vtkGetMacro(DeltaAngleMajor, double);
@@ -549,6 +667,7 @@ public:
   ///@{
   /**
    * Set/Get the angle between 2 minor ticks on the last arc.
+   * Default: 22.5.
    */
   vtkSetMacro(DeltaAngleMinor, double);
   vtkGetMacro(DeltaAngleMinor, double);
@@ -556,10 +675,12 @@ public:
 
   ///@{
   /**
-   * Set/Get the angle between 2 radial axes.
+   * Set/Get requested delta angle for radial axes.
+   * If set to 0, compute it depending on count.
+   * Default: 45.
    */
-  vtkSetMacro(DeltaAngleRadialAxes, double);
-  vtkGetMacro(DeltaAngleRadialAxes, double);
+  vtkSetMacro(RequestedDeltaAngleRadialAxes, double);
+  vtkGetMacro(RequestedDeltaAngleRadialAxes, double);
   ///@}
 
   //------------------------------------------------
@@ -567,36 +688,39 @@ public:
   ///@{
   /**
    * Turn on and off the visibility of non-polar radial axes.
+   * Default: true.
    */
-  vtkSetMacro(RadialAxesVisibility, vtkTypeBool);
-  vtkGetMacro(RadialAxesVisibility, vtkTypeBool);
-  vtkBooleanMacro(RadialAxesVisibility, vtkTypeBool);
+  vtkSetMacro(RadialAxesVisibility, bool);
+  vtkGetMacro(RadialAxesVisibility, bool);
+  vtkBooleanMacro(RadialAxesVisibility, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of titles for non-polar radial axes.
+   * Default: true.
    */
-  vtkSetMacro(RadialTitleVisibility, vtkTypeBool);
-  vtkGetMacro(RadialTitleVisibility, vtkTypeBool);
-  vtkBooleanMacro(RadialTitleVisibility, vtkTypeBool);
+  vtkSetMacro(RadialTitleVisibility, bool);
+  vtkGetMacro(RadialTitleVisibility, bool);
+  vtkBooleanMacro(RadialTitleVisibility, bool);
   ///@}
 
   ///@{
   /**
    * Turn on and off the visibility of arcs for polar axis.
+   * Default: true.
    */
-  vtkSetMacro(PolarArcsVisibility, vtkTypeBool);
-  vtkGetMacro(PolarArcsVisibility, vtkTypeBool);
-  vtkBooleanMacro(PolarArcsVisibility, vtkTypeBool);
+  vtkSetMacro(PolarArcsVisibility, bool);
+  vtkGetMacro(PolarArcsVisibility, bool);
+  vtkBooleanMacro(PolarArcsVisibility, bool);
   ///@}
 
   ///@{
   /**
    * Enable/Disable labels 2D mode (always facing the camera).
    */
-  void SetUse2DMode(int val);
-  int GetUse2DMode();
+  void SetUse2DMode(bool enable);
+  bool GetUse2DMode();
   ///@}
 
   ///@{
@@ -604,7 +728,7 @@ public:
    * Set/Get the polar axis title text property.
    */
   virtual void SetPolarAxisTitleTextProperty(vtkTextProperty* p);
-  vtkGetObjectMacro(PolarAxisTitleTextProperty, vtkTextProperty);
+  vtkTextProperty* GetPolarAxisTitleTextProperty();
   ///@}
 
   ///@{
@@ -612,7 +736,7 @@ public:
    * Set/Get the polar axis labels text property.
    */
   virtual void SetPolarAxisLabelTextProperty(vtkTextProperty* p);
-  vtkGetObjectMacro(PolarAxisLabelTextProperty, vtkTextProperty);
+  vtkTextProperty* GetPolarAxisLabelTextProperty();
   ///@}
 
   ///@{
@@ -620,7 +744,7 @@ public:
    * Set/Get the last radial axis text property.
    */
   virtual void SetLastRadialAxisTextProperty(vtkTextProperty* p);
-  vtkGetObjectMacro(LastRadialAxisTextProperty, vtkTextProperty);
+  vtkTextProperty* GetLastRadialAxisTextProperty();
   ///@}
 
   ///@{
@@ -628,7 +752,7 @@ public:
    * Set/Get the secondary radial axes text property.
    */
   virtual void SetSecondaryRadialAxesTextProperty(vtkTextProperty* p);
-  vtkGetObjectMacro(SecondaryRadialAxesTextProperty, vtkTextProperty);
+  vtkTextProperty* GetSecondaryRadialAxesTextProperty();
   ///@}
 
   ///@{
@@ -636,7 +760,7 @@ public:
    * Get/Set polar axis actor properties.
    */
   virtual void SetPolarAxisProperty(vtkProperty*);
-  vtkGetObjectMacro(PolarAxisProperty, vtkProperty);
+  vtkProperty* GetPolarAxisProperty();
   ///@}
 
   ///@{
@@ -644,7 +768,7 @@ public:
    * Get/Set last radial axis actor properties.
    */
   virtual void SetLastRadialAxisProperty(vtkProperty* p);
-  vtkGetObjectMacro(LastRadialAxisProperty, vtkProperty);
+  vtkProperty* GetLastRadialAxisProperty();
   ///@}
 
   ///@{
@@ -652,7 +776,7 @@ public:
    * Get/Set secondary radial axes actors properties.
    */
   virtual void SetSecondaryRadialAxesProperty(vtkProperty* p);
-  vtkGetObjectMacro(SecondaryRadialAxesProperty, vtkProperty);
+  vtkProperty* GetSecondaryRadialAxesProperty();
   ///@}
 
   ///@{
@@ -677,6 +801,7 @@ public:
    * The bounds are used only when no Input or Prop is specified. The bounds
    * are specified according to (xmin,xmax, ymin,ymax, zmin,zmax), making
    * sure that the min's are less than the max's.
+   * Default: (-1, 1, -1, 1, -1, 1).
    */
   vtkSetVector6Macro(Bounds, double);
   double* GetBounds() override;
@@ -687,12 +812,23 @@ public:
 
   ///@{
   /**
-   * Ratio
+   * Ratio.
+   * Default: 1.
    */
   vtkSetClampMacro(Ratio, double, 0.001, 100.0);
   vtkGetMacro(Ratio, double);
   ///@}
 
+  ///@{
+  /**
+   * Set/Get the number of line per degree to draw polar arc.
+   * Default: 0.2.
+   */
+  vtkSetClampMacro(PolarArcResolutionPerDegree, double, VTK_MINIMUM_POLAR_ARC_RESOLUTION_PER_DEG,
+    VTK_MAXIMUM_POLAR_ARC_RESOLUTION_PER_DEG);
+  vtkGetMacro(PolarArcResolutionPerDegree, double);
+  ///@}
+
 protected:
   vtkPolarAxesActor();
   ~vtkPolarAxesActor() override;
@@ -741,6 +877,7 @@ protected:
    * attributes
    * then PolarAxis itself is in charge of ticks drawing
    */
+  VTK_DEPRECATED_IN_9_3_0("Use SetDeltaRangeMajor instead or enable AxisTickMatchesPolarAxes")
   void AutoComputeTicksProperties();
 
   /**
@@ -807,187 +944,210 @@ protected:
    */
   static double ComputeEllipseAngle(double angleInDegrees, double ratio);
 
+  /**
+   * Compute delta range of polar axes.
+   */
+  virtual void ComputeDeltaRangePolarAxes(vtkIdType);
+
   /**
    * Compute delta angle of radial axes.
    */
   virtual void ComputeDeltaAngleRadialAxes(vtkIdType);
   /**
    * Coordinates of the pole
-   * Default: (0,0,0).
    */
-  double Pole[3];
+  double Pole[3] = { 0.0, 0.0, 0.0 };
 
   /**
    * Number of radial axes
    */
-  int NumberOfRadialAxes;
+  int NumberOfRadialAxes = 0;
 
   /**
    * Requested Number of radial axes
+   * If set to 0, compute it depending on angle.
    */
-  int RequestedNumberOfRadialAxes;
+  int RequestedNumberOfRadialAxes = 0;
 
   /**
-   * Whether the number of polar axis ticks and arcs should be automatically calculated.
-   * Default: TRUE
+   * Number of polar axes
    */
-  bool AutoSubdividePolarAxis;
+  int NumberOfPolarAxes = 5;
+
+  /**
+   * Requested Number of polar axes
+   * If set to 0, compute it depending on range.
+   */
+  int RequestedNumberOfPolarAxes = 5;
 
   /**
    * Ratio for elliptical representation of the polar axes actor.
    */
-  double Ratio;
+  double Ratio = 1.0;
+
+  /**
+   * Polar arc resolution (number of line) per degree.
+   */
+  double PolarArcResolutionPerDegree = 0.2;
 
   /**
    * Define the range values displayed on the polar Axis.
    */
-  double Range[2];
+  double Range[2] = { 0.0, 10.0 };
 
   /**
-   * Step between 2 minor ticks, in range value (values displayed on the axis).
+   * Range between 2 minor ticks.
    */
-  double DeltaRangeMinor;
+  double DeltaRangeMinor = 0.5;
 
   /**
-   * Step between 2 major ticks, in range value (values displayed on the axis).
+   * Range between 2 major ticks (values displayed on the axis).
    */
-  double DeltaRangeMajor;
+  double DeltaRangeMajor = 1.0;
+
+  /**
+   * DEPRECATED: Automatically subdivide polar axis.
+   */
+  bool AutoSubdividePolarAxis = false;
+
+  /**
+   * Range between 2 polar axes.
+   */
+  double DeltaRangePolarAxes = 0.0;
+
+  /**
+   * Requested delta range for polar axes.
+   * If set to 0, compute it depending on count.
+   */
+  double RequestedDeltaRangePolarAxes = 0.0;
 
   /**
    * Angle between 2 minor ticks on the last arc.
    */
-  double DeltaAngleMinor;
+  double DeltaAngleMinor = 22.5;
 
   /**
    * Angle between 2 major ticks on the last arc.
    */
-  double DeltaAngleMajor;
+  double DeltaAngleMajor = 45.0;
 
   /**
    * Angle between 2 radial Axes.
    */
-  double DeltaAngleRadialAxes;
+  double DeltaAngleRadialAxes = 45.0;
+
+  /**
+   * Requested delta angle for radial axes.
+   * If set to 0, compute it depending on count.
+   */
+  double RequestedDeltaAngleRadialAxes = 45.0;
 
   /**
    * Minimum polar radius.
-   * Default: 0.0
    */
-  double MinimumRadius;
+  double MinimumRadius = 0.0;
 
   /**
    * Maximum polar radius.
-   * Default: 1
    */
-  double MaximumRadius;
+  double MaximumRadius = 1.0;
 
   /**
    * Enable/Disable log scale
-   * Default: 0
    */
-  bool Log;
-
-  /**
-   * Auto-scale polar radius (with respect to average length scale of x-y bounding box).
-   */
-  bool AutoScaleRadius;
+  bool Log = false;
 
   /**
    * Minimum polar angle
-   * Default: 0.
    */
-  double MinimumAngle;
+  double MinimumAngle = 0.0;
 
   /**
    * Maximum polar angle
-   * Default: 90.
    */
-  double MaximumAngle;
+  double MaximumAngle = 90.0;
 
   /**
    * Smallest radial angle distinguishable from polar axis
    */
-  double SmallestVisiblePolarAngle;
+  double SmallestVisiblePolarAngle = 0.5;
 
   // Structures for principal polar arc
-  vtkPolyData* PolarArcs;
-  vtkPolyDataMapper* PolarArcsMapper;
-  vtkActor* PolarArcsActor;
+  vtkNew<vtkPolyData> PolarArcs;
+  vtkNew<vtkPolyDataMapper> PolarArcsMapper;
+  vtkNew<vtkActor> PolarArcsActor;
 
   ///@{
   /**
    * Structures for secondary polar arcs
    */
-  vtkPolyData* SecondaryPolarArcs;
-  vtkPolyDataMapper* SecondaryPolarArcsMapper;
-  vtkActor* SecondaryPolarArcsActor;
+  vtkNew<vtkPolyData> SecondaryPolarArcs;
+  vtkNew<vtkPolyDataMapper> SecondaryPolarArcsMapper;
+  vtkNew<vtkActor> SecondaryPolarArcsActor;
   ///@}
 
   /**
    * Camera attached to the polar axes system
    */
-  vtkCamera* Camera;
+  vtkSmartPointer<vtkCamera> Camera;
 
   /**
    * Control variables for polar axis
    */
-  vtkAxisActor* PolarAxis;
+  vtkNew<vtkAxisActor> PolarAxis;
 
   /**
    * Control variables for non-polar radial axes
    */
-  vtkAxisActor** RadialAxes;
+  vtkSmartPointer<vtkAxisActor>* RadialAxes = nullptr;
 
   ///@{
   /**
    * Title to be used for the polar axis
    * NB: Non-polar radial axes use the polar angle as title and have no labels
    */
-  char* PolarAxisTitle;
-  char* PolarLabelFormat;
+  char* PolarAxisTitle = nullptr;
+  char* PolarLabelFormat = nullptr;
   ///@}
 
   /**
    * String to format angle values displayed on the radial axes.
    */
-  char* RadialAngleFormat;
+  char* RadialAngleFormat = nullptr;
 
   /**
    * Display angle units (degrees) to label radial axes
-   * Default is true
    */
-  bool RadialUnits;
+  bool RadialUnits = true;
 
   /**
    * If enabled the actor will not be visible at a certain distance from the camera.
-   * Default is true
    */
-  int EnableDistanceLOD;
+  bool EnableDistanceLOD = true;
 
   /**
-   * Default is 0.80
    * This determines at what fraction of camera far clip range, actor is not visible.
    */
-  double DistanceLODThreshold;
+  double DistanceLODThreshold = 0.7;
 
   /**
    * If enabled the actor will not be visible at a certain view angle.
-   * Default is true.
    */
-  int EnableViewAngleLOD;
+  bool EnableViewAngleLOD = true;
 
   /**
    * This determines at what view angle to geometry will make the geometry not visible.
    * Default is 0.3.
    */
-  double ViewAngleLODThreshold;
+  double ViewAngleLODThreshold = 0.3;
 
   ///@{
   /**
    * Visibility of polar axis and its title, labels, ticks (major only)
    */
-  vtkTypeBool PolarAxisVisibility;
-  vtkTypeBool PolarTitleVisibility;
-  vtkTypeBool PolarLabelVisibility;
+  bool PolarAxisVisibility = true;
+  bool PolarTitleVisibility = true;
+  bool PolarLabelVisibility = true;
   ///@}
 
   /**
@@ -997,12 +1157,12 @@ protected:
    * For an ellipse, tick directions are defined from ellipse center to tick origin and
    * the orthogonal direction of the ellipse plane.
    */
-  int TickLocation;
+  int TickLocation = vtkAxisActor::VTK_TICKS_BOTH;
 
   /**
    * Hold visibility for all present ticks
    */
-  vtkTypeBool PolarTickVisibility;
+  bool PolarTickVisibility = true;
 
   /**
    * If On, the ticks are drawn from the angle of the polarAxis (i.e. this->MinimumAngle)
@@ -1010,7 +1170,7 @@ protected:
    * 0.0, i.e.
    * the angle on the major radius of the ellipse.
    */
-  int ArcTicksOriginToPolarAxis;
+  bool ArcTicksOriginToPolarAxis = true;
 
   /**
    * If On, the radial axes are drawn from the angle of the polarAxis (i.e. this->MinimalRadius)
@@ -1018,153 +1178,184 @@ protected:
    * 0.0, i.e.
    * the angle on the major radius of the ellipse.
    */
-  int RadialAxesOriginToPolarAxis;
+  bool RadialAxesOriginToPolarAxis = true;
 
   /**
    * Hold visibility of major/minor ticks for the polar axis and the last radial axis
    */
-  vtkTypeBool AxisTickVisibility, AxisMinorTickVisibility;
+  bool AxisTickVisibility = true, AxisMinorTickVisibility = false;
+
+  /**
+   * Enable / Disable major ticks matching polar axes range (and minor half angle)
+   */
+  bool AxisTickMatchesPolarAxes = true;
 
   /**
    * Enable / Disable major/minor tick visibility on the last arc displayed
    */
-  vtkTypeBool ArcTickVisibility, ArcMinorTickVisibility;
+  bool ArcTickVisibility = true, ArcMinorTickVisibility = false;
+
+  /**
+   * Enable / Disable major ticks matching radial axes angle (and minor half angle)
+   */
+  bool ArcTickMatchesRadialAxes = true;
 
   /**
    * Defines the length of the ticks located on the last arc
    */
-  double PolarAxisMajorTickSize, LastRadialAxisMajorTickSize, ArcMajorTickSize;
+  double PolarAxisMajorTickSize = 0.0, LastRadialAxisMajorTickSize = 0.0, ArcMajorTickSize = 0.0;
 
   /**
-   * Set the ratios between major tick Size for each ticks location
+   * Set the ratios between major and minor tick Size for each ticks location
    */
-  double PolarAxisTickRatioSize, LastAxisTickRatioSize, ArcTickRatioSize;
+  double PolarAxisTickRatioSize = 0.3, LastAxisTickRatioSize = 0.3, ArcTickRatioSize = 0.3;
 
   /**
-   * Defines the tickness of the major ticks.
+   * Set the ratio between maximum radius and major tick size
    */
-  double PolarAxisMajorTickThickness, LastRadialAxisMajorTickThickness, ArcMajorTickThickness;
+  double TickRatioRadiusSize = 0.02;
+
+  /**
+   * Defines the thickness of the major ticks.
+   */
+  double PolarAxisMajorTickThickness = 1.0, LastRadialAxisMajorTickThickness = 1.0,
+         ArcMajorTickThickness = 1.0;
 
   /**
    * Set the ratios between major tick thickness for each ticks location
    */
-  double PolarAxisTickRatioThickness, LastAxisTickRatioThickness, ArcTickRatioThickness;
+  double PolarAxisTickRatioThickness = 0.5, LastAxisTickRatioThickness = 0.5,
+         ArcTickRatioThickness = 0.5;
 
   ///@{
   /**
    * Visibility of radial axes and their titles
    */
-  vtkTypeBool RadialAxesVisibility;
-  vtkTypeBool RadialTitleVisibility;
+  bool RadialAxesVisibility = true;
+  bool RadialTitleVisibility = true;
   ///@}
 
   /**
    * Define the alignment of the title related to the radial axis. (BOTTOM or EXTERN)
    */
-  int RadialAxisTitleLocation;
+  int RadialAxisTitleLocation = VTK_TITLE_BOTTOM;
 
   /**
    * Define the alignment of the title related to the polar axis. (BOTTOM or EXTERN)
    */
-  int PolarAxisTitleLocation;
+  int PolarAxisTitleLocation = VTK_TITLE_BOTTOM;
 
   /**
    * Define the location of the exponent of the labels values, located on the polar axis.
    * it could be: LABEL, EXTERN, BOTTOM
    */
-  int ExponentLocation;
+  int ExponentLocation = VTK_EXPONENT_LABELS;
 
   /**
    * Visibility of polar arcs
    */
-  vtkTypeBool PolarArcsVisibility;
+  bool PolarArcsVisibility = true;
 
   /**
    * Visibility of the inner axes (overridden to 0 if RadialAxesVisibility is set to 0)
    */
-  vtkTypeBool DrawRadialGridlines;
+  bool DrawRadialGridlines = true;
 
   /**
    * Visibility of the inner arcs (overridden to 0 if PolarArcsVisibility is set to 0)
    */
-  vtkTypeBool DrawPolarArcsGridlines;
+  bool DrawPolarArcsGridlines = true;
 
   /**
    * Keep the arc major ticks vtkPoints instances
    */
-  vtkPoints* ArcMajorTickPts;
+  vtkNew<vtkPoints> ArcMajorTickPts;
 
   /**
    * Keep the arc minor ticks vtkPoints instances
    */
-  vtkPoints* ArcMinorTickPts;
+  vtkNew<vtkPoints> ArcMinorTickPts;
 
   ///@{
   /**
    * vtk object for arc Ticks
    */
-  vtkPolyData* ArcTickPolyData;
-  vtkPolyData* ArcMinorTickPolyData;
-  vtkPolyDataMapper* ArcTickPolyDataMapper;
-  vtkPolyDataMapper* ArcMinorTickPolyDataMapper;
-  vtkActor* ArcTickActor;
-  vtkActor* ArcMinorTickActor;
+  vtkNew<vtkPolyData> ArcTickPolyData;
+  vtkNew<vtkPolyData> ArcMinorTickPolyData;
+  vtkNew<vtkPolyDataMapper> ArcTickPolyDataMapper;
+  vtkNew<vtkPolyDataMapper> ArcMinorTickPolyDataMapper;
+  vtkNew<vtkActor> ArcTickActor;
+  vtkNew<vtkActor> ArcMinorTickActor;
   ///@}
 
   ///@{
   /**
    * Text properties of polar axis title and labels
    */
-  vtkTextProperty* PolarAxisTitleTextProperty;
-  vtkTextProperty* PolarAxisLabelTextProperty;
+  vtkSmartPointer<vtkTextProperty> PolarAxisTitleTextProperty;
+  vtkSmartPointer<vtkTextProperty> PolarAxisLabelTextProperty;
   ///@}
 
   /**
    * Text properties of last radial axis
    */
-  vtkTextProperty* LastRadialAxisTextProperty;
+  vtkSmartPointer<vtkTextProperty> LastRadialAxisTextProperty;
 
   /**
    * Text properties of secondary radial axes
    */
-  vtkTextProperty* SecondaryRadialAxesTextProperty;
+  vtkSmartPointer<vtkTextProperty> SecondaryRadialAxesTextProperty;
 
   /**
    * General properties of polar axis
    * Behavior may be override by polar axis ticks 's actor property.
    */
-  vtkProperty* PolarAxisProperty;
+  vtkSmartPointer<vtkProperty> PolarAxisProperty;
 
   /**
    * General properties of last radial axis
    */
-  vtkProperty* LastRadialAxisProperty;
+  vtkSmartPointer<vtkProperty> LastRadialAxisProperty;
 
   /**
    * General properties of radial axes
    */
-  vtkProperty* SecondaryRadialAxesProperty;
+  vtkSmartPointer<vtkProperty> SecondaryRadialAxesProperty;
 
   vtkTimeStamp BuildTime;
 
   /**
    * Title scale factor
    */
-  double TitleScale;
+  double TitleScale = -1.0;
 
   /**
    * Label scale factor
    */
-  double LabelScale;
+  double LabelScale = -1.0;
 
   /**
    * Text screen size
    */
-  double ScreenSize;
+  double ScreenSize = 10.0;
+
+  ///@{
+  /**
+   * Handles title (x,y), label (y) and exponent (y) available offsets.
+   */
+  double PolarTitleOffset[2] = { 20.0, 10.0 }, RadialTitleOffset[2] = { 20.0, 0.0 };
+  double PolarLabelOffset = 10.0, PolarExponentOffset = 5.0;
+  ///@}
 
 private:
+  static constexpr int VTK_MAXIMUM_NUMBER_OF_POLAR_AXES = 20;
+  static constexpr int VTK_MAXIMUM_NUMBER_OF_RADIAL_AXES = 50;
+  static constexpr double VTK_MINIMUM_POLAR_ARC_RESOLUTION_PER_DEG = 0.05;
+  static constexpr double VTK_MAXIMUM_POLAR_ARC_RESOLUTION_PER_DEG = 100.0;
+
   vtkPolarAxesActor(const vtkPolarAxesActor&) = delete;
   void operator=(const vtkPolarAxesActor&) = delete;
 };
 
+VTK_ABI_NAMESPACE_END
 #endif
