@@ -1460,6 +1460,16 @@ void vtkCamera::ShallowCopy(vtkCamera* source)
     this->ProjectionTransform->Register(this);
   }
 
+  if (this->ExplicitProjectionTransformMatrix != nullptr)
+  {
+    this->ExplicitProjectionTransformMatrix->Delete();
+  }
+  this->ExplicitProjectionTransformMatrix = source->ExplicitProjectionTransformMatrix;
+  if (this->ExplicitProjectionTransformMatrix != nullptr)
+  {
+    this->ExplicitProjectionTransformMatrix->Register(this);
+  }
+
   if (this->Transform != nullptr)
   {
     this->Transform->Delete();
@@ -1519,6 +1529,16 @@ void vtkCamera::ShallowCopy(vtkCamera* source)
   {
     this->ModelViewTransform->Register(this);
   }
+
+  if (this->ExplicitProjectionTransformMatrix != nullptr)
+  {
+    this->ExplicitProjectionTransformMatrix->Delete();
+  }
+  this->ExplicitProjectionTransformMatrix = source->ExplicitProjectionTransformMatrix;
+  if (this->ExplicitProjectionTransformMatrix != nullptr)
+  {
+    this->ExplicitProjectionTransformMatrix->Register(this);
+  }
 }
 
 //------------------------------------------------------------------------------
@@ -1571,6 +1591,24 @@ void vtkCamera::DeepCopy(vtkCamera* source)
     this->UserViewTransform->DeepCopy(source->UserViewTransform);
   }
 
+  if (source->ExplicitProjectionTransformMatrix == nullptr)
+  {
+    if (this->ExplicitProjectionTransformMatrix != nullptr)
+    {
+      this->ExplicitProjectionTransformMatrix->UnRegister(this);
+      this->ExplicitProjectionTransformMatrix = nullptr;
+    }
+  }
+  else
+  {
+    if (this->ExplicitProjectionTransformMatrix == nullptr)
+    {
+      this->ExplicitProjectionTransformMatrix =
+        static_cast<vtkMatrix4x4*>(source->ExplicitProjectionTransformMatrix->NewInstance());
+    }
+    this->ExplicitProjectionTransformMatrix->DeepCopy(source->ExplicitProjectionTransformMatrix);
+  }
+
   if (source->ViewTransform == nullptr)
   {
     if (this->ViewTransform != nullptr)
@@ -1606,6 +1644,23 @@ void vtkCamera::DeepCopy(vtkCamera* source)
     this->ProjectionTransform->DeepCopy(source->ProjectionTransform);
   }
 
+  if (source->ExplicitProjectionTransformMatrix == nullptr)
+  {
+    if (this->ExplicitProjectionTransformMatrix != nullptr)
+    {
+      this->ExplicitProjectionTransformMatrix->UnRegister(this);
+      this->ExplicitProjectionTransformMatrix = nullptr;
+    }
+  }
+  else
+  {
+    if (this->ExplicitProjectionTransformMatrix == nullptr)
+    {
+      this->ExplicitProjectionTransformMatrix = vtkMatrix4x4::New();
+    }
+    this->ExplicitProjectionTransformMatrix->DeepCopy(source->ExplicitProjectionTransformMatrix);
+  }
+
   if (source->Transform == nullptr)
   {
     if (this->Transform != nullptr)
@@ -1739,6 +1794,8 @@ void vtkCamera::PartialCopy(vtkCamera* source)
   while (i < 3)
   {
     this->FocalPoint[i] = source->FocalPoint[i];
+    this->FocalPointShift[i] = source->FocalPointShift[i];
+    this->NearPlaneShift[i] = source->NearPlaneShift[i];
     this->Position[i] = source->Position[i];
     this->ViewUp[i] = source->ViewUp[i];
     this->DirectionOfProjection[i] = source->DirectionOfProjection[i];
@@ -1762,13 +1819,20 @@ void vtkCamera::PartialCopy(vtkCamera* source)
   this->Distance = source->Distance;
   this->UseHorizontalViewAngle = source->UseHorizontalViewAngle;
   this->UseOffAxisProjection = source->UseOffAxisProjection;
+  this->UseExplicitProjectionTransformMatrix = source->UseExplicitProjectionTransformMatrix;
   this->OffAxisClippingAdjustment = source->OffAxisClippingAdjustment;
 
   this->FocalDisk = source->FocalDisk;
   this->FocalDistance = source->FocalDistance;
   this->EyeSeparation = source->EyeSeparation;
+  this->UseExplicitProjectionTransformMatrix = source->UseExplicitProjectionTransformMatrix;
 
   this->ViewingRaysMTime = source->ViewingRaysMTime;
+
+  this->ExplicitAspectRatio = source->ExplicitAspectRatio;
+  this->UseExplicitAspectRatio = source->UseExplicitAspectRatio;
+  this->NearPlaneScale = source->NearPlaneScale;
+  this->ShiftScaleThreshold = source->ShiftScaleThreshold;
 }
 
 //------------------------------------------------------------------------------
