@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkHardwarePicker.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkHardwarePicker.h"
 
 #include "vtkAbstractMapper3D.h"
@@ -44,6 +32,7 @@
 
 #include <limits>
 
+VTK_ABI_NAMESPACE_BEGIN
 namespace
 {
 constexpr double DEFAULT_VALUE = std::numeric_limits<double>::quiet_NaN();
@@ -588,3 +577,4 @@ void vtkHardwarePicker::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "PCoords: (" << this->PCoords[0] << ", " << this->PCoords[1] << ", "
      << this->PCoords[2] << ")" << endl;
 }
+VTK_ABI_NAMESPACE_END
