@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkRenderer.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkRenderer.h"
 
 #include "vtkAssemblyPath.h"
@@ -41,6 +29,7 @@
 
 #include <sstream>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkCxxSetObjectMacro(vtkRenderer, Information, vtkInformation);
 vtkCxxSetObjectMacro(vtkRenderer, Delegate, vtkRendererDelegate);
 vtkCxxSetObjectMacro(vtkRenderer, BackgroundTexture, vtkTexture);
@@ -1225,15 +1214,15 @@ void vtkRenderer::ResetCameraClippingRange(const double bounds[6])
   {
     this->ActiveCamera->GetViewPlaneNormal(vn);
     this->ActiveCamera->GetPosition(position);
-    this->ExpandBounds(expandedBounds, this->ActiveCamera->GetModelTransformMatrix());
   }
   else
   {
     this->ActiveCamera->GetEyePosition(position);
     this->ActiveCamera->GetEyePlaneNormal(vn);
-    this->ExpandBounds(expandedBounds, this->ActiveCamera->GetModelViewTransformMatrix());
   }
 
+  this->ExpandBounds(expandedBounds, this->ActiveCamera->GetModelTransformMatrix());
+
   a = -vn[0];
   b = -vn[1];
   c = -vn[2];
@@ -1264,6 +1253,12 @@ void vtkRenderer::ResetCameraClippingRange(const double bounds[6])
   {
     minGap = 0.1 * this->ActiveCamera->GetParallelScale();
   }
+  else if (this->ActiveCamera->GetUseOffAxisProjection())
+  {
+    double offAxisAdustment = this->ActiveCamera->GetOffAxisClippingAdjustment();
+    range[0] -= offAxisAdustment;
+    range[1] += offAxisAdustment;
+  }
   else
   {
     double angle = vtkMath::RadiansFromDegrees(this->ActiveCamera->GetViewAngle());
@@ -1334,7 +1329,7 @@ void vtkRenderer::ResetCameraClippingRange(
 
 // Automatically set up the camera based on the visible actors.
 // Use a screen space bounding box to zoom closer to the data.
-void vtkRenderer::ResetCameraScreenSpace()
+void vtkRenderer::ResetCameraScreenSpace(const double offsetRatio)
 {
   double allBounds[6];
 
@@ -1346,7 +1341,7 @@ void vtkRenderer::ResetCameraScreenSpace()
   }
   else
   {
-    this->ResetCameraScreenSpace(allBounds);
+    this->ResetCameraScreenSpace(allBounds, offsetRatio);
   }
 
   // Here to let parallel/distributed compositing intercept
@@ -1355,8 +1350,8 @@ void vtkRenderer::ResetCameraScreenSpace()
 }
 
 // Alternative version of ResetCameraScreenSpace(bounds[6]);
-void vtkRenderer::ResetCameraScreenSpace(
-  double xmin, double xmax, double ymin, double ymax, double zmin, double zmax)
+void vtkRenderer::ResetCameraScreenSpace(double xmin, double xmax, double ymin, double ymax,
+  double zmin, double zmax, const double offsetRatio)
 {
   double bounds[6];
 
@@ -1367,11 +1362,11 @@ void vtkRenderer::ResetCameraScreenSpace(
   bounds[4] = zmin;
   bounds[5] = zmax;
 
-  this->ResetCameraScreenSpace(bounds);
+  this->ResetCameraScreenSpace(bounds, offsetRatio);
 }
 
 // Use a screen space bounding box to zoom closer to the data.
-void vtkRenderer::ResetCameraScreenSpace(const double bounds[6])
+void vtkRenderer::ResetCameraScreenSpace(const double bounds[6], const double offsetRatio)
 {
   // Make sure all bounds are visible to project into screen space
   this->ResetCamera(bounds);
@@ -1438,8 +1433,7 @@ void vtkRenderer::ResetCameraScreenSpace(const double bounds[6])
   // Now the focal point is at the center of the box
 
   const vtkRecti box(xmin, ymin, xmax - xmin, ymax - ymin);
-  // We let a 10% offset around the zoomed data
-  this->ZoomToBoxUsingViewAngle(box, 0.9);
+  this->ZoomToBoxUsingViewAngle(box, offsetRatio);
 }
 
 // Display to world using vtkVector3d
@@ -1456,7 +1450,7 @@ vtkVector3d vtkRenderer::DisplayToWorld(const vtkVector3d& display)
   return vtkVector3d(world4.GetData());
 }
 
-void vtkRenderer::ZoomToBoxUsingViewAngle(const vtkRecti& box, const double offsetRatio)
+void vtkRenderer::ZoomToBoxUsingViewAngle(const vtkRecti& box, double offsetRatio)
 {
   const int* size = this->GetSize();
   double zf1 = size[0] / static_cast<double>(box.GetWidth());
@@ -1485,19 +1479,37 @@ void vtkRenderer::SetRenderWindow(vtkRenderWindow* renwin)
 // Given a pixel location, return the Z value
 double vtkRenderer::GetZ(int x, int y)
 {
-  float* zPtr;
-  double z;
+  double z = 1.0;
 
-  zPtr = this->RenderWindow->GetZbufferData(x, y, x, y);
-  if (zPtr)
+  // use a hardware selector because calling this->RenderWindow->
+  // GetZbufferData(int,int,int,int) directly from here always
+  // results in a z-buffer value of 1.0, meaning it is using a
+  // cleared depth buffer.
   {
-    z = *zPtr;
-    delete[] zPtr;
-  }
-  else
-  {
-    z = 1.0;
+    vtkNew<vtkHardwareSelector> hsel;
+    hsel->SetActorPassOnly(true);
+    hsel->SetCaptureZValues(true);
+    hsel->SetRenderer(this);
+    hsel->SetArea(x, y, x, y);
+    vtkSmartPointer<vtkSelection> sel;
+    sel.TakeReference(hsel->Select());
+
+    // find the closest z-buffer value
+    if (sel && sel->GetNode(0))
+    {
+      double closestDepth = 1.0;
+      unsigned int numPicked = sel->GetNumberOfNodes();
+      for (unsigned int pIdx = 0; pIdx < numPicked; pIdx++)
+      {
+        vtkSelectionNode* selnode = sel->GetNode(pIdx);
+        double adepth = selnode->GetProperties()->Get(vtkSelectionNode::ZBUFFER_VALUE());
+        if (adepth < closestDepth)
+          closestDepth = adepth;
+      }
+      z = closestDepth;
+    }
   }
+
   return z;
 }
 
@@ -1939,6 +1951,7 @@ vtkAssemblyPath* vtkRenderer::PickProp(double selectionX1, double selectionY1, d
     // store the list of picked props
     vtkProp* closestProp = nullptr;
     double closestDepth = 2.0;
+    unsigned int closestIdx = 0;
     this->PickResultProps = vtkPropCollection::New();
     unsigned int numPicked = sel->GetNumberOfNodes();
     for (unsigned int pIdx = 0; pIdx < numPicked; pIdx++)
@@ -1954,6 +1967,7 @@ vtkAssemblyPath* vtkRenderer::PickProp(double selectionX1, double selectionY1, d
         {
           closestProp = aProp;
           closestDepth = adepth;
+          closestIdx = pIdx;
         }
       }
     }
@@ -1961,6 +1975,16 @@ vtkAssemblyPath* vtkRenderer::PickProp(double selectionX1, double selectionY1, d
     {
       return nullptr;
     }
+    if (closestIdx != 0)
+    {
+      // reorder the selection so that the closest / picked prop is the first selection node
+      std::string nodeNameZero = sel->GetNodeNameAtIndex(0);
+      std::string nodeNameClosest = sel->GetNodeNameAtIndex(closestIdx);
+
+      vtkSmartPointer<vtkSelectionNode> temp = sel->GetNode(nodeNameZero);
+      sel->SetNode(nodeNameZero, sel->GetNode(nodeNameClosest));
+      sel->SetNode(nodeNameClosest, temp);
+    }
     closestProp->InitPathTraversal();
     this->PickedProp = closestProp->GetNextPath();
     this->PickedProp->Register(this);
@@ -2035,7 +2059,7 @@ void vtkRenderer::ExpandBounds(double bounds[6], vtkMatrix4x4* matrix)
   bounds[5] = max[2];
 }
 
-int vtkRenderer::Transparent()
+vtkTypeBool vtkRenderer::Transparent()
 {
   return this->PreserveColorBuffer;
 }
@@ -2067,7 +2091,8 @@ double vtkRenderer::GetTiledAspectRatio()
 
 int vtkRenderer::CaptureGL2PSSpecialProp(vtkProp* prop)
 {
-  if (this->GL2PSSpecialPropCollection && !this->GL2PSSpecialPropCollection->IsItemPresent(prop))
+  if (this->GL2PSSpecialPropCollection &&
+    this->GL2PSSpecialPropCollection->IndexOfFirstOccurence(prop) < 0)
   {
     this->GL2PSSpecialPropCollection->AddItem(prop);
     return 1;
@@ -2121,3 +2146,4 @@ const std::array<double, 16>& vtkRenderer::GetProjectionTransformationMatrix()
   }
   return this->ProjectionTransformationMatrix;
 }
+VTK_ABI_NAMESPACE_END
