@@ -1,17 +1,5 @@
-/*=========================================================================
-
-Program:   Visualization Toolkit
-Module:    vtkCocoaRenderWindow.mm
-
-Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-All rights reserved.
-See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-This software is distributed WITHOUT ANY WARRANTY; without even
-the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtk_glew.h"
 
@@ -31,7 +19,9 @@ PURPOSE.  See the above copyright notice for more information.
 
 #import <sstream>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkCocoaRenderWindow);
+VTK_ABI_NAMESPACE_END
 
 //----------------------------------------------------------------------------
 // This is a private class and an implementation detail, do not use it.
@@ -199,6 +189,7 @@ vtkStandardNewMacro(vtkCocoaRenderWindow);
 
 @end
 
+VTK_ABI_NAMESPACE_BEGIN
 //----------------------------------------------------------------------------
 vtkCocoaRenderWindow::vtkCocoaRenderWindow()
 {
@@ -690,7 +681,8 @@ void vtkCocoaRenderWindow::CreateAWindow()
   // been specified already.  This is the case for a 'pure vtk application'.
   // If you are using vtk in a 'regular Mac application' you should call
   // SetRootWindow() and SetWindowId() so that a window is not created here.
-  if (!this->GetRootWindow() && !this->GetWindowId() && !this->GetParentId())
+  if (!this->GetRootWindow() && !this->GetWindowId() && !this->GetParentId() &&
+    this->GetConnectContextToNSView())
   {
     // Ordinarily, only .app bundles get proper mouse and keyboard interaction,
     // but here we change the 'activation policy' to behave as if we were a
@@ -736,12 +728,12 @@ void vtkCocoaRenderWindow::CreateAWindow()
     }
     else
     {
-      if ((this->Size[0] + this->Size[1]) == 0)
+      if ((this->Size[0] == 0) && (this->Size[1] == 0))
       {
         this->Size[0] = 300;
         this->Size[1] = 300;
       }
-      if ((this->Position[0] + this->Position[1]) == 0)
+      if ((this->Position[0] == 0) && (this->Position[1] == 0))
       {
         this->Position[0] = 50;
         this->Position[1] = 50;
@@ -791,7 +783,7 @@ void vtkCocoaRenderWindow::CreateAWindow()
   }
 
   // create an NSView if one has not been specified
-  if (!this->GetWindowId())
+  if (!this->GetWindowId() && this->GetConnectContextToNSView())
   {
     // For NSViews that display OpenGL, the OS defaults to drawing magnified,
     // not in high resolution. There is a tradeoff here between better visual
@@ -886,6 +878,7 @@ void vtkCocoaRenderWindow::CreateAWindow()
       this->SetWindowId(glView);
       this->ViewCreated = 1;
       [glView setVTKRenderWindow:this];
+      [window makeFirstResponder:glView];
 
 #if !VTK_OBJC_IS_ARC
       [glView release];
@@ -1093,6 +1086,37 @@ void vtkCocoaRenderWindow::Initialize()
   }
 }
 
+//----------------------------------------------------------------------------
+void vtkCocoaRenderWindow::Render()
+{
+  // Is this the first Render?
+  bool neverRendered = this->NeverRendered;
+
+  // Do the superclass stuff.
+  this->vtkOpenGLRenderWindow::Render();
+
+  // Under certain circumstances, we want the Render() to be followed by a
+  // round of event handling.  For simple VTK programs that don't implement
+  // their own runloop, this is needed for macOS to draw the window and its
+  // decorations on-screen.  The runloop is triggered if and only if:
+  // (1) we created the NSWindow ourselves (it wasn't given to us),
+  // (2) this is the first time the window is rendered,
+  // (3) we are not being called from within another Render() call,
+  // (4) we have passed through Initialize(), and
+  // (5) the window is mapped to screen.
+  if (this->WindowCreated && neverRendered && !this->InRender && this->OnScreenInitialized &&
+    this->Mapped)
+  {
+    // Setting InRender=1 guards against triggering an additional render
+    // within the current render.  The runloop is run only until "distantPast"
+    // to ensure that it stops and returns after pending events are processed.
+    this->InRender = 1;
+    NSRunLoop* mainRunLoop = [NSRunLoop mainRunLoop];
+    [mainRunLoop runUntilDate:[NSDate distantPast]];
+    this->InRender = 0;
+  }
+}
+
 //----------------------------------------------------------------------------
 int* vtkCocoaRenderWindow::GetSize()
 {
@@ -1639,3 +1663,5 @@ void vtkCocoaRenderWindow::SetConnectContextToNSView(bool connect)
 {
   this->ConnectContextToNSView = connect;
 }
+
+VTK_ABI_NAMESPACE_END
