@@ -1,16 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkOpenGLGlyph3DHelper.h"
 
 #include "vtkOpenGLHelper.h"
@@ -51,6 +40,7 @@
 #include <numeric>
 
 //------------------------------------------------------------------------------
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkOpenGLGlyph3DHelper);
 
 //------------------------------------------------------------------------------
@@ -64,7 +54,7 @@ vtkOpenGLGlyph3DHelper::vtkOpenGLGlyph3DHelper()
   // GCMCMatrix could be modified accordingly but Shift and Scale coefficients are
   // computed with the glyph source and not the point cloud.
   // Disabling it for now.
-  this->SetVBOShiftScaleMethod(vtkOpenGLVertexBufferObject::DISABLE_SHIFT_SCALE);
+  this->SetVBOShiftScaleMethod(ShiftScaleMethodType::DISABLE_SHIFT_SCALE);
 }
 
 //------------------------------------------------------------------------------
@@ -88,6 +78,17 @@ void vtkOpenGLGlyph3DHelper::GetShaderTemplate(
   shaders[vtkShader::Vertex]->SetSource(vtkGlyph3DVS);
 }
 
+//------------------------------------------------------------------------------
+void vtkOpenGLGlyph3DHelper::ReplaceShaderValues(
+  std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
+{
+  this->Superclass::ReplaceShaderValues(shaders, ren, actor);
+#ifdef GL_ES_VERSION_3_0
+  this->ReplaceShaderPointSize(shaders, ren, actor);
+#endif
+}
+
+//------------------------------------------------------------------------------
 void vtkOpenGLGlyph3DHelper::ReplaceShaderPositionVC(
   std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
 {
@@ -111,6 +112,7 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderPositionVC(
   this->Superclass::ReplaceShaderPositionVC(shaders, ren, actor);
 }
 
+//------------------------------------------------------------------------------
 void vtkOpenGLGlyph3DHelper::ReplaceShaderColor(
   std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
 {
@@ -177,6 +179,7 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderColor(
   this->Superclass::ReplaceShaderColor(shaders, ren, actor);
 }
 
+//------------------------------------------------------------------------------
 void vtkOpenGLGlyph3DHelper::ReplaceShaderNormal(
   std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer* ren, vtkActor* actor)
 {
@@ -239,6 +242,7 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderClip(
   this->Superclass::ReplaceShaderClip(shaders, ren, actor);
 }
 
+//------------------------------------------------------------------------------
 void vtkOpenGLGlyph3DHelper::ReplaceShaderPicking(
   std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer*, vtkActor*)
 {
@@ -253,6 +257,18 @@ void vtkOpenGLGlyph3DHelper::ReplaceShaderPicking(
   shaders[vtkShader::Fragment]->SetSource(FSSource);
 }
 
+//------------------------------------------------------------------------------
+void vtkOpenGLGlyph3DHelper::ReplaceShaderPointSize(
+  std::map<vtkShader::Type, vtkShader*> shaders, vtkRenderer*, vtkActor*)
+{
+  std::string VSSource = shaders[vtkShader::Vertex]->GetSource();
+  // Point size
+  vtkShaderProgram::Substitute(VSSource, "//VTK::PointSizeGLES30::Dec", "uniform float pointSize;");
+  vtkShaderProgram::Substitute(
+    VSSource, "//VTK::PointSizeGLES30::Impl", "gl_PointSize = pointSize;");
+  shaders[vtkShader::Vertex]->SetSource(VSSource);
+}
+
 void vtkOpenGLGlyph3DHelper::GlyphRender(vtkRenderer* ren, vtkActor* actor, vtkIdType numPts,
   std::vector<unsigned char>& colors, std::vector<float>& matrices,
   std::vector<float>& normalMatrices, std::vector<vtkIdType>& pickIds, vtkMTimeType pointMTime,
@@ -297,7 +313,11 @@ void vtkOpenGLGlyph3DHelper::GlyphRender(vtkRenderer* ren, vtkActor* actor, vtkI
 
   if (selecting_points)
   {
+#ifndef GL_ES_VERSION_3_0
     ostate->vtkglPointSize(6.0);
+#else
+    (void)ostate;
+#endif
     representation = GL_POINTS;
   }
 
@@ -346,7 +366,12 @@ void vtkOpenGLGlyph3DHelper::GlyphRender(vtkRenderer* ren, vtkActor* actor, vtkI
           }
           program->SetUniform3f("mapperIndex", selector->GetPropColorValue());
         }
-
+#ifdef GL_ES_VERSION_3_0
+        if (selecting_points)
+        {
+          program->SetUniformf("pointSize", 6.0);
+        }
+#endif
         glDrawRangeElements(mode, 0, static_cast<GLuint>(numVerts - 1),
           static_cast<GLsizei>(this->Primitives[i].IBO->IndexCount), GL_UNSIGNED_INT, nullptr);
       }
@@ -361,12 +386,6 @@ void vtkOpenGLGlyph3DHelper::SetMapperShaderParameters(
   vtkOpenGLHelper& cellBO, vtkRenderer* ren, vtkActor* actor)
 {
   this->Superclass::SetMapperShaderParameters(cellBO, ren, actor);
-
-  vtkHardwareSelector* selector = ren->GetSelector();
-  if (selector)
-  {
-    cellBO.Program->SetUniform3f("mapperIndex", selector->GetPropColorValue());
-  }
 }
 
 void vtkOpenGLGlyph3DHelper::GlyphRenderInstances(vtkRenderer* ren, vtkActor* actor,
@@ -617,3 +636,4 @@ void vtkOpenGLGlyph3DHelper::SetLODColoring(bool val)
 {
   this->InstanceCulling->SetColorLOD(val);
 }
+VTK_ABI_NAMESPACE_END
