@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkOpenGLImageSliceMapper.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkOpenGLImageSliceMapper.h"
 
 #include "vtk_glew.h"
@@ -26,6 +14,7 @@
 #include "vtkLookupTable.h"
 #include "vtkMapper.h"
 #include "vtkMath.h"
+#include "vtkMatrix3x3.h"
 #include "vtkMatrix4x4.h"
 #include "vtkObjectFactory.h"
 #include "vtkOpenGLCamera.h"
@@ -52,6 +41,7 @@
 
 #include "vtkOpenGLError.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkOpenGLImageSliceMapper);
 
 //------------------------------------------------------------------------------
@@ -262,9 +252,9 @@ void vtkOpenGLImageSliceMapper::RenderTexturedPolygon(
     input->GetMTime() > loadTime || orientationChanged || sliceChanged || recursive)
   {
     // get the data to load as a texture
-    int xsize;
-    int ysize;
-    int bytesPerPixel;
+    int xsize = this->TextureSize[0];
+    int ysize = this->TextureSize[1];
+    int bytesPerPixel = this->TextureBytesPerPixel;
 
     // whether to try to use the input data directly as the texture
     bool reuseData = true;
@@ -394,17 +384,19 @@ void vtkOpenGLImageSliceMapper::RenderPolygon(
     vtkImageSliceMapper::GetDimensionIndices(this->Orientation, xdim, ydim);
     double* origin = this->DataOrigin;
     double* spacing = this->DataSpacing;
-    double xshift = origin[xdim] - (0.5 - extent[2 * xdim]) * spacing[xdim];
+    double xshift = -(0.5 - extent[2 * xdim]) * spacing[xdim];
     double xscale = this->TextureSize[xdim] * spacing[xdim];
-    double yshift = origin[ydim] - (0.5 - extent[2 * ydim]) * spacing[ydim];
+    double yshift = -(0.5 - extent[2 * ydim]) * spacing[ydim];
     double yscale = this->TextureSize[ydim] * spacing[ydim];
     vtkIdType ncoords = points->GetNumberOfPoints();
     double coord[3];
     double tcoord[2];
+    double invDirection[9];
 
     polyPoints->DeepCopy(points);
     if (textured)
     {
+      vtkMatrix3x3::Invert(this->DataDirection, invDirection);
       polyTCoords->SetNumberOfTuples(ncoords);
     }
 
@@ -412,7 +404,10 @@ void vtkOpenGLImageSliceMapper::RenderPolygon(
     {
       if (textured)
       {
+        // convert points from 3D model coords to 2D texture coords
         points->GetPoint(i, coord);
+        vtkMath::Subtract(coord, origin, coord);
+        vtkMatrix3x3::MultiplyPoint(invDirection, coord, coord);
         tcoord[0] = (coord[0] - xshift) / xscale;
         tcoord[1] = (coord[1] - yshift) / yscale;
         polyTCoords->SetTuple(i, tcoord);
@@ -612,6 +607,7 @@ void vtkOpenGLImageSliceMapper::Render(vtkRenderer* ren, vtkImageSlice* prop)
   // update the input information
   vtkImageData* input = this->GetInput();
   input->GetSpacing(this->DataSpacing);
+  vtkMatrix3x3::DeepCopy(this->DataDirection, input->GetDirectionMatrix());
   input->GetOrigin(this->DataOrigin);
   vtkInformation* inputInfo = this->GetInputInformation(0, 0);
   inputInfo->Get(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(), this->DataWholeExtent);
@@ -708,3 +704,4 @@ void vtkOpenGLImageSliceMapper::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
 }
+VTK_ABI_NAMESPACE_END
