@@ -1,16 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkOpenGLPolyDataMapper
  * @brief   PolyDataMapper using OpenGL to render.
@@ -28,11 +17,13 @@
 #include "vtkRenderingOpenGL2Module.h" // For export macro
 #include "vtkShader.h"                 // for methods
 #include "vtkStateStorage.h"           // used for ivars
+#include "vtkWrappingHints.h"          // For VTK_MARSHALAUTO
 
 #include <map>    // for map
 #include <tuple>  // for tuple
 #include <vector> // for vector
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkCellArray;
 class vtkGenericOpenGLResourceFreeCallback;
 class vtkMatrix4x4;
@@ -49,7 +40,7 @@ class vtkTextureObject;
 class vtkTransform;
 class vtkOpenGLShaderProperty;
 
-class VTKRENDERINGOPENGL2_EXPORT vtkOpenGLPolyDataMapper : public vtkPolyDataMapper
+class VTKRENDERINGOPENGL2_EXPORT VTK_MARSHALAUTO vtkOpenGLPolyDataMapper : public vtkPolyDataMapper
 {
 public:
   static vtkOpenGLPolyDataMapper* New();
@@ -61,6 +52,13 @@ public:
    */
   void RenderPiece(vtkRenderer* ren, vtkActor* act) override;
 
+  /**
+   * Unique hash based on availability of scalars, normals, tcoords, lookup tables
+   * and related attributes that distinguish the rendering requirements of different
+   * polydata.
+   */
+  MapperHashType GenerateHash(vtkPolyData* polydata) override;
+
   ///@{
   /**
    * Implemented by sub classes. Actual rendering is done here.
@@ -77,7 +75,9 @@ public:
    */
   void ReleaseGraphicsResources(vtkWindow*) override;
 
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_IS_INTERNAL)
   vtkGetMacro(PopulateSelectionSettings, int);
+  VTK_MARSHALEXCLUDE(VTK_MARSHAL_EXCLUDE_REASON_IS_INTERNAL)
   void SetPopulateSelectionSettings(int v) { this->PopulateSelectionSettings = v; }
 
   /**
@@ -111,6 +111,7 @@ public:
    * If this class should override the process id using a data-array,
    * set this variable to the name of the array to use. It must be a
    * point-array.
+   * The array's DataType *MUST* be VTK_UNSIGNED_INT.
    */
   vtkSetStringMacro(ProcessIdArrayName);
   vtkGetStringMacro(ProcessIdArrayName);
@@ -124,7 +125,8 @@ public:
    * to provide a cell array that can be used to render in the composite id in
    * selection passes. Set to NULL (default) to not override the composite id
    * color set by vtkCompositePainter if any.
-   * The array *MUST* be a cell array and of type vtkUnsignedIntArray.
+   * The array *MUST* be a cell array.
+   * The array's DataType *MUST* be VTK_UNSIGNED_INT.
    */
   vtkSetStringMacro(CompositeIdArrayName);
   vtkGetStringMacro(CompositeIdArrayName);
@@ -141,26 +143,7 @@ public:
   /**\brief A convenience method for enabling/disabling
    *   the VBO's shift+scale transform.
    */
-  virtual void SetVBOShiftScaleMethod(int m);
-  virtual int GetVBOShiftScaleMethod() { return this->ShiftScaleMethod; }
-
-  /**\brief Pause per-render updates to VBO shift+scale parameters.
-   *
-   * For large datasets, re-uploading the VBO during user interaction
-   * can cause stutters in the framerate. Interactors can use this
-   * method to force UpdateCameraShiftScale to return immediately
-   * (without changes) while users are zooming/rotating/etc. and then
-   * re-enable shift-scale just before a still render.
-   *
-   * This setting has no effect unless the shift-scale method is set
-   * to NEAR_PLANE_SHIFT_SCALE or FOCAL_POINT_SHIFT_SCALE.
-   *
-   * Changing this setting does **not** mark the mapper as modified as
-   * that would force a VBO upload – defeating its own purpose.
-   */
-  virtual void SetPauseShiftScale(bool pauseShiftScale) { this->PauseShiftScale = pauseShiftScale; }
-  vtkGetMacro(PauseShiftScale, bool);
-  vtkBooleanMacro(PauseShiftScale, bool);
+  void SetVBOShiftScaleMethod(int method) override;
 
   /**
    * Allow the shader code to set the point size (with gl_PointSize variable)
@@ -450,8 +433,6 @@ protected:
   vtkMatrix3x3* TempMatrix3;
   vtkNew<vtkTransform> VBOInverseTransform;
   vtkNew<vtkMatrix4x4> VBOShiftScale;
-  int ShiftScaleMethod; // for points
-  bool PauseShiftScale;
   bool UseProgramPointSize;
 
   // if set to true, tcoords will be passed to the
@@ -512,7 +493,7 @@ protected:
   // used to occasionally invoke timers
   unsigned int TimerQueryCounter;
 
-  // stores the mapping from vtk cells to gl_PrimitiveId
+  // stores the mapping from OpenGL primitives IDs (gl_PrimitiveId) to VTK cells IDs
   vtkNew<vtkOpenGLCellToVTKCellMap> CellCellMap;
 
   // compute and set the maximum point and cell ID used in selection
@@ -525,9 +506,13 @@ protected:
 
   vtkNew<vtkCellArray> SelectionArrays[4];
 
+  vtkMTimeType EnvironmentTextureTime = 0;
+  vtkTexture* EnvironmentTexture = nullptr;
+
 private:
   vtkOpenGLPolyDataMapper(const vtkOpenGLPolyDataMapper&) = delete;
   void operator=(const vtkOpenGLPolyDataMapper&) = delete;
 };
 
+VTK_ABI_NAMESPACE_END
 #endif
