@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkOpenGLRenderWindow.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkOpenGLRenderWindow.h"
 #include "vtk_glew.h"
 
@@ -48,10 +36,15 @@
 #include "vtkTimerLog.h"
 #include "vtkUnsignedCharArray.h"
 
+#include "vtksys/SystemTools.hxx"
+
 #include "BlueNoiseTexture64x64.h"
 #include "vtkTextureObjectVS.h" // a pass through shader
 
+#include <cstdlib>
 #include <sstream>
+#include <string>
+#include <type_traits>
 using std::ostringstream;
 
 #include <cassert>
@@ -59,13 +52,18 @@ using std::ostringstream;
 // Initialize static member that controls global maximum number of multisamples
 // (off by default on Apple because it causes problems on some Mac models).
 #if defined(__APPLE__)
+VTK_ABI_NAMESPACE_BEGIN
 static int vtkOpenGLRenderWindowGlobalMaximumNumberOfMultiSamples = 0;
+VTK_ABI_NAMESPACE_END
 #else
+VTK_ABI_NAMESPACE_BEGIN
 static int vtkOpenGLRenderWindowGlobalMaximumNumberOfMultiSamples = 8;
+VTK_ABI_NAMESPACE_END
 #endif
 
 // Some linux drivers have issues reading a multisampled texture,
 // so we check the driver's "Renderer" against this list of strings.
+VTK_ABI_NAMESPACE_BEGIN
 struct vtkOpenGLRenderWindowDriverInfo
 {
   const char* Vendor;
@@ -83,11 +81,10 @@ static const vtkOpenGLRenderWindowDriverInfo vtkOpenGLRenderWindowMSAATextureBug
   { "X.Org", "", "AMD" },
 };
 
-const char* defaultWindowName = "Visualization Toolkit - OpenGL";
+static const char* defaultWindowName = "Visualization Toolkit - OpenGL";
 
-const char* ResolveShader =
-  R"***(
-  //VTK::System::Dec
+static const char* ResolveShader =
+  R"***(//VTK::System::Dec
   in vec2 texCoord;
   uniform sampler2DMS tex;
   uniform int samplecount;
@@ -104,10 +101,10 @@ const char* ResolveShader =
 
     for (int i = 0; i < samplecount; i++)
     {
-      vec4 sample = texelFetch(tex, itexcoords, i);
+      vec4 sampleValue = texelFetch(tex, itexcoords, i);
       // apply gamma correction and sum
-      accumulate += pow(sample.rgb, vec3(gamma));
-      alpha += sample.a;
+      accumulate += pow(sampleValue.rgb, vec3(gamma));
+      alpha += sampleValue.a;
     }
 
     // divide and reverse gamma correction
@@ -116,9 +113,8 @@ const char* ResolveShader =
   }
   )***";
 
-const char* DepthBlitShader =
-  R"***(
-  //VTK::System::Dec
+static const char* DepthBlitShader =
+  R"***(//VTK::System::Dec
   in vec2 texCoord;
   uniform sampler2D tex;
   uniform vec2 texLL;
@@ -131,9 +127,39 @@ const char* DepthBlitShader =
   }
   )***";
 
-const char* FlipShader =
-  R"***(
-  //VTK::System::Dec
+static const char* DepthReadShader =
+  R"***(//VTK::System::Dec
+  in vec2 texCoord;
+  uniform sampler2D tex;
+  //VTK::Output::Dec
+
+  void main()
+  {
+    // define the number of bits in the depth attachment as an integer `depthSize`.
+    //VTK::DepthSize::Impl
+    float maxNBitUintValue = float(1 << depthSize) - 1.0f;
+    float depth = texture(tex, texCoord).r;
+    // scale up to max n-bit unsigned integer.
+    float z = floor(depth * maxNBitUintValue);
+    // extract 8-bit unsigned integers from the n-bit unsigned integer.
+    // assume maxNBits == 32, we'll skip unnecessary 8-bit values during reconstruction.
+    // gl_FragData[0] = vec4(z & 0xFF, (z >> 8) & 0xFF, (z >> 16) & 0xFF, (z >> 24) & 0xFF);
+    float r = mod(z, 256.0f);
+    z -= r;
+    z /= 256.0f;
+    float g = mod(z, 256.0f);
+    z -= g;
+    z /= 256.0f;
+    float b = mod(z, 256.0f);
+    z -= b;
+    z /= 256.0f;
+    float a = mod(z, 256.0f);
+    gl_FragData[0] = vec4(r / 255.0f, g / 255.0f, b / 255.0f, a / 255.0f);
+  }
+  )***";
+
+static const char* FlipShader =
+  R"***(//VTK::System::Dec
   in vec2 texCoord;
   uniform sampler2D tex;
   //VTK::Output::Dec
@@ -144,6 +170,198 @@ const char* FlipShader =
   }
   )***";
 
+#ifdef GL_ES_VERSION_3_0
+namespace
+{
+// helpers to go from GL_ defines to vtkType*
+template <GLint GLType>
+struct GLTypeToVTKHelper
+{
+};
+
+template <>
+struct GLTypeToVTKHelper<GL_UNSIGNED_BYTE>
+{
+  using vtk_type = vtkTypeUInt8;
+};
+
+template <>
+struct GLTypeToVTKHelper<GL_UNSIGNED_INT>
+{
+  using vtk_type = vtkTypeUInt32;
+};
+
+template <>
+struct GLTypeToVTKHelper<GL_FLOAT>
+{
+  using vtk_type = vtkTypeFloat32;
+};
+
+template <>
+struct GLTypeToVTKHelper<GL_INT>
+{
+  using vtk_type = vtkTypeInt32;
+};
+
+int GetNumberOfColorComponents(const GLint& glFormat)
+{
+  switch (glFormat)
+  {
+    case GL_RGB:
+      return 3;
+    case GL_RGBA:
+    case GL_RGBA_INTEGER:
+      return 4;
+    default:
+      return 0;
+  }
+}
+
+template <typename T1, typename T2>
+bool RGBToRGBA(const T1* const rgb, T2* rgba, const std::size_t& n)
+{
+  constexpr bool is_T1_uint_or_int = std::is_integral<T1>::value || std::is_unsigned<T1>::value;
+  constexpr bool is_T2_uint_or_int = std::is_integral<T2>::value || std::is_unsigned<T2>::value;
+  constexpr bool is_T1_float32 = std::is_floating_point<T1>::value;
+  constexpr bool is_T2_float32 = std::is_floating_point<T2>::value;
+  if ((is_T1_uint_or_int && is_T2_uint_or_int) || (is_T1_float32 && is_T2_float32))
+  {
+    std::size_t idx = 0;
+    while (idx < n)
+    {
+      (*rgba++) = rgb[idx++];
+      (*rgba++) = rgb[idx++];
+      (*rgba++) = rgb[idx++];
+      (*rgba++) = is_T2_uint_or_int ? 255 : 1.0f;
+    }
+  }
+  else if (is_T1_uint_or_int && is_T2_float32)
+  {
+    std::size_t idx = 0;
+    while (idx < n)
+    {
+      (*rgba++) = rgb[idx++] / 255.0f;
+      (*rgba++) = rgb[idx++] / 255.0f;
+      (*rgba++) = rgb[idx++] / 255.0f;
+      (*rgba++) = 1.0f;
+      idx++;
+    }
+  }
+  else if (is_T1_float32 && is_T2_uint_or_int)
+  {
+    std::size_t idx = 0;
+    while (idx < n)
+    {
+      (*rgba++) = static_cast<T2>(rgb[idx++] * 255.0f);
+      (*rgba++) = static_cast<T2>(rgb[idx++] * 255.0f);
+      (*rgba++) = static_cast<T2>(rgb[idx++] * 255.0f);
+      (*rgba++) = 1.0f;
+      idx++;
+    }
+  }
+  else
+  {
+    return false;
+  }
+  return true;
+}
+
+template <typename T1, typename T2>
+bool RGBAToRGB(const T1* const rgba, T2* rgb, const std::size_t& n)
+{
+  constexpr bool is_T1_uint_or_int = std::is_integral<T1>::value || std::is_unsigned<T1>::value;
+  constexpr bool is_T2_uint_or_int = std::is_integral<T2>::value || std::is_unsigned<T2>::value;
+  constexpr bool is_T1_float32 = std::is_floating_point<T1>::value;
+  constexpr bool is_T2_float32 = std::is_floating_point<T2>::value;
+  if ((is_T1_uint_or_int && is_T2_uint_or_int) || (is_T1_float32 && is_T2_float32))
+  {
+    std::size_t idx = 0;
+    while (idx < n)
+    {
+      (*rgb++) = rgba[idx++];
+      (*rgb++) = rgba[idx++];
+      (*rgb++) = rgba[idx++];
+      idx++;
+    }
+  }
+  else if (is_T1_uint_or_int && is_T2_float32)
+  {
+    std::size_t idx = 0;
+    while (idx < n)
+    {
+      (*rgb++) = rgba[idx++] / 255.0f;
+      (*rgb++) = rgba[idx++] / 255.0f;
+      (*rgb++) = rgba[idx++] / 255.0f;
+      idx++;
+    }
+  }
+  else if (is_T1_float32 && is_T2_uint_or_int)
+  {
+    std::size_t idx = 0;
+    while (idx < n)
+    {
+      (*rgb++) = static_cast<T2>(rgba[idx++] * 255.0f);
+      (*rgb++) = static_cast<T2>(rgba[idx++] * 255.0f);
+      (*rgb++) = static_cast<T2>(rgba[idx++] * 255.0f);
+      idx++;
+    }
+  }
+  else
+  {
+    return false;
+  }
+  return true;
+}
+
+/**
+ * Read pixels with parameters <sourceFormat, SourceGLType> and cast into <destFormat,
+ * DestinationGLType> Acceptable glformats: GL_RGB, GL_RGBA, GL_RGBA_INTEGER Acceptable gltypes:
+ * GL_UNSIGNED_BYTE, GL_UNSIGNED_INT, GL_INT, GL_FLOAT
+ */
+template <GLint SourceGLType, GLint DestinationGLType>
+bool ConvertGLColor(GLint sourceFormat, const vtkRecti& rect, void* data, GLint destFormat)
+{
+  using SourceTypeNative = typename GLTypeToVTKHelper<SourceGLType>::vtk_type;
+  using DestinationTypeNative = typename GLTypeToVTKHelper<DestinationGLType>::vtk_type;
+
+  const int numSrcComponents = ::GetNumberOfColorComponents(sourceFormat);
+  const int numDestComponents = ::GetNumberOfColorComponents(destFormat);
+
+  // read pixels in the source color format.
+  std::vector<SourceTypeNative> srcPixels(rect.GetWidth() * rect.GetHeight() * numSrcComponents);
+  glReadPixels(rect.GetLeft(), rect.GetBottom(), rect.GetWidth(), rect.GetHeight(), sourceFormat,
+    SourceGLType, srcPixels.data());
+
+  // cast pixels to destination color format
+  auto dstPixels = static_cast<DestinationTypeNative*>(data);
+  if (sourceFormat == destFormat)
+  {
+    std::copy(srcPixels.begin(), srcPixels.end(), dstPixels);
+    return true;
+  }
+  else if (numSrcComponents == 3 && numDestComponents == 4)
+  {
+    return RGBToRGBA(srcPixels.data(), dstPixels, srcPixels.size());
+  }
+  else if (numSrcComponents == 4 && numDestComponents == 3)
+  {
+    return RGBAToRGB(srcPixels.data(), dstPixels, srcPixels.size());
+  }
+  else
+  {
+    return false;
+  }
+}
+
+// Convenient when both source, dest types are the same.
+template <GLint SameGLType>
+bool ConvertGLColor(GLint sourceFormat, const vtkRecti& rect, void* data, GLint destFormat)
+{
+  return ConvertGLColor<SameGLType, SameGLType>(sourceFormat, rect, data, destFormat);
+}
+}
+#endif
+
 //------------------------------------------------------------------------------
 void vtkOpenGLRenderWindow::SetGlobalMaximumNumberOfMultiSamples(int val)
 {
@@ -174,12 +392,16 @@ vtkOpenGLRenderWindow::vtkOpenGLRenderWindow()
   this->ResolveQuad = nullptr;
   this->DepthBlitQuad = nullptr;
   this->FlipQuad = nullptr;
+  this->DepthReadQuad = nullptr;
   this->FramebufferFlipY = false;
 
   this->Initialized = false;
   this->GlewInitValid = false;
 
-  this->MultiSamples = vtkOpenGLRenderWindowGlobalMaximumNumberOfMultiSamples;
+  this->MultiSamples = vtksys::SystemTools::HasEnv("VTK_TESTING")
+    ? 0
+    : vtkOpenGLRenderWindowGlobalMaximumNumberOfMultiSamples;
+
   delete[] this->WindowName;
   this->WindowName = new char[strlen(defaultWindowName) + 1];
   strcpy(this->WindowName, defaultWindowName);
@@ -190,6 +412,8 @@ vtkOpenGLRenderWindow::vtkOpenGLRenderWindow()
   this->DisplayFramebuffer->SetContext(this);
   this->ResolveFramebuffer = vtkOpenGLFramebufferObject::New();
   this->ResolveFramebuffer->SetContext(this);
+  this->DepthFramebuffer = vtkOpenGLFramebufferObject::New();
+  this->DepthFramebuffer->SetContext(this);
 
   this->DrawPixelsTextureObject = nullptr;
 
@@ -204,6 +428,7 @@ vtkOpenGLRenderWindow::vtkOpenGLRenderWindow()
   // this->DepthRenderBufferObject = 0;
   this->AlphaBitPlanes = 8;
   this->Capabilities = nullptr;
+  this->RenderBufferTargetDepthSize = 32;
 
   this->TQuad2DVBO = nullptr;
   this->NoiseTextureObject = nullptr;
@@ -233,6 +458,11 @@ vtkOpenGLRenderWindow::~vtkOpenGLRenderWindow()
     this->ResolveFramebuffer->Delete();
     this->ResolveFramebuffer = nullptr;
   }
+  if (this->DepthFramebuffer)
+  {
+    this->DepthFramebuffer->Delete();
+    this->DepthFramebuffer = nullptr;
+  }
 
   if (this->DrawPixelsTextureObject != nullptr)
   {
@@ -313,9 +543,13 @@ void vtkOpenGLRenderWindow::ReleaseGraphicsResources(vtkWindow* renWin)
   delete this->FlipQuad;
   this->FlipQuad = nullptr;
 
+  delete this->DepthReadQuad;
+  this->DepthReadQuad = nullptr;
+
   this->RenderFramebuffer->ReleaseGraphicsResources(renWin);
   this->DisplayFramebuffer->ReleaseGraphicsResources(renWin);
   this->ResolveFramebuffer->ReleaseGraphicsResources(renWin);
+  this->DepthFramebuffer->ReleaseGraphicsResources(renWin);
 
   // release the registered resources
   if (this->NoiseTextureObject)
@@ -497,9 +731,9 @@ void vtkOpenGLRenderWindow::OpenGLInitContext()
     // get this system's supported maximum line width
     // we do it here and store it to avoid repeated glGet
     // calls when the result should not change
-    GLfloat lineWidthRange[2];
     this->MaximumHardwareLineWidth = 1.0;
 #if defined(GL_SMOOTH_LINE_WIDTH_RANGE) && defined(GL_ALIASED_LINE_WIDTH_RANGE)
+    GLfloat lineWidthRange[2];
     if (this->LineSmoothing)
     {
       glGetFloatv(GL_SMOOTH_LINE_WIDTH_RANGE, lineWidthRange);
@@ -645,25 +879,41 @@ int vtkOpenGLRenderWindow::GetColorBufferSizes(int* rgba)
 
     glGetFramebufferAttachmentParameteriv(
       GL_DRAW_FRAMEBUFFER, attachment, GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE, &size);
-    if (glGetError() == GL_NO_ERROR)
+    if (auto error = glGetError())
+    {
+      vtkWarningMacro(<< "Failed to get red color buffer size (" << error << ')');
+    }
+    else
     {
       rgba[0] = static_cast<int>(size);
     }
     glGetFramebufferAttachmentParameteriv(
       GL_DRAW_FRAMEBUFFER, attachment, GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE, &size);
-    if (glGetError() == GL_NO_ERROR)
+    if (auto error = glGetError())
+    {
+      vtkWarningMacro(<< "Failed to get green color buffer size (" << error << ')');
+    }
+    else
     {
       rgba[1] = static_cast<int>(size);
     }
     glGetFramebufferAttachmentParameteriv(
       GL_DRAW_FRAMEBUFFER, attachment, GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE, &size);
-    if (glGetError() == GL_NO_ERROR)
+    if (auto error = glGetError())
+    {
+      vtkWarningMacro(<< "Failed to get blue color buffer size (" << error << ')');
+    }
+    else
     {
       rgba[2] = static_cast<int>(size);
     }
     glGetFramebufferAttachmentParameteriv(
       GL_DRAW_FRAMEBUFFER, attachment, GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE, &size);
-    if (glGetError() == GL_NO_ERROR)
+    if (auto error = glGetError())
+    {
+      vtkWarningMacro(<< "Failed to get alpha color buffer size (" << error << ')');
+    }
+    else
     {
       rgba[3] = static_cast<int>(size);
     }
@@ -709,6 +959,8 @@ int vtkOpenGLRenderWindow::GetColorBufferInternalFormat(int attachmentPoint)
     }
     vtkOpenGLClearErrorMacro();
   }
+#else
+  (void)attachmentPoint;
 #endif
 
   return format;
@@ -821,9 +1073,11 @@ int vtkOpenGLRenderWindow::ReadPixels(
   }
 
   // Must clear previous errors first.
+#ifdef VTK_REPORT_OPENGL_ERRORS
   while (glGetError() != GL_NO_ERROR)
   {
   }
+#endif
 
   this->GetState()->vtkglDisable(GL_SCISSOR_TEST);
 
@@ -864,9 +1118,109 @@ int vtkOpenGLRenderWindow::ReadPixels(
       this->ResolveFramebuffer->ActivateReadBuffer(0);
     }
   }
-
+#ifdef GL_ES_VERSION_3_0
+  // Open GL ES is very strict about the internal formats and data types that can be
+  // used in `glReadPixels`. Even the slightest mistake will result in GL_INVALID_OPERATION
+  // These restrictions are documented in the `Errors` section here -
+  // https://docs.gl/es3/glReadPixels This block of code queries the pixel format and data type of
+  // the currently bound read framebuffer. If those parameters agree with the arguments to this
+  // function, we're good. Otherwise, we need to use current parameters and cast as needed. Example,
+  // current bound read frame buffer is <GL_RGBA, GL_UNSIGNED_BYTE>, whereas this function was
+  // invoked with <GL_RGB, GL_FLOAT>. In this case, `glReadPixels` will be invoked with <GL_RGBA,
+  // GL_UNSIGNED_BYTE>. The result will be cast into <GL_RGB, GL_FLOAT>. Such color down-cast will
+  // skip the alpha channel. Color up-cast sets alpha = 255
+  bool castOk = false;
+  GLint currentGlType = 0;
+  glGetIntegerv(GL_IMPLEMENTATION_COLOR_READ_TYPE, &currentGlType);
+  GLint currentGlFormat = 0;
+  glGetIntegerv(GL_IMPLEMENTATION_COLOR_READ_FORMAT, &currentGlFormat);
+  if (currentGlType == gltype)
+  {
+    if (currentGlFormat == glformat)
+    {
+      glReadPixels(rect.GetLeft(), rect.GetBottom(), rect.GetWidth(), rect.GetHeight(), glformat,
+        gltype, data);
+      castOk = true;
+    }
+    else if (gltype == GL_UNSIGNED_BYTE)
+    {
+      castOk = ConvertGLColor<GL_UNSIGNED_BYTE>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_UNSIGNED_INT)
+    {
+      castOk = ConvertGLColor<GL_UNSIGNED_INT>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_INT)
+    {
+      castOk = ConvertGLColor<GL_INT>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_FLOAT)
+    {
+      castOk = ConvertGLColor<GL_FLOAT>(currentGlFormat, rect, data, glformat);
+    }
+  }
+  else if (currentGlType == GL_UNSIGNED_BYTE)
+  {
+    if (gltype == GL_FLOAT)
+    {
+      castOk = ConvertGLColor<GL_UNSIGNED_BYTE, GL_FLOAT>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_UNSIGNED_INT)
+    {
+      castOk =
+        ConvertGLColor<GL_UNSIGNED_BYTE, GL_UNSIGNED_INT>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_INT)
+    {
+      castOk = ConvertGLColor<GL_UNSIGNED_BYTE, GL_INT>(currentGlFormat, rect, data, glformat);
+    }
+  }
+  else if (currentGlType == GL_UNSIGNED_INT)
+  {
+    if (gltype == GL_FLOAT)
+    {
+      castOk = ConvertGLColor<GL_UNSIGNED_INT, GL_FLOAT>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_UNSIGNED_BYTE)
+    {
+      castOk =
+        ConvertGLColor<GL_UNSIGNED_INT, GL_UNSIGNED_BYTE>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_INT)
+    {
+      castOk = ConvertGLColor<GL_UNSIGNED_INT, GL_INT>(currentGlFormat, rect, data, glformat);
+    }
+  }
+  else if (currentGlType == GL_FLOAT)
+  {
+    if (gltype == GL_UNSIGNED_INT)
+    {
+      castOk = ConvertGLColor<GL_FLOAT, GL_UNSIGNED_INT>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_UNSIGNED_BYTE)
+    {
+      castOk = ConvertGLColor<GL_FLOAT, GL_UNSIGNED_BYTE>(currentGlFormat, rect, data, glformat);
+    }
+    else if (gltype == GL_INT)
+    {
+      castOk = ConvertGLColor<GL_FLOAT, GL_INT>(currentGlFormat, rect, data, glformat);
+    }
+  }
+  if (auto error = glGetError())
+  {
+    vtkErrorMacro(<< "Failed to read pixels in <glformat, gltype>=" << '<' << glformat << ','
+                  << gltype << ">. Error code : " << error);
+  }
+  else if (!castOk)
+  {
+    vtkErrorMacro(<< "Failed to cast pixel format and/or type from <glformat, gltype> "
+                  << currentGlFormat << ',' << currentGlType << " to " << glformat << ','
+                  << gltype);
+  }
+#else
   glReadPixels(
     rect.GetLeft(), rect.GetBottom(), rect.GetWidth(), rect.GetHeight(), glformat, gltype, data);
+#endif
 
   this->GetState()->PopReadFramebufferBinding();
 
@@ -1001,8 +1355,15 @@ void vtkOpenGLRenderWindow::Frame()
     this->RenderFramebuffer->Bind(GL_READ_FRAMEBUFFER);
     this->RenderFramebuffer->ActivateReadBuffer(0);
 
-    this->GetState()->vtkglBlitFramebuffer(0, 0, fbsize[0], fbsize[1], 0, 0, fbsize[0], fbsize[1],
-      (copiedColor ? 0 : GL_COLOR_BUFFER_BIT) | GL_DEPTH_BUFFER_BIT, GL_NEAREST);
+    if (this->FramebufferFlipY)
+    {
+      this->TextureDepthBlit(this->RenderFramebuffer->GetDepthAttachmentAsTextureObject());
+    }
+    else
+    {
+      this->GetState()->vtkglBlitFramebuffer(0, 0, fbsize[0], fbsize[1], 0, 0, fbsize[0], fbsize[1],
+        (copiedColor ? 0 : GL_COLOR_BUFFER_BIT) | GL_DEPTH_BUFFER_BIT, GL_NEAREST);
+    }
 
     this->GetState()->vtkglViewport(0, 0, this->Size[0], this->Size[1]);
     this->GetState()->vtkglScissor(0, 0, this->Size[0], this->Size[1]);
@@ -1018,6 +1379,10 @@ void vtkOpenGLRenderWindow::Frame()
       {
         this->BlitDisplayFramebuffer();
       }
+      if (this->FrameBlitMode == BlitToCurrentWithDepth)
+      {
+        this->BlitDisplayFramebufferColorAndDepth();
+      }
     }
   }
 }
@@ -1120,6 +1485,48 @@ bool vtkOpenGLRenderWindow::ResolveFlipRenderFramebuffer()
   return copiedColor;
 }
 
+//------------------------------------------------------------------------------
+bool vtkOpenGLRenderWindow::ReadDepthComponent(int depthSize)
+{
+  bool readDepthBuffer = false;
+  if (!this->DepthReadQuad)
+  {
+    std::string shader = DepthReadShader;
+    std::ostringstream os;
+    os << "int depthSize = " << depthSize << ';';
+    vtkShaderProgram::Substitute(shader, "//VTK::DepthSize::Impl", os.str());
+    this->DepthReadQuad = new vtkOpenGLQuadHelper(this, nullptr, shader.c_str(), "");
+    if (!this->DepthReadQuad->Program || !this->DepthReadQuad->Program->GetCompiled())
+    {
+      vtkErrorMacro("Couldn't build the shader program for reading depth component.");
+    }
+  }
+  else
+  {
+    this->GetShaderCache()->ReadyShaderProgram(this->DepthReadQuad->Program);
+  }
+
+  if (this->DepthReadQuad->Program && this->DepthReadQuad->Program->GetCompiled())
+  {
+    auto ostate = this->GetState();
+    vtkOpenGLState::ScopedglEnableDisable depthTestSaver(ostate, GL_DEPTH_TEST);
+    ostate->vtkglDisable(GL_DEPTH_TEST);
+    vtkOpenGLState::ScopedglEnableDisable blendSaver(ostate, GL_BLEND);
+    ostate->vtkglDisable(GL_BLEND);
+    auto tex = this->GetBufferNeedsResolving()
+      ? this->ResolveFramebuffer->GetDepthAttachmentAsTextureObject()
+      : this->RenderFramebuffer->GetDepthAttachmentAsTextureObject();
+    tex->Activate();
+    this->DepthReadQuad->Program->SetUniformi("tex", tex->GetTextureUnit());
+    this->DepthReadQuad->Render();
+    tex->Deactivate();
+    readDepthBuffer = true;
+  }
+
+  return readDepthBuffer;
+}
+
+//------------------------------------------------------------------------------
 void vtkOpenGLRenderWindow::BlitDisplayFramebuffersToHardware()
 {
   auto ostate = this->GetState();
@@ -1161,6 +1568,12 @@ void vtkOpenGLRenderWindow::BlitDisplayFramebuffer()
     this->Size[1], GL_COLOR_BUFFER_BIT, GL_NEAREST);
 }
 
+void vtkOpenGLRenderWindow::BlitDisplayFramebufferColorAndDepth()
+{
+  this->BlitDisplayFramebuffer(0, 0, 0, this->Size[0], this->Size[1], 0, 0, this->Size[0],
+    this->Size[1], GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT, GL_NEAREST);
+}
+
 void vtkOpenGLRenderWindow::BlitDisplayFramebuffer(int right, int srcX, int srcY, int srcWidth,
   int srcHeight, int destX, int destY, int destWidth, int destHeight, int bufferMode,
   int interpolation)
@@ -1875,9 +2288,58 @@ int vtkOpenGLRenderWindow::GetZbufferData(int x1, int y1, int x2, int y2, float*
     this->ResolveFramebuffer->ActivateReadBuffer(0);
   }
 
+#ifdef GL_ES_VERSION_3_0
+  {
+    const int depthSize = this->GetDepthBufferSize();
+    this->GetState()->PushDrawFramebufferBinding();
+    auto readFramebuffer = resolveMSAA ? this->ResolveFramebuffer : this->RenderFramebuffer;
+    int* fbsize = readFramebuffer->GetLastSize();
+    this->DepthFramebuffer->Bind(GL_DRAW_FRAMEBUFFER);
+    this->DepthFramebuffer->ActivateDrawBuffer(0);
+    this->GetState()->vtkglViewport(0, 0, fbsize[0], fbsize[1]);
+    this->GetState()->vtkglScissor(0, 0, fbsize[0], fbsize[1]);
+
+    bool readDepth = this->ReadDepthComponent(depthSize);
+    this->GetState()->PopDrawFramebufferBinding();
+    this->GetState()->PopReadFramebufferBinding();
+    if (!readDepth)
+    {
+      vtkErrorMacro(<< "Failed to read depth component!");
+      return VTK_ERROR;
+    }
+    else
+    {
+      const auto maxDepthValueAsInteger = float(1 << depthSize) - 1.0f;
+      this->GetState()->PushReadFramebufferBinding();
+      this->DepthFramebuffer->Bind(GL_READ_FRAMEBUFFER);
+      this->DepthFramebuffer->ActivateReadBuffer(0);
+      std::vector<vtkTypeUInt8> z_data_quarters(width * height * 4, 0);
+      const vtkRecti rect(x_low, y_low, width, height);
+      ConvertGLColor<GL_UNSIGNED_BYTE>(GL_RGBA, rect, z_data_quarters.data(), GL_RGBA);
+      this->GetState()->PopReadFramebufferBinding();
+      for (int i = 0, j = 0; i < width * height; ++i)
+      {
+        vtkTypeUInt32 z_int = z_data_quarters[j++];
+        z_int += (z_data_quarters[j++] << 8);
+#if defined(GL_DEPTH_COMPONENT24) || defined(GL_DEPTH_COMPONENT32)
+        z_int += (z_data_quarters[j++] << 16);
+#else
+        ++j;
+#endif
+#ifdef GL_DEPTH_COMPONENT32
+        z_int += (z_data_quarters[j++] << 24);
+#else
+        ++j;
+#endif
+        z_data[i] = z_int / maxDepthValueAsInteger;
+      }
+    }
+  }
+#else
   glReadPixels(x_low, y_low, width, height, GL_DEPTH_COMPONENT, GL_FLOAT, z_data);
 
   this->GetState()->PopReadFramebufferBinding();
+#endif
 
   if (glGetError() != GL_NO_ERROR)
   {
@@ -2067,8 +2529,8 @@ int vtkOpenGLRenderWindow::CreateFramebuffers(int width, int height)
 #else
       this->MultiSamples ? false : true, // textures
 #endif
-      1, VTK_UNSIGNED_CHAR, // 1 color buffer uchar
-      true, 32,             // depth buffer
+      1, VTK_UNSIGNED_CHAR,                    // 1 color buffer uchar
+      true, this->RenderBufferTargetDepthSize, // depth buffer
       this->MultiSamples, this->StencilCapable != 0);
     this->LastMultiSamples = this->MultiSamples;
     this->GetState()->PopFramebufferBindings();
@@ -2082,9 +2544,9 @@ int vtkOpenGLRenderWindow::CreateFramebuffers(int width, int height)
   {
     this->GetState()->PushFramebufferBindings();
     this->DisplayFramebuffer->PopulateFramebuffer(width, height,
-      true,                 // textures
-      2, VTK_UNSIGNED_CHAR, // 1 color buffer uchar
-      true, 32,             // depth buffer
+      true,                                    // textures
+      2, VTK_UNSIGNED_CHAR,                    // 1 color buffer uchar
+      true, this->RenderBufferTargetDepthSize, // depth buffer
       0, this->StencilCapable != 0);
     this->GetState()->PopFramebufferBindings();
   }
@@ -2097,13 +2559,27 @@ int vtkOpenGLRenderWindow::CreateFramebuffers(int width, int height)
   {
     this->GetState()->PushFramebufferBindings();
     this->ResolveFramebuffer->PopulateFramebuffer(width, height,
-      true,                 // textures
-      1, VTK_UNSIGNED_CHAR, // 1 color buffer uchar
-      true, 32,             // depth buffer
+      true,                                    // textures
+      1, VTK_UNSIGNED_CHAR,                    // 1 color buffer uchar
+      true, this->RenderBufferTargetDepthSize, // depth buffer
       0, this->StencilCapable != 0);
     this->GetState()->PopFramebufferBindings();
   }
 
+  if (!this->DepthFramebuffer->GetFBOIndex())
+  {
+    this->GetState()->PushFramebufferBindings();
+    this->DepthFramebuffer->PopulateFramebuffer(width, height,
+      true,                 // textures
+      1, VTK_UNSIGNED_CHAR, // 1 color buffer uchar
+      false, 0,             // depth buffer
+      0, this->StencilCapable != 0);
+    this->GetState()->PopFramebufferBindings();
+  }
+  else
+  {
+    this->DepthFramebuffer->Resize(width, height);
+  }
   return 1;
 }
 
@@ -2186,7 +2662,15 @@ int vtkOpenGLRenderWindow::SupportsOpenGL()
     this->OpenGLSupportResult = 1;
     this->OpenGLSupportMessage = "The system appears to support OpenGL 3.2/3.1";
   }
-
+#else
+#ifdef GL_ES_VERSION_2_0
+  this->OpenGLSupportResult = 1;
+  this->OpenGLSupportMessage = "The system appears to support OpenGL ES 2.0";
+#endif
+#ifdef GL_ES_VERSION_3_0
+  this->OpenGLSupportResult = 1;
+  this->OpenGLSupportMessage = "The system appears to support OpenGL ES 3.0";
+#endif
 #endif
 
   if (this->OpenGLSupportResult)
@@ -2319,3 +2803,4 @@ void vtkOpenGLRenderWindow::Render()
     vtkOpenGLRenderUtilities::MarkDebugEvent("Completed vtkOpenGLRenderWIndow::Render");
   }
 }
+VTK_ABI_NAMESPACE_END
