@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkOpenGLRenderWindow.h
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkOpenGLRenderWindow
  * @brief   OpenGL rendering window
@@ -29,10 +17,12 @@
 #include "vtkRenderWindow.h"
 #include "vtkRenderingOpenGL2Module.h" // For export macro
 #include "vtkType.h"                   // for ivar
+#include "vtkWrappingHints.h"          // For VTK_MARSHALAUTO
 #include <map>                         // for ivar
 #include <set>                         // for ivar
 #include <string>                      // for ivar
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkIdList;
 class vtkOpenGLBufferObject;
 class vtkOpenGLFramebufferObject;
@@ -48,7 +38,7 @@ class vtkTextureUnitManager;
 class vtkGenericOpenGLResourceFreeCallback;
 class vtkOpenGLState;
 
-class VTKRENDERINGOPENGL2_EXPORT vtkOpenGLRenderWindow : public vtkRenderWindow
+class VTKRENDERINGOPENGL2_EXPORT VTK_MARSHALAUTO vtkOpenGLRenderWindow : public vtkRenderWindow
 {
 public:
   vtkTypeMacro(vtkOpenGLRenderWindow, vtkRenderWindow);
@@ -57,7 +47,7 @@ public:
   /**
    * Begin the rendering process.
    */
-  void Start(void) override;
+  void Start() override;
 
   /**
    * A termination method performed at the end of the rendering process
@@ -285,7 +275,7 @@ public:
    * should be possible to call them multiple times, even changing WindowId
    * in-between.  This is what WindowRemap does.
    */
-  virtual void Initialize(void) {}
+  virtual void Initialize() {}
 
   std::set<vtkGenericOpenGLResourceFreeCallback*> Resources;
 
@@ -377,10 +367,17 @@ public:
   void ReleaseGraphicsResources(vtkWindow*) override;
 
   /**
-   * Blit a display framebuffer into a currently bound draw destination
+   * Blit a display framebuffer into a currently bound draw destination,
+   * color only
    */
   void BlitDisplayFramebuffer();
 
+  /**
+   * Blit a display framebuffer into a currently bound draw destination,
+   * color and depth
+   */
+  void BlitDisplayFramebufferColorAndDepth();
+
   /**
    * Blit a display buffer into a currently bound draw destination
    */
@@ -403,20 +400,46 @@ public:
    */
   enum FrameBlitModes
   {
-    BlitToHardware, // hardware buffers
-    BlitToCurrent,  // currently bound draw framebuffer
-    NoBlit          // no blit, GUI or external code will handle the blit
+    BlitToHardware,         // hardware buffers
+    BlitToCurrent,          // currently bound draw framebuffer, without depth buffer
+    BlitToCurrentWithDepth, // currently bound draw framebuffer, including depth buffer
+    NoBlit                  // no blit, GUI or external code will handle the blit
   };
 
   ///@{
   /**
    * SetGet how to handle blits at the end of a Frame() call.
    * Only happens when SwapBuffers is true.
+   *
+   * The “blit” operation means copying something (pixels) from one region
+   * in memory to another. This is a common OpenGL operation: check out the
+   * glBlitFramebuffer OpenGL method. VTK OpenGL rasterizes 3D/2D geometry
+   * into color/depth attachments of an in-memory framebuffer: in the graphics
+   * world, this is called offscreen rendering.
+   *
+   * The FrameBlitMode tells VTK where to copy the offscreen
+   * buffer. Either the default hardware framebuffer or the currently bound
+   * framebuffer. Here are the available modes:
+   * - BlitToHardware: This mode blits to the default framebuffer managed by
+   *   platform(Win32/X/Cocoa) render windows. This is the default mode when VTK
+   *   is used through one of the platform render windows (with vtkRenderWindow).
+   * - BlitToCurrent: This mode blits to the currently bound framebuffer.
+   *   It is useful when an external framebuffer is bound just before
+   *   the vtkRenderWindow::Frame() call. You’ll need this when integrating
+   *   VTK with other UI frameworks because these UI frameworks create/have
+   *   their own framebuffers. This only blits the color buffer.
+   * - BlitToCurrentWithDepth: Almost the same as BlitToCurrent, but this mode
+   *   also blits the depth buffer.
+   * - NoBlit: no blit. The GUI or external code will handle the blit.
    */
   vtkSetClampMacro(FrameBlitMode, FrameBlitModes, BlitToHardware, NoBlit);
   vtkGetMacro(FrameBlitMode, FrameBlitModes);
   void SetFrameBlitModeToBlitToHardware() { this->SetFrameBlitMode(BlitToHardware); }
   void SetFrameBlitModeToBlitToCurrent() { this->SetFrameBlitMode(BlitToCurrent); }
+  void SetFrameBlitModeToBlitToCurrentWithDepth()
+  {
+    this->SetFrameBlitMode(BlitToCurrentWithDepth);
+  }
   void SetFrameBlitModeToNoBlit() { this->SetFrameBlitMode(NoBlit); }
   ///@}
 
@@ -429,6 +452,14 @@ public:
   vtkBooleanMacro(FramebufferFlipY, bool);
   ///@}
 
+  ///@{
+  /**
+   * Give a target bit size for depth buffers of created Framebuffer Objects. Default is 32.
+   */
+  vtkSetMacro(RenderBufferTargetDepthSize, int);
+  vtkGetMacro(RenderBufferTargetDepthSize, int);
+  ///@}
+
   ///@{
   // copy depth values from a source framebuffer to a destination framebuffer
   // using texture maps to do the copy. The source framebufferobject must be texture
@@ -465,6 +496,10 @@ protected:
   // a FSQ we use to flip framebuffer texture
   vtkOpenGLQuadHelper* FlipQuad;
 
+  // a FSQ we use to read depth component on platforms with OpenGL ES implementations
+  // because `glReadPixels` cannot be used to read GL_DEPTH_COMPONENT
+  vtkOpenGLQuadHelper* DepthReadQuad;
+
   // flip quad helpers Y tcoord
   bool FramebufferFlipY;
 
@@ -473,6 +508,15 @@ protected:
   // true if the color buffer was copied.
   virtual bool ResolveFlipRenderFramebuffer();
 
+  // On GLES, the depth attachment buffer cannot be downloaded from
+  // the GPU with `glReadPixels`.
+  // This method reads the depth buffer bits.
+  // The depth attachment size can be 8,16,24 or 32. The values are split into 4 8-bit numbers.
+  // These are stored in the form of an RGBA color attachment in DepthFrameBuffer.
+  // `glReadPixels` can read that RGBA format and reconstruct full 8,16,24 or 32-bit integer
+  // followed by scaling down to 0-1.
+  bool ReadDepthComponent(int depthSize);
+
   // used in testing for opengl support
   // in the SupportsOpenGL() method
   bool OpenGLSupportTested;
@@ -483,7 +527,7 @@ protected:
     const vtkRecti& rect, int front, int glFormat, int glType, void* data, int right = 0);
 
   /**
-   * Create the offScreen framebuffer
+   * Create the offScreen framebuffers or resize them if they are already created.
    * Return if the creation was successful or not.
    * \pre positive_width: width>0
    * \pre positive_height: height>0
@@ -498,6 +542,10 @@ protected:
   // framebuffer
   vtkOpenGLFramebufferObject* ResolveFramebuffer;
 
+  // used when we need to read depth component
+  // with OpenGL ES 3
+  vtkOpenGLFramebufferObject* DepthFramebuffer;
+
   /**
    * Create a not-off-screen window.
    */
@@ -547,6 +595,9 @@ protected:
   // keep track of in case we need to recreate the framebuffer
   int LastMultiSamples;
 
+  // how much bits to use for depth of created Framebuffers
+  int RenderBufferTargetDepthSize;
+
   int ScreenSize[2];
 
 private:
@@ -558,4 +609,5 @@ private:
   vtkOpenGLState* State;
 };
 
+VTK_ABI_NAMESPACE_END
 #endif
