@@ -1,16 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkShaderProgram.h"
 #include "vtkObjectFactory.h"
 
@@ -29,6 +18,7 @@
 #include <sstream>
 #include <vtksys/SystemTools.hxx>
 
+VTK_ABI_NAMESPACE_BEGIN
 namespace
 {
 
@@ -71,6 +61,9 @@ vtkStandardNewMacro(vtkShaderProgram);
 vtkCxxSetObjectMacro(vtkShaderProgram, VertexShader, vtkShader);
 vtkCxxSetObjectMacro(vtkShaderProgram, FragmentShader, vtkShader);
 vtkCxxSetObjectMacro(vtkShaderProgram, GeometryShader, vtkShader);
+vtkCxxSetObjectMacro(vtkShaderProgram, ComputeShader, vtkShader);
+vtkCxxSetObjectMacro(vtkShaderProgram, TessControlShader, vtkShader);
+vtkCxxSetObjectMacro(vtkShaderProgram, TessEvaluationShader, vtkShader);
 vtkCxxSetObjectMacro(vtkShaderProgram, TransformFeedback, vtkTransformFeedback);
 
 vtkShaderProgram::vtkShaderProgram()
@@ -81,6 +74,12 @@ vtkShaderProgram::vtkShaderProgram()
   this->FragmentShader->SetType(vtkShader::Fragment);
   this->GeometryShader = vtkShader::New();
   this->GeometryShader->SetType(vtkShader::Geometry);
+  this->ComputeShader = vtkShader::New();
+  this->ComputeShader->SetType(vtkShader::Compute);
+  this->TessControlShader = vtkShader::New();
+  this->TessControlShader->SetType(vtkShader::TessControl);
+  this->TessEvaluationShader = vtkShader::New();
+  this->TessEvaluationShader->SetType(vtkShader::TessEvaluation);
 
   this->TransformFeedback = nullptr;
 
@@ -90,6 +89,9 @@ vtkShaderProgram::vtkShaderProgram()
   this->VertexShaderHandle = 0;
   this->FragmentShaderHandle = 0;
   this->GeometryShaderHandle = 0;
+  this->ComputeShaderHandle = 0;
+  this->TessControlShaderHandle = 0;
+  this->TessEvaluationShaderHandle = 0;
   this->Linked = false;
   this->Bound = false;
 
@@ -114,6 +116,21 @@ vtkShaderProgram::~vtkShaderProgram()
     this->GeometryShader->Delete();
     this->GeometryShader = nullptr;
   }
+  if (this->ComputeShader)
+  {
+    this->ComputeShader->Delete();
+    this->ComputeShader = nullptr;
+  }
+  if (this->TessControlShader)
+  {
+    this->TessControlShader->Delete();
+    this->TessControlShader = nullptr;
+  }
+  if (this->TessEvaluationShader)
+  {
+    this->TessEvaluationShader->Delete();
+    this->TessEvaluationShader = nullptr;
+  }
   if (this->TransformFeedback)
   {
     this->TransformFeedback->Delete();
@@ -217,14 +234,62 @@ bool vtkShaderProgram::AttachShader(const vtkShader* shader)
   }
   else if (shader->GetType() == vtkShader::Geometry)
   {
+// only use GS if supported
+#ifdef GL_GEOMETRY_SHADER
     if (this->GeometryShaderHandle != 0)
     {
       glDetachShader(
         static_cast<GLuint>(this->Handle), static_cast<GLuint>(this->GeometryShaderHandle));
     }
-// only use GS if supported
-#ifdef GL_GEOMETRY_SHADER
     this->GeometryShaderHandle = shader->GetHandle();
+#else
+    this->Error = "Geometry shaders are not supported in this build of VTK";
+    return false;
+#endif
+  }
+  else if (shader->GetType() == vtkShader::Compute)
+  {
+// only use CS if supported
+#ifdef GL_COMPUTE_SHADER
+    if (this->ComputeShaderHandle != 0)
+    {
+      glDetachShader(
+        static_cast<GLuint>(this->Handle), static_cast<GLuint>(this->ComputeShaderHandle));
+    }
+    this->ComputeShaderHandle = shader->GetHandle();
+#else
+    this->Error = "Compute shaders are not supported in this build of VTK";
+    return false;
+#endif
+  }
+  else if (shader->GetType() == vtkShader::TessControl)
+  {
+// only use TCS if supported
+#ifdef GL_TESS_CONTROL_SHADER
+    if (this->TessControlShaderHandle != 0)
+    {
+      glDetachShader(
+        static_cast<GLuint>(this->Handle), static_cast<GLuint>(this->TessControlShaderHandle));
+    }
+    this->TessControlShaderHandle = shader->GetHandle();
+#else
+    this->Error = "Tessellation shaders are not supported in this build of VTK";
+    return false;
+#endif
+  }
+  else if (shader->GetType() == vtkShader::TessEvaluation)
+  {
+// only use TES if supported
+#ifdef GL_TESS_EVALUATION_SHADER
+    if (this->TessEvaluationShaderHandle != 0)
+    {
+      glDetachShader(
+        static_cast<GLuint>(this->Handle), static_cast<GLuint>(this->TessEvaluationShaderHandle));
+    }
+    this->TessEvaluationShaderHandle = shader->GetHandle();
+#else
+    this->Error = "Tessellation shaders are not supported in this build of VTK";
+    return false;
 #endif
   }
   else
@@ -283,8 +348,8 @@ bool vtkShaderProgram::DetachShader(const vtkShader* shader)
         this->Linked = false;
         return true;
       }
-#ifdef GL_GEOMETRY_SHADER
     case vtkShader::Geometry:
+#ifdef GL_GEOMETRY_SHADER
       if (this->GeometryShaderHandle != shader->GetHandle())
       {
         this->Error = "The supplied shader was not attached to this program.";
@@ -297,6 +362,63 @@ bool vtkShaderProgram::DetachShader(const vtkShader* shader)
         this->Linked = false;
         return true;
       }
+#else
+      this->Error = "Geometry shaders are not supported in this build of VTK";
+      return false;
+#endif
+    case vtkShader::Compute:
+#ifdef GL_COMPUTE_SHADER
+      if (this->ComputeShaderHandle != shader->GetHandle())
+      {
+        this->Error = "The supplied shader was not attached to this program.";
+        return false;
+      }
+      else
+      {
+        glDetachShader(static_cast<GLuint>(this->Handle), static_cast<GLuint>(shader->GetHandle()));
+        this->ComputeShaderHandle = 0;
+        this->Linked = false;
+        return true;
+      }
+#else
+      this->Error = "Compute shaders are not supported in this build of VTK";
+      return false;
+#endif
+    case vtkShader::TessControl:
+#ifdef GL_TESS_CONTROL_SHADER
+      if (this->TessControlShaderHandle != shader->GetHandle())
+      {
+        this->Error = "The supplied shader was not attached to this program.";
+        return false;
+      }
+      else
+      {
+        glDetachShader(static_cast<GLuint>(this->Handle), static_cast<GLuint>(shader->GetHandle()));
+        this->TessControlShaderHandle = 0;
+        this->Linked = false;
+        return true;
+      }
+#else
+      this->Error = "Tessellation control shaders are not supported in this build of VTK";
+      return false;
+#endif
+    case vtkShader::TessEvaluation:
+#ifdef GL_TESS_EVALUATION_SHADER
+      if (this->TessEvaluationShaderHandle != shader->GetHandle())
+      {
+        this->Error = "The supplied shader was not attached to this program.";
+        return false;
+      }
+      else
+      {
+        glDetachShader(static_cast<GLuint>(this->Handle), static_cast<GLuint>(shader->GetHandle()));
+        this->TessEvaluationShaderHandle = 0;
+        this->Linked = false;
+        return true;
+      }
+#else
+      this->Error = "Tessellation evaluation shaders are not supported in this build of VTK";
+      return false;
 #endif
     case vtkShader::Unknown:
     default:
@@ -389,10 +511,19 @@ bool vtkShaderProgram::Bind()
 {
   if (this->FileNamePrefixForDebugging != nullptr && this->FileNamePrefixForDebugging[0] != 0)
   {
-    const char* exts[3] = { "VS.glsl", "FS.glsl", "GS.glsl" };
-    vtkShader* shaders[3] = { this->VertexShader, this->FragmentShader, this->GeometryShader };
-    for (int cc = 0; cc < 3; cc++)
+    const char* exts[5] = {
+      "VS.glsl",
+      "TCS.glsl",
+      "TES.glsl",
+      "GS.glsl",
+      "FS.glsl",
+    };
+    vtkShader* shaders[5] = { this->VertexShader, this->TessControlShader,
+      this->TessEvaluationShader, this->GeometryShader, this->FragmentShader };
+    std::string sources[5] = {};
+    for (int cc = 0; cc < 5; cc++)
     {
+      sources[cc] = shaders[cc]->GetSource();
       std::string fname = this->FileNamePrefixForDebugging;
       fname += exts[cc];
       if (vtksys::SystemTools::FileExists(fname))
@@ -408,7 +539,15 @@ bool vtkShaderProgram::Bind()
         ofp << shaders[cc]->GetSource();
       }
     }
-    this->CompileShader();
+    if (!this->CompileShader())
+    {
+      // fallback to previous source code.
+      vtkWarningMacro(<< "Falling back to last working source code");
+      for (int cc = 0; cc < 5; cc++)
+      {
+        shaders[cc]->SetSource(sources[cc]);
+      }
+    }
   }
   if (!this->Linked && !this->Link())
   {
@@ -423,50 +562,19 @@ bool vtkShaderProgram::Bind()
 // return 0 if there is an issue
 int vtkShaderProgram::CompileShader()
 {
-  if (!this->GetVertexShader()->Compile())
+  if (!this->GetVertexShader()->GetSource().empty() && !this->GetVertexShader()->Compile())
   {
-    int lineNum = 1;
-    std::istringstream stream(this->GetVertexShader()->GetSource());
-    std::stringstream sstm;
-    std::string aline;
-    while (std::getline(stream, aline))
-    {
-      sstm << lineNum << ": " << aline << "\n";
-      lineNum++;
-    }
-    vtkErrorMacro(<< sstm.str());
-    vtkErrorMacro(<< this->GetVertexShader()->GetError());
+    this->ReportShaderError(this->GetVertexShader());
     return 0;
   }
-  if (!this->GetFragmentShader()->Compile())
+  if (!this->GetFragmentShader()->GetSource().empty() && !this->GetFragmentShader()->Compile())
   {
-    int lineNum = 1;
-    std::istringstream stream(this->GetFragmentShader()->GetSource());
-    std::stringstream sstm;
-    std::string aline;
-    while (std::getline(stream, aline))
-    {
-      sstm << lineNum << ": " << aline << "\n";
-      lineNum++;
-    }
-    vtkErrorMacro(<< sstm.str());
-    vtkErrorMacro(<< this->GetFragmentShader()->GetError());
+    this->ReportShaderError(this->GetFragmentShader());
     return 0;
   }
-#ifdef GL_GEOMETRY_SHADER
   if (!this->GetGeometryShader()->GetSource().empty() && !this->GetGeometryShader()->Compile())
   {
-    int lineNum = 1;
-    std::istringstream stream(this->GetGeometryShader()->GetSource());
-    std::stringstream sstm;
-    std::string aline;
-    while (std::getline(stream, aline))
-    {
-      sstm << lineNum << ": " << aline << "\n";
-      lineNum++;
-    }
-    vtkErrorMacro(<< sstm.str());
-    vtkErrorMacro(<< this->GetGeometryShader()->GetError());
+    this->ReportShaderError(this->GetGeometryShader());
     return 0;
   }
   if (!this->GetGeometryShader()->GetSource().empty() &&
@@ -475,7 +583,54 @@ int vtkShaderProgram::CompileShader()
     vtkErrorMacro(<< this->GetError());
     return 0;
   }
-#endif
+  if (!this->GetComputeShader()->GetSource().empty())
+  {
+    if (!this->GetComputeShader()->Compile())
+    {
+      this->ReportShaderError(this->GetComputeShader());
+      return 0;
+    }
+    if (!this->AttachShader(this->GetComputeShader()))
+    {
+      vtkErrorMacro(<< this->GetError());
+      return 0;
+    }
+
+    if (!this->Link())
+    {
+      vtkErrorMacro(<< "Links failed: " << this->GetError());
+      return 0;
+    }
+
+    this->Compiled = true;
+    return 1;
+  }
+  if (!this->GetTessControlShader()->GetSource().empty())
+  {
+    if (!this->GetTessControlShader()->Compile())
+    {
+      this->ReportShaderError(this->GetTessControlShader());
+      return 0;
+    }
+    if (!this->AttachShader(this->GetTessControlShader()))
+    {
+      vtkErrorMacro(<< this->GetError());
+      return 0;
+    }
+  }
+  if (!this->GetTessEvaluationShader()->GetSource().empty())
+  {
+    if (!this->GetTessEvaluationShader()->Compile())
+    {
+      this->ReportShaderError(this->GetTessEvaluationShader());
+      return 0;
+    }
+    if (!this->AttachShader(this->GetTessEvaluationShader()))
+    {
+      vtkErrorMacro(<< this->GetError());
+      return 0;
+    }
+  }
   if (!this->AttachShader(this->GetVertexShader()))
   {
     vtkErrorMacro(<< this->GetError());
@@ -518,9 +673,13 @@ void vtkShaderProgram::ReleaseGraphicsResources(vtkWindow* win)
     this->DetachShader(this->VertexShader);
     this->DetachShader(this->FragmentShader);
     this->DetachShader(this->GeometryShader);
+    this->DetachShader(this->TessControlShader);
+    this->DetachShader(this->TessEvaluationShader);
     this->VertexShader->Cleanup();
     this->FragmentShader->Cleanup();
     this->GeometryShader->Cleanup();
+    this->TessControlShader->Cleanup();
+    this->TessEvaluationShader->Cleanup();
     this->Compiled = false;
   }
 
@@ -648,7 +807,7 @@ bool vtkShaderProgram::SetUniformMatrix4x4(const char* name, float* matrix)
   return this->SetUniformMatrix4x4v(name, 1, matrix);
 }
 
-bool vtkShaderProgram::SetUniformMatrix4x4v(const char* name, const int count, float* matrix)
+bool vtkShaderProgram::SetUniformMatrix4x4v(const char* name, int count, float* matrix)
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -679,7 +838,7 @@ bool vtkShaderProgram::SetUniformMatrix(const char* name, vtkMatrix3x3* matrix)
   return true;
 }
 
-bool vtkShaderProgram::SetUniform1fv(const char* name, const int count, const float* v)
+bool vtkShaderProgram::SetUniform1fv(const char* name, int count, const float* v)
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -692,7 +851,7 @@ bool vtkShaderProgram::SetUniform1fv(const char* name, const int count, const fl
   return true;
 }
 
-bool vtkShaderProgram::SetUniform1iv(const char* name, const int count, const int* v)
+bool vtkShaderProgram::SetUniform1iv(const char* name, int count, const int* v)
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -705,7 +864,7 @@ bool vtkShaderProgram::SetUniform1iv(const char* name, const int count, const in
   return true;
 }
 
-bool vtkShaderProgram::SetUniform3fv(const char* name, const int count, const float* f)
+bool vtkShaderProgram::SetUniform3fv(const char* name, int count, const float* f)
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -718,7 +877,7 @@ bool vtkShaderProgram::SetUniform3fv(const char* name, const int count, const fl
   return true;
 }
 
-bool vtkShaderProgram::SetUniform3fv(const char* name, const int count, const float (*v)[3])
+bool vtkShaderProgram::SetUniform3fv(const char* name, int count, const float (*v)[3])
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -731,7 +890,7 @@ bool vtkShaderProgram::SetUniform3fv(const char* name, const int count, const fl
   return true;
 }
 
-bool vtkShaderProgram::SetUniform4fv(const char* name, const int count, const float* f)
+bool vtkShaderProgram::SetUniform4fv(const char* name, int count, const float* f)
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -744,7 +903,7 @@ bool vtkShaderProgram::SetUniform4fv(const char* name, const int count, const fl
   return true;
 }
 
-bool vtkShaderProgram::SetUniform4fv(const char* name, const int count, const float (*v)[4])
+bool vtkShaderProgram::SetUniform4fv(const char* name, int count, const float (*v)[4])
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -770,7 +929,7 @@ bool vtkShaderProgram::SetUniform2f(const char* name, const float v[2])
   return true;
 }
 
-bool vtkShaderProgram::SetUniform2fv(const char* name, const int count, const float* f)
+bool vtkShaderProgram::SetUniform2fv(const char* name, int count, const float* f)
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -783,7 +942,7 @@ bool vtkShaderProgram::SetUniform2fv(const char* name, const int count, const fl
   return true;
 }
 
-bool vtkShaderProgram::SetUniform2fv(const char* name, const int count, const float (*f)[2])
+bool vtkShaderProgram::SetUniform2fv(const char* name, int count, const float (*f)[2])
 {
   GLint location = static_cast<GLint>(this->FindUniform(name));
   if (location == -1)
@@ -970,6 +1129,22 @@ bool vtkShaderProgram::IsAttributeUsed(const char* cname)
   return (result != -1);
 }
 
+//------------------------------------------------------------------------------
+void vtkShaderProgram::ReportShaderError(vtkShader* shader)
+{
+  int lineNum = 1;
+  std::istringstream stream(shader->GetSource());
+  std::stringstream sstm;
+  std::string aline;
+  while (std::getline(stream, aline))
+  {
+    sstm << lineNum << ": " << aline << "\n";
+    lineNum++;
+  }
+  vtkErrorMacro(<< sstm.str());
+  vtkErrorMacro(<< shader->GetError());
+}
+
 //------------------------------------------------------------------------------
 void vtkShaderProgram::PrintSelf(ostream& os, vtkIndent indent)
 {
@@ -977,3 +1152,4 @@ void vtkShaderProgram::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "FileNamePrefixForDebugging: "
      << (this->FileNamePrefixForDebugging ? this->FileNamePrefixForDebugging : "(null)") << endl;
 }
+VTK_ABI_NAMESPACE_END
