@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkXRenderWindowInteractor.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 // Must be included first to avoid conflicts with X11's `Status` define.
 #include <vtksys/SystemTools.hxx>
@@ -39,10 +27,12 @@
 
 #include <climits>
 #include <cmath>
+#include <cstdint>
 #include <map>
 #include <set>
 #include <sstream>
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkXRenderWindowInteractor);
 
 template <int EventType>
@@ -80,25 +70,35 @@ public:
 
   void DestroyLocalTimer(int id) { this->LocalToTimer.erase(id); }
 
-  void GetTimeToNextTimer(timeval& tv)
+  /**
+   * This interactor uses `select` to coordinate timers.
+   * Returns true if `select` needs to block until some time elapses or a user interaction event
+   * occurs. Returns false if `select` needs to block indefinitely until a user interaction event
+   * occurs. The `tv` arg is populated with a time-interval that can be used by `select` when it
+   * needs to use a timeout.
+   */
+  bool GetTimeToNextTimer(timeval& tv)
   {
-    uint64_t lowestDelta = 1000000;
+    bool useTimeout = false; // whether `select` must block for some time.
+    uint64_t delta = 0;      // in microsecs
     if (!this->LocalToTimer.empty())
     {
       timeval ctv;
       gettimeofday(&ctv, nullptr);
+      delta = VTK_UNSIGNED_INT_MAX; // arbitrary high value for time interval
       for (auto& timer : this->LocalToTimer)
       {
-        uint64_t delta = (ctv.tv_sec - timer.second.lastFire.tv_sec) * 1000000 + ctv.tv_usec -
-          timer.second.lastFire.tv_usec;
-        if (delta < lowestDelta)
-        {
-          lowestDelta = delta;
-        }
+        uint64_t duration = timer.second.duration * 1000; // in microsecs
+        uint64_t elapsed = (ctv.tv_sec - timer.second.lastFire.tv_sec) * 1000000 + ctv.tv_usec -
+          timer.second.lastFire.tv_usec; // in microsecs
+        // 0 lets the timer fire immediately.
+        delta = duration > elapsed ? std::min<uint64_t>(duration - elapsed, delta) : 0;
+        useTimeout = true;
       }
     }
-    tv.tv_sec = lowestDelta / 1000000;
-    tv.tv_usec = lowestDelta % 1000000;
+    tv.tv_sec = delta / 1000000;
+    tv.tv_usec = delta % 1000000;
+    return useTimeout;
   }
 
   void FireTimers(vtkXRenderWindowInteractor* rwi)
@@ -107,38 +107,57 @@ public:
     {
       timeval ctv;
       gettimeofday(&ctv, nullptr);
-      std::vector<unsigned long> expired;
-      for (auto& timer : this->LocalToTimer)
+      std::vector<LocalToTimerType::value_type> timers(
+        this->LocalToTimer.begin(), this->LocalToTimer.end());
+      for (auto& timer : timers)
       {
         int64_t delta = (ctv.tv_sec - timer.second.lastFire.tv_sec) * 1000000 + ctv.tv_usec -
           timer.second.lastFire.tv_usec;
         if (delta / 1000 >= static_cast<int64_t>(timer.second.duration))
         {
           int timerId = rwi->GetVTKTimerId(timer.first);
-          rwi->InvokeEvent(vtkCommand::TimerEvent, &timerId);
-          if (rwi->IsOneShotTimer(timerId))
+          if (timerId != 0)
           {
-            expired.push_back(timer.first);
-          }
-          else
-          {
-            timer.second.lastFire.tv_sec = ctv.tv_sec;
-            timer.second.lastFire.tv_usec = ctv.tv_usec;
+            rwi->InvokeEvent(vtkCommand::TimerEvent, &timerId);
+            if (rwi->IsOneShotTimer(timerId))
+            {
+              this->DestroyLocalTimer(timer.first);
+            }
+            else
+            {
+              auto it = this->LocalToTimer.find(timer.first);
+              if (it != this->LocalToTimer.end())
+              {
+                it->second.lastFire.tv_sec = ctv.tv_sec;
+                it->second.lastFire.tv_usec = ctv.tv_usec;
+              }
+            }
           }
         }
       }
-      for (auto exp : expired)
-      {
-        this->DestroyLocalTimer(exp);
-      }
     }
   }
 
   static std::set<vtkXRenderWindowInteractor*> Instances;
 
+  struct FDWaitInformation
+  {
+    // whether application was terminated
+    bool Done = false;
+    // whether `WaitForEvents` invokes `select` with a timeout argument.
+    bool UseTimeout = false;
+    // the number of events dispatched by `ProcessEvents()`
+    uint64_t NumEventsDispatched = 0;
+    // the timeout value provided to `select`. waits until an event occurs or this interval expires.
+    timeval WaitInterval;
+    // file descriptors that are monitored for activity in `WaitForEvents`
+    std::vector<int> RwiFileDescriptors;
+  } FDWaitInfo;
+
 private:
   int TimerIdCount;
-  std::map<int, vtkXRenderWindowInteractorTimer> LocalToTimer;
+  typedef std::map<int, vtkXRenderWindowInteractorTimer> LocalToTimerType;
+  LocalToTimerType LocalToTimer;
 };
 
 std::set<vtkXRenderWindowInteractor*> vtkXRenderWindowInteractorInternals::Instances;
@@ -207,18 +226,22 @@ void vtkXRenderWindowInteractor::TerminateApp()
 
 void vtkXRenderWindowInteractor::ProcessEvents()
 {
-  std::vector<int> rwiFileDescriptors;
-  fd_set in_fds;
-  struct timeval tv;
-  struct timeval minTv;
-
-  bool wait = true;
-  bool done = true;
-  minTv.tv_sec = 1000;
-  minTv.tv_usec = 1000;
-  XEvent event;
+  auto& internals = (*this->Internal);
+  auto& done = internals.FDWaitInfo.Done;
+  auto& evCount = internals.FDWaitInfo.NumEventsDispatched;
+  auto& waitTv = internals.FDWaitInfo.WaitInterval;
+  auto& rwiFileDescriptors = internals.FDWaitInfo.RwiFileDescriptors;
+  auto& useTimeout = internals.FDWaitInfo.UseTimeout;
 
+  // reset vars which help VTK wait for new events or timer timeouts.
+  done = true;
+  evCount = 0;
+  rwiFileDescriptors.clear();
   rwiFileDescriptors.reserve(vtkXRenderWindowInteractorInternals::Instances.size());
+  useTimeout = false;
+
+  // these file descriptors will be polled for new events. after pending events are processed, if
+  // any.
   for (auto rwi : vtkXRenderWindowInteractorInternals::Instances)
   {
     rwiFileDescriptors.push_back(ConnectionNumber(rwi->DisplayId));
@@ -227,20 +250,18 @@ void vtkXRenderWindowInteractor::ProcessEvents()
   for (auto rwi = vtkXRenderWindowInteractorInternals::Instances.begin();
        rwi != vtkXRenderWindowInteractorInternals::Instances.end();)
   {
-
+    XEvent event;
     if (XPending((*rwi)->DisplayId) == 0)
     {
       // get how long to wait for the next timer
-      (*rwi)->Internal->GetTimeToNextTimer(tv);
-      minTv.tv_sec = std::min(tv.tv_sec, minTv.tv_sec);
-      minTv.tv_usec = std::min(tv.tv_usec, minTv.tv_usec);
+      useTimeout = (*rwi)->Internal->GetTimeToNextTimer(waitTv);
     }
     while (XPending((*rwi)->DisplayId) != 0)
     {
       // If events are pending, dispatch them to the right RenderWindowInteractor
       XNextEvent((*rwi)->DisplayId, &event);
       (*rwi)->DispatchEvent(&event);
-      wait = false;
+      evCount++;
     }
     (*rwi)->FireTimers();
 
@@ -271,35 +292,55 @@ void vtkXRenderWindowInteractor::ProcessEvents()
       ++rwi;
     }
   }
-
-  if (wait && !done)
-  {
-    // select will wait until 'tv' elapses or something else wakes us
-    FD_ZERO(&in_fds);
-    for (auto rwiFileDescriptor : rwiFileDescriptors)
-    {
-      FD_SET(rwiFileDescriptor, &in_fds);
-    }
-    int maxFileDescriptor = *std::max_element(rwiFileDescriptors.begin(), rwiFileDescriptors.end());
-    select(maxFileDescriptor + 1, &in_fds, nullptr, nullptr, &minTv);
-  }
-
   this->Done = done;
 }
 
+//------------------------------------------------------------------------------
+void vtkXRenderWindowInteractor::WaitForEvents()
+{
+  auto& internals = (*this->Internal);
+  auto& fdWaitInfo = (internals.FDWaitInfo);
+  fd_set in_fds;
+
+  // select will wait until 'tv' elapses or something else wakes us
+  FD_ZERO(&in_fds);
+  int maxFd = -1;
+  timeval* timeout = fdWaitInfo.UseTimeout ? &fdWaitInfo.WaitInterval : nullptr;
+  for (const auto& rwiFileDescriptor : fdWaitInfo.RwiFileDescriptors)
+  {
+    FD_SET(rwiFileDescriptor, &in_fds);
+    maxFd = std::max<int>(maxFd, rwiFileDescriptor);
+  }
+  vtkDebugMacro(<< "wait");
+  select(maxFd + 1, &in_fds, nullptr, nullptr, timeout);
+}
+
 //------------------------------------------------------------------------------
 // This will start up the X event loop. If you
 // call this method it will loop processing X events until the
 // loop is exited.
 void vtkXRenderWindowInteractor::StartEventLoop()
 {
+  // cannot process events without an X display or window.
+  if (!this->DisplayId || !this->WindowId)
+  {
+    return;
+  }
   for (auto rwi : vtkXRenderWindowInteractorInternals::Instances)
   {
     rwi->Done = false;
   }
   do
   {
+    auto& internals = (*this->Internal);
+    auto& fdWaitInfo = (internals.FDWaitInfo);
+    // process pending events.
     this->ProcessEvents();
+    // wait for events only if no events were dispatched and application is not yet terminated.
+    if (!fdWaitInfo.NumEventsDispatched && !fdWaitInfo.Done)
+    {
+      this->WaitForEvents();
+    }
   } while (!this->Done);
 }
 
@@ -342,19 +383,25 @@ void vtkXRenderWindowInteractor::Initialize()
   size = ren->GetActualSize();
   size[0] = ((size[0] > 0) ? size[0] : 300);
   size[1] = ((size[1] > 0) ? size[1] : 300);
-  XSync(this->DisplayId, False);
+  if (this->DisplayId)
+  {
+    XSync(this->DisplayId, False);
+  }
 
   ren->Start();
   ren->End();
 
   this->WindowId = reinterpret_cast<Window>(ren->GetGenericWindowId());
 
-  XWindowAttributes attribs;
-  //  Find the current window size
-  XGetWindowAttributes(this->DisplayId, this->WindowId, &attribs);
+  if (this->DisplayId && this->WindowId)
+  {
+    XWindowAttributes attribs;
+    //  Find the current window size
+    XGetWindowAttributes(this->DisplayId, this->WindowId, &attribs);
 
-  size[0] = attribs.width;
-  size[1] = attribs.height;
+    size[0] = attribs.width;
+    size[1] = attribs.height;
+  }
   ren->SetSize(size[0], size[1]);
 
   this->Enable();
@@ -374,9 +421,15 @@ void vtkXRenderWindowInteractor::Finalize()
   if (this->OwnDisplay && this->DisplayId)
   {
     XCloseDisplay(this->DisplayId);
-    this->DisplayId = nullptr;
-    this->OwnDisplay = false;
   }
+
+  // disconnect from the display, even if we didn't own it
+  this->DisplayId = nullptr;
+  this->OwnDisplay = false;
+
+  // revert to uninitialized state
+  this->Initialized = false;
+  this->Enabled = false;
 }
 
 //------------------------------------------------------------------------------
@@ -387,6 +440,12 @@ void vtkXRenderWindowInteractor::Enable()
   {
     return;
   }
+  // When we're attached to an offscreen render window,
+  // there is no real X Display or X Window.
+  if (!this->WindowId || !this->DisplayId)
+  {
+    return;
+  }
 
   // Add the event handler to the system.
   // If we change the types of events processed by this handler, then
@@ -544,14 +603,15 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
         // just getting the last configure event
         event = &result;
       }
-      int width = (reinterpret_cast<XConfigureEvent*>(event))->width;
-      int height = (reinterpret_cast<XConfigureEvent*>(event))->height;
+      XConfigureEvent* configureEvent = reinterpret_cast<XConfigureEvent*>(event);
+      int width = configureEvent->width;
+      int height = configureEvent->height;
       if (width != this->Size[0] || height != this->Size[1])
       {
         bool resizeSmaller = width <= this->Size[0] && height <= this->Size[1];
         this->UpdateSizeNoXResize(width, height);
-        xp = (reinterpret_cast<XButtonEvent*>(event))->x;
-        yp = (reinterpret_cast<XButtonEvent*>(event))->y;
+        xp = configureEvent->x;
+        yp = configureEvent->y;
         this->SetEventPosition(xp, this->Size[1] - yp - 1);
         // only render if we are currently accepting events
         if (this->Enabled)
@@ -581,17 +641,18 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
       {
         return;
       }
-      int ctrl = ((reinterpret_cast<XButtonEvent*>(event))->state & ControlMask) ? 1 : 0;
-      int shift = ((reinterpret_cast<XButtonEvent*>(event))->state & ShiftMask) ? 1 : 0;
-      int alt = ((reinterpret_cast<XButtonEvent*>(event))->state & Mod1Mask) ? 1 : 0;
-      xp = (reinterpret_cast<XButtonEvent*>(event))->x;
-      yp = (reinterpret_cast<XButtonEvent*>(event))->y;
+      XButtonEvent* buttonEvent = reinterpret_cast<XButtonEvent*>(event);
+      int ctrl = (buttonEvent->state & ControlMask) ? 1 : 0;
+      int shift = (buttonEvent->state & ShiftMask) ? 1 : 0;
+      int alt = (buttonEvent->state & Mod1Mask) ? 1 : 0;
+      xp = buttonEvent->x;
+      yp = buttonEvent->y;
 
       // check for double click
       static int MousePressTime = 0;
       int repeat = 0;
       // 400 ms threshold by default is probably good to start
-      int eventTime = static_cast<int>(reinterpret_cast<XButtonEvent*>(event)->time);
+      int eventTime = static_cast<int>(buttonEvent->time);
       if ((eventTime - MousePressTime) < 400)
       {
         MousePressTime -= 2000; // no double click next time
@@ -604,7 +665,7 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
 
       this->SetEventInformationFlipY(xp, yp, ctrl, shift, 0, repeat);
       this->SetAltKey(alt);
-      switch ((reinterpret_cast<XButtonEvent*>(event))->button)
+      switch (buttonEvent->button)
       {
         case Button1:
           this->InvokeEvent(vtkCommand::LeftButtonPressEvent, nullptr);
@@ -631,14 +692,15 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
       {
         return;
       }
-      int ctrl = ((reinterpret_cast<XButtonEvent*>(event))->state & ControlMask) ? 1 : 0;
-      int shift = ((reinterpret_cast<XButtonEvent*>(event))->state & ShiftMask) ? 1 : 0;
-      int alt = ((reinterpret_cast<XButtonEvent*>(event))->state & Mod1Mask) ? 1 : 0;
-      xp = (reinterpret_cast<XButtonEvent*>(event))->x;
-      yp = (reinterpret_cast<XButtonEvent*>(event))->y;
+      XButtonEvent* buttonEvent = reinterpret_cast<XButtonEvent*>(event);
+      int ctrl = (buttonEvent->state & ControlMask) ? 1 : 0;
+      int shift = (buttonEvent->state & ShiftMask) ? 1 : 0;
+      int alt = (buttonEvent->state & Mod1Mask) ? 1 : 0;
+      xp = buttonEvent->x;
+      yp = buttonEvent->y;
       this->SetEventInformationFlipY(xp, yp, ctrl, shift);
       this->SetAltKey(alt);
-      switch ((reinterpret_cast<XButtonEvent*>(event))->button)
+      switch (buttonEvent->button)
       {
         case Button1:
           this->InvokeEvent(vtkCommand::LeftButtonReleaseEvent, nullptr);
@@ -662,7 +724,7 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
         XEnterWindowEvent* e = reinterpret_cast<XEnterWindowEvent*>(event);
         this->SetEventInformationFlipY(
           e->x, e->y, (e->state & ControlMask) != 0, (e->state & ShiftMask) != 0);
-        this->SetAltKey(((reinterpret_cast<XButtonEvent*>(event))->state & Mod1Mask) ? 1 : 0);
+        this->SetAltKey((e->state & Mod1Mask) ? 1 : 0);
         this->InvokeEvent(vtkCommand::EnterEvent, nullptr);
       }
     }
@@ -675,7 +737,7 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
         XLeaveWindowEvent* e = reinterpret_cast<XLeaveWindowEvent*>(event);
         this->SetEventInformationFlipY(
           e->x, e->y, (e->state & ControlMask) != 0, (e->state & ShiftMask) != 0);
-        this->SetAltKey(((reinterpret_cast<XButtonEvent*>(event))->state & Mod1Mask) ? 1 : 0);
+        this->SetAltKey((e->state & Mod1Mask) ? 1 : 0);
         this->InvokeEvent(vtkCommand::LeaveEvent, nullptr);
       }
     }
@@ -687,13 +749,19 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
       {
         return;
       }
-      int ctrl = ((reinterpret_cast<XButtonEvent*>(event))->state & ControlMask) ? 1 : 0;
-      int shift = ((reinterpret_cast<XButtonEvent*>(event))->state & ShiftMask) ? 1 : 0;
-      int alt = ((reinterpret_cast<XButtonEvent*>(event))->state & Mod1Mask) ? 1 : 0;
-      vtkKeySym keySym = XLookupKeysym(reinterpret_cast<XKeyEvent*>(event), 0);
-      char keyCode = keySym < 128 ? static_cast<char>(keySym) : 0;
-      xp = (reinterpret_cast<XKeyEvent*>(event))->x;
-      yp = (reinterpret_cast<XKeyEvent*>(event))->y;
+      XKeyEvent* keyEvent = reinterpret_cast<XKeyEvent*>(event);
+      int ctrl = (keyEvent->state & ControlMask) ? 1 : 0;
+      int shift = (keyEvent->state & ShiftMask) ? 1 : 0;
+      int alt = (keyEvent->state & Mod1Mask) ? 1 : 0;
+
+      // XLookupString provide a keycode as a char in Basic Latin and Latin1
+      // unicode blocks. We care only for the first char of the keycode.
+      char keyCode;
+      vtkKeySym keySym;
+      XLookupString(keyEvent, &keyCode, 1, &keySym, nullptr);
+
+      xp = keyEvent->x;
+      yp = keyEvent->y;
       this->SetEventInformationFlipY(xp, yp, ctrl, shift, keyCode, 1, XKeysymToString(keySym));
       this->SetAltKey(alt);
       this->InvokeEvent(vtkCommand::KeyPressEvent, nullptr);
@@ -707,13 +775,19 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
       {
         return;
       }
-      int ctrl = ((reinterpret_cast<XButtonEvent*>(event))->state & ControlMask) ? 1 : 0;
-      int shift = ((reinterpret_cast<XButtonEvent*>(event))->state & ShiftMask) ? 1 : 0;
-      int alt = ((reinterpret_cast<XButtonEvent*>(event))->state & Mod1Mask) ? 1 : 0;
-      vtkKeySym keySym = XLookupKeysym(reinterpret_cast<XKeyEvent*>(event), 0);
-      char keyCode = keySym < 128 ? static_cast<char>(keySym) : 0;
-      xp = (reinterpret_cast<XKeyEvent*>(event))->x;
-      yp = (reinterpret_cast<XKeyEvent*>(event))->y;
+      XKeyEvent* keyEvent = reinterpret_cast<XKeyEvent*>(event);
+      int ctrl = (keyEvent->state & ControlMask) ? 1 : 0;
+      int shift = (keyEvent->state & ShiftMask) ? 1 : 0;
+      int alt = (keyEvent->state & Mod1Mask) ? 1 : 0;
+
+      // XLookupString provide a keycode as a char in Basic Latin and Latin1
+      // unicode blocks. We care only for the first char of the keycode.
+      char keyCode;
+      vtkKeySym keySym;
+      XLookupString(keyEvent, &keyCode, 1, &keySym, nullptr);
+
+      xp = keyEvent->x;
+      yp = keyEvent->y;
       this->SetEventInformationFlipY(xp, yp, ctrl, shift, keyCode, 1, XKeysymToString(keySym));
       this->SetAltKey(alt);
       this->InvokeEvent(vtkCommand::KeyReleaseEvent, nullptr);
@@ -726,9 +800,10 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
       {
         return;
       }
-      int ctrl = ((reinterpret_cast<XButtonEvent*>(event))->state & ControlMask) ? 1 : 0;
-      int shift = ((reinterpret_cast<XButtonEvent*>(event))->state & ShiftMask) ? 1 : 0;
-      int alt = ((reinterpret_cast<XButtonEvent*>(event))->state & Mod1Mask) ? 1 : 0;
+      XMotionEvent* motionEvent = reinterpret_cast<XMotionEvent*>(event);
+      int ctrl = (motionEvent->state & ControlMask) ? 1 : 0;
+      int shift = (motionEvent->state & ShiftMask) ? 1 : 0;
+      int alt = (motionEvent->state & Mod1Mask) ? 1 : 0;
 
       // Note that even though the (x,y) location of the pointer is event structure,
       // we must call XQueryPointer for the hints (motion event compression) to
@@ -875,18 +950,14 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
         }
 
         // Recover the position
-        int xWindow, yWindow;
-        int xRoot = event->xclient.data.l[2] >> 16;
-        int yRoot = event->xclient.data.l[2] & 0xffff;
-        Window root = DefaultRootWindow(this->DisplayId);
-        Window child;
-        XTranslateCoordinates(
-          this->DisplayId, root, this->WindowId, xRoot, yRoot, &xWindow, &yWindow, &child);
-
-        // Convert it to VTK compatible location
-        double location[2];
-        location[0] = static_cast<double>(xWindow);
-        location[1] = static_cast<double>(this->Size[1] - yWindow - 1);
+        int location[2];
+        unsigned int keys;
+        this->GetMousePositionAndModifierKeysState(&location[0], &location[1], &keys);
+        int ctrl = keys & ControlMask ? 1 : 0;
+        int shift = keys & ShiftMask ? 1 : 0;
+        int alt = keys & Mod1Mask ? 1 : 0;
+        this->SetEventInformationFlipY(location[0], location[1], ctrl, shift);
+        this->SetAltKey(alt);
         this->InvokeEvent(vtkCommand::UpdateDropLocationEvent, location);
 
         // Reply that we are ready to copy the dragged data
@@ -950,13 +1021,18 @@ void vtkXRenderWindowInteractor::DispatchEvent(XEvent* event)
 
 //------------------------------------------------------------------------------
 void vtkXRenderWindowInteractor::GetMousePosition(int* x, int* y)
+{
+  unsigned int keys;
+  this->GetMousePositionAndModifierKeysState(x, y, &keys);
+}
+
+//------------------------------------------------------------------------------
+void vtkXRenderWindowInteractor::GetMousePositionAndModifierKeysState(
+  int* x, int* y, unsigned int* keys)
 {
   Window root, child;
   int root_x, root_y;
-  unsigned int keys;
-
-  XQueryPointer(this->DisplayId, this->WindowId, &root, &child, &root_x, &root_y, x, y, &keys);
-
+  XQueryPointer(this->DisplayId, this->WindowId, &root, &child, &root_x, &root_y, x, y, keys);
   *y = this->Size[1] - *y - 1;
 }
 
@@ -972,3 +1048,4 @@ void vtkXRenderWindowInteractor::GetMousePosition(int* x, int* y)
 // {
 //   vtkXRenderWindowInteractorCallback(w, client_data, event, ctd);
 // }
+VTK_ABI_NAMESPACE_END
