@@ -1,19 +1,8 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkSmartVolumeMapper.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include <cassert>
 
+#include "vtkAnariVolumeInterface.h"
 #include "vtkCellData.h"
 #include "vtkCellDataToPointData.h"
 #include "vtkColorTransferFunction.h"
@@ -38,6 +27,7 @@
 #include "vtkVolume.h"
 #include "vtkVolumeProperty.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkSmartVolumeMapper);
 
 //------------------------------------------------------------------------------
@@ -60,6 +50,8 @@ vtkSmartVolumeMapper::vtkSmartVolumeMapper()
   this->LowResGPUNecessary = 0;
   this->InterpolationMode = VTK_RESLICE_CUBIC;
 
+  this->UseJittering = 0;
+
   // If the render window has a desired update greater than or equal to the
   // interactive update rate, we apply certain optimizations to ensure that the
   // rendering is interactive.
@@ -140,6 +132,7 @@ vtkSmartVolumeMapper::vtkSmartVolumeMapper()
   cb->Delete();
 
   this->OSPRayMapper = nullptr;
+  this->AnariMapper = nullptr;
 
   this->Transfer2DYAxisArray = nullptr;
 
@@ -187,6 +180,11 @@ vtkSmartVolumeMapper::~vtkSmartVolumeMapper()
     this->OSPRayMapper->Delete();
     this->OSPRayMapper = nullptr;
   }
+  if (this->AnariMapper)
+  {
+    this->AnariMapper->Delete();
+    this->AnariMapper = nullptr;
+  }
 
   this->SetTransfer2DYAxisArray(nullptr);
 
@@ -249,6 +247,13 @@ void vtkSmartVolumeMapper::Render(vtkRenderer* ren, vtkVolume* vol)
       }
       this->OSPRayMapper->Render(ren, vol);
       break;
+    case vtkSmartVolumeMapper::AnariRenderMode:
+      if (!this->AnariMapper)
+      {
+        this->AnariMapper = vtkAnariVolumeInterface::New();
+      }
+      this->AnariMapper->Render(ren, vol);
+      break;
     case vtkSmartVolumeMapper::InvalidRenderMode:
       // Silently fail - a render mode that is not
       // valid was selected so we will render nothing
@@ -402,6 +407,10 @@ void vtkSmartVolumeMapper::ComputeRenderMode(vtkRenderer* ren, vtkVolume* vol)
       this->CurrentRenderMode = vtkSmartVolumeMapper::OSPRayRenderMode;
       break;
 
+    case vtkSmartVolumeMapper::AnariRenderMode:
+      this->CurrentRenderMode = vtkSmartVolumeMapper::AnariRenderMode;
+      break;
+
       // This should never happen since the SetRequestedRenderMode
       // protects against invalid states
     default:
@@ -466,6 +475,7 @@ void vtkSmartVolumeMapper::ComputeRenderMode(vtkRenderer* ren, vtkVolume* vol)
       this->GPUMapper->SetBlendMode(this->GetBlendMode());
       this->GPUMapper->SetFinalColorWindow(this->FinalColorWindow);
       this->GPUMapper->SetFinalColorLevel(this->FinalColorLevel);
+      this->GPUMapper->SetUseJittering(this->UseJittering);
       this->GPUMapper->SetSampleDistance(this->SampleDistance);
       this->GPUMapper->SetTransfer2DYAxisArray(this->Transfer2DYAxisArray);
       this->GPUMapper->SetComputeNormalFromOpacity(this->ComputeNormalFromOpacity);
@@ -503,6 +513,7 @@ void vtkSmartVolumeMapper::ComputeRenderMode(vtkRenderer* ren, vtkVolume* vol)
         this->GPULowResMapper->SetBlendMode(this->GetBlendMode());
         this->GPULowResMapper->SetFinalColorWindow(this->FinalColorWindow);
         this->GPULowResMapper->SetFinalColorLevel(this->FinalColorLevel);
+        this->GPULowResMapper->SetUseJittering(this->UseJittering);
         this->GPULowResMapper->SetSampleDistance(this->SampleDistance);
       }
       else
@@ -515,6 +526,9 @@ void vtkSmartVolumeMapper::ComputeRenderMode(vtkRenderer* ren, vtkVolume* vol)
     case vtkSmartVolumeMapper::OSPRayRenderMode:
       break;
 
+    case vtkSmartVolumeMapper::AnariRenderMode:
+      break;
+
       // The user selected a RequestedRenderMode that is
       // not supported. In this case the mapper will just
       // silently fail.
@@ -833,6 +847,12 @@ void vtkSmartVolumeMapper::SetRequestedRenderModeToOSPRay()
   this->SetRequestedRenderMode(vtkSmartVolumeMapper::OSPRayRenderMode);
 }
 
+//------------------------------------------------------------------------------
+void vtkSmartVolumeMapper::SetRequestedRenderModeToAnari()
+{
+  this->SetRequestedRenderMode(vtkSmartVolumeMapper::AnariRenderMode);
+}
+
 //------------------------------------------------------------------------------
 void vtkSmartVolumeMapper::ReleaseGraphicsResources(vtkWindow* w)
 {
@@ -931,3 +951,4 @@ void vtkSmartVolumeMapper::SetVectorMode(int mode)
     this->Modified();
   }
 }
+VTK_ABI_NAMESPACE_END
