@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkSmartVolumeMapper.h
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkSmartVolumeMapper
  * @brief   Adaptive volume mapper
@@ -76,18 +64,21 @@
 #include "vtkImageReslice.h"                 // for VTK_RESLICE_NEAREST, VTK_RESLICE_CUBIC
 #include "vtkRenderingVolumeOpenGL2Module.h" // For export macro
 #include "vtkVolumeMapper.h"
+#include "vtkWrappingHints.h" // For VTK_MARSHALAUTO
 
+VTK_ABI_NAMESPACE_BEGIN
 class vtkFixedPointVolumeRayCastMapper;
 class vtkGPUVolumeRayCastMapper;
 class vtkImageResample;
 class vtkMultiBlockVolumeMapper;
 class vtkOSPRayVolumeInterface;
+class vtkAnariVolumeInterface;
 class vtkRenderWindow;
 class vtkVolume;
 class vtkVolumeProperty;
 class vtkImageMagnitude;
 
-class VTKRENDERINGVOLUMEOPENGL2_EXPORT vtkSmartVolumeMapper : public vtkVolumeMapper
+class VTKRENDERINGVOLUMEOPENGL2_EXPORT VTK_MARSHALAUTO vtkSmartVolumeMapper : public vtkVolumeMapper
 {
 public:
   static vtkSmartVolumeMapper* New();
@@ -140,8 +131,9 @@ public:
     RayCastRenderMode = 1,
     GPURenderMode = 2,
     OSPRayRenderMode = 3,
-    UndefinedRenderMode = 4,
-    InvalidRenderMode = 5
+    AnariRenderMode = 4,
+    UndefinedRenderMode = 5,
+    InvalidRenderMode = 6
   };
 
   /**
@@ -177,6 +169,12 @@ public:
    */
   void SetRequestedRenderModeToOSPRay();
 
+  /**
+   * Set the requested render mode to vtkSmartVolumeMapper::AnariRenderMode.
+   * This option will use ANARI to do rendering exclusively.
+   */
+  void SetRequestedRenderModeToAnari();
+
   ///@{
   /**
    * Get the requested render mode.
@@ -233,6 +231,17 @@ public:
   void CreateCanonicalView(vtkRenderer* ren, vtkVolume* volume, vtkVolume* volume2,
     vtkImageData* image, int blend_mode, double viewDirection[3], double viewUp[3]);
 
+  ///@{
+  /**
+   * If UseJittering is on, each ray traversal direction will be
+   * perturbed slightly using a noise-texture to get rid of wood-grain
+   * effect. This is only used by the GPU mapper.
+   */
+  vtkSetClampMacro(UseJittering, vtkTypeBool, 0, 1);
+  vtkGetMacro(UseJittering, vtkTypeBool);
+  vtkBooleanMacro(UseJittering, vtkTypeBool);
+  ///@}
+
   ///@{
   /**
    * If the DesiredUpdateRate of the vtkRenderWindow that caused the Render
@@ -375,9 +384,10 @@ public:
 
   vtkSetMacro(LowResMode, int);
   vtkGetMacro(LowResMode, int)
-    ///@}
+  ///@}
 
-    protected : vtkSmartVolumeMapper();
+protected:
+  vtkSmartVolumeMapper();
   ~vtkSmartVolumeMapper() override;
 
   /**
@@ -490,6 +500,11 @@ public:
    */
   int InitializedBlendMode;
 
+  /**
+   * Enable / disable stochastic jittering
+   */
+  vtkTypeBool UseJittering;
+
   /**
    * The distance between sample points along the ray
    */
@@ -578,6 +593,8 @@ private:
   void operator=(const vtkSmartVolumeMapper&) = delete;
 
   vtkOSPRayVolumeInterface* OSPRayMapper;
+  vtkAnariVolumeInterface* AnariMapper;
 };
 
+VTK_ABI_NAMESPACE_END
 #endif
