@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   ParaView
-  Module:    vtkSMParaViewPipelineController.cxx
-
-  Copyright (c) Kitware, Inc.
-  All rights reserved.
-  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkSMParaViewPipelineController.h"
 
 #include "vtkClientServerStreamInstantiator.h"
@@ -283,12 +271,6 @@ bool vtkSMParaViewPipelineController::CreateAnimationHelpers(vtkSMProxy* proxy)
   return true;
 }
 
-//----------------------------------------------------------------------------
-void vtkSMParaViewPipelineController::DoMaterialSetup(vtkSMProxy* vtkNotUsed(proxy))
-{
-  // expected to be overridden by rendering capable subclass
-}
-
 //----------------------------------------------------------------------------
 bool vtkSMParaViewPipelineController::InitializeSession(vtkSMSession* session)
 {
@@ -349,7 +331,6 @@ bool vtkSMParaViewPipelineController::InitializeSession(vtkSMSession* session)
     {
       this->InitializeProxy(materialLib);
       materialLib->UpdateVTKObjects();
-      this->DoMaterialSetup(materialLib.Get());
       pxm->RegisterProxy("materiallibrary", materialLib);
     }
 #endif
@@ -523,7 +504,7 @@ bool vtkSMParaViewPipelineController::RegisterPipelineProxy(
 
   // Now register the proxy itself.
   // If proxyname is nullptr, the proxy manager makes up a name.
-  if (proxyname == nullptr)
+  if (proxyname == nullptr || proxyname[0] == 0)
   {
     auto pname = proxy->GetSessionProxyManager()->RegisterProxy("sources", proxy);
 
@@ -613,7 +594,7 @@ bool vtkSMParaViewPipelineController::RegisterViewProxy(vtkSMProxy* proxy, const
   this->ProcessInitializationHelperRegistration(proxy);
 
   // Now register the proxy itself.
-  if (proxyname == nullptr)
+  if (proxyname == nullptr || proxyname[0] == 0)
   {
     auto pname = proxy->GetSessionProxyManager()->RegisterProxy("views", proxy);
 
@@ -738,7 +719,8 @@ bool vtkSMParaViewPipelineController::UnRegisterViewProxy(
 }
 
 //----------------------------------------------------------------------------
-bool vtkSMParaViewPipelineController::RegisterRepresentationProxy(vtkSMProxy* proxy)
+bool vtkSMParaViewPipelineController::RegisterRepresentationProxy(
+  vtkSMProxy* proxy, const char* name)
 {
   if (!proxy)
   {
@@ -752,7 +734,20 @@ bool vtkSMParaViewPipelineController::RegisterRepresentationProxy(vtkSMProxy* pr
   this->ProcessInitializationHelperRegistration(proxy);
 
   // Register the proxy itself.
-  proxy->GetSessionProxyManager()->RegisterProxy("representations", proxy);
+  if (name == nullptr || name[0] == 0)
+  {
+    auto pname = proxy->GetSessionProxyManager()->RegisterProxy("representations", proxy);
+
+    // assign a name for logging
+    proxy->SetLogName(pname.c_str());
+  }
+  else
+  {
+    proxy->GetSessionProxyManager()->RegisterProxy("representations", name, proxy);
+
+    // assign a name for logging
+    proxy->SetLogName(name);
+  }
   return true;
 }
 
@@ -991,7 +986,7 @@ bool vtkSMParaViewPipelineController::RegisterTextureProxy(vtkSMProxy* proxy, co
     return false;
   }
 
-  SM_SCOPED_TRACE(RegisterTextureProxy).arg("proxy", proxy);
+  SM_SCOPED_TRACE(RegisterTextureProxy).arg("proxy", proxy).arg("filename", filename);
 
   // Register proxies created for proxy list domains.
   this->RegisterProxiesForProxyListDomains(proxy);
@@ -1005,6 +1000,28 @@ bool vtkSMParaViewPipelineController::RegisterTextureProxy(vtkSMProxy* proxy, co
   return true;
 }
 
+//----------------------------------------------------------------------------
+bool vtkSMParaViewPipelineController::RegisterTextureProxy(
+  vtkSMProxy* proxy, const char* trivialProducerKey, const char* proxyname)
+{
+  if (!proxy)
+  {
+    return false;
+  }
+
+  SM_SCOPED_TRACE(RegisterTextureProxy)
+    .arg("proxy", proxy)
+    .arg("trivial_producer_key", trivialProducerKey)
+    .arg("proxyname", proxyname);
+
+  // Register proxies created for proxy list domains.
+  this->RegisterProxiesForProxyListDomains(proxy);
+
+  // Register the proxy itself.
+  proxy->GetSessionProxyManager()->RegisterProxy("textures", proxyname, proxy);
+  return true;
+}
+
 //----------------------------------------------------------------------------
 bool vtkSMParaViewPipelineController::RegisterExtractorProxy(
   vtkSMProxy* proxy, const char* proxyname)
