@@ -16,8 +16,7 @@
 
 int main(int argc, char** argv)
 {
-  auto opts = vtkm::cont::InitializeOptions::DefaultAnyDevice;
-  vtkm::cont::InitializeResult config = vtkm::cont::Initialize(argc, argv, opts);
+  vtkm::cont::Initialize(argc, argv);
 
   const char* input = "data/kitchen.vtk";
   vtkm::io::VTKDataSetReader reader(input);
