@@ -13,7 +13,6 @@
 #include <vtkm/cont/vtkm_cont_export.h>
 
 #include <vtkm/Assert.h>
-#include <vtkm/Deprecated.h>
 #include <vtkm/Flags.h>
 #include <vtkm/Types.h>
 
@@ -130,6 +129,11 @@ struct ArrayHandleCheck
 {
 };
 
+/// @brief Checks that the given type is a `vtkm::cont::ArrayHandle`.
+///
+/// If the type is not a `vtkm::cont::ArrayHandle` or a subclass, a static assert will
+/// cause a compile exception. This is a good way to ensure that a template argument
+/// that is assumed to be an array handle type actually is.
 #define VTKM_IS_ARRAY_HANDLE(T) VTKM_STATIC_ASSERT(::vtkm::cont::internal::ArrayHandleCheck<T>{})
 
 } // namespace internal
@@ -155,10 +159,7 @@ struct GetTypeInParentheses<void(T)>
   VTKM_IS_ARRAY_HANDLE(Superclass);                                                                \
                                                                                                    \
   VTKM_CONT                                                                                        \
-  classname()                                                                                      \
-    : Superclass()                                                                                 \
-  {                                                                                                \
-  }                                                                                                \
+  classname() {}                                                                                   \
                                                                                                    \
   VTKM_CONT                                                                                        \
   classname(const Thisclass& src)                                                                  \
@@ -187,6 +188,18 @@ struct GetTypeInParentheses<void(T)>
   }                                                                                                \
                                                                                                    \
   VTKM_CONT                                                                                        \
+  explicit classname(const std::vector<vtkm::cont::internal::Buffer>& buffers)                     \
+    : Superclass(buffers)                                                                          \
+  {                                                                                                \
+  }                                                                                                \
+                                                                                                   \
+  VTKM_CONT                                                                                        \
+  explicit classname(std::vector<vtkm::cont::internal::Buffer>&& buffers) noexcept                 \
+    : Superclass(std::move(buffers))                                                               \
+  {                                                                                                \
+  }                                                                                                \
+                                                                                                   \
+  VTKM_CONT                                                                                        \
   Thisclass& operator=(const Thisclass& src)                                                       \
   {                                                                                                \
     this->Superclass::operator=(src);                                                              \
@@ -201,7 +214,10 @@ struct GetTypeInParentheses<void(T)>
   }                                                                                                \
                                                                                                    \
   using ValueType = typename__ Superclass::ValueType;                                              \
-  using StorageTag = typename__ Superclass::StorageTag
+  using StorageTag = typename__ Superclass::StorageTag;                                            \
+  using StorageType = typename__ Superclass::StorageType;                                          \
+  using ReadPortalType = typename__ Superclass::ReadPortalType;                                    \
+  using WritePortalType = typename__ Superclass::WritePortalType
 
 /// \brief Macro to make default methods in ArrayHandle subclasses.
 ///
@@ -260,12 +276,9 @@ VTKM_CONT_EXPORT VTKM_CONT bool ArrayHandleIsOnDevice(
   const std::vector<vtkm::cont::internal::Buffer>& buffers,
   vtkm::cont::DeviceAdapterId device);
 
-VTKM_CONT_EXPORT VTKM_CONT vtkm::cont::DeviceAdapterId ArrayHandleGetDeviceAdapterId(
-  const std::vector<vtkm::cont::internal::Buffer>& buffers);
-
 } // namespace detail
 
-/// \brief Manages an array-worth of data.
+/// @brief Manages an array-worth of data.
 ///
 /// `ArrayHandle` manages as array of data that can be manipulated by VTKm
 /// algorithms. The `ArrayHandle` may have up to two copies of the array, one
@@ -295,26 +308,15 @@ public:
   using StorageTag = StorageTag_;
   using StorageType = vtkm::cont::internal::Storage<ValueType, StorageTag>;
 
+  /// The type of portal used when accessing data in a read-only mode.
   using ReadPortalType = typename StorageType::ReadPortalType;
+  /// The type of portal used when accessing data in a read-write mode.
   using WritePortalType = typename StorageType::WritePortalType;
 
-  // TODO: Deprecate this
-  template <typename Device>
-  struct VTKM_DEPRECATED(1.6, "Use ReadPortalType and WritePortalType.") ExecutionTypes
-  {
-    using Portal = WritePortalType;
-    using PortalConst = ReadPortalType;
-  };
-
-  using PortalControl VTKM_DEPRECATED(1.6, "Use ArrayHandle::WritePortalType instead.") =
-    WritePortalType;
-  using PortalConstControl VTKM_DEPRECATED(1.6, "Use ArrayHandle::ReadPortalType instead.") =
-    ReadPortalType;
-
   /// Constructs an empty ArrayHandle.
   ///
   VTKM_CONT ArrayHandle()
-    : Buffers(static_cast<std::size_t>(StorageType::GetNumberOfBuffers()))
+    : Buffers(StorageType::CreateBuffers())
   {
   }
 
@@ -342,28 +344,22 @@ public:
   {
   }
 
-  ///@{
   /// Special constructor for subclass specializations that need to set the
   /// initial state array. Used when pulling data from other sources.
   ///
-  VTKM_CONT ArrayHandle(const std::vector<vtkm::cont::internal::Buffer>& buffers)
+  VTKM_CONT explicit ArrayHandle(const std::vector<vtkm::cont::internal::Buffer>& buffers)
     : Buffers(buffers)
   {
-    VTKM_ASSERT(static_cast<vtkm::IdComponent>(this->Buffers.size()) == this->GetNumberOfBuffers());
   }
 
-  VTKM_CONT ArrayHandle(std::vector<vtkm::cont::internal::Buffer>&& buffers) noexcept
+  /// Special constructor for subclass specializations that need to set the
+  /// initial state array. Used when pulling data from other sources.
+  ///
+  VTKM_CONT explicit ArrayHandle(std::vector<vtkm::cont::internal::Buffer>&& buffers) noexcept
     : Buffers(std::move(buffers))
   {
-    VTKM_ASSERT(static_cast<vtkm::IdComponent>(this->Buffers.size()) == this->GetNumberOfBuffers());
   }
 
-  VTKM_CONT ArrayHandle(const vtkm::cont::internal::Buffer* buffers)
-    : Buffers(buffers, buffers + StorageType::GetNumberOfBuffers())
-  {
-  }
-  ///@}
-
   /// Destructs an empty ArrayHandle.
   ///
   /// Implemented so that it is defined exclusively in the control environment.
@@ -373,7 +369,7 @@ public:
   ///
   VTKM_CONT ~ArrayHandle() {}
 
-  /// \brief Copies an ArrayHandle
+  /// @brief Shallow copies an ArrayHandle
   ///
   VTKM_CONT
   vtkm::cont::ArrayHandle<ValueType, StorageTag>& operator=(
@@ -383,7 +379,7 @@ public:
     return *this;
   }
 
-  /// \brief Move and Assignment of an ArrayHandle
+  /// @brief Move and Assignment of an ArrayHandle
   ///
   VTKM_CONT
   vtkm::cont::ArrayHandle<ValueType, StorageTag>& operator=(
@@ -393,7 +389,7 @@ public:
     return *this;
   }
 
-  /// Like a pointer, two \c ArrayHandles are considered equal if they point
+  /// Like a pointer, two `ArrayHandle`s are considered equal if they point
   /// to the same location in memory.
   ///
   VTKM_CONT
@@ -420,45 +416,11 @@ public:
     return true; // different valuetype and/or storage
   }
 
-  VTKM_CONT static constexpr vtkm::IdComponent GetNumberOfBuffers()
-  {
-    return StorageType::GetNumberOfBuffers();
-  }
-
   /// Get the storage.
   ///
   VTKM_CONT StorageType GetStorage() const { return StorageType{}; }
 
-  /// Get the array portal of the control array.
-  /// Since worklet invocations are asynchronous and this routine is a synchronization point,
-  /// exceptions maybe thrown for errors from previously executed worklets.
-  ///
-  /// \deprecated Use `WritePortal` instead.
-  ///
-  VTKM_CONT
-  VTKM_DEPRECATED(1.6,
-                  "Use ArrayHandle::WritePortal() instead. "
-                  "Note that the returned portal will lock the array while it is in scope.")
-
-  /// \cond NOPE
-  WritePortalType GetPortalControl() const { return this->WritePortal(); }
-  /// \endcond
-
-  /// Get the array portal of the control array.
-  /// Since worklet invocations are asynchronous and this routine is a synchronization point,
-  /// exceptions maybe thrown for errors from previously executed worklets.
-  ///
-  /// \deprecated Use `ReadPortal` instead.
-  ///
-  VTKM_CONT
-  VTKM_DEPRECATED(1.6,
-                  "Use ArrayHandle::ReadPortal() instead. "
-                  "Note that the returned portal will lock the array while it is in scope.")
-  /// \cond NOPE
-  ReadPortalType GetPortalConstControl() const { return this->ReadPortal(); }
-  /// \endcond
-
-  /// \brief Get an array portal that can be used in the control environment.
+  /// @brief Get an array portal that can be used in the control environment.
   ///
   /// The returned array can be used in the control environment to read values from the array. (It
   /// is not possible to write to the returned portal. That is `Get` will work on the portal, but
@@ -471,11 +433,16 @@ public:
   VTKM_CONT ReadPortalType ReadPortal() const
   {
     vtkm::cont::Token token;
+    return this->ReadPortal(token);
+  }
+  /// @copydoc ReadPortalType
+  VTKM_CONT ReadPortalType ReadPortal(vtkm::cont::Token& token) const
+  {
     return StorageType::CreateReadPortal(
       this->GetBuffers(), vtkm::cont::DeviceAdapterTagUndefined{}, token);
   }
 
-  /// \brief Get an array portal that can be used in the control environment.
+  /// @brief Get an array portal that can be used in the control environment.
   ///
   /// The returned array can be used in the control environment to reand and write values to the
   /// array.
@@ -487,7 +454,11 @@ public:
   VTKM_CONT WritePortalType WritePortal() const
   {
     vtkm::cont::Token token;
-
+    return this->WritePortal(token);
+  }
+  /// @copydoc WritePortal
+  VTKM_CONT WritePortalType WritePortal(vtkm::cont::Token& token) const
+  {
     return StorageType::CreateWritePortal(
       this->GetBuffers(), vtkm::cont::DeviceAdapterTagUndefined{}, token);
   }
@@ -499,16 +470,21 @@ public:
     return StorageType::GetNumberOfValues(this->GetBuffers());
   }
 
-  ///@{
-  ///  \brief Allocates an array large enough to hold the given number of values.
+  /// @copydoc vtkm::cont::UnknownArrayHandle::GetNumberOfComponentsFlat
+  VTKM_CONT vtkm::IdComponent GetNumberOfComponentsFlat() const
+  {
+    return StorageType::GetNumberOfComponentsFlat(this->GetBuffers());
+  }
+
+  /// @brief Allocates an array large enough to hold the given number of values.
   ///
   /// The allocation may be done on an already existing array. If so, then the data
   /// are preserved as best as possible if the preserve flag is set to `vtkm::CopyFlag::On`.
   /// If the preserve flag is set to `vtkm::CopyFlag::Off` (the default), any existing data
   /// could be wiped out.
   ///
-  /// This method can throw `ErrorBadAllocation` if the array cannot be allocated or
-  /// `ErrorBadValue` if the allocation is not feasible (for example, the
+  /// This method can throw `vtkm::cont::ErrorBadAllocation` if the array cannot be allocated or
+  /// `vtkm::cont::ErrorBadValue` if the allocation is not feasible (for example, the
   /// array storage is read-only).
   ///
   VTKM_CONT void Allocate(vtkm::Id numberOfValues,
@@ -518,16 +494,15 @@ public:
     StorageType::ResizeBuffers(numberOfValues, this->GetBuffers(), preserve, token);
   }
 
+  /// @copydoc Allocate
   VTKM_CONT void Allocate(vtkm::Id numberOfValues,
                           vtkm::CopyFlag preserve = vtkm::CopyFlag::Off) const
   {
     vtkm::cont::Token token;
     this->Allocate(numberOfValues, preserve, token);
   }
-  ///@}
 
-  ///@{
-  /// \brief Allocates an array and fills it with an initial value.
+  /// @brief Allocates an array and fills it with an initial value.
   ///
   /// `AllocateAndFill` behaves similar to `Allocate` except that after allocation it fills
   /// the array with a given `fillValue`. This method is convenient when you wish to initialize
@@ -560,6 +535,7 @@ public:
     }
   }
 
+  /// @copydoc AllocateAndFill
   VTKM_CONT void AllocateAndFill(vtkm::Id numberOfValues,
                                  const ValueType& fillValue,
                                  vtkm::CopyFlag preserve = vtkm::CopyFlag::Off) const
@@ -567,20 +543,13 @@ public:
     vtkm::cont::Token token;
     this->AllocateAndFill(numberOfValues, fillValue, preserve, token);
   }
-  ///@}
 
-  VTKM_DEPRECATED(1.6, "Use Allocate(n, vtkm::CopyFlag::On) instead of Shrink(n).")
-  VTKM_CONT void Shrink(vtkm::Id numberOfValues)
-  {
-    this->Allocate(numberOfValues, vtkm::CopyFlag::On);
-  }
-
-  /// @{
-  /// \brief Fills the array with a given value.
+  /// @brief Fills the array with a given value.
   ///
-  /// After calling this method, every entry in the array from `startIndex` to `endIndex`.
-  /// of the array is set to `fillValue`. If `startIndex` or `endIndex` is not specified,
-  /// then the fill happens from the begining or end, respectively.
+  /// After calling this method, every entry in the array from `startIndex` (inclusive)
+  /// to `endIndex` (exclusive) of the array is set to `fillValue`. If `startIndex` or
+  /// `endIndex` is not specified, then the fill happens from the begining or end,
+  /// respectively.
   ///
   VTKM_CONT void Fill(const ValueType& fillValue,
                       vtkm::Id startIndex,
@@ -589,17 +558,18 @@ public:
   {
     StorageType::Fill(this->GetBuffers(), fillValue, startIndex, endIndex, token);
   }
+  /// @copydoc Fill
   VTKM_CONT void Fill(const ValueType& fillValue, vtkm::Id startIndex, vtkm::Id endIndex) const
   {
     vtkm::cont::Token token;
     this->Fill(fillValue, startIndex, endIndex, token);
   }
+  /// @copydoc Fill
   VTKM_CONT void Fill(const ValueType& fillValue, vtkm::Id startIndex = 0) const
   {
     vtkm::cont::Token token;
     this->Fill(fillValue, startIndex, this->GetNumberOfValues(), token);
   }
-  /// @}
 
   /// Releases any resources being used in the execution environment (that are
   /// not being shared by the control environment).
@@ -673,25 +643,6 @@ public:
     return StorageType::CreateWritePortal(this->GetBuffers(), device, token);
   }
 
-  VTKM_CONT VTKM_DEPRECATED(1.6, "PrepareForInput now requires a vtkm::cont::Token object.")
-    ReadPortalType PrepareForInput(vtkm::cont::DeviceAdapterId device) const
-  {
-    vtkm::cont::Token token;
-    return this->PrepareForInput(device, token);
-  }
-  VTKM_CONT VTKM_DEPRECATED(1.6, "PrepareForOutput now requires a vtkm::cont::Token object.")
-    WritePortalType PrepareForOutput(vtkm::Id numberOfValues, vtkm::cont::DeviceAdapterId device)
-  {
-    vtkm::cont::Token token;
-    return this->PrepareForOutput(numberOfValues, device, token);
-  }
-  VTKM_CONT VTKM_DEPRECATED(1.6, "PrepareForInPlace now requires a vtkm::cont::Token object.")
-    WritePortalType PrepareForInPlace(vtkm::cont::DeviceAdapterId device) const
-  {
-    vtkm::cont::Token token;
-    return this->PrepareForInPlace(device, token);
-  }
-
   /// Returns true if the ArrayHandle's data is on the given device. If the data are on the given
   /// device, then preparing for that device should not require any data movement.
   ///
@@ -708,19 +659,6 @@ public:
     return this->IsOnDevice(vtkm::cont::DeviceAdapterTagUndefined{});
   }
 
-  /// Returns a DeviceAdapterId for a device currently allocated on. If there is no device
-  /// with an up-to-date copy of the data, VTKM_DEVICE_ADAPTER_UNDEFINED is
-  /// returned.
-  ///
-  /// Note that in a multithreaded environment the validity of this result can
-  /// change.
-  ///
-  VTKM_CONT
-  VTKM_DEPRECATED(1.7, "Use ArrayHandle::IsOnDevice.") DeviceAdapterId GetDeviceAdapterId() const
-  {
-    return detail::ArrayHandleGetDeviceAdapterId(this->Buffers);
-  }
-
   /// Synchronizes the control array with the execution array. If either the
   /// user array or control array is already valid, this method does nothing
   /// (because the data is already available in the control environment).
@@ -776,9 +714,15 @@ public:
     }
   }
 
-  /// Returns the internal `Buffer` structures that hold the data.
+  /// \brief Returns the internal `Buffer` structures that hold the data.
   ///
-  VTKM_CONT vtkm::cont::internal::Buffer* GetBuffers() const { return this->Buffers.data(); }
+  /// Note that great care should be taken when modifying buffers outside of the ArrayHandle.
+  ///
+  VTKM_CONT const std::vector<vtkm::cont::internal::Buffer>& GetBuffers() const
+  {
+    return this->Buffers;
+  }
+  VTKM_CONT std::vector<vtkm::cont::internal::Buffer>& GetBuffers() { return this->Buffers; }
 
 private:
   mutable std::vector<vtkm::cont::internal::Buffer> Buffers;
@@ -789,11 +733,13 @@ protected:
     this->Buffers[static_cast<std::size_t>(index)] = buffer;
   }
 
-  // BufferContainer must be an iteratable container of Buffer objects.
-  template <typename BufferContainer>
-  VTKM_CONT void SetBuffers(const BufferContainer& buffers)
+  VTKM_CONT void SetBuffers(const std::vector<vtkm::cont::internal::Buffer>& buffers)
   {
-    std::copy(buffers.begin(), buffers.end(), this->Iterators->Buffers.begin());
+    this->Buffers = buffers;
+  }
+  VTKM_CONT void SetBuffers(std::vector<vtkm::cont::internal::Buffer>&& buffers)
+  {
+    this->Buffers = std::move(buffers);
   }
 };
 
@@ -915,6 +861,25 @@ namespace internal
 namespace detail
 {
 
+VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer>&);
+template <typename T, typename S, typename... Args>
+VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer>& buffers,
+                                        const vtkm::cont::ArrayHandle<T, S>& array,
+                                        const Args&... args);
+template <typename... Args>
+VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer>& buffers,
+                                        const vtkm::cont::internal::Buffer& buffer,
+                                        const Args&... args);
+
+template <typename... Args>
+VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer>& buffers,
+                                        const std::vector<vtkm::cont::internal::Buffer>& addbuffs,
+                                        const Args&... args);
+template <typename Arg0, typename... Args>
+VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer>& buffers,
+                                        const Arg0& arg0,
+                                        const Args&... args);
+
 VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer>&)
 {
   // Nothing left to add.
@@ -925,9 +890,7 @@ VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer
                                         const vtkm::cont::ArrayHandle<T, S>& array,
                                         const Args&... args)
 {
-  vtkm::cont::internal::Buffer* arrayBuffers = array.GetBuffers();
-  buffers.insert(buffers.end(), arrayBuffers, arrayBuffers + array.GetNumberOfBuffers());
-  CreateBuffersImpl(buffers, args...);
+  CreateBuffersImpl(buffers, array.GetBuffers(), args...);
 }
 
 template <typename... Args>
@@ -948,11 +911,6 @@ VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer
   CreateBuffersImpl(buffers, args...);
 }
 
-template <typename Arg0, typename... Args>
-VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer>& buffers,
-                                        const Arg0& arg0,
-                                        const Args&... args);
-
 template <typename T, typename S, typename... Args>
 VTKM_CONT inline void CreateBuffersResolveArrays(std::vector<vtkm::cont::internal::Buffer>& buffers,
                                                  std::true_type,
@@ -1001,7 +959,7 @@ VTKM_CONT inline void CreateBuffersImpl(std::vector<vtkm::cont::internal::Buffer
 ///   - `ArrayHandle`: The buffers from the `ArrayHandle` are added to the list.
 ///   - `Buffer`: A copy of the buffer is added to the list.
 ///   - `std::vector<Buffer>`: A copy of all buffers in this vector are added to the list.
-///   - Anything else: A buffer with the given object attached as metadata is
+///   - Anything else: A buffer with the given object attached as metadata is added to the list.
 ///
 template <typename... Args>
 VTKM_CONT inline std::vector<vtkm::cont::internal::Buffer> CreateBuffers(const Args&... args)
