@@ -18,6 +18,7 @@
 #include <vtkm/cont/vtkm_cont_export.h>
 
 #include <iostream>
+#include <memory>
 #include <sstream>
 #include <string>
 #include <typeindex>
@@ -143,7 +144,7 @@
 /// \endcode
 
 /// \def VTKM_LOG_SCOPE_FUNCTION(level)
-/// Equivalent to VTKM_LOG_SCOPE(__func__);
+/// Equivalent to `VTKM_LOG_SCOPE(level, __func__)`
 
 /// \def VTKM_LOG_ALWAYS_S(level, ...)
 /// This ostream-style log message is always emitted, even when logging is
@@ -187,6 +188,15 @@
 /// and then cast appropriately, as described here:
 /// https://gitlab.kitware.com/vtk/vtk-m/-/issues/358#note_550157
 
+#define VTKM_CONCAT_IMPL(s1, s2) s1##s2
+#define VTKM_CONCAT(s1, s2) VTKM_CONCAT_IMPL(s1, s2)
+
+#ifdef __COUNTER__
+#define VTKM_ANONYMOUS_VARIABLE VTKM_CONCAT(vtk_m_anonymous_, __COUNTER__)
+#else
+#define VTKM_ANONYMOUS_VARIABLE VTKM_CONCAT(vtk_m_anonymous_, __LINE__)
+#endif
+
 #if defined(VTKM_ENABLE_LOGGING)
 
 #define VTKM_LOG_IF_S(level, cond, ...) \
@@ -198,14 +208,12 @@
 #define VTKM_LOG_S(level, ...) VTKM_LOG_IF_S(level, true, __VA_ARGS__)
 #define VTKM_LOG_F(level, ...) VTKM_LOG_IF_F(level, true, __VA_ARGS__)
 
-#define VTKM_LOG_SCOPE(level, ...) vtkm::cont::LogScope(level, __FILE__, __LINE__, __VA_ARGS__)
+#define VTKM_LOG_SCOPE(level, ...) \
+  vtkm::cont::detail::LogScope VTKM_ANONYMOUS_VARIABLE { level, __FILE__, __LINE__, __VA_ARGS__ }
 
 #define VTKM_LOG_SCOPE_FUNCTION(level) VTKM_LOG_SCOPE(level, __func__)
 #define VTKM_LOG_ALWAYS_S(level, ...) VTKM_LOG_S(level, __VA_ARGS__)
 
-// VTKM_LOG_ERROR_CONTEXT is disabled as it is deprecated
-#define VTKM_LOG_ERROR_CONTEXT(desc, data)
-
 
 // Convenience macros:
 
@@ -289,7 +297,7 @@ namespace cont
 /// Log levels for use with the logging macros.
 enum class LogLevel
 {
-  /// Used with SetStderrLogLevel to silence the log. Do not actually log to
+  /// A placeholder used to silence all logging. Do not actually log to
   /// this level.
   Off = -9, //loguru::Verbosity_OFF,
 
@@ -306,36 +314,41 @@ enum class LogLevel
   /// output.
   Info = 0, //loguru::Verbosity_INFO,
 
-  /// The range 1-255 are reserved to application use.
+  /// The first in a range of logging levels reserved for code that uses VTK-m.
+  /// Internal VTK-m code will not log on these levels but will report these logs.
   UserFirst = 1,
-  /// The range 1-255 are reserved to application use.
+  /// The last in a range of logging levels reserved for code that uses VTK-m.
   UserLast = 255,
 
-  /// Information about which devices are enabled/disabled
+  /// Information about which devices are enabled/disabled.
   DevicesEnabled,
 
   /// General timing data and algorithm flow information, such as filter
   /// execution, worklet dispatches, and device algorithm calls.
   Perf,
 
-  /// Host-side resource allocations/frees (e.g. ArrayHandle control buffers)
+  /// Host-side resource allocations/frees (e.g. ArrayHandle control buffers).
   MemCont,
 
-  /// Device-side resource allocations/frees (e.g ArrayHandle device buffers)
+  /// Device-side resource allocations/frees (e.g ArrayHandle device buffers).
   MemExec,
 
-  /// Host->device / device->host data copies
+  /// Transferring of data between a host and device.
   MemTransfer,
 
-  /// Details on Device-side Kernel Launches
+  /// Details on device-side kernel launches.
   KernelLaunches,
 
-  /// When a dynamic object is (or isn't) resolved via CastAndCall, etc.
+  /// Reports when a dynamic object is (or is not) resolved via a CastAndCall or other
+  /// casting method.
   Cast,
 
-  /// 1024-2047 are reserved for application usage.
+  /// The first in a range of logging levels reserved for code that uses VTK-m.
+  /// Internal VTK-m code will not log on these levels but will report these logs.
+  /// These are used similarly to those in the UserFirst range but are at a lower
+  /// precedence that also includes more verbose reporting from VTK-m.
   UserVerboseFirst = 1024,
-  /// 1024-2047 are reserved for application usage.
+  /// The last in a range of logging levels reserved for code that uses VTK-m.
   UserVerboseLast = 2047
 };
 
@@ -408,7 +421,7 @@ VTKM_CONT
 std::string GetLogLevelName(vtkm::cont::LogLevel level);
 
 /**
- * The name to identify the current thread in the log output.
+ * Specifies a humman-readable name to identify the current thread in the log output.
  * @{
  */
 VTKM_CONT_EXPORT
@@ -434,7 +447,7 @@ VTKM_CONT
 std::string GetStackTrace(vtkm::Int32 skip = 0);
 
 //@{
-/// Convert a size in bytes to a human readable string (e.g. "64 bytes",
+/// Convert a size in bytes to a human readable string (such as "64 bytes",
 /// "1.44 MiB", "128 GiB", etc). @a prec controls the fixed point precision
 /// of the stringified number.
 VTKM_CONT_EXPORT
@@ -450,7 +463,7 @@ VTKM_CONT inline std::string GetHumanReadableSize(T&& bytes, int prec = 2)
 
 //@{
 /// Returns "%1 (%2 bytes)" where %1 is the result from GetHumanReadableSize
-/// and two is the exact number of bytes.
+/// and %2 is the exact number of bytes.
 VTKM_CONT_EXPORT
 VTKM_CONT
 std::string GetSizeString(vtkm::UInt64 bytes, int prec = 2);
@@ -488,25 +501,43 @@ inline VTKM_CONT std::string TypeToString(const T&)
  *
  * \param level  Desired LogLevel value for the log message.
  * \param cond   When false this function is no-op.
+ * \param file   The source file where the log entry was genearted.
+ * \param line   The line in the source file where the log entry was generated.
  * \param format Printf like format string.
  */
 VTKM_CONT_EXPORT
 VTKM_CONT
 void LogCond(LogLevel level, bool cond, const char* file, unsigned line, const char* format...);
 
+namespace detail
+{
+
 /**
  * \brief Logs a scoped message with a printf-like format.
  *
  * The indentation level will be determined based on its LogLevel and it will
- * print out its wall time upon exiting its scope.
- *
- * \param level  Desired LogLevel value for the log message.
- * \param cond   When false this function is no-op.
- * \param format Printf like format string.
+ * print out its wall time upon exiting its scope. The scope starts from when
+ * the object is created to when it is destroyed.
  */
-VTKM_CONT_EXPORT
-VTKM_CONT
-void LogScope(LogLevel level, const char* file, unsigned line, const char* format...);
+class VTKM_CONT_EXPORT LogScope
+{
+  struct InternalStruct;
+  std::unique_ptr<InternalStruct> Internals;
+
+public:
+  /*
+   * \param level  Desired LogLevel value for the log message.
+   * \param cond   When false this function is no-op.
+   * \param format Printf like format string.
+   */
+  VTKM_CONT
+  LogScope(LogLevel level, const char* file, unsigned line, const char* format...);
+
+  VTKM_CONT ~LogScope();
+};
+
+
+} // namespace detail
 
 /**
  * \brief Conditionally logs a message with a stream-like interface.
