@@ -11,7 +11,7 @@
 #ifndef vtk_m_filter_zfp_ZFPCompressor2D_h
 #define vtk_m_filter_zfp_ZFPCompressor2D_h
 
-#include <vtkm/filter/NewFilterField.h>
+#include <vtkm/filter/Filter.h>
 #include <vtkm/filter/zfp/vtkm_filter_zfp_export.h>
 
 namespace vtkm
@@ -20,16 +20,19 @@ namespace filter
 {
 namespace zfp
 {
-/// \brief Compress a scalar field using ZFP
 
-/// Takes as input a 1D array and generates on
+/// \brief Compress a scalar field using ZFP.
+///
+/// Takes as input a 2D array and generates an
 /// output of compressed data.
 /// @warning
-/// This filter is currently only supports 1D volumes.
-class VTKM_FILTER_ZFP_EXPORT ZFPCompressor2D : public vtkm::filter::NewFilterField
+/// This filter is currently only supports 2D structured cell sets.
+class VTKM_FILTER_ZFP_EXPORT ZFPCompressor2D : public vtkm::filter::Filter
 {
 public:
+  /// @brief Specifies the rate of compression.
   void SetRate(vtkm::Float64 _rate) { rate = _rate; }
+  /// @copydoc SetRate
   vtkm::Float64 GetRate() { return rate; }
 
 private:
@@ -37,12 +40,8 @@ private:
 
   vtkm::Float64 rate = 0;
 };
+
 } // namespace zfp
-class VTKM_DEPRECATED(1.8, "Use vtkm::filter::zfp::ZFPCompressor2D.") ZFPCompressor2D
-  : public vtkm::filter::zfp::ZFPCompressor2D
-{
-  using zfp::ZFPCompressor2D::ZFPCompressor2D;
-};
 } // namespace filter
 } // namespace vtkm
 
