@@ -18,6 +18,10 @@ namespace vtkm
 namespace rendering
 {
 
+/// @brief A mapper that produces unoriented glyphs.
+///
+/// This mapper is meant to be used with scalar fields. The glyphs can be optionally
+/// sized based on the field.
 class VTKM_RENDERING_EXPORT MapperGlyphScalar : public vtkm::rendering::MapperGlyphBase
 {
 public:
@@ -25,20 +29,23 @@ public:
 
   ~MapperGlyphScalar();
 
+  /// @brief Specify the shape of the glyphs.
   vtkm::rendering::GlyphType GetGlyphType() const;
+  /// @copydoc GetGlyphType
   void SetGlyphType(vtkm::rendering::GlyphType glyphType);
 
-  void RenderCells(const vtkm::cont::UnknownCellSet& cellset,
-                   const vtkm::cont::CoordinateSystem& coords,
-                   const vtkm::cont::Field& scalarField,
-                   const vtkm::cont::ColorTable& colorTable,
-                   const vtkm::rendering::Camera& camera,
-                   const vtkm::Range& scalarRange) override;
-
   vtkm::rendering::Mapper* NewCopy() const override;
 
 protected:
   vtkm::rendering::GlyphType GlyphType;
+
+  void RenderCellsImpl(const vtkm::cont::UnknownCellSet& cellset,
+                       const vtkm::cont::CoordinateSystem& coords,
+                       const vtkm::cont::Field& scalarField,
+                       const vtkm::cont::ColorTable& colorTable,
+                       const vtkm::rendering::Camera& camera,
+                       const vtkm::Range& scalarRange,
+                       const vtkm::cont::Field& ghostField) override;
 };
 }
 } //namespace vtkm::rendering
