@@ -196,21 +196,23 @@ void ConnectivityTracer::SetColorMap(const vtkm::cont::ArrayHandle<vtkm::Vec4f_3
 void ConnectivityTracer::SetVolumeData(const vtkm::cont::Field& scalarField,
                                        const vtkm::Range& scalarBounds,
                                        const vtkm::cont::UnknownCellSet& cellSet,
-                                       const vtkm::cont::CoordinateSystem& coords)
+                                       const vtkm::cont::CoordinateSystem& coords,
+                                       const vtkm::cont::Field& ghostField)
 {
   //TODO: Need a way to tell if we have been updated
   ScalarField = scalarField;
+  GhostField = ghostField;
   ScalarBounds = scalarBounds;
   CellSet = cellSet;
   Coords = coords;
   MeshConnIsConstructed = false;
 
-  const bool isSupportedField = ScalarField.IsFieldCell() || ScalarField.IsFieldPoint();
+  const bool isSupportedField = ScalarField.IsCellField() || ScalarField.IsPointField();
   if (!isSupportedField)
   {
     throw vtkm::cont::ErrorBadValue("Field not accociated with cell set or points");
   }
-  FieldAssocPoints = ScalarField.IsFieldPoint();
+  FieldAssocPoints = ScalarField.IsPointField();
 
   this->Integrator = Volume;
 
@@ -435,14 +437,12 @@ public:
 
     FloatType minDistance = static_cast<FloatType>(1e32);
     FloatType maxDistance = static_cast<FloatType>(-1);
-    int hitCount = 0;
     for (vtkm::Int32 i = 0; i < numFaces; ++i)
     {
       FloatType dist = distances[i];
 
       if (dist != -1)
       {
-        hitCount++;
         if (dist < minDistance)
         {
           minDistance = dist;
@@ -564,14 +564,12 @@ public:
       vtkm::Int32 maxFace = -1;
       FloatType minDistance = static_cast<FloatType>(1e32);
       FloatType maxDistance = static_cast<FloatType>(-1);
-      int hitCount = 0;
       for (int i = 0; i < numFaces; ++i)
       {
         FloatType dist = distances[i];
 
         if (dist != -1)
         {
-          hitCount++;
           if (dist < minDistance)
           {
             minDistance = dist;
@@ -896,8 +894,8 @@ public:
     InvDeltaScalar = (minScalar == maxScalar) ? 1.f : 1.f / (maxScalar - minScalar);
   }
 
-
   using ControlSignature = void(FieldIn,
+                                WholeArrayIn,
                                 WholeArrayIn,
                                 FieldIn,
                                 FieldIn,
@@ -906,11 +904,15 @@ public:
                                 WholeArrayIn,
                                 WholeArrayInOut,
                                 FieldIn);
-  using ExecutionSignature = void(_1, _2, _3, _4, _5, _6, _7, _8, WorkIndex, _9);
+  using ExecutionSignature = void(_1, _2, _3, _4, _5, _6, _7, _8, _9, WorkIndex, _10);
 
-  template <typename ScalarPortalType, typename ColorMapType, typename FrameBufferType>
+  template <typename ScalarPortalType,
+            typename GhostPortalType,
+            typename ColorMapType,
+            typename FrameBufferType>
   VTKM_EXEC inline void operator()(const vtkm::Id& currentCell,
                                    ScalarPortalType& scalarPortal,
+                                   GhostPortalType& ghostPortal,
                                    const FloatType& enterDistance,
                                    const FloatType& exitDistance,
                                    FloatType& currentDistance,
@@ -923,6 +925,8 @@ public:
 
     if (rayStatus != RAY_ACTIVE)
       return;
+    if (int(ghostPortal.Get(currentCell)) != 0)
+      return;
 
     vtkm::Vec4f_32 color;
     BOUNDS_CHECK(frameBuffer, pixelIndex * 4 + 0);
@@ -1232,6 +1236,7 @@ void ConnectivityTracer::SampleCells(Ray<FloatType>& rays, detail::RayTracking<F
 
     dispatcher.Invoke(rays.HitIdx,
                       vtkm::rendering::raytracing::GetScalarFieldArray(this->ScalarField),
+                      GhostField.GetData().ExtractComponent<vtkm::UInt8>(0),
                       *(tracker.EnterDist),
                       *(tracker.ExitDist),
                       tracker.CurrentDistance,
