@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   ParaView
-  Module:    vtkPVRepresentedDataInformation.h
-
-  Copyright (c) Kitware, Inc.
-  All rights reserved.
-  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkPVRepresentedDataInformation
  *
@@ -41,6 +29,11 @@ protected:
   vtkPVRepresentedDataInformation();
   ~vtkPVRepresentedDataInformation() override;
 
+  /**
+   * Simplifies a composite dataset by merging leafs of partitioned datasets for faster processing.
+   */
+  vtkSmartPointer<vtkCompositeDataSet> SimplifyCompositeDataSet(vtkCompositeDataSet* cd) override;
+
 private:
   vtkPVRepresentedDataInformation(const vtkPVRepresentedDataInformation&) = delete;
   void operator=(const vtkPVRepresentedDataInformation&) = delete;
