@@ -16,10 +16,11 @@
 pqSignalAdaptorComboBox::pqSignalAdaptorComboBox(QComboBox* p)
   : QObject(p)
 {
-  QObject::connect(p, SIGNAL(currentIndexChanged(const QString&)), this,
-    SIGNAL(currentTextChanged(const QString&)));
+  QObject::connect(
+    p, &QComboBox::currentTextChanged, this, &pqSignalAdaptorComboBox::currentTextChanged);
 
-  QObject::connect(p, SIGNAL(currentIndexChanged(int)), this, SIGNAL(currentIndexChanged(int)));
+  QObject::connect(p, QOverload<int>::of(&QComboBox::currentIndexChanged), this,
+    &pqSignalAdaptorComboBox::currentIndexChanged);
 }
 
 //----------------------------------------------------------------------------
