@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkPythonArgs.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 /*-----------------------------------------------------------------------
 The vtkPythonArgs class was created in Oct 2010 by David Gobbi.
 
@@ -26,11 +14,13 @@ resulting in wrapper code that is faster and more compact.
 
 #include "vtkPythonArgs.h"
 #include "PyVTKReference.h"
+#include "vtkABINamespace.h"
 #include "vtkPythonUtil.h"
 
 #include "vtkObject.h"
 #include "vtkSmartPointerBase.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 //------------------------------------------------------------------------------
 // Extract various C++ types from python objects.  The rules are
 // identical to PyArg_ParseTuple except that range checking is done
@@ -51,7 +41,7 @@ inline bool vtkPythonGetValue(PyObject* o, long& a)
 {
   VTK_PYTHON_FLOAT_CHECK();
 
-  a = PyInt_AsLong(o);
+  a = PyLong_AsLong(o);
   return (a != static_cast<long>(-1) || !PyErr_Occurred());
 }
 
@@ -95,25 +85,17 @@ Py_ssize_t vtkPythonGetStringSize(PyObject* o)
 {
   if (PyBytes_Check(o))
   {
-    return PyBytes_GET_SIZE(o);
+    return PyBytes_Size(o);
   }
   else if (PyByteArray_Check(o))
   {
-    return PyByteArray_GET_SIZE(o);
+    return PyByteArray_Size(o);
   }
   else if (PyUnicode_Check(o))
   {
-#ifdef VTK_PY3K
     Py_ssize_t size;
     PyUnicode_AsUTF8AndSize(o, &size);
     return size;
-#else
-    PyObject* s = _PyUnicode_AsDefaultEncodedString(o, nullptr);
-    if (s)
-    {
-      return PyBytes_GET_SIZE(s);
-    }
-#endif
   }
   return 0;
 }
@@ -122,33 +104,18 @@ bool vtkPythonGetStringValue(PyObject* o, const char*& a, const char* exctext)
 {
   if (PyBytes_Check(o))
   {
-    a = PyBytes_AS_STRING(o);
+    a = PyBytes_AsString(o);
     return true;
   }
   else if (PyByteArray_Check(o))
   {
-    a = PyByteArray_AS_STRING(o);
+    a = PyByteArray_AsString(o);
     return true;
   }
   else if (PyUnicode_Check(o))
   {
-#ifdef VTK_PY3K
-    a = PyUnicode_AsUTF8(o);
+    a = PyUnicode_AsUTF8AndSize(o, nullptr);
     return true;
-#else
-    PyObject* s = _PyUnicode_AsDefaultEncodedString(o, nullptr);
-    if (s)
-    {
-      a = PyBytes_AS_STRING(s);
-      return true;
-    }
-
-    if (exctext)
-    {
-      // set a more specific error message
-      exctext = "(unicode conversion error)";
-    }
-#endif
   }
 
   if (exctext)
@@ -170,24 +137,10 @@ inline bool vtkPythonGetStdStringValue(PyObject* o, std::string& a, const char*
   }
   else if (PyUnicode_Check(o))
   {
-#ifdef VTK_PY3K
     Py_ssize_t len;
     const char* val = PyUnicode_AsUTF8AndSize(o, &len);
     a = std::string(val, len);
     return true;
-#else
-    PyObject* s = _PyUnicode_AsDefaultEncodedString(o, nullptr);
-    if (s)
-    {
-      char* val;
-      Py_ssize_t len;
-      PyBytes_AsStringAndSize(s, &val, &len);
-      a = std::string(val, len);
-      return true;
-    }
-
-    exctext = "(unicode conversion error)";
-#endif
   }
 
   PyErr_SetString(PyExc_TypeError, exctext);
@@ -204,23 +157,14 @@ static bool vtkPythonGetValue(PyObject* o, const void*& a, Py_buffer* view, char
   void* p = nullptr;
   Py_ssize_t sz = 0;
   const char* format = nullptr;
-#ifndef VTK_PY3K
-  PyBufferProcs* b = Py_TYPE(o)->tp_as_buffer;
-#endif
 
-#if PY_VERSION_HEX < 0x02060000
-  (void)view;
-#else
-#ifdef VTK_PY3K
   PyObject* bytes = nullptr;
   if (PyUnicode_Check(o))
   {
     bytes = PyUnicode_AsUTF8String(o);
     PyBytes_AsStringAndSize(bytes, reinterpret_cast<char**>(&p), &sz);
   }
-  else
-#endif
-    if (PyObject_CheckBuffer(o))
+  else if (PyObject_CheckBuffer(o))
   {
     int flags = (PyBUF_ANY_CONTIGUOUS | PyBUF_FORMAT);
     if (btype == '\0')
@@ -253,51 +197,18 @@ static bool vtkPythonGetValue(PyObject* o, const void*& a, Py_buffer* view, char
       }
     }
   }
-#ifndef VTK_PY3K
-  else
-#endif
-#endif
-#ifndef VTK_PY3K
-  // use the old buffer interface
-  if (b && b->bf_getreadbuffer && b->bf_getsegcount)
-  {
-    if (b->bf_getsegcount(o, nullptr) == 1)
-    {
-      sz = b->bf_getreadbuffer(o, 0, &p);
-    }
-    else
-    {
-      PyErr_SetString(PyExc_TypeError, "buffer must be single-segment");
-      return false;
-    }
-  }
-#endif
 
-#ifdef VTK_PY3K
   if (bytes && btype == '\0')
-#else
-    if (p && sz >= 0 && sz <= VTK_INT_MAX && btype == '\0' &&
-      (format == nullptr || format[0] == 'c' || format[0] == 'B'))
-#endif
   {
     // check for pointer mangled as string
     int s = static_cast<int>(sz);
     a = vtkPythonUtil::UnmanglePointer(reinterpret_cast<char*>(p), &s, "p_void");
-#ifdef VTK_PY3K
     Py_DECREF(bytes);
     if (s != 0)
     {
       PyErr_SetString(PyExc_TypeError, "requires a _addr_p_void string");
       return false;
     }
-#else
-    if (s == -1)
-    {
-      // matched _addr_ but not p_void, assume it isn't a swig ptr string:
-      // use the buffer's pointer as the argument
-      a = p;
-    }
-#endif
     return true;
   }
   else if (p && sz >= 0)
@@ -335,16 +246,34 @@ inline bool vtkPythonGetValue(PyObject* o, std::string& a)
 
 inline bool vtkPythonGetValue(PyObject* o, char& a)
 {
-  static const char exctext[] = "a string of length 1 is required";
+  static const char exctext[] = "string of length 1, ord < 256 is required";
   const char* b;
   if (vtkPythonGetStringValue(o, b, exctext))
   {
-    if (b[0] == '\0' || b[1] == '\0')
+    if (PyObject_Length(o) == 1)
     {
-      a = b[0];
-      return true;
+      if (b[0] == '\0' || b[1] == '\0')
+      {
+        a = b[0];
+        return true;
+      }
+      else if (PyUnicode_Check(o))
+      {
+        if (b[0] == '\xc2' && b[2] == '\0')
+        {
+          // one unicode character, and in the range [128,191]
+          a = b[1];
+          return true;
+        }
+        else if (b[0] == '\xc3' && b[2] == '\0')
+        {
+          // one unicode character, and in the range [192,255]
+          a = b[1] | '\x40';
+          return true;
+        }
+      }
     }
-    PyErr_SetString(PyExc_TypeError, exctext);
+    PyErr_SetString(PyExc_ValueError, exctext);
   }
   return false;
 }
@@ -541,13 +470,13 @@ inline bool vtkPythonGetArray(PyObject* o, T* a, size_t n)
 
     if (PyTuple_Check(o))
     {
-      m = PyTuple_GET_SIZE(o);
+      m = PyTuple_Size(o);
       if (m == static_cast<Py_ssize_t>(n))
       {
         bool r = true;
         for (Py_ssize_t i = 0; i < m && r; i++)
         {
-          PyObject* s = PyTuple_GET_ITEM(o, i);
+          PyObject* s = PyTuple_GetItem(o, i);
           r = vtkPythonGetValue(s, a[i]);
         }
         return r;
@@ -555,13 +484,13 @@ inline bool vtkPythonGetArray(PyObject* o, T* a, size_t n)
     }
     else if (PyList_Check(o))
     {
-      m = PyList_GET_SIZE(o);
+      m = PyList_Size(o);
       if (m == static_cast<Py_ssize_t>(n))
       {
         bool r = true;
         for (Py_ssize_t i = 0; i < m && r; i++)
         {
-          PyObject* s = PyList_GET_ITEM(o, i);
+          PyObject* s = PyList_GetItem(o, i);
           r = vtkPythonGetValue(s, a[i]);
         }
         return r;
@@ -659,7 +588,7 @@ bool vtkPythonGetNArray(PyObject* o, T* a, int ndim, const size_t* dims)
 
     if (PyList_Check(o))
     {
-      m = PyList_GET_SIZE(o);
+      m = PyList_Size(o);
       if (m == static_cast<Py_ssize_t>(n))
       {
         bool r = true;
@@ -667,7 +596,7 @@ bool vtkPythonGetNArray(PyObject* o, T* a, int ndim, const size_t* dims)
         {
           for (Py_ssize_t i = 0; i < m && r; i++)
           {
-            PyObject* s = PyList_GET_ITEM(o, i);
+            PyObject* s = PyList_GetItem(o, i);
             r = vtkPythonGetNArray(s, a, ndim - 1, dims + 1);
             a += inc;
           }
@@ -676,7 +605,7 @@ bool vtkPythonGetNArray(PyObject* o, T* a, int ndim, const size_t* dims)
         {
           for (Py_ssize_t i = 0; i < m && r; i++)
           {
-            PyObject* s = PyList_GET_ITEM(o, i);
+            PyObject* s = PyList_GetItem(o, i);
             r = vtkPythonGetValue(s, a[i]);
           }
         }
@@ -729,7 +658,7 @@ inline bool vtkPythonSetArray(PyObject* o, const T* a, size_t n)
 
     if (PyList_Check(o))
     {
-      m = PyList_GET_SIZE(o);
+      m = PyList_Size(o);
       if (m == static_cast<Py_ssize_t>(n))
       {
         bool r = true;
@@ -739,8 +668,7 @@ inline bool vtkPythonSetArray(PyObject* o, const T* a, size_t n)
           PyObject* s = vtkPythonArgs::BuildValue(a[i]);
           if (s)
           {
-            Py_DECREF(PyList_GET_ITEM(o, i));
-            PyList_SET_ITEM(o, i, s);
+            PyList_SetItem(o, i, s);
             r = true;
           }
         }
@@ -781,10 +709,10 @@ inline bool vtkPythonSetArray(PyObject* o, const char* a, size_t n)
 
     if (PyByteArray_Check(o))
     {
-      m = PyByteArray_GET_SIZE(o);
+      m = PyByteArray_Size(o);
       if (m == static_cast<Py_ssize_t>(n))
       {
-        char* b = PyByteArray_AS_STRING(o);
+        char* b = PyByteArray_AsString(o);
         for (Py_ssize_t i = 0; i < m; i++)
         {
           b[i] = a[i];
@@ -837,7 +765,7 @@ bool vtkPythonSetNArray(PyObject* o, const T* a, int ndim, const size_t* dims)
 
     if (PyList_Check(o))
     {
-      m = PyList_GET_SIZE(o);
+      m = PyList_Size(o);
       if (m == static_cast<Py_ssize_t>(n))
       {
         bool r = true;
@@ -845,7 +773,7 @@ bool vtkPythonSetNArray(PyObject* o, const T* a, int ndim, const size_t* dims)
         {
           for (Py_ssize_t i = 0; i < m && r; i++)
           {
-            PyObject* s = PyList_GET_ITEM(o, i);
+            PyObject* s = PyList_GetItem(o, i);
             r = vtkPythonSetNArray(s, a, ndim - 1, dims + 1);
             a += inc;
           }
@@ -858,8 +786,7 @@ bool vtkPythonSetNArray(PyObject* o, const T* a, int ndim, const size_t* dims)
             PyObject* s = vtkPythonArgs::BuildValue(a[i]);
             if (s)
             {
-              Py_DECREF(PyList_GET_ITEM(o, i));
-              PyList_SET_ITEM(o, i, s);
+              PyList_SetItem(o, i, s);
               r = true;
             }
           }
@@ -923,7 +850,7 @@ inline PyObject* vtkPythonBuildTuple(const T* a, size_t n)
     for (Py_ssize_t i = 0; i < m; i++)
     {
       PyObject* o = vtkPythonArgs::BuildValue(a[i]);
-      PyTuple_SET_ITEM(t, i, o);
+      PyTuple_SetItem(t, i, o);
     }
     return t;
   }
@@ -951,7 +878,7 @@ VTK_PYTHON_BUILD_TUPLE(unsigned long long)
 VTK_PYTHON_BUILD_TUPLE(std::string)
 
 // For an array of smart pointers
-PyObject* vtkPythonArgs::BuildTuple(vtkSmartPointerBase* a, size_t n)
+PyObject* vtkPythonArgs::BuildTuple(const vtkSmartPointerBase* a, size_t n)
 {
   if (a)
   {
@@ -962,11 +889,11 @@ PyObject* vtkPythonArgs::BuildTuple(vtkSmartPointerBase* a, size_t n)
       vtkObjectBase* ob = a[i].GetPointer();
       if (ob)
       {
-        PyTuple_SET_ITEM(t, i, vtkPythonUtil::GetObjectFromPointer(ob));
+        PyTuple_SetItem(t, i, vtkPythonUtil::GetObjectFromPointer(ob));
       }
       else
       {
-        PyTuple_SET_ITEM(t, i, Py_None);
+        PyTuple_SetItem(t, i, Py_None);
         Py_INCREF(Py_None);
       }
     }
@@ -1017,9 +944,9 @@ PyObject* vtkPythonArgs::GetSelfFromFirstArg(PyObject* self, PyObject* args)
   if (PyType_Check(self))
   {
     PyTypeObject* pytype = (PyTypeObject*)self;
-    if (PyTuple_GET_SIZE(args) > 0)
+    if (PyTuple_Size(args) > 0)
     {
-      self = PyTuple_GET_ITEM(args, 0);
+      self = PyTuple_GetItem(args, 0);
       if (PyObject_TypeCheck(self, pytype))
       {
         return self;
@@ -1027,8 +954,8 @@ PyObject* vtkPythonArgs::GetSelfFromFirstArg(PyObject* self, PyObject* args)
     }
 
     char buf[256];
-    snprintf(
-      buf, sizeof(buf), "unbound method requires a %.200s as the first argument", pytype->tp_name);
+    snprintf(buf, sizeof(buf), "unbound method requires a %.200s as the first argument",
+      vtkPythonUtil::GetTypeName(pytype));
     PyErr_SetString(PyExc_TypeError, buf);
     return nullptr;
   }
@@ -1042,7 +969,7 @@ PyObject* vtkPythonArgs::GetSelfFromFirstArg(PyObject* self, PyObject* args)
 
 PyObject* vtkPythonArgs::GetArgAsPythonObject(bool& valid)
 {
-  PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);
+  PyObject* o = PyTuple_GetItem(this->Args, this->I++);
   valid = true;
   return o;
 }
@@ -1055,7 +982,7 @@ PyObject* vtkPythonArgs::GetArgAsPythonObject(PyObject* o, bool& valid)
 
 vtkObjectBase* vtkPythonArgs::GetArgAsVTKObject(const char* classname, bool& valid)
 {
-  PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);
+  PyObject* o = PyTuple_GetItem(this->Args, this->I++);
   vtkObjectBase* r = vtkPythonArgs::GetArgAsVTKObject(o, classname, valid);
   if (!valid)
   {
@@ -1073,7 +1000,7 @@ vtkObjectBase* vtkPythonArgs::GetArgAsVTKObject(PyObject* o, const char* classna
 
 void* vtkPythonArgs::GetArgAsSpecialObject(const char* classname, PyObject** p)
 {
-  PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);
+  PyObject* o = PyTuple_GetItem(this->Args, this->I++);
   void* r = vtkPythonArgs::GetArgAsSpecialObject(o, classname, p);
   if (r == nullptr)
   {
@@ -1090,7 +1017,7 @@ void* vtkPythonArgs::GetArgAsSpecialObject(PyObject* o, const char* classname, P
 
 int vtkPythonArgs::GetArgAsEnum(const char* enumname, bool& valid)
 {
-  PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);
+  PyObject* o = PyTuple_GetItem(this->Args, this->I++);
   int i = vtkPythonArgs::GetArgAsEnum(o, enumname, valid);
   if (!valid)
   {
@@ -1105,7 +1032,7 @@ int vtkPythonArgs::GetArgAsEnum(PyObject* o, const char* enumname, bool& valid)
   PyTypeObject* pytype = vtkPythonUtil::FindEnum(enumname);
   if (pytype && PyObject_TypeCheck(o, pytype))
   {
-    i = PyInt_AsLong(o);
+    i = PyLong_AsLong(o);
     valid = true;
   }
   else
@@ -1113,7 +1040,7 @@ int vtkPythonArgs::GetArgAsEnum(PyObject* o, const char* enumname, bool& valid)
     std::string errstring = "expected enum ";
     errstring += enumname;
     errstring += ", got ";
-    errstring += Py_TYPE(o)->tp_name;
+    errstring += vtkPythonUtil::GetTypeNameForObject(o);
     PyErr_SetString(PyExc_TypeError, errstring.c_str());
     valid = false;
   }
@@ -1143,7 +1070,7 @@ bool vtkPythonArgs::GetVTKObject(PyObject* o, vtkSmartPointerBase& v, const char
 #define VTK_PYTHON_GET_ARG(T)                                                                      \
   bool vtkPythonArgs::GetValue(T& a)                                                               \
   {                                                                                                \
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);                                         \
+    PyObject* o = PyTuple_GetItem(this->Args, this->I++);                                          \
     if (PyVTKReference_Check(o))                                                                   \
     {                                                                                              \
       o = PyVTKReference_GetValue(o);                                                              \
@@ -1181,7 +1108,7 @@ VTK_PYTHON_GET_ARG(unsigned long long)
 #define VTK_PYTHON_GET_FILEPATH_ARG(T)                                                             \
   bool vtkPythonArgs::GetFilePath(T& a)                                                            \
   {                                                                                                \
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);                                         \
+    PyObject* o = PyTuple_GetItem(this->Args, this->I++);                                          \
     if (PyVTKReference_Check(o))                                                                   \
     {                                                                                              \
       o = PyVTKReference_GetValue(o);                                                              \
@@ -1205,7 +1132,7 @@ VTK_PYTHON_GET_FILEPATH_ARG(std::string)
 #define VTK_PYTHON_GET_ARRAY_ARG(T)                                                                \
   bool vtkPythonArgs::GetArray(T* a, size_t n)                                                     \
   {                                                                                                \
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);                                         \
+    PyObject* o = PyTuple_GetItem(this->Args, this->I++);                                          \
     if (vtkPythonGetArray(o, a, n))                                                                \
     {                                                                                              \
       return true;                                                                                 \
@@ -1233,7 +1160,7 @@ VTK_PYTHON_GET_ARRAY_ARG(std::string)
 // For an array of smart pointers
 bool vtkPythonArgs::GetArray(vtkSmartPointerBase* a, size_t n, const char* classname)
 {
-  PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);
+  PyObject* o = PyTuple_GetItem(this->Args, this->I++);
   if (a)
   {
     Py_ssize_t m = static_cast<Py_ssize_t>(n);
@@ -1275,7 +1202,7 @@ bool vtkPythonArgs::GetArray(vtkSmartPointerBase* a, size_t n, const char* class
 #define VTK_PYTHON_GET_NARRAY_ARG(T)                                                               \
   bool vtkPythonArgs::GetNArray(T* a, int ndim, const size_t* dims)                                \
   {                                                                                                \
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);                                         \
+    PyObject* o = PyTuple_GetItem(this->Args, this->I++);                                          \
     if (vtkPythonGetNArray(o, a, ndim, dims))                                                      \
     {                                                                                              \
       return true;                                                                                 \
@@ -1315,7 +1242,7 @@ bool vtkPythonArgs::GetFunction(PyObject* arg, PyObject*& o)
 
 bool vtkPythonArgs::GetFunction(PyObject*& o)
 {
-  PyObject* arg = PyTuple_GET_ITEM(this->Args, this->I++);
+  PyObject* arg = PyTuple_GetItem(this->Args, this->I++);
   return vtkPythonArgs::GetFunction(arg, o);
 }
 
@@ -1325,7 +1252,7 @@ bool vtkPythonArgs::GetFunction(PyObject*& o)
 #define VTK_PYTHON_GET_BUFFER(T, btype)                                                            \
   bool vtkPythonArgs::GetBuffer(T*& a, Py_buffer* buf)                                             \
   {                                                                                                \
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);                                         \
+    PyObject* o = PyTuple_GetItem(this->Args, this->I++);                                          \
     void* v;                                                                                       \
     if (vtkPythonGetValue(o, v, buf, btype))                                                       \
     {                                                                                              \
@@ -1338,7 +1265,7 @@ bool vtkPythonArgs::GetFunction(PyObject*& o)
                                                                                                    \
   bool vtkPythonArgs::GetBuffer(const T*& a, Py_buffer* buf)                                       \
   {                                                                                                \
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->I++);                                         \
+    PyObject* o = PyTuple_GetItem(this->Args, this->I++);                                          \
     const void* v;                                                                                 \
     if (vtkPythonGetValue(o, v, buf, btype))                                                       \
     {                                                                                              \
@@ -1395,7 +1322,7 @@ VTK_PYTHON_GET_BUFFER(unsigned long long, 'Q')
   {                                                                                                \
     if (this->M + i < this->N)                                                                     \
     {                                                                                              \
-      PyObject* m = PyTuple_GET_ITEM(this->Args, this->M + i);                                     \
+      PyObject* m = PyTuple_GetItem(this->Args, this->M + i);                                      \
       PyObject* o = vtkPythonArgs::BuildValue(a);                                                  \
       int r = PyVTKReference_SetValue(m, o);                                                       \
       if (r == 0)                                                                                  \
@@ -1413,7 +1340,7 @@ VTK_PYTHON_GET_BUFFER(unsigned long long, 'Q')
   {                                                                                                \
     if (this->M + i < this->N)                                                                     \
     {                                                                                              \
-      PyObject* m = PyTuple_GET_ITEM(this->Args, this->M + i);                                     \
+      PyObject* m = PyTuple_GetItem(this->Args, this->M + i);                                      \
       PyObject* o = vtkPythonArgs::BuildTuple(a, n);                                               \
       int r = PyVTKReference_SetValue(m, o);                                                       \
       if (r == 0)                                                                                  \
@@ -1463,7 +1390,7 @@ VTK_PYTHON_SET_ARGN(unsigned long long)
   {                                                                                                \
     if (this->M + i < this->N)                                                                     \
     {                                                                                              \
-      PyObject* o = PyTuple_GET_ITEM(this->Args, this->M + i);                                     \
+      PyObject* o = PyTuple_GetItem(this->Args, this->M + i);                                      \
       if (vtkPythonSetArray(o, a, n))                                                              \
       {                                                                                            \
         return true;                                                                               \
@@ -1497,7 +1424,7 @@ VTK_PYTHON_SET_ARRAY_ARG(unsigned long long)
   {                                                                                                \
     if (this->M + i < this->N)                                                                     \
     {                                                                                              \
-      PyObject* o = PyTuple_GET_ITEM(this->Args, this->M + i);                                     \
+      PyObject* o = PyTuple_GetItem(this->Args, this->M + i);                                      \
       if (vtkPythonSetNArray(o, a, ndim, dims))                                                    \
       {                                                                                            \
         return true;                                                                               \
@@ -1529,7 +1456,7 @@ bool vtkPythonArgs::SetContents(int i, PyObject* seq)
 {
   if (this->M + i < this->N)
   {
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->M + i);
+    PyObject* o = PyTuple_GetItem(this->Args, this->M + i);
     Py_ssize_t l = PySequence_Size(o);
     if (l >= 0 && PySequence_SetSlice(o, 0, l, seq) != -1)
     {
@@ -1605,7 +1532,6 @@ bool vtkPythonArgs::RefineArgTypeError(Py_ssize_t i)
 
     PyErr_Fetch(&exc, &val, &frame);
 
-#ifdef VTK_PY3K
     const char* cp = "";
     if (val && !PyUnicode_Check(val))
     {
@@ -1614,15 +1540,6 @@ bool vtkPythonArgs::RefineArgTypeError(Py_ssize_t i)
     }
     newval = PyUnicode_FromFormat(
       "%s argument %" PY_FORMAT_SIZE_T "d: %V", this->MethodName, i + 1, val, cp);
-#else
-    const char* cp = "";
-    if (val && PyString_Check(val))
-    {
-      cp = PyString_AsString(val);
-    }
-    newval =
-      PyString_FromFormat("%s argument %" PY_FORMAT_SIZE_T "d: %s", this->MethodName, i + 1, cp);
-#endif
 
     Py_XDECREF(val);
     PyErr_Restore(exc, newval, frame);
@@ -1638,7 +1555,7 @@ bool vtkPythonSequenceError(PyObject* o, size_t n, size_t m)
   if (m == n)
   {
     snprintf(text, sizeof(text), "expected a sequence of %lld value%s, got %s",
-      static_cast<long long>(n), ((n == 1) ? "" : "s"), Py_TYPE(o)->tp_name);
+      static_cast<long long>(n), ((n == 1) ? "" : "s"), vtkPythonUtil::GetTypeNameForObject(o));
   }
   else
   {
@@ -1656,7 +1573,7 @@ size_t vtkPythonArgs::GetArgSize(int i)
   size_t size = 0;
   if (this->M + i < this->N)
   {
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->M + i);
+    PyObject* o = PyTuple_GetItem(this->Args, this->M + i);
     if (PySequence_Check(o))
     {
       size = PySequence_Size(o);
@@ -1672,7 +1589,7 @@ size_t vtkPythonArgs::GetStringSize(int i)
   size_t size = 0;
   if (this->M + i < this->N)
   {
-    PyObject* o = PyTuple_GET_ITEM(this->Args, this->M + i);
+    PyObject* o = PyTuple_GetItem(this->Args, this->M + i);
     size = vtkPythonGetStringSize(o);
     if (size == 0 && PySequence_Check(o))
     {
@@ -1690,7 +1607,7 @@ bool vtkPythonArgs::CheckSizeHint(int i, size_t m, size_t n)
   {
     if (m != n)
     {
-      PyObject* o = PyTuple_GET_ITEM(this->Args, this->M + i);
+      PyObject* o = PyTuple_GetItem(this->Args, this->M + i);
       return vtkPythonSequenceError(o, n, m);
     }
   }
@@ -1715,3 +1632,4 @@ vtkPythonArgs::Array<T>::Array(size_t n)
 
 // Instantiate the Array class template over all types:
 vtkPythonArgsTemplateMacro(template class VTKWRAPPINGPYTHONCORE_EXPORT vtkPythonArgs::Array);
+VTK_ABI_NAMESPACE_END
