@@ -1,24 +1,6 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkParseData.c
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
-/*-------------------------------------------------------------------------
-  Copyright (c) 2010 David Gobbi.
-
-  Contributed to the VisualizationToolkit by the author in May 2010
-  under the terms of the Visualization Toolkit 2008 copyright.
--------------------------------------------------------------------------*/
-
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-FileCopyrightText: Copyright (c) 2010 David Gobbi
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkParseData.h"
 #include <stdlib.h>
 #include <string.h>
@@ -141,6 +123,8 @@ void vtkParse_InitFunction(FunctionInfo* func)
   func->Template = NULL;
   func->NumberOfParameters = 0;
   func->Parameters = NULL;
+  func->MarshalPropertyName = NULL;
+  func->MarshalExcludeReason = NULL;
   func->ReturnValue = NULL;
   func->NumberOfPreconds = 0;
   func->Preconds = NULL;
@@ -160,6 +144,7 @@ void vtkParse_InitFunction(FunctionInfo* func)
   func->IsExplicit = 0;
   func->IsExcluded = 0;
   func->IsDeprecated = 0;
+  func->IsMarshalExcluded = 0;
 
 #ifndef VTK_PARSE_LEGACY_REMOVE
   /* everything below here is legacy information, *
@@ -196,6 +181,9 @@ void vtkParse_CopyFunction(FunctionInfo* func, const FunctionInfo* orig)
   func->Signature = orig->Signature;
   func->Template = NULL;
 
+  func->MarshalPropertyName = orig->MarshalPropertyName;
+  func->MarshalExcludeReason = orig->MarshalExcludeReason;
+
   if (orig->Template)
   {
     func->Template = (TemplateInfo*)malloc(sizeof(TemplateInfo));
@@ -249,6 +237,7 @@ void vtkParse_CopyFunction(FunctionInfo* func, const FunctionInfo* orig)
   func->IsLegacy = orig->IsLegacy;
   func->IsExcluded = orig->IsExcluded;
   func->IsDeprecated = orig->IsDeprecated;
+  func->IsMarshalExcluded = orig->IsMarshalExcluded;
 
 #ifndef VTK_PARSE_LEGACY_REMOVE
   /* everything below here is legacy information, *
@@ -407,7 +396,7 @@ void vtkParse_CopyEnum(EnumInfo* item, const EnumInfo* orig)
 /* Free an Enum struct */
 void vtkParse_FreeEnum(EnumInfo* enum_info)
 {
-  free(enum_info);
+  vtkParse_FreeClass(enum_info);
 }
 
 /* Initialize a Using struct */
@@ -440,6 +429,7 @@ void vtkParse_FreeUsing(UsingInfo* using_info)
 void vtkParse_InitClass(ClassInfo* cls)
 {
   cls->ItemType = VTK_CLASS_INFO;
+  cls->MarshalType = VTK_MARSHAL_NONE;
   cls->Access = VTK_ACCESS_PUBLIC;
   cls->Name = NULL;
   cls->Comment = NULL;
@@ -480,6 +470,7 @@ void vtkParse_CopyClass(ClassInfo* cls, const ClassInfo* orig)
 {
   int i, n;
 
+  cls->MarshalType = orig->MarshalType;
   cls->ItemType = orig->ItemType;
   cls->Access = orig->Access;
   cls->Name = orig->Name;
